.TH  host 1 "Release 0.10" Monet
.SH NAME
host \- look up host names using domain server
.SH SYNOPSIS 
.na
.nf
\fBhost\fP [\fB-v\fP] [\fB-a\fP] [\fB-t\fP \fIquerytype\fP] [\fIoptions\fP]  \fIname\fP  [\fIserver\fP] 
.br
\fBhost\fP [\fB-v\fP] [\fB-a\fP] [\fB-t\fP \fIquerytype\fP] [\fIoptions\fP]  \fB-l\fP \fIdomain\fP  [\fIserver\fP] 
.br
\fBhost\fP [\fB-v\fP] [\fIoptions\fP]  \fB-H\fP \fIdomain\fP
.br
\fBhost\fP [\fB-v\fP] [\fIoptions\fP]  \fB-C\fP \fIdomain\fP
.br
\fBhost\fP [\fB-v\fP] [\fIoptions\fP]  \fB-A\fP \fIhost\fP
.SH DESCRIPTION 
.I host
looks for information about Internet hosts or domains.
It gets this information from a set of interconnected servers
that are spread across the world.
By default, it simply converts between host names and Internet
addresses.  However, with the \fB-t\fP, \fB-a\fP and \fB-v\fP options,
it can be used to find all of the information about this host or
domain that is maintained by the domain server.
.PP
The arguments can be either host names (domain names) or host numbers.
The program first attempts to interpret them as host numbers.
If this fails, it will treat them as host names.
.sp
A host number consists of
four decimal numbers separated by dots, e.g. \fB192.16.199.1\fP
.sp
A host name or domain name
consists of names separated by dots, e.g. \fBnikhefh.nikhef.nl\fP.
.sp
For single names without a trailing dot, the local domain
is automatically tacked on the end.  Thus a user in domain "nikhef.nl"
can say \fBhost nikhapo\fP, and it will actually look up "nikhapo.nikhef.nl".
In all other cases, the name is tried unchanged.
Single names with trailing dot are considered top-level domains.
.sp
This same convention is used for mail.
Note that the usual convention for any name that does not end with
a trailing dot is to try first with the local domain appended.
.sp
The actual suffix to tack on the end is usually obtained
by looking at the results of a "hostname" call, and using everything
starting at the first dot.
This can be overridden by specifying the local domain in the
file \fB/etc/resolv.conf\fP.  (See below for a description of
how to customize the host name lookup.) 
.SH ARGUMENTS
The first argument is the host name (domain name) you want to look up.
If this is a number, an "inverse query" is done, i.e. the domain
system looks in a separate set of databases used to convert numbers
to names.
.sp
If the \fB\-l\fP option is given, the first argument is a domain (zone)
name for which a complete listing is given.
.PP
The second argument is optional.  It allows you to specify a particular
server to query.  If you don't specify this argument, the default server
(normally the local machine) is used.
.SH OPTIONS
There are a number of options that can be used before the
specified name.  Most of these options are meaningful only to the
staff who have to maintain the domain database.
.TP 8
.B \-r
causes recursion to be turned off in the request.
This means that the nameserver will return only data it has in
its own database.  It will not ask other servers for more 
information.
.TP
.B \-w
causes the program to wait forever for a response.  Normally
it will time out after 10 seconds per nameserver address tried.
.TP
.BI \-s " seconds"
specifies a new nameserver timeout value. The program will wait
for a nameserver reply in one attempt of this number of seconds.
Normally it does 2 attempts of 5 seconds per nameserver address tried.
.TP
.B \-i
generates an inverse query for the \fBin-addr.arpa\fP domain
in case a numeric address was specified for the host or domain.
Useful only for domain listing, since for numeric host lookups
such inverse query is done anyway.
.TP
.B \-q
be quiet and suppress various warning messages.
.TP
.B \-v
causes printout to be in a "verbose" format.  This
is the official domain master file format, which is documented 
in the man page for \fInamed\fP.  Without this option, output still follows
this format in general terms, but TTL and class fields are not shown.
In addition, the verbose option prints extra information about the
actions that are taken.
Note that \fB\-vv\fP is "very verbose".
.TP
.B \-d
turns on debugging.  Network transactions are shown in detail.
Note that \fB\-dd\fP prints even more debugging output.
.TP
.BI \-f " filename"
writes resource record output to given file as well as to standard output.
.TP
.BI \-c " class"
allows you to specify a particular class. Supported are
\fBIN\fP, \fBCHAOS\fP, \fBHS\fP, and the wildcard \fBany\fP or \fB*\fP.
.br
The default class is \fBIN\fP.
.TP
.BI \-t " querytype"
allows you to specify a particular type of information
to be looked up.  Supported types are listed below.
The wildcard may be written as either \fBany\fP or \fB*\fP.
Types may be given in upper or lower case.
.br
Note that the default is type \fBA\fP in host lookups,
and \fBA\fP, \fBNS\fP, and \fBPTR\fP in domain listings.
.TP
.B \-a
(for "all") is equivalent to \fB-t any\fP.
.TP
.B \-m
is equivalent to \fB-t mailb\fP.
In addition, \fBMR\fP and \fBMG\fP records will be recursively
expanded into \fBMB\fP records.
.TP
.B \-e
excludes information about hosts that are not residing directly
in the given domain during zone listings, such as some glue records.
For normal queries, it suppresses the printing of additional
information and nameserver records.
.TP
.B \-p
causes the primary nameserver of a domain to be contacted for
zone transfers during domain listings. Normally, zone transfers
are obtained from one of the authoritative servers that responds.
Note that a specifiec server given on the command line overrules
this option.
.TP
.B \-S
prints statistics about resource records found during zone listings
of domains.
.TP
.B \-T
prints the time-to-live values during non-verbose output.
By default the TTL is shown only in verbose mode.
.TP
.BI \-l " domain"
causes a listing of a complete domain.
.PP
E.g. the command
.br
	\fBhost -l nikhef.nl\fP
.br
will give a listing of all hosts in the "nikhef.nl" domain.  The \fB-t\fP
option is used to filter what information is presented, as you 
would expect.  The default is address information from A records,
supplemented with data from PTR and NS records.  The command
.br
	\fBhost -v -a -l nikhef.nl\fP
.br
will give a complete download of the zone data for "nikhef.nl",
in the official master file format.
.SH "SPECIAL OPTIONS"
.TP 8
.B \-H
can be specified instead of the \fB\-l\fP option. It will print
the count of the unique hostnames encountered within the domain.
It will not count pseudo names like "localhost", nor addresses
associated with domain names itself.
By default, it will not print any resource records.
.br
Combined with the \fB\-S\fP option, it will give a complete
statistics survey of the domain.
.TP
.B \-C
can be specified instead of the \fB\-l\fP option. It causes the SOA
records for the specified domain to be compared as found at each of
the authoritative nameservers for the domain.
.TP
.B \-A
enters a special address check mode.
.br
If the first argument is a host name, its addresses will be retrieved
and for each of the addresses will be checked whether they map back
to the given host.
.br
If the first argument is a dotted quad host address, its name will be
retrieved, and it will be checked whether the given address is listed
among the known addresses belonging to that host.
.SH QUERYTYPES
The following querytypes are supported.
.TP 8
.B A
Host address (dotted quad)
.TP
.B NS
Authoritative nameserver (domain)
.TP
.B MD
Mail destination (domain)
.TP
.B MF
Mail forwarder (domain)
.TP
.B CNAME
Canonical name for an alias (domain)
.TP
.B SOA
Marks the start of a zone of authority
(nameserver, mailbox, serial, refresh, retry, expiration, minimum)
.TP
.B MB
Mailbox domain name (domain)
.TP
.B MG
Mail group member (domain)
.TP
.B MR
Mail rename domain name (domain)
.TP
.B NULL
Null resource record (no format or data)
.TP
.B WKS
Well-known service description (address, protocol, list of services)
.TP
.B PTR
Domain name pointer (domain)
.TP
.B HINFO
Host information (CPU type, OS type)
.TP
.B MINFO
Mailbox or mail list information (request domain, error domain)
.TP
.B MX
Mail exchanger (preference, domain)
.TP
.B TXT
Descriptive text (string)
.TP
.B UINFO
User information (string)
.TP
.B UID
User identification (number)
.TP
.B GID
Group identification (number)
.TP
.B UNSPEC
Unspecified binary data (data)
.TP
.B ANY
Matches any of the above information.
.TP
.B MAILB
Matches any of types \fBMB\fP, \fBMR\fP, \fBMG\fP, or \fBMINFO\fP.
.TP
.B MAILA
Matches any of types \fBMD\fP, or \fBMF\fP.
.SH CUSTOMIZING HOST NAME LOOKUP
In general, if the name supplied by the user does not
have any dots in it, a default domain is appended to the end.
This domain can be defined in \fB/etc/resolv.conf\fP, but is normally derived
by taking the local hostname after its first dot.
.sp
The user can override
this, and specify a different default domain, using the environment
variable
.IR LOCALDOMAIN .
.sp
In addition, the user can supply his own abbreviations for host names.
They should be in a file consisting of one line per abbreviation.
Each line contains an abbreviation, a space, and then the full
host name.  This file must be pointed to by an environment variable
.IR HOSTALIASES ,
which is the name of the file.
.SH EXAMPLE
If a name is specified, you may see output of three different kinds.
Here is an example that shows all of them:
.br
   % host sun4.rutgers.edu
.br
   sun4.rutgers.edu   CNAME    ATHOS.RUTGERS.EDU
.br
   ATHOS.RUTGERS.EDU  A        128.6.5.46
.br
   ATHOS.RUTGERS.EDU  A        128.6.4.4
.br
   ATHOS.RUTGERS.EDU  MX       ARAMIS.RUTGERS.EDU
.br
The user has typed the command "host sun4.rutgers.edu".
The first line indicates
that the name "sun4.rutgers.edu" is actually a nickname.  The official
host name is "ATHOS.RUTGERS.EDU'.  The next two lines show the
address.  If a system has more than one network interface, there
will be a separate address for each.  The last line indicates
that ATHOS.RUTGERS.EDU does not receive its own mail.  Mail for
it is taken by ARAMIS.RUTGERS.EDU.  There may be more than one
such line, since some systems have more than one other system
that will handle mail for them.
.sp
Technically, every system that
can receive mail is supposed to have an entry of this kind.  If
the system receives its own mail, there should be an entry
the mentions the system itself, for example
.br
   nikhefh.nikhef.nl  MX       nikhefh.nikhef.nl
.br
However, many systems that receive
their own mail do not bother to mention that fact.  If a system
has a MX entry, but no address, this indicates
that it is not really part of the Internet, but a system that is
on the network will forward mail to it.  Systems on Usenet, Bitnet,
and a number of other networks have entries of this kind.
.SH SPECIAL CONSIDERATIONS
The complete set of resource information about a requested host
is available from an authoritative nameserver only. Therefore,
if you query another server with the "-a" option, only a subset
of the data may be presented, since this option asks for any data
that the latter server currently knows about.
.br
Note that the "-v" option shows whether an answer is authoritative
or not.
.PP
When listing a domain with the "-l" option, information will be fetched
from authoritative nameservers for that domain. This is implemented by
doing a complete zone transfer and then filtering out the information
the you have asked for.
.br
Note that direct contact with such nameservers must be possible for
this option to work.
.br
This option should be used only if it is absolutely necessary.
.SH AUTHOR
This program is originally from Rutgers University.
.br
Extensively rewritten by
.br
Eric Wassenaar, Nikhef-H, <e07@nikhef.nl>
.SH "SEE ALSO"
named(8), resolver(4), resolver(3)
