/*******************************************************************************
 *
 *	link.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part determines if two nodes are connected or not.
 *	It divides the original nodestructure into several parts: nodes without
 *	connections, nodes with only one connection and nodes with more
 *	connections.
 *	The nodes with connected to only one node with more links are placed
 *	just above/under this connecting node.
 *
 ******************************************************************************/

# include "layout.h"


	/* determination if two nodenames are linked or not
	   returns FALSE if not linked and returns TRUE if linked */

extern int linked_or_not(hostnr_one,hostnr_two)
int hostnr_one;		/* the ranknumber of the first node we are looking for */
int hostnr_two;		/* the ranknumber of the second node we are looking for */
{
	struct linkentity * dummylink1;		/* structure to be searched */

	dummylink1 = link;
	while (dummylink1 != NULL)
	{
		if ( ((hostnr_one == dummylink1->nr_one) && (hostnr_two == dummylink1->nr_two))
		  || ((hostnr_one == dummylink1->nr_two) && (hostnr_two == dummylink1->nr_one)) )
			return TRUE;

		dummylink1 = dummylink1->next;
	}
	return FALSE;
}


	/* the nodes with only one link have to be placed near the node with more links it is connected to */
	/* the nodes with one link only connected to other nodes with one link are placed in the border */

extern void place_one_link_nodes(sizecheck, upperboundary)
int sizecheck;			/* check sizes for the boxes */
int * upperboundary;		/* area for one_link_node - one_link_node */
{
	struct hostentity * dummyhost1;		/* node with one link to determine the coordinates for */
	struct hostentity * dummyhost2;		/* node with more than one link */
	struct hostentity * dummyhost3;		/* node with one link to more_link_node */
	struct hostentity * newhost;
	struct hostentity * dummy;		/* nodes with one link connected to another one_link_node */
	int Linked;				/* one_link_node linked to node_with_more_links */
	int Above_center;			/* more_link_node is on tophalf of window or not */
	int MoreToOne;
	int is_taken, is_taken_2, space;
	long int tmpy;
	int x_offset, y_offset;			/* coordinates for one_link_nodes connected to one_link_node */
	int nodecount;				/* how many of these are there */
	int Adjusted_parent;			/* y_coord. more_link_node on lower_half of window is corrected */

	int highest = 0;
	int the_width = 0;
	long int total_width;
	float temp_value;


	/* get one_link_nodes only connected to another one_link_node */

	dummyhost1 = host_one_link;
	newhost = NULL;
	dummy = NULL;

	while (dummyhost1 != NULL)
	{
		Linked = FALSE;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			{
				if (linked_or_not(dummyhost1->number,dummyhost2->number))
					Linked = TRUE;
			}

			dummyhost2 = dummyhost2->next;
		}

		if (! Linked)
			dummy = addnodestructure(dummy, dummyhost1->id, dummyhost1->hostname,
						 dummyhost1->x_coord, dummyhost1->y_coord,
						 dummyhost1->width, dummyhost1->height,
						 dummyhost1->link, dummyhost1->number, TRUE);
		else
			newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
						   dummyhost1->x_coord, dummyhost1->y_coord,
						   dummyhost1->width, dummyhost1->height,
						   dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* make (reduced) one_link_nodestructure known */

	host_one_link = newhost;

	/* Determine boundary to use for all one_link_nodes (connected to other
	   one_link_nodes). Put the y_coordinates at 0 for these nodes ! */

	*upperboundary = 25;	nodecount = 0;
	dummyhost1 = dummy;
	while (dummyhost1 != NULL)
	{
		if (dummyhost1->height > highest)
			highest = dummyhost1->height;
		if (dummyhost1->width > the_width)
			the_width = dummyhost1->width;
		dummyhost1->y_coord = 0;
		nodecount ++;
		dummyhost1 = dummyhost1->next;
	}

	if (nodecount != 0)
	{
		total_width = nodecount  * (the_width + 25);
		*upperboundary += highest + 10;
		temp_value = (float) total_width / WINDOWwidth;

		while (temp_value > 0)
		{
			temp_value --;		*upperboundary += highest + 10;
		}

		WINDOWheight += (*upperboundary - boundary);
		fprintf(stderr,"place_one_link_nodes: WINDOWheigth adjusted to %ld\n", WINDOWheight);
	}

	/* place the one_link_nodes connected to another one_link_node on the topborder */

	x_offset = 100;	y_offset = WINDOWheight - (highest + 10);

	dummyhost1 = dummy;
	newhost = host;

	while (dummyhost1 != NULL)
	{
		dummyhost2 = dummy;

		while (dummyhost2 != NULL)
		{
			if ( (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			  && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				{
					if ((x_offset + the_width) >= (WINDOWwidth - 100))
					{
						x_offset = 100;
						y_offset -= (2 * (highest + 10));
					}

					if (dummyhost2->y_coord == 0)
					{
						dummyhost1->x_coord = x_offset;
						dummyhost1->y_coord = y_offset;
						x_offset += the_width + 25;
					}
					else
					{
						dummyhost1->x_coord = dummyhost2->x_coord;
						dummyhost1->y_coord = dummyhost2->y_coord - dummyhost1->height - 10;
					}

					newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
								   dummyhost1->x_coord, dummyhost1->y_coord,
								   dummyhost1->width, dummyhost1->height,
								   dummyhost1->link, dummyhost1->number, FALSE);
				}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* save nodes with more links, also the nodes only connected to one_link_nodes
	   and one_link_nodes connected to the "special" more_link_nodes
	   and one_link_nodes only connected to another one_link node */

	host = newhost;
	newhost = NULL;

	/* place the rest of the one_link_nodes close above the more_links_node (tophalf of window)
	   or direct under the more_links_node (bottomhalf of the window) */

	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{

		if ((dummyhost1->y_coord + (dummyhost1->height / 2) + (dummyhost1->link * (DEFAULTheight + 5) / 2)) >=
		    (WINDOWheight / 2))
			Above_center = TRUE;
		else
		{
			Above_center = FALSE;	MoreToOne = FALSE;

		/* nodes with more links but only to one_link_nodes are special, so skip them */

			dummyhost2 = host_more_to_one;
			while (dummyhost2 != NULL)
			{
				if (dummyhost2->number == dummyhost1->number)
					MoreToOne = TRUE;

		/* nodes with only one link connected to a more_link_node with only links to
		   one_link_nodes are special, so skip them */

				dummyhost3 = host;
				while (dummyhost3 != NULL)
				{
					if (dummyhost3->number == dummyhost2->number)
						MoreToOne = TRUE;
					dummyhost3 = dummyhost3->next;
				}

				dummyhost2 = dummyhost2->next;
			}

			if (! MoreToOne)
			{
				if (dummyhost1->link > 0)
				{
					dummyhost3 = host;
					while (dummyhost3 != NULL)
					{
						if ( (dummyhost1->number != dummyhost3->number)
						  && (linked_or_not(dummyhost1->number, dummyhost3->number)) )
							dummyhost1->y_coord += dummyhost3->height;
						dummyhost3 = dummyhost3->next;
					}
				}
			}
		}

		nodecount = 1;
		Adjusted_parent = FALSE;
		dummyhost2 = host_one_link;

		while (dummyhost2 != NULL)
		{
			if ( (dummyhost1->number != dummyhost2->number)
			  && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
			{
				dummyhost2->x_coord = dummyhost1->x_coord;

				is_taken = TRUE;	space = 5;
				while (is_taken)
				{
	if (Above_center)
		tmpy = dummyhost1->y_coord + (nodecount * (dummyhost2->height + 5)) + space;
	else
	{
		if (! Adjusted_parent)
		{
			dummyhost1->y_coord += (dummyhost1->link * 5);
			dummyhost3 = host;
			while (dummyhost3 != NULL)
			{
				if ( (dummyhost1->number != dummyhost3->number)
				  && (linked_or_not(dummyhost1->number, dummyhost3->number)) )
					dummyhost1->y_coord += dummyhost3->height;
				dummyhost3 = dummyhost3->next;
			}

			Adjusted_parent = TRUE;
		}
		tmpy = dummyhost1->y_coord - (nodecount * (dummyhost2->height + 5)) - space;
	}

#ifdef DEBUG
	fprintf(stderr,"place_one_link_nodes: checking [1] on %s at %ld %ld\n", dummyhost2->hostname,
		dummyhost2->x_coord, tmpy);
#endif

	is_taken = check_free(host, dummyhost2->x_coord, tmpy,
			      dummyhost2->width, dummyhost2->height,
			      dummyhost2->number, FALSE);
	space += 5;
				}

				if (! is_taken)
				{
					is_taken_2 = TRUE;
					while (is_taken_2)
					{
#ifdef DEBUG
	fprintf(stderr,"place_one_link_nodes: checking [2] on %s at %ld %ld\n", dummyhost2->hostname,
		dummyhost2->x_coord, tmpy);
#endif

	is_taken_2 = check_free(newhost, dummyhost2->x_coord, tmpy,
		      dummyhost2->width, dummyhost2->height,
		      dummyhost2->number, FALSE);
	if (is_taken_2)
	{
		if (Above_center)
			tmpy = dummyhost1->y_coord + (nodecount * (dummyhost2->height + 5)) + space;
		else
			tmpy = dummyhost1->y_coord - (nodecount * (dummyhost2->height + 5)) - space;
		space += 5;
	}
					}
				}

	if ( (! is_taken) && (! is_taken_2) )
		dummyhost2->y_coord = tmpy;
	else
	{
		fprintf(stderr,"place_one_linked_node: Can't place %s without overlap\n", dummyhost2->hostname);
		dummyhost2->x_coord = WINDOWwidth - dummyhost2->width;
		if (Above_center)
			dummyhost2->y_coord = WINDOWheight / 2 + 25;
		else
			dummyhost2->y_coord = 25;

		fprintf(stderr,"It is placed at %ld %ld\n", dummyhost2->x_coord, dummyhost2->y_coord);
	}

				newhost = addnodestructure(newhost, dummyhost2->id, dummyhost2->hostname,
							   dummyhost2->x_coord, dummyhost2->y_coord,
							   dummyhost2->width, dummyhost2->height,
							   dummyhost2->link, dummyhost2->number,  TRUE);
				nodecount ++;
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* make additions of one_link_nodes connected to the more_links_nodes known */

	dummyhost1 = newhost;

	while (dummyhost1 != NULL)
	{
		host = addnodestructure(host, dummyhost1->id, dummyhost1->hostname,
 					dummyhost1->x_coord, dummyhost1->y_coord,
					dummyhost1->width, dummyhost1->height,
 					dummyhost1->link, dummyhost1->number, FALSE);
		dummyhost1 = dummyhost1->next;
	}
}


	/* get the nodes without links. They are not needed for the iterations of the layoutprogram.
	   These nodes are finally placed in the border of the window */
	/* also the nodes with only one link are gathered and placed close to the more_linked_nodes
	   at the end of the program */

extern void determine_links_for_a_node(sizecheck)
int sizecheck;			/* check maximum sizes for the boxes */
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	struct hostentity * newhost;
	int links;				/* number of links of a node */

	newhost = NULL;
	dummyhost1 = host;
	host_zero_links = NULL;
	host_one_link = NULL;

	while (dummyhost1 != NULL)
	{
		links = 0;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->number, dummyhost2->number))
				links ++;

			dummyhost2 = dummyhost2->next;
		}

#ifdef DEBUG
		fprintf(stderr,"determine_links_for_a_node: node %s has %d links\n",dummyhost1->hostname,links);
#endif

		if ( (links == 0) || (links == 1) )
		{
			/* vector determination will otherwise be wrong */

 			number_of_nodes --;
			if (links == 0)
				host_zero_links = addnodestructure(host_zero_links, dummyhost1->id, dummyhost1->hostname,
								   dummyhost1->x_coord,	dummyhost1->y_coord,
								   dummyhost1->width, dummyhost1->height,
								   0, dummyhost1->number, TRUE);
			else
				host_one_link = addnodestructure(host_one_link, dummyhost1->id, dummyhost1->hostname,
								 dummyhost1->x_coord, dummyhost1->y_coord,
								 dummyhost1->width, dummyhost1->height,
								 1, dummyhost1->number, TRUE);
		}
		else
		{
			/* make temp list with nodes with TWO or more links and copy this back later */
			/* make more_linked_nodes look more important */

			if ( (links > 2) && (links <= 5) )
				dummyhost1->width += 0.33 * DEFAULTwidth;
			else if ( (links > 5) && (links < 10) )
				dummyhost1->width += 0.67 * DEFAULTwidth;
			else if (links >= 10)
				dummyhost1->width += 1.00 * DEFAULTwidth;

			if ( (links > 5) && (dummyhost1->height <= 90) )
				dummyhost1->height += 10;

			if (sizecheck)
			{
				if (dummyhost1->width > (WINDOWwidth / 5) )
					dummyhost1->width = WINDOWwidth / 5;
				if (dummyhost1->height > 100 )
					dummyhost1->height = 100;
			}

			newhost = addnodestructure(newhost,dummyhost1->id, dummyhost1->hostname,
						   dummyhost1->x_coord, dummyhost1->y_coord,
						   dummyhost1->width, dummyhost1->height,
						   links, dummyhost1->number, TRUE);
		}

		dummyhost1 = dummyhost1->next;
	}

	/* after the complete cycle : make changes known to the original hoststructure */

	host = newhost;

	/* eventually resize the box for the more_links_node so it will not be smaller
	   than the largest box of a/the one_link_node(s) it is connected to */

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		dummyhost2 = host_one_link;
		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->number, dummyhost2->number))
			{
				if (dummyhost1->width < dummyhost2->width)
					dummyhost1->width = dummyhost2->width;
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}
}
