/*******************************************************************************
 *
 *	main.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part is the mainpart of the program.
 *	It reads the commandline (and decides what to do) and it reads the
 *	input of nodes (with properties: a) connected to which other nodes
 *				     and b) name, coordinates and sizes).
 *	Variables are set, other procedures are called.
 *
 ******************************************************************************/

# include "layout.h"
#include "string.h"
#include "strings.h"

int number_of_nodes = 0;		/* keep track of total hosts - networks */
int WINDOWwidth  = 1000;		/* x-size of the window (default) */
int WINDOWheight = 1000;		/* y-size of the window (default) */
int DEFAULTwidth  = 150;		/* width for nodes with originally zero width */
int DEFAULTheight =  15;		/* height for nodes with originally zero height */
int boundary      = 100;		/* surrounding area in window */
int lower_y_boundary = 100;		/* area for unlinked nodes (changed in place_unlinked_nodes) */
int upper_y_boundary = 100;		/* area for one_link_node linked to other one_link_node
					   (changed in place_one_link_nodes) */


struct hostentity *host		    = NULL;	/* name of the node structure */
struct hostentity *host_zero_links  = NULL;	/* structurename with nodes without links */
struct hostentity *host_one_link    = NULL;	/* structurename with nodes with only one links */
struct hostentity *host_more_to_one = NULL;	/* structurename with nodes with more links, but only to nodes with one link */
struct hostentity *host_1	    = NULL;	/* 3 structures needed for the final */
struct hostentity *host_2	    = NULL;	/* determination of the coordinates */
struct hostentity *host_more	    = NULL;	/* !! host_1 + host_2 + host_more = host !! */

struct linkentity *link = NULL;			/* name of the link structure */

struct nodeheightentity *nodeheight = NULL;	/* name of the nodename - totalheight structure */


	/* Scans the input and builds-up a list with hostnames and their
	   coordinates and sizes . Copies this list into the output */
	/* gets everything done:
		layout of more_linked_nodes, placing of 1_link_node with another 1_link_node,
		placing of unlinked nodes
		printing of postscript etc */
   

void main(argc,argv)
int argc;
char *argv[];
{
	char line[MaxLine];			/* one line of input */
	int contr;				/* controls the scan-loop */
	char id[MaxEnt];			/* unique identifier + hostname */
	char name[MaxEnt];			/* hostname */
	long int x_coord,y_coord;			/* initial x/y_coord of host */
	int width,height;			/* initial width/height of host */
	char site_1[MaxEnt], site_2[MaxEnt];	/* first/second sitename of the connection */
	int cntrl;				/* counter used for scanning arguments */
	unsigned int it;			/* counts number of iterations to make */
	unsigned int iterations = 0;		/* the number of iterations */
	long UTC1, UTC2;			/* time used for determining layout */
	int printhostinput = FALSE;		/* print input of hoststructure or not */
	int printlinkinput = FALSE;		/* print input of linkstructure or not */
	int printhost = TRUE;			/* print hoststructure output or not */
	int printlink = TRUE;			/* print linkstructure output or not */
	int rand_coord = FALSE;			/* flag to control randomly generated coordinates */
	int postscript = FALSE;			/* flag to control postscript output */
	int colorpostscript = FALSE;		/* flag to control colorpostscript output */
	int adjust_size = TRUE;			/* flag to control max. width/height for a node */
	int AutoWindow = TRUE;			/* flag to control autosizing of the window */
	int HighLight = FALSE;			/* flag to control highlighting of a node and its connections */
	char nodenamestr[MaxEnt];		/* node to highlight */
	char interfacestring[MaxEnt];		/* string for scanning the line with connections between nodes */
	int length_int = 0;
	char dummystr[MaxEnt];
	char outputstr[MaxEnt];			/* name of postscript_outputfile */
	float factor = 1.0;			/* factor to multiply the vector by (force determination) */
	int rank = 1;				/* the rank of the read-in-nodes */
	long int max_boxwidth, max_boxheight;
	int WindowWidth_saved, WindowHeight_saved;


	time(&UTC1);
	fprintf(stderr,"%s started at : %s\n",argv[0], ctime(&UTC1));

	strcpy(interfacestring,"interface=");

	if (argc > 21)
		usage(argv[0]);
	else if (argc == 1)
		iterations = 10;

	for (cntrl = 1 ; cntrl < argc ; cntrl++)
		switch(argv[cntrl][0])
		{
			case '-':
				switch(argv[cntrl][1])
				{
	case 'a':
		AutoWindow = FALSE;
		break;
	case 'c':
		if ((cntrl + 1) < argc)
		{
			strcpy(interfacestring,argv[cntrl + 1]);
			length_int = strlen(interfacestring);
			cntrl ++;
		}
		else
		{
			fprintf(stderr,"You must enter the connectionname of the linked nodes\n");
			usage(argv[0]);
		}
		break;
	case 'f':
		if ((cntrl + 1) < argc)
		{
			factor = atof(argv[cntrl + 1]);
			if (factor <= 0)
			{
				fprintf(stderr,"You entered %lf as factor, this is set to 1\n",factor);
				factor = 1;
			}
			else if (factor > 5)
				fprintf(stderr,"%lf is a large factor\n",factor);
			cntrl ++;
		}
		else
		{
			fprintf(stderr,"You must enter the factor to multiply the forces with\n");
			usage(argv[0]);
		}
		break;
	case 'h':
		if ((cntrl + 1) < argc)
		{
			WINDOWheight = atoi(argv[cntrl + 1]);
			if (WINDOWheight < 500)
			{
				fprintf(stderr,"Too small: height resized to 500\n");
				WINDOWheight = 500;
			}
			cntrl ++;
		}
		else
		{
			fprintf(stderr,"You must enter the height of the window\n");
			usage(argv[0]);
		}
		break;
	case 'H':
		HighLight = TRUE;
		if ((cntrl + 1) < argc)
		{
			switch(argv[(cntrl + 1)][0])
			{
				case '-':
					fprintf(stderr,"You forgot to enter the nodename (to highlight)\n");
					usage(argv[0]);
					break;
				default:
					strcpy(nodenamestr,argv[(cntrl + 1)]);
					cntrl ++;
					break;
			}
			break;
		}
		break;
	case 'i':
		if ((cntrl + 1) < argc)
		{
			iterations = atoi(argv[cntrl + 1]);
			if (iterations < 1)
			{
				fprintf(stderr,"Number of iterations will be 1\n");
				iterations = 1;
			}
			cntrl ++;
		}
		else
		{
			fprintf(stderr,"You must enter the number of iterations to be made\n");
			usage(argv[0]);
		}
		break;
	case 'l':
		printlink = FALSE;
		break;
	case 'n':
		printhost = FALSE;
		break;
	case 'p':
		postscript = TRUE;
		if ((cntrl + 1) < argc)
		{
			switch(argv[(cntrl + 1)][0])
			{
				case '-':
					strcpy(outputstr,"/tmp/psfile");
					break;
				case '>':
					strcpy(outputstr,"/tmp/psfile");
					break;
				case '|':
					strcpy(outputstr,"/tmp/psfile");
					break;
				default:
					strcpy(outputstr,argv[(cntrl + 1)]);
					cntrl ++;
					break;
			}
			break;
		}
		else
			strcpy(outputstr,"/tmp/psfile");
		break;
	case 'P':
		colorpostscript = TRUE;
		if ((cntrl + 1) < argc)
		{
			switch(argv[(cntrl + 1)][0])
			{
				case '-':
					strcpy(outputstr,"/tmp/psfile");
					break;
				case '>':
					strcpy(outputstr,"/tmp/psfile");
					break;
				case '|':
					strcpy(outputstr,"/tmp/psfile");
					break;
				default:
					strcpy(outputstr,argv[(cntrl + 1)]);
					cntrl ++;
					break;
			}
			break;
		}
		else
			strcpy(outputstr,"/tmp/psfile");
		break;
	case 'r':
		rand_coord = TRUE;
		break;
	case 's':
		adjust_size = FALSE;
		break;
	case 'w':
		if ((cntrl + 1) < argc)
		{
			WINDOWwidth = atoi(argv[cntrl + 1]);
			if (WINDOWwidth < 500)
			{
				fprintf(stderr,"Too small : width resized to 500\n");
				WINDOWwidth = 500;
			}
			cntrl ++;
		}
		else
		{
			fprintf(stderr,"You must enter the width of the window\n");
			usage(argv[0]);
		}
		break;
	case '?':
		usage(argv[0]);
		break;
	default:
		fprintf(stderr,"Option '%s' does not exist\n",argv[cntrl]);
		usage(argv[0]);
		break;
				}
				break;
			default:
				fprintf(stderr,"Option '%s' must be preceded by '-'\n",argv[cntrl]);
				usage(argv[0]);
				break;
		}

	WindowWidth_saved = WINDOWwidth;	WindowHeight_saved = WINDOWheight;

	if (iterations < 1)
		iterations = 1;

	if ((postscript) && (colorpostscript))
	{
		fprintf(stderr,"It is only possible to get b/w-postscript OR colorpostscript not both\n");
		fprintf(stderr,"We will default to black_and_white-postscript\n");
		colorpostscript = FALSE;
	}

#ifdef DEBUG
	fprintf(stderr,"\n%s runs with the following parameters :\n",argv[0]);
	fprintf(stderr,"number of iterations is %d\n",iterations);
	fprintf(stderr,"width of the 2D window is %d\n",WINDOWwidth);
	fprintf(stderr,"height of the 2D window is %d\n",WINDOWheight);
	fprintf(stderr,"forcevector divide factor is %f\n",factor);
	if (postscript)
		fprintf(stderr,"Postscript output is made\n");
	else
		fprintf(stderr,"No postscript output is made\n");
	if (rand_coord)
		fprintf(stderr,"Random coordinates for all nodes\n");
	else
		fprintf(stderr,"No random coordinates for the nodes (only those with coordinates == 0)\n");
	if (adjust_size)
		fprintf(stderr,"width and height of the box for a node is checked\n");
	else
		fprintf(stderr,"No size_check for the boxes\n");

	fprintf(stderr,"\nStarting to read the nodes\n\n");
#endif

	/* Scan the input for hostnames with their coordinates and sizes,
	   build-up the hoststructure (with function addnodestructure) */
	/* Scan the input for connections (links) with their two sites ,
	   build up the linkstructure (with function addlinkstructure) */

	while (gets(line))
	{
		sscanf(line,"%s",dummystr);

		if ( ((length_int != 0) && (strcmp(dummystr,interfacestring) == 0))
		   || (strcmp(dummystr,"interface=") == 0)
		   || (strcmp(dummystr,"Interface=") == 0) )

		/* get the connections between nodes */

		{
#ifdef DEBUG
			printlinkinput = TRUE;
#endif

			contr = sscanf(line,"%s %s %s",dummystr,site_1,site_2);
			if (contr == 3)
			{
				link = addlinkstructure(link,interfacestring,site_1,site_2, 0, 0);
			}
			else
			{
				fprintf(stderr,"addlink: wrong data (%s %s)\n",site_1,site_2);
				continue;
			}
		}

		else /* get the node : id, name, initial coordinates and sizes */
		{
#ifdef DEBUG
			printhostinput = TRUE;
#endif

			/* skip the interfaces for the node structure */

			if ( (strncmp("interface:",line,10) != 0) && (strncmp("ip.interface:",line,12) != 0) )
			{
				contr = sscanf(line,"%s %s %ld %ld %ld %ld",id,name,&x_coord,&y_coord,&width,&height);
				if (contr == 6)
				{
					host = addnodestructure(host,id,name,x_coord,y_coord,width,height,0,rank,FALSE);
					rank ++;
				}
				else
				{
					fprintf(stderr,"addhost: wrong data (%s %s %ld %ld %ld %ld)\n",
						id,name,x_coord,y_coord,width,height);
					continue;
				}
			}
		}
	}

	if (number_of_nodes == 0)
	{
		fprintf(stderr,"No nodes are read\n");
		fprintf(stderr,"Checking if we (only) have connections\n");
		print_link(link, FALSE);
		exit(2);
	}

	sort_the_nodestructure();
	rank_number_into_link();

	/* print these structures on standard output */

	if (printhostinput)
		print_node(host, FALSE);
	if (printlinkinput)
		print_link(link, FALSE);

#ifdef DEBUG
	fprintf(stderr,"Starting with the actual layout\n\n");
#endif

	fprintf(stderr,"%s: total number of nodes %d\n",argv[0],number_of_nodes);

	/* get the nodes without links and save them in another structure */
	/* get the nodes with only one link and save them in another structure */

#ifdef DEBUG
	fprintf(stderr,"Determination of number of links for all nodes\n");
	determine_links_for_a_node(adjust_size);
#endif

	fprintf(stderr,"%s: number of nodes (with more than 1 link) to process %d\n",argv[0],number_of_nodes);

	/* get the nodes with the most links and place them in center */

#ifdef DEBUG
	if (rand_coord)
		fprintf(stderr,"\nNodes are randomly placed in the window (more linked nodes near the center) !!\n\n");
	else
		fprintf(stderr,"\nPlacing nodes in the window\n\n");
#endif

	if (adjust_size)
	{
		max_boxwidth = WINDOWwidth / 5;
		max_boxheight = WINDOWheight / 10;
		if (max_boxheight > 50)
			max_boxheight = 50;

		fprintf(stderr,"\nadjusting the sizes for the number of nodes\n");
		fprintf(stderr,"max. boxwidth  for a node will be : %d\n", max_boxwidth);
		fprintf(stderr,"max. boxheight for a node will be : %d\n", max_boxheight);

		if (rand_coord)
			place_linked_node(max_boxwidth, max_boxheight, TRUE);
		else
			place_linked_node(max_boxwidth, max_boxheight, FALSE);
	}
	else
	{
		if (rand_coord)
			place_linked_node(WINDOWwidth, WINDOWheight, TRUE);
		else
			place_linked_node(WINDOWwidth, WINDOWheight, FALSE);
	}

	/* put all effective heights in a structure (speed-up) */

	determine_height();

	/* get the changes for attracting (linked) nodes and repelling (unlinked) nodes */
	/* only do this with nodes with more links (otherwise it will take too long to finish) */

	time(&UTC2);
	fprintf(stderr,"\n%s started with %u iterations at : %s",
		argv[0], iterations, ctime(&UTC2));

	for (it=1; it <= iterations ; it++)
	{
		fprintf(stderr,"Doing iteration %u\n", it);
		layout_changes(factor);
	}

	time(&UTC2);
	fprintf(stderr,"%s finished with the iterations at : %s\n",
		argv[0], ctime(&UTC2));

	/* perform an automatic sizing of the window if AutoWindow is set */

	nodes_into_window(AutoWindow);

	/* place the extra nodes (more links, but only to nodes with one links) in the emptiest sector */

	place_more_linked_nodes(adjust_size);

#ifdef DEBUG
	fprintf(stderr,"\n%s: number of nodes (added specials) %d\n",argv[0],number_of_nodes);
#endif

	/* place the nodes with one link, close to the node, with more links, it is linked to */

	place_one_link_nodes(adjust_size, &upper_y_boundary);

#ifdef DEBUG
	fprintf(stderr,"%s: number of nodes (added one_link_nodes) %d\n",argv[0],number_of_nodes);
#endif

	/* place the linkless nodes on the bottomborder of the window */

	place_unlinked_nodes(&lower_y_boundary);

	/* fix boundaries => size of the window (if necessary) */

	if (! AutoWindow)
		fix_boundaries( WindowWidth_saved, WindowHeight_saved,
				&lower_y_boundary, upper_y_boundary);

	/* make postscript outputfile and print the final results */

	if (printhost)
		print_node(host, TRUE);
	if (printlink)
		print_link(link, TRUE);
	if (postscript)
		make_postscript(outputstr, FALSE, HighLight, nodenamestr,
				lower_y_boundary, upper_y_boundary);
	else if (colorpostscript)
		make_postscript(outputstr, TRUE, HighLight, nodenamestr,
				lower_y_boundary, upper_y_boundary);

	time(&UTC2);
	fprintf(stderr,"\n%s ended at : %s",argv[0], ctime(&UTC2));
	fprintf(stderr,"\nSeconds used for %s: %ld\n",argv[0], (UTC2 - UTC1));
}
