/*******************************************************************************
 *
 *	morelinked.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part divides the more_link_nodes structure in three.
 *	It tries to fix the overlap for these nodes and it tries to
 *	reduce the unused gaps between all nodes.
 *	Finally it tries to fit the "freely (by this program) determined
 *	windowsizes" into the specified windowsizes.
 *
 ******************************************************************************/


# include "layout.h"

	/* place the nodes with only one or two links to other nodes in a
	   separate structure, and reduce the original hoststructure */

static void determine_nodes_1_or_2()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	int links;

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		links = 0;
		dummyhost2 = host;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			&& (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				links ++;
			dummyhost2 = dummyhost2->next;
		}

		if (links == 1)
			host_1 = addnodestructure(host_1, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		else if (links == 2)
			host_2 = addnodestructure(host_2, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		else
			host_more = addnodestructure(host_more, dummyhost1->id, dummyhost1->hostname,
						      dummyhost1->x_coord, dummyhost1->y_coord,
						      dummyhost1->width, dummyhost1->height,
						      dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}
}


	/* check the nodes with only 1 link to another more_linked_node (instead of 2)
	   and check nodes with 0 links to more_linked_nodes (instead of 1) */
	/* we get one structure with nodes, we have to place in the window
	   and two structures with nodes we can place close to these previous nodes */

static void check_nodes_1_or_2()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	struct hostentity * newhost;
	struct hostentity * dummy;
	int linknumber;

	dummy = NULL;		newhost = NULL;

	dummyhost1 = host_2;
	while (dummyhost1 != NULL)
	{
		linknumber = 0;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			&& (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				linknumber ++;
			dummyhost2 = dummyhost2->next;
		}

		if (linknumber == 2)
			newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		else
			dummy = addnodestructure(dummy, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* add the nodes with less than 2 links to other two_links_nodes
	   to the structure with the "certain" coordinates ('host_more') */

	dummyhost1 = dummy;
	while (dummyhost1 != NULL)
	{
		host_more = addnodestructure(host_more, dummyhost1->id, dummyhost1->hostname,
					     dummyhost1->x_coord, dummyhost1->y_coord,
					     dummyhost1->width, dummyhost1->height,
					     dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* save the reduced structure with nodes connected to two other more_linked_nodes
	   these nodes can later be placed in between the two connecting nodes */

	host_2 = newhost;

	dummy = NULL;		newhost = NULL;

	dummyhost1 = host_1;
	while (dummyhost1 != NULL)
	{
		linknumber = 0;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			&& (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				linknumber ++;
			dummyhost2 = dummyhost2->next;
		}

		if (linknumber == 1)
			newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		else
			dummy = addnodestructure(dummy, dummyhost1->id, dummyhost1->hostname,
						  dummyhost1->x_coord, dummyhost1->y_coord,
						  dummyhost1->width, dummyhost1->height,
						  dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* add the nodes with less than 1 link to two_links_nodes
	   to the structure with the "certain" coordinates ('host_more') */

	dummyhost1 = dummy;
	while (dummyhost1 != NULL)
	{
		host_more = addnodestructure(host_more, dummyhost1->id, dummyhost1->hostname,
					     dummyhost1->x_coord, dummyhost1->y_coord,
					     dummyhost1->width, dummyhost1->height,
					     dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	/* save the reduced structure with nodes connected to one other "certain_node"
	   these nodes can later be placed close to the connecting node */

	host_1 = newhost;
}


	/* perform an extra size-reduction (if necessary) */
	/* WINDOWwidth and -height are adjusted in this routine ! */

static void adjust_window(autowindow)
int autowindow;		/* TRUE : WINDOWsizes are adjusted to the needed space for all nodes */
			/* FALSE: default/given WINDOWwidth and -height are used */
{
	long int min_x, min_y;
	long int max_x, max_y;


	get_min_max_coord(host, &min_x, &max_x, TRUE);
	get_min_max_coord(host, &min_y, &max_y, FALSE);

	if ( ((max_x - min_x) > (WINDOWwidth - (2 * boundary)))
	  || ((max_y - min_y) > (WINDOWheight - (2 * boundary))) )
	{
		get_min_max_coord(host, &min_x, &max_x, TRUE);
		get_min_max_coord(host, &min_y, &max_y, FALSE);
	}

	if (autowindow)
	{
		/* adjust WINDOWwidth and WINDOWheight */

		WINDOWwidth = max_x - min_x + 2 * boundary;
		WINDOWheight = max_y - min_y + 2 * boundary;

		fprintf(stderr,"adjust_window: new sizes: width %d, height %d\n", WINDOWwidth, WINDOWheight);
	}
	else
	{
		/* if the needed space is smaller than the windowsize:
			adjust the nodes so we fill the entire window */

		if ( ((max_x - min_x) <= (WINDOWwidth - (2 * boundary)))
		  && ((max_y - min_y) <= (WINDOWheight - (2 * boundary))) )
		{
			resizing(TRUE);
			resizing(FALSE);
		}

		get_min_max_coord(host, &min_x, &max_x, TRUE);
		get_min_max_coord(host, &min_y, &max_y, FALSE);

		WINDOWwidth = max_x - min_x + 2 * boundary;
		fprintf(stderr,"adjust_window: WINDOWwidth is now %d   ",WINDOWwidth);

		WINDOWheight = max_y - min_y + 2 * boundary;
		fprintf(stderr,"WINDOWheight is now %d\n",WINDOWheight);
	}
}


	/* put the nodes into the window */
	/* Check every node with more than one connection to another node,
	   Check overlap : multiply every coordinate of the nodes, until
	   we have no more overlap. After this we can move every
	   node towards (boundary,boundary) etc. */

extern void nodes_into_window(autowindow)
int autowindow;		/* if TRUE : WINDOW is adjusted to the needed space for all nodes */
			/* if FALSE: default/given WINDOWwidth and WINDOWheight are used */
{
	long int min_x, min_y;		/* offset in x/y direction from (boundary,boundary) */
	long int max_x, max_y;


	host_1 = NULL;	host_2 = NULL;	host_more = NULL;

	/* sort the host structure into 3 structures:
	   a> only nodes with one connection to other nodes
	   b> nodes with only two connections to other nodes
	   c> nodes with more connections to other nodes + specials of category a and b. */
	/* check for overlap */

	determine_nodes_1_or_2();
	check_nodes_1_or_2();
	check_nodes_overlap();
	reduce_gaps(host_more, 100, TRUE, 1, WINDOWwidth);
	reduce_gaps(host_more, 100, FALSE, 1, WINDOWheight);

	/* place the nodes connected to 2 more_linked_nodes in the window
	   place the nodes connected to 1 more_linked_node in the window */
	/* check for overlap with already placed nodes */

	place_node_2();
	check_nodes_overlap();
	reduce_gaps(host_more, 100, TRUE, 1, WINDOWwidth);
	reduce_gaps(host_more, 100, FALSE, 1, WINDOWheight);
	place_node_1();
	check_nodes_overlap();
	reduce_gaps(host_more, 100, TRUE, 1, WINDOWwidth);
	reduce_gaps(host_more, 100, FALSE, 1, WINDOWheight);


	/* we worked with host_1, host_2 and host_more, but the rest of the program
	   still works with host. So we copy the structure host_more into host */

	host = host_more;

	fprintf(stderr,"adjusting nodes (with 2 links) between connecting nodes\n");
	adjust_some_node_2();
	check_nodes_overlap();
	fprintf(stderr,"alligning nodes with small differences in coordinates\n");
	allign_nodes();
	check_nodes_overlap();

	/* determines if we simply can move everything in one time into the window
	   or that we have to move every node separately */

	reduce_gaps(host, 50, TRUE, 1, WINDOWwidth);
	reduce_gaps(host, 50, FALSE, 1, WINDOWheight);

	get_min_max_coord(host, &min_x, &max_x, TRUE);
	get_min_max_coord(host, &min_y, &max_y, FALSE);

#ifdef DEBUG
	fprintf(stderr,"nodes_into_window: min/max x %ld %ld, and min/max y %ld %ld\n", min_x,max_x, min_y,max_y);
#endif

	if ((max_x - min_x) < (WINDOWwidth - (2 * boundary)))
		fprintf(stderr,"x-dimension fits in WINDOW(width)\n");
	else
		reduce_gaps(host, 25, TRUE, 1, WINDOWwidth);
	if ((max_y - min_y) < (WINDOWheight - (2 * boundary)))
		fprintf(stderr,"y-dimension fits in WINDOW(height)\n");
	else
		reduce_gaps(host, 25, FALSE, 1, WINDOWheight);

	adjust_window(autowindow);
}


	/* finally fix the boundaries (if necessary) */

extern void fix_boundaries(Ww_given, Wh_given, low_y_boun, high_y_boun)
int Ww_given, Wh_given;
int *low_y_boun, high_y_boun;
{
	long int min_x, max_x, min_y, max_y;
	long int tmp_min, tmp_max;
	int old_boundary;
	struct hostentity * dummy;


#ifdef DEBUG
	fprintf(stderr,"fix_boundaries: Windowsizes: width %d, height %d\n", WINDOWwidth, WINDOWheight);
	fprintf(stderr,"fix_boundaries: checking gaps (final)\n");
#endif

	reduce_gaps(host, 25, TRUE, 1, WINDOWwidth);
	reduce_gaps(host, 25, FALSE, 1, WINDOWheight);

	get_min_max_coord(host, &min_x, &max_x, TRUE);

	WINDOWwidth = max_x - min_x + 2 * boundary;

	if (WINDOWwidth > Ww_given)
	{
		old_boundary = boundary;	boundary = 25;
		adjust_coord(host, TRUE, TRUE, boundary);

		/* adjust also the x_coord for all unlinked nodes */

		dummy = host_zero_links;
		while (dummy != NULL)
		{
			dummy->x_coord -= (old_boundary - boundary);
			dummy = dummy->next;
		}

		get_min_max_coord(host, &min_x, &max_x, TRUE);
		get_min_max_coord(host_zero_links, &tmp_min, &tmp_max, TRUE);

		if (tmp_min < min_x)
			min_x = tmp_min;
		if (tmp_max > max_x)
			max_x = tmp_max;

		WINDOWwidth = max_x - min_x + 2 * boundary;
	}

	get_min_max_coord(host, &min_y, &max_y, FALSE);

	WINDOWheight = max_y - min_y + *low_y_boun + 10;

	if (WINDOWheight > Wh_given)
	{

	adjust_coord(host, FALSE, TRUE, *low_y_boun);

	get_min_max_coord(host, &min_y, &max_y, FALSE);

	dummy = host_zero_links;
	if (dummy != NULL)
		get_min_max_coord(host_zero_links, &tmp_min, &tmp_max, FALSE);
	else
	{
		tmp_min = 500;	tmp_max = 500;
	}
		if (tmp_max < (min_y - 25))
		{
			*low_y_boun -= (min_y - tmp_max - 25);
			adjust_coord(host, FALSE, TRUE, *low_y_boun);
			get_min_max_coord(host, &min_y, &max_y, FALSE);
		}

		if (tmp_max > max_y)
			max_y = tmp_max;

		WINDOWheight = max_y + 25;

	}

	fprintf(stderr,"fix_boundaries: Wwidth IS %d (finally)\n",WINDOWwidth);
	fprintf(stderr,"fix_boundaries: Wheight IS %d (finally)\n", WINDOWheight);
}
