/*******************************************************************************
 *
 *	move_node.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part moves all nodes towards (boundary,boundary).
 *	First it determines the distance of the closest node and reduces all coordinates
 *	of all nodes with this distance.
 *	Secondly it determines the largest unused space in x or y direction and moves
 *	nodes across this gap.
 *	When all nodes occupy only a part of the window, we multiply the coordinates
 *	so they are spread all over the window.
 *
 ******************************************************************************/

# include "layout.h"


	/* move all nodes towards (boundary,boundary) with the same distance */

static void move_towards_boundary(nodestruct)
struct hostentity * nodestruct;
{
	struct hostentity * dummyhost1;
	long int min, max;


	get_min_max_coord(nodestruct, &min, &max, TRUE);
	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{

#ifdef DEBUG
		fprintf(stderr,"move_towards_boundary: [OLD] %s at %ld %ld (%ld %ld)  ",
			dummyhost1->hostname,
			dummyhost1->x_coord, dummyhost1->y_coord,
			dummyhost1->width, dummyhost1->height);
#endif

		dummyhost1->x_coord -= min - boundary;

#ifdef DEBUG
		fprintf(stderr,"[NEW-x] %ld\n", dummyhost1->hostname, dummyhost1->x_coord);
#endif

		dummyhost1 = dummyhost1->next;
	}

	get_min_max_coord(nodestruct, &min, &max, FALSE);

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		dummyhost1->y_coord -= min - boundary; 

#ifdef DEBUG
		fprintf(stderr,"[NEW-y] %ld\n", dummyhost1->y_coord);
#endif

		dummyhost1 = dummyhost1->next;
	}
}


	/* determine minimum space left_of/under the "box" of a node */

static void determine_space(x1, w1, x2, w2, free)
long int x1, x2;
long int w1, w2;
long int *free;
{
	if (x1 > x2)
	{
		if ( (x1 + w1) < (x2 + w2))
		{
			*free = 0;
		}
		if ( (x2 + w2) < x1)
		{
			if ( (x1 - (x2 + w2)) < *free)
				*free = x1 - x2 - w2;
		}
		else
			*free = 0;
	}
}


	/* adjust the coordinates of nodes below/above the limit with the free-space available */
	/* if full_or_half != 1 : compare x with windowpart, and after that fix y !!
				Or compare y and fix x_coordinates !! */

static void adjust_nodes(limit, free_space, is_x, full_or_half, size)
long int limit, free_space;
int is_x;
int full_or_half;	/* 1: all coordinates, 2: upper half, 3: lower half */
int size;		/* the size of the window */
{
	struct hostentity * dummyhost1;


	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{

#ifdef DEBUG
		fprintf(stderr,"adjust_nodes: %s at %ld %ld",
			dummyhost1->hostname, dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		if (is_x)
		{
			if (full_or_half == 1)
			{
				if ((dummyhost1->x_coord + dummyhost1->width) < limit)
					dummyhost1->x_coord += free_space;
			}

			else if (full_or_half == 2)
			{
				if ( ((dummyhost1->x_coord + dummyhost1->width) > (size / 2) )
				  && (dummyhost1->y_coord >= limit) )
					dummyhost1->y_coord -= free_space;
			}
			else /* if (full_or_half == 3) */
			{
				if ( ((dummyhost1->x_coord + dummyhost1->width) < (size / 2) )
				  && (dummyhost1->y_coord >= limit) )
					dummyhost1->y_coord -= free_space;
			}
		}
		else
		{
			if (full_or_half == 1)
			{
				if ((dummyhost1->y_coord + dummyhost1->height) < limit)
					dummyhost1->y_coord += free_space;
			}
			else if (full_or_half == 2)
			{
				if ( ((dummyhost1->y_coord + dummyhost1->height) > (size / 2) )
				  && (dummyhost1->x_coord >= limit) )
					dummyhost1->x_coord -= free_space;
			}
			else /* if (full_or_half == 3) */
			{
				if ( ((dummyhost1->y_coord + dummyhost1->height) < (size / 2) )
				 && (dummyhost1->x_coord >= limit) )
					dummyhost1->x_coord -= free_space;
			}
		}

#ifdef DEBUG
		fprintf(stderr," \t NEW: %s at %ld %ld\n", dummyhost1->hostname,
			dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		dummyhost1 = dummyhost1->next;
	}
}


	/* get the maximum free space for the nodes */
	/* there tends to be a gap between a x/y-coordinate of a node and the rest of the nodes.
	   we can reduce the size of the WINDOW if we correct for this unused x/y-space */
	/* This is a RECURSIVE routine */

extern void reduce_gaps(nodestruct, distance, is_x, full_or_half, size)
struct hostentity * nodestruct;
long int distance;
int is_x;
int full_or_half;	/* 1: all coordinates, 2: upper half, 3: lower half */
int size;		/* the size of the window (before adjusting) */
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	long int cor1,  cor2,  x_cor1,  x_cor2,  y_cor1,  y_cor2;
	long int size1, size2, x_size1, x_size2, y_size1, y_size2;
	long int free, max, where;


	move_towards_boundary(nodestruct);

	max = 0;
	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		free = size;

		x_cor1 = dummyhost1->x_coord;	x_size1 = dummyhost1->width;
		y_cor1 = dummyhost1->y_coord;
		y_size1 = get_height(dummyhost1);

		if (is_x)
		{
			cor1 = x_cor1;		size1 = x_size1;
		}
		else
		{
			cor1 = y_cor1;		size1 = y_size1;
		}

		if ( (full_or_half == 1)
		|| ( (full_or_half == 2) && ((cor1 + size1) > (size / 2)) )
		|| ( (full_or_half == 3) && (cor1 < (size / 2)) ) )
		{
			dummyhost2 = host;
			while (dummyhost2 != NULL)
			{
				if (dummyhost1->number != dummyhost2->number)
				{
					x_cor2 = dummyhost2->x_coord;	x_size2 = dummyhost2->width;
					y_cor2 = dummyhost2->y_coord;
					y_size2 = get_height(dummyhost2);

					if (is_x)
					{
						cor2 = x_cor2;		size2 = x_size2;
					}
					else
					{
						cor2 = y_cor2;		size2 = y_size2;
					}

					if (full_or_half == 1)
						determine_space(cor1, size1, cor2, size2, &free);

					if ( ( (full_or_half == 2) && ((cor2 + size2) > (size / 2)) )
					  || ( (full_or_half == 3) && (cor2 < (size / 2)) ) )
					{
	if (is_x)
		determine_space(y_cor1, y_size1, y_cor2, y_size2, &free);
	else
		determine_space(x_cor1, x_size1, x_cor2, x_size2, &free);
					}
				}
				dummyhost2 = dummyhost2->next;
			}

			if (free == size)
				free = 0;

			if (free > max)
			{
				max = free;
				if (full_or_half == 1)
					where = cor1;
				else if ( (full_or_half != 1) && (is_x) )
					where = y_cor1;
				else
					where = x_cor1;
			}
		}

		dummyhost1 = dummyhost1->next;
	}

	if (max > distance)
	{
		max = max - distance;
		if (is_x)
		{
			if (full_or_half == 1)
			{
				fprintf(stderr,"reducing gap (%ld) in x direction with %ld\n", (distance + max), max);
				adjust_nodes(where, max, TRUE, full_or_half, size);
				reduce_gaps(nodestruct, distance, TRUE, full_or_half, size);
			}
			else
			{
   				if (full_or_half == 2)
					fprintf(stderr,"Y-Gap (%ld) fixing for upper x-part of window with %ld\n",
						(distance + max), max);
				else
					fprintf(stderr,"Y-Gap (%ld) fixing for lower x-part of window with %ld\n",
						(distance + max), max);
				adjust_nodes(where, max, TRUE, full_or_half, size);
			}
		}
		else
		{
			if (full_or_half == 1)
			{
				fprintf(stderr,"reducing gap (%ld) in y direction with %ld\n", (distance + max), max);
				adjust_nodes(where, max, FALSE, full_or_half, size);
				reduce_gaps(nodestruct, distance, FALSE, full_or_half, size);
			}
			else
			{
				if (full_or_half == 2)
					fprintf(stderr,"X-Gap (%ld) fixing for upper y-part of window with %ld\n",
						(distance + max), max);
				else
					fprintf(stderr,"X-Gap (%ld) fixing for lower y-part of window with %ld\n",
						(distance + max), max);
				adjust_nodes(where, max, FALSE, full_or_half, size);
			}
		}
	}

	move_towards_boundary(nodestruct);
}


	/* if the nodes are only in a part of the window we can multiply
	   all coordinates in order to get them spread in the entire window */

extern void resizing(is_x)
int is_x;
{
	struct hostentity * dummyhost1;	/* host to determine the changes for */
	long int x1, y1;		/* temp. values for host1 */
	long int w1, h1;
	long int new;			/* new coordinates for the node */
	float scale;			/* scale factor for new layout */
	long int min, max;		/* minimum x and y coordinates for all nodes */


	fprintf(stderr,"Resizing and rearranging ");
	if (is_x)
		fprintf(stderr,"in x direction\n");
	else
		fprintf(stderr,"in y direction\n");

	/* get scaling factors for all nodes (whole window) */

	get_min_max_coord(host, &min, &max, is_x);

	if (max != min)
		scale = (float) (WINDOWwidth  - 2 * boundary) / (max - min);
	else
		scale = 1.0;

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{

#ifdef DEBUG
		fprintf(stderr,"resizing: node %s at %ld %ld  ",
			dummyhost1->hostname, dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		if (is_x)
		{
			x1 = dummyhost1->x_coord;	w1 = dummyhost1->width;
			new = boundary + ((x1 - min) * scale);
			if (new < boundary)
				new = boundary;
			if ((new + w1) > (WINDOWwidth - boundary))
				new = WINDOWwidth - boundary - w1;
			dummyhost1->x_coord = new;
		}
		else
		{
			y1 = dummyhost1->y_coord;
			h1 = get_height(dummyhost1);
			new = boundary + ((y1 - min) * scale);
			if (new < boundary)
				new = boundary;
			if ((new + h1) > (WINDOWheight - boundary))
				new = WINDOWheight - boundary - h1;
			dummyhost1->y_coord = new;
		}

#ifdef DEBUG
		fprintf(stderr,"to %ld %ld\n", dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		dummyhost1 = dummyhost1->next;
	}
}
