/*******************************************************************************
 *
 *	output.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part produces all output. It prints the nodestructure, the
 *	linkstructure, and the (color) postscriptoutput.
 *
 ******************************************************************************/

# include "layout.h"


	/* printroutine for the nodes (to stderr or stdout) */
      
extern void print_node(all, final)
struct hostentity *all;
int final;
{
	struct hostentity *dummyhost1;

	if (! final)
		fprintf(stderr,"printing hoststructure\n");
	dummyhost1 = all;

	while (dummyhost1 != NULL)
	{
		if (final)
		{
			printf("%s\t%s\t%ld\t%ld\t%ld\t%ld\n",dummyhost1->id, dummyhost1->hostname,
				dummyhost1->x_coord, dummyhost1->y_coord,
				dummyhost1->width, dummyhost1->height);
		}
		else
		{
			fprintf(stderr,"%s %s  %ld  %ld  %ld  %ld\n",dummyhost1->id, dummyhost1->hostname,
				dummyhost1->x_coord, dummyhost1->y_coord,
				dummyhost1->width, dummyhost1->height);
		}

		dummyhost1 = dummyhost1->next;
	}
}


	/* printroutine for the links (to stderr or stdout) */

extern void print_link(all, final)
struct linkentity *all;
int final;
{
	struct linkentity *dummylink1;

	if (! final)
		fprintf(stderr,"printing linkstructure\n");
	dummylink1 = all;

	while (dummylink1 != NULL)
	{
		if (final)
			printf("%s\t%s\t%s\n",dummylink1->linktype, dummylink1->id_one , dummylink1->id_two);
		else
			fprintf(stderr,"%s  %s  %s\n",dummylink1->linktype,dummylink1->id_one,dummylink1->id_two);

		dummylink1 = dummylink1->next;
	}
}


	/* find a good place to position the interface between the two connecting nodes
	   if the new position is a straight line choose that */

static void choose_attachment(x1, y1, w1, h1, x2, y2, w2, h2, x_int, y_int, w_int, h_int)
long int x1, y1, x2, y2;	/* coordinates of the two connecting nodes */
long int w1, h1, w2, h2;	/* sizes of the two connecting nodes */
long int *x_int, *y_int;	/* coordinates of the interface */
long int w_int, h_int;		/* sizes of the interface box */
{
	long int xr1, yr1, xr2, yr2;

	nearest_point(x1, x2, w1, w2, &xr1, &xr2);
	nearest_point(y1, y2, h1, h2, &yr1, &yr2);

	if (xr1 == xr2)
	{
		*x_int = xr1;		*y_int = (yr1 + yr2) / 2;
	}
	else if (yr1 == yr2)
	{
		*x_int = (xr1 + xr2) / 2;		*y_int = yr1;
	}
	else
	{
		*x_int = xr1 + (xr2 - xr1);
		if (x1 > x2)
			*x_int -= 3;
		else if (x1 < x2)
			*x_int += 3;
		if (y1 > y2)
			*y_int = yr1 - (h1 / 2);
		else if (y1 < y2)
			*y_int = yr1 + (h1 / 2);
	}
}


	/* draw a rectangular box (for a node or interface) */

static void draw_box(psfile, x1, y1, w1, h1)
FILE *psfile;
long int x1, y1, w1, h1;
{
	fprintf(psfile,"%04ld %04ld mt ",x1, y1);
	fprintf(psfile,"%04ld %04ld lt ",(x1 + w1), y1);
	fprintf(psfile,"%04ld %04ld lt ",(x1 + w1), (y1 + h1));
	fprintf(psfile,"%04ld %04ld lt ",x1, (y1 + h1));
	fprintf(psfile,"%04ld %04ld lt\n",x1, y1);
	fprintf(psfile,"sk\n");
}


	/* produce the general information in the postscriptoutputfile */

static void header(psfile, color, fontsize, lower_y_boundary, upper_y_boundary)
FILE *psfile;
int color, fontsize;
int lower_y_boundary, upper_y_boundary;
{
	long UTC;
	float xfact, yfact;	/* scalefactor for an a4 postscript picture */
	long int tmpx, tmpy, tmpWw;


#ifdef DEBUG
	fprintf(stderr,"header: producing general info for postscript\n");
#endif

	time(&UTC);

	fprintf(psfile,"%%!\n");
	fprintf(psfile,"%% postscript output from layout dd. %s",ctime(&UTC));
	fprintf(psfile,"/mt {moveto} def\n");
	fprintf(psfile,"/lt {lineto} def\n");
	fprintf(psfile,"/slw {setlinewidth} def\n");
	if (color)
		fprintf(psfile,"/rgb {setrgbcolor} def\n");
	else
		fprintf(psfile,"/gy {setgray} def\n");
	fprintf(psfile,"/sk {stroke} def\n");
	fprintf(psfile,"%% Print text in Times-Roman %d\n",fontsize);
	fprintf(psfile,"/Times-Roman findfont %d scalefont setfont\n",fontsize);

	/* stuff for A3-format (edit later in the postscriptfile */

	fprintf(psfile,"%%%% extra stuff for A3 format\n");
	fprintf(psfile,"%%statusdict begin\n");
	fprintf(psfile,"%%a3 a3tray\n");
	fprintf(psfile,"%%0 setjobtimeout\n");
	fprintf(psfile,"%%end\n");
	fprintf(psfile,"%%/sfactor {2 sqrt} def\n");
	fprintf(psfile,"%%sfactor sfactor scale\n");
	fprintf(psfile,"%%%% that's all folks (for A3 format)\n");

	if (WINDOWwidth <= 450 )
		xfact = 1.0;
	else
	{
		xfact = (float) WINDOWwidth / 450;
		xfact = 1.0 / xfact;
	}

	if (WINDOWheight <= 500 )
		yfact = 1.0;
	else
	{
		yfact = (float) WINDOWheight / 500;
		yfact = 1.0 / yfact;
	}

	fprintf(psfile,"%3.2f %3.2f scale\n",xfact, yfact);
	if (WINDOWwidth > WINDOWheight)
	{
		tmpx = (int) 50 / xfact;
		fprintf(psfile,"90 rotate\n");
		tmpWw = WINDOWwidth * 1.1;
		fprintf(psfile,"%d -%d translate\n", tmpx, tmpWw);
	}
	else
	{
		tmpx = (int) 50 / xfact;
		tmpy = (int) 40 / yfact;
		fprintf(psfile,"%d %d translate\n",tmpx,tmpy);
	}

	/* windowframe in red (?) */

	fprintf(psfile,"1 slw  ");
	if (color)
		fprintf(psfile,"1.0 0.0 0.0 rgb\n");
	else
		fprintf(psfile,"0.1 gy\n");
	draw_box(psfile, 0, 0, WINDOWwidth, WINDOWheight);

	/* border (used for unlinked nodes / one_link_node connected to another one_link_node) */

/* Used only for testing purposes */
#ifdef DEBUG
	if (color)
		fprintf(psfile,"0.8 0.3 0.3 rgb\n");

	fprintf(psfile,"%04ld %04ld mt ",boundary, lower_y_boundary);
	fprintf(psfile,"%04ld %04ld lt ",(WINDOWwidth - boundary), lower_y_boundary);
	fprintf(psfile,"%04ld %04ld lt ",(WINDOWwidth - boundary), (WINDOWheight - upper_y_boundary));
	fprintf(psfile,"%04ld %04ld lt ",boundary, (WINDOWheight - upper_y_boundary));
	fprintf(psfile,"%04ld %04ld lt\n",boundary, lower_y_boundary);
	fprintf(psfile,"sk\n");
#endif
}


	/* make outputfile with postscript commands */

extern void make_postscript(filename, color, highlight, nodename, lower_y_boundary, upper_y_boundary)
char filename[];
int color;		/* produce colorpostscript or not */
int highlight;		/* highlight a node and its connections */
char nodename[MaxEnt];
int lower_y_boundary, upper_y_boundary;		/* y_boundaries */
{
	struct hostentity * dummyhost1;	/* hoststructure */
	struct hostentity * dummyhost2;	/* hoststructure */
	FILE *psfile;			/* outputfile */
	long int x1, y1, x2, y2;	/* coordinates */
	long int w1, h1, w2, h2;	/* sizes */
	long int x1_center, y1_center, x2_center, y2_center;	/* center of a node */
	long int x_int, y_int;		/* center for a interface between node 1 and node 2 */
	int INTwidth = 4;		/* width for an interface */
	int fontsize = DEFAULTheight - 4;	/* fontsize for nodename */
	int nodecount, count;


	if (color)
		fprintf(stderr,"\ncreating color postscriptfile %s with the layoutresults\n", filename);
	else
		fprintf(stderr,"\ncreating postscriptfile %s with the layoutresults\n", filename);

	if ((psfile=fopen(filename,"w")) == NULL)
	{
		fprintf(stderr,"I can't create %s !!\n", filename);
		fprintf(stderr,"creating psfile.ps instead\n");
		if ((psfile=fopen("psfile.ps","w")) == NULL)
		{
			fprintf(stderr,"I can't create psfile.ps !!\n");
			fprintf(stderr,"creating /tmp/psfile instead\n");
			if ((psfile=fopen("/tmp/psfile","w")) == NULL)
			{
				fprintf(stderr,"I can't create /tmp/psfile !!\n");
				exit(1);
			}
		}
	}

	/* start of postscriptfile */

	header(psfile, color, fontsize, lower_y_boundary, upper_y_boundary);

	/* where are the linked nodes */

	fprintf(psfile,"%%\n%% Here come the nodes with links\n%%\n");

	nodecount = 0;
	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{

#ifdef DEBUG
		fprintf(stderr,"make_postscript: linked node %s\n",dummyhost1->hostname);
#endif

		nodecount ++;
		x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;
		w1 = dummyhost1->width;		h1 = dummyhost1->height;

		/* text in light_blue (?) */

		if (color)
			fprintf(psfile,"0.0 1.0 1.0 rgb\n");
		else
			fprintf(psfile,"0.1 gy\n");
		fprintf(psfile,"%4.0f %4.0f mt ",(float) (x1 + ((w1 - (strlen(dummyhost1->hostname) * 7)) / 2)),
				(float) (y1 + ((h1 - fontsize) / 2)));
		fprintf(psfile,"(%s) show\n",dummyhost1->hostname);

		/* box in dark_blue (?) */

		if ( (highlight)
		  && ((strcmp(dummyhost1->hostname,nodename)) == 0) )
			fprintf(psfile,"3 slw  ");
		else
			fprintf(psfile,"1 slw  ");

		if (color)
		{
			fprintf(psfile,"sk\n");
			fprintf(psfile,"0.0 0.0 1.0 rgb\n");
		}
		draw_box(psfile, x1, y1, w1, h1);

		/* lines and interfaces between connecting nodes */

		dummyhost2 = host;

		for (count=1; count <= nodecount ;count++)
			dummyhost2 = dummyhost2->next;

		while (dummyhost2 != NULL)
		{
			if ( (strcmp(dummyhost1->hostname,dummyhost2->hostname) != 0)
			   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
			{
				x2 = dummyhost2->x_coord;	y2 = dummyhost2->y_coord;
				w2 = dummyhost2->width;		h2 = dummyhost2->height;

				nearest_point(x1, x2, w1, w2, &x1_center, &x2_center);
				nearest_point(y1, y2, h1, h2, &y1_center, &y2_center);

				choose_attachment(x1, y1, w1, h1, x2, y2, w2, h2, &x_int, &y_int, INTwidth, INTwidth);

			/* interface in green (?) */

				if ( (highlight)
				&& ( ((strcmp(dummyhost1->hostname,nodename)) == 0)
				  || ((strcmp(dummyhost2->hostname,nodename)) == 0) ) )
				{
					fprintf(psfile,"2.0 slw  ");
					if (color)
						fprintf(psfile,"0.0 1.0 0.0 rgb\n");
					else
						fprintf(psfile,"0.1 gy\n");
				}
				else
				{
					fprintf(psfile,"0.5 slw  ");
					if (color)
						fprintf(psfile,"0.0 0.8 0.0 rgb\n");
					else
						fprintf(psfile,"0.3 gy\n");
				}

				fprintf(psfile,"%04ld %04ld mt ",x1_center, y1_center );
				fprintf(psfile,"%04ld %04ld lt ",x_int, y_int);
				fprintf(psfile,"%04ld %04ld lt\n",x2_center, y2_center);

				draw_box(psfile, x_int, y_int, INTwidth, INTwidth);
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* where are the unlinked nodes */

	dummyhost1 = host_zero_links;

	if (dummyhost1 != NULL)
	{
		fprintf(psfile,"%%\n%% Here come the nodes without links\n%%\n");

		if (! color)
			fprintf(psfile,"0.1 gy\n");
	}

	while (dummyhost1 != NULL)
	{

#ifdef DEBUG
		fprintf(stderr,"make_postscript: UNlinked node %s\n",dummyhost1->hostname);
#endif

		x1 = dummyhost1->x_coord;	y1 = dummyhost1->y_coord;
		w1 = dummyhost1->width;		h1 = dummyhost1->height;

		/* text in light_blue (?) */

		if (color)
			fprintf(psfile,"0.0 1.0 1.0 rgb\n");
		fprintf(psfile,"%4.0f %4.0f mt ",(float) (x1 + ((w1 - (strlen(dummyhost1->hostname) * 7)) / 2)),
				(float) (y1 + ((h1 - fontsize) / 2)));
		fprintf(psfile,"(%s) show\n",dummyhost1->hostname);

		/* box in dark_blue (?) */

		if ( (highlight)
		  && ((strcmp(dummyhost1->hostname,nodename)) == 0) )
			fprintf(psfile,"3 slw  ");
		else
			fprintf(psfile,"1 slw  ");

		if (color)
		{
			fprintf(psfile,"sk\n");
			fprintf(psfile,"0.0 0.0 1.0 rgb\n");
		}

		draw_box(psfile, x1, y1, w1, h1);

		dummyhost1 = dummyhost1->next;
	}

	/* end of postscriptfile */

	fprintf(psfile,"showpage\n");
	fclose(psfile);

#ifdef DEBUG
	fprintf(stderr,"make_postscript: finished postscript\n");
#endif
}
