/*******************************************************************************
 *
 *	place.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part determines where to put the nodes with only links to
 *	nodes with one link. It checks for overlap at the upperleft part of
 *	the window and by moving to the upperright, lowerright, lowerleft, and
 *	finally back to the upperleft part it gets the first free place along
 *	the axis.
 *	The nodes without connection are placed on the bottompart of the window.
 *
 ******************************************************************************/

# include "layout.h"


	/* determine where on the x/y-axis the more_links_nodes with only links to one_link_nodes
	   can be placed */

static void where_to_put(hoststr, width, height, h_node_only, number, x, y)
struct hostentity * hoststr;
long int width, height;		/* height of node AND connecting one_link_nodes */
long int h_node_only;		/* height of node */
int number;
long int *x, *y;
{
	long int x_value, y_value;
	int Is_taken;


	/* get place along the upper x-axis, resp. along the right y_axis */

	x_value = boundary;
	y_value = WINDOWheight - boundary - height;

	Is_taken = check_free(hoststr, x_value, y_value,
			      width, height, number, TRUE);
	if (! Is_taken)
		y_value += height - h_node_only;

	while ( (Is_taken)
	     && (x_value < (WINDOWwidth - boundary - width))
	     && (y_value > boundary) )
	{
		if (Is_taken)
		{
			if (x_value < (WINDOWwidth - boundary - width))
				x_value += width / 2;
			else
			{
				x_value = WINDOWwidth - boundary - width;
				if (y_value > boundary)
					y_value -= height * 3;
				else
					y_value = boundary;
			}
			Is_taken = check_free(hoststr, x_value, y_value,
					      width, height, number, TRUE);
		}
		if (! Is_taken)
			y_value += height - h_node_only;
	}

	/* otherwise: get place along the lower x-axis, resp. along the left y_axis */

	if (Is_taken)
	{
		x_value = WINDOWwidth - boundary - width;
		y_value = boundary;

		Is_taken = check_free(hoststr, x_value, y_value,
				      width, height, number, TRUE);

		while ( (Is_taken)
		     && (x_value > boundary)
		     && (y_value < (WINDOWheight - boundary - height)) )
		{
			if (Is_taken)
			{
				if (x_value > boundary)
					x_value -= width / 2;
				else
				{
					x_value = boundary;
					if (y_value < (WINDOWheight - boundary - height))
						y_value += height * 3;
					else
						y_value = WINDOWheight - boundary - height;
				}
				Is_taken = check_free(hoststr, x_value, y_value,
						      width, height, number, TRUE);
			}
		}
	}

	/* could not place the node somewhere along the 4 axis ! */

	if (Is_taken)
	{
		fprintf(stderr,"where_to_put: couldn't place this node (number %d)\n", number);
		fprintf(stderr,"NEEDS TO BE IMPLEMENTED\n");
	}
	else
	{
		*x = x_value;	*y = y_value;
	}
}


	/* put the nodes with more links, but only to one-link nodes, on the window */
	/* and add the one_link_nodes connected to this more_links_node */

extern void place_more_linked_nodes(sizecheck)
int sizecheck;
{
	struct hostentity * dummyhost1;		/* node to determine the number of links for */
	struct hostentity * dummyhost2;		/* one_link_node connected to special more_link_node */
	struct hostentity * newhost;
	int nodecount;				/* number of linked one_link_nodes to a more_links_node */
	long int x, y;				/* x_value and y_value */
	long int h;
	int Found;


	newhost = host;
	dummyhost1 = host_more_to_one;

	while (dummyhost1 != NULL)
	{
		h = get_height(dummyhost1);

		where_to_put(newhost, dummyhost1->width, h, dummyhost1->height, dummyhost1->number, &x, &y);
		dummyhost1->x_coord = x;	dummyhost1->y_coord = y;

#ifdef DEBUG
		fprintf(stderr,"place_more_linked_nodes: %s %ld %ld %ld %ld\n",dummyhost1->hostname,dummyhost1->x_coord,
				dummyhost1->y_coord,dummyhost1->width,dummyhost1->height);
#endif

		newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
					   dummyhost1->x_coord, dummyhost1->y_coord,
					   dummyhost1->width, dummyhost1->height,
					   dummyhost1->link, dummyhost1->number, TRUE);

		/* place one_link_node close to this special more_links_node */

		nodecount = 1;
		dummyhost2 = host_one_link;

		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->number, dummyhost2->number))
			{
				dummyhost2->x_coord = dummyhost1->x_coord;
				dummyhost2->y_coord = dummyhost1->y_coord - dummyhost1->height
						    - (nodecount * (dummyhost2->height + 5));
				nodecount ++;

#ifdef DEBUG
				fprintf(stderr,"place_more: %s linked and placed on: %ld %ld %ld %ld\n",dummyhost2->hostname,
					dummyhost2->x_coord, dummyhost2->y_coord,
					dummyhost2->width, dummyhost2->height);
#endif

			/* place these also in the temp. structure (with more_linked_nodes) */

				newhost = addnodestructure(newhost, dummyhost2->id,dummyhost2->hostname,
							   dummyhost2->x_coord, dummyhost2->y_coord,
							   dummyhost2->width, dummyhost2->height,
							   dummyhost2->link, dummyhost2->number, FALSE);
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}

	/* save the more_linked nodes and the special more_link_nodes with their connecting one_link_nodes.
	   replace the complete one_link_nodestructure by the structure without the one_link_nodes
	   connected to more_linked_nodes */

	host = newhost;
	newhost = NULL;

	dummyhost1 = host_one_link;

	while (dummyhost1 != NULL)
	{
		Found = FALSE;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (dummyhost1->number == dummyhost2->number)
				Found = TRUE;
			dummyhost2 = dummyhost2->next;
		}

		if (! Found)
			newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
						   dummyhost1->x_coord, dummyhost1->y_coord,
						   dummyhost1->width, dummyhost1->height,
						   dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	host_one_link = newhost;
}


	/* place the unlinked nodes in the bottomborder of the window */

extern void place_unlinked_nodes(lowerboundary)
int *lowerboundary;		/* area for unlinked nodes */
{
	struct hostentity * dummyhost1;	/* node without links to determine the coordinates for */
	long int x_offset = 100;	/* starting x_value */
	long int unlinked_y_coord;	/* y_coordinate for unlinked nodes */
	int unlinked_nr = 0;
	int highest = 0;
	int nodecount = 0;
	long int total_width = 0;
	float temp_value;


	*lowerboundary = 25;

	/* determine boundary to use for all zero_links_nodes */

	dummyhost1 = host_zero_links;
	while (dummyhost1 != NULL)
	{
		if (dummyhost1->height > highest)
			highest = dummyhost1->height;
		nodecount ++;
		total_width += dummyhost1->width + 25;
		dummyhost1 = dummyhost1->next;
	}

	if (nodecount != 0)
	{
		temp_value = (float) total_width / WINDOWwidth;
		*lowerboundary += highest + 10;
		while (temp_value > 0)
		{
			temp_value --;		*lowerboundary += highest + 10;
		}
	}

	WINDOWheight += (*lowerboundary - boundary);

	if (nodecount == 0)
		adjust_coord(host, FALSE, TRUE, *lowerboundary);
	else
	{
		if (*lowerboundary > (boundary + 25))
			adjust_coord(host, FALSE, TRUE, (*lowerboundary - boundary + 25));
		else if (*lowerboundary == (boundary + 25))
		{
			WINDOWheight += 25;
			adjust_coord(host, FALSE, TRUE, boundary);
		}
		else
		{
			WINDOWheight += 25;
			adjust_coord(host, FALSE, TRUE, (*lowerboundary));
		}
	}

	fprintf(stderr,"place_unlinked_nodes: WINDOWheight adjusted to %ld\n", WINDOWheight);

	/* place the unlinked nodes in this area */

	unlinked_y_coord = 25;
	dummyhost1 = host_zero_links;

	while (dummyhost1 != NULL)
	{
		unlinked_nr ++;
		if ( (x_offset + dummyhost1->width) <= (WINDOWwidth - 100) )
		{
			dummyhost1->x_coord = x_offset;
			x_offset += dummyhost1->width + 25;
		}
		else
		{
			x_offset = 100;
			unlinked_y_coord += (highest + 10);
			dummyhost1->x_coord = x_offset;
			if (unlinked_y_coord > (*lowerboundary - highest))
			{
				unlinked_y_coord = *lowerboundary - highest;
				fprintf(stderr,"Too many unlinked nodes (%d) to place on WINDOWbottompart\n",unlinked_nr);
			}
		}
		dummyhost1->y_coord = unlinked_y_coord;

		printf("%s %s %ld %ld %ld %ld\n",dummyhost1->id, dummyhost1->hostname,
			dummyhost1->x_coord, dummyhost1->y_coord,
			dummyhost1->width, dummyhost1->height);

#ifdef DEBUG
		fprintf(stderr,"place_unlinked_nodes: %s %s %ld %ld %ld %ld\n",dummyhost1->id, dummyhost1->hostname,
				dummyhost1->x_coord, dummyhost1->y_coord,
				dummyhost1->width, dummyhost1->height);
#endif

		dummyhost1 = dummyhost1->next;
	}
}
