/*******************************************************************************
 *
 *	place_adjust.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	Februari 1992.
 *	email: a89@nikhef.nl
 *
 *
 *	This part places the nodes on the window in a random order. However
 *	the more links a node has the closer it is placed to the center of the
 *	window.
 *	Nodes connected to two other nodes get coordinates in between these two.
 *	Nodes with coordinates close to coordinates of connecting nodes are
 *	adjusted.
 *
 ******************************************************************************/

# include "layout.h"


	/* Determine randomplacement of the nodes in the window.
	   Nodes with more links closer to the center. */

static void random_placement(hoststr, tmpstr, grpid, rand)
struct hostentity ** hoststr;
struct hostentity * tmpstr;
int grpid, rand;
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	struct hostentity * newhost;
	long int x_center, y_center, x_size, y_size;
	long int x_total, y_total;
	int nr_links, links;
	long x_rand, y_rand;
	long UTC;

	time (&UTC);
	srandom( (UTC % 10000) );
/* Temporarily */
	srandom (number_of_nodes);
/* TEMPORARILY */


	if (grpid == 3)
	{
		x_size = WINDOWwidth - 2 * boundary;
		y_size = WINDOWheight - 2 * boundary;
	}
	else
	{
		dummyhost1 = *hoststr;
		if (dummyhost1 != NULL)
		{
			if (grpid == 2) /* nodes with 3 or 4 links */
			{
				x_size = (WINDOWwidth - 2 * boundary) / 2;
				y_size = (WINDOWheight - 2 * boundary) / 2;
			}
			else /* grpid=1 : nodes with 1 or 2 links */
			{
				x_size = (WINDOWwidth - 2 * boundary) / 4;
				y_size = (WINDOWheight - 2 * boundary) / 4;
			}
		}
		else
		{
			x_size = WINDOWwidth - 2 * boundary;
			y_size = WINDOWheight - 2 * boundary;
		}
	}

	newhost = *hoststr;	dummyhost1 = tmpstr;
	while (dummyhost1 != NULL)
	{
		/* generate random coordinates
		   Nodes with more links get closer to the center of the window */

		links = 0;
		dummyhost2 = host;
		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->number, dummyhost2->number))
				links ++;
			dummyhost2 = dummyhost2->next;
		}

		x_rand = random() % 100;	y_rand = random() % 100;
		if (x_rand > 50)
			x_rand -= 100;
		if (y_rand > 50)
			y_rand -= 100;

		if (grpid == 3)
		{
			x_center = WINDOWwidth / 2;	y_center = WINDOWheight / 2;
		}
		else
		{
			x_total = y_total = nr_links = 0;
			dummyhost2 = *hoststr;
			if (dummyhost2 == NULL)
			{
				x_center = WINDOWwidth / 2;	y_center = WINDOWheight / 2;
			}
			else
			{
				while (dummyhost2 != NULL)
				{
					if (linked_or_not (dummyhost1->number, dummyhost2->number))
					{
						nr_links++;
						x_total += dummyhost2->x_coord;		y_total += dummyhost2->y_coord;
					}
					dummyhost2 = dummyhost2->next;
				}
				if (nr_links != 0)
				{
					x_center = x_total / nr_links;		y_center = y_total / nr_links;
				}
				else
				{
					x_center = WINDOWwidth / 2;	y_center = WINDOWheight / 2;
				}
			}
		}

		if ((rand) || (dummyhost1->x_coord == 0))
		{
			dummyhost1->x_coord = x_center + x_rand * x_size / (50 * links);

			if (dummyhost1->x_coord < boundary)
				dummyhost1->x_coord = boundary;
			if ( (dummyhost1->x_coord + dummyhost1->width) > (WINDOWwidth - boundary))
				dummyhost1->x_coord = WINDOWwidth - boundary - dummyhost1->width;
		}

		if ((rand) || (dummyhost1->y_coord == 0))
		{
			dummyhost1->y_coord = y_center + y_rand * y_size / (50 * links);

			if (dummyhost1->y_coord < boundary)
				dummyhost1->y_coord = boundary;
			if ( (dummyhost1->y_coord + dummyhost1->height) > (WINDOWheight - boundary))
				dummyhost1->y_coord = WINDOWheight - boundary - dummyhost1->height;
		}

#ifdef DEBUG
		fprintf(stderr,"random_placement: %s (%d) at %ld %ld\n",dummyhost1->hostname,
			grpid, dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		newhost = addnodestructure(newhost, dummyhost1->id, dummyhost1->hostname,
					   dummyhost1->x_coord, dummyhost1->y_coord,
					   dummyhost1->width, dummyhost1->height,
					   dummyhost1->link, dummyhost1->number, TRUE);
		dummyhost1 = dummyhost1->next;
	}

	*hoststr = newhost;
}


	/* get number of links for the nodes and then place them on the window
	   First the nodes with the most links, and second/third the nodes with lesser
	   links closer to the already placed nodes. */

extern void place_linked_node(max_x_size, max_y_size, rand)
long int max_x_size, max_y_size;	/* maximum sizes for a node */
int rand;				/* all nodes random coordinates or not */
{
	struct hostentity * dummyhost1;	/* node to determine the number of links for */
	struct hostentity * dummyhost2;	/* are there links to this node */
	struct hostentity * newhost;	/* nodes with more links (to more_linked_nodes) with adjusted linkcount */
	struct hostentity * host_1or2;
	struct hostentity * host_3or4;
	struct hostentity * host_many;
	int links;			/* number of links for a node */


	dummyhost1 = host;		/* all nodes with more links */
	host_more_to_one = NULL;
	host_1or2 = NULL;
	host_3or4 = NULL;
	host_many = NULL;

	while (dummyhost1 != NULL)
	{
		/* adjust sizes for a node */

		if (dummyhost1->width > max_x_size)
		{
			fprintf(stderr,"place_linked_node: adjusting width for %s (%ld) to %ld\n",
				dummyhost1->hostname, dummyhost1->width, max_x_size);
			dummyhost1->width = max_x_size;
		}
		if (dummyhost1->height > max_y_size)
		{
			fprintf(stderr,"place_linked_node: adjusting height for %s (%ld) to %ld\n",
				dummyhost1->hostname, dummyhost1->height, max_y_size);
			dummyhost1->height = max_y_size;
		}
		links = 0;
		dummyhost2 = host;

		while (dummyhost2 != NULL)
		{
			if (linked_or_not(dummyhost1->number, dummyhost2->number))
				links ++;
			dummyhost2 = dummyhost2->next;
		}

#ifdef DEBUG
		fprintf(stderr,"place_linked_node: node %s has %d links\n",dummyhost1->hostname,links);
#endif

		if (links == 0)
		{
#ifdef DEBUG
			fprintf(stderr,"place_linked_node: %s has only links to one_link_nodes\n",dummyhost1->hostname);
#endif
			/* make list with nodes with only links to one_link_nodes */

			host_more_to_one = addnodestructure(host_more_to_one, dummyhost1->id, dummyhost1->hostname,
							    dummyhost1->x_coord, dummyhost1->y_coord,
							    dummyhost1->width, dummyhost1->height,
							    dummyhost1->link,  dummyhost1->number, TRUE);
		}
		else if ((links == 1) || (links == 2))
		{
			links = dummyhost1->link - links;
			host_1or2 = addnodestructure(host_1or2, dummyhost1->id, dummyhost1->hostname,
						     dummyhost1->x_coord, dummyhost1->y_coord,
						     dummyhost1->width, dummyhost1->height,
						     links,  dummyhost1->number, TRUE);
		}
		else if ((links == 3) || (links == 4))
		{
			links = dummyhost1->link - links;
			host_3or4 = addnodestructure(host_3or4, dummyhost1->id, dummyhost1->hostname,
						     dummyhost1->x_coord, dummyhost1->y_coord,
						     dummyhost1->width, dummyhost1->height,
						     links,  dummyhost1->number, TRUE);
		}
		else
		{
			links = dummyhost1->link - links;
			host_many = addnodestructure(host_many, dummyhost1->id, dummyhost1->hostname,
						     dummyhost1->x_coord, dummyhost1->y_coord,
						     dummyhost1->width, dummyhost1->height,
						     links,  dummyhost1->number, TRUE);
		}

		dummyhost1 = dummyhost1->next;
	}

	/* determine random coordinates. */

	newhost = NULL;
	random_placement(&newhost, host_many, 3, rand);
	random_placement(&newhost, host_3or4, 2, rand);
	random_placement(&newhost, host_1or2, 1, rand);

	/* make changes known to the original hoststructure */

	host = newhost;
}


	/* check for overlap with the new coordinates */

extern int check_free(hoststr, x1, y1, w1, h1, nr1, All)
struct hostentity * hoststr;
long int x1, y1;
long int w1, h1, nr1;
int All;	/* TRUE : large box (node + connecting nodes) */
		/* FALSE: use only dimensions of the node (do not add the connecting nodes) */
{
	struct hostentity * dummyhost1;

	long int x2, y2;
	long int w2, h2;
	int Overlap = FALSE;


	dummyhost1 = hoststr;

	while (dummyhost1 != NULL)
	{
		if (nr1 != dummyhost1->number)
		{
			x2 = dummyhost1->x_coord;	y2 = dummyhost1->y_coord;
			w2 = dummyhost1->width;
			if (All)
				h2 = get_height(dummyhost1);
			else
				h2 = dummyhost1->height;

			if ( ((x1 <= x2) && (x1 + w1 >= x2)) || ((x2 <= x1) && (x2 + w2 >= x1)) )
			{
				if ( ( (y1 <= y2) && ((y1 + h1) >= y2) )
				  || ( (y2 <= y1) && ((y2 + h2) >= y1) ) )
					Overlap = TRUE;
			}
		}

		dummyhost1 = dummyhost1->next;
	}

	return Overlap;
}


	/* determine if we can give the node with two links to more_links_nodes
	   coordinates in between these two connecting nodes */

static void check_coordinates(x1, y1, x2, y2, node)
long int x1, y1, x2, y2;
struct hostentity * node;
{
	int is_taken, contr;
	long int tmpx, tmpy;
	long int gap;
	long int temp_height;


	if ( ((x1 > node->x_coord) && (x2 > node->x_coord)) || ((x1 < node->x_coord) && (x2 < node->x_coord)) )
	{
		tmpx = (x1 + x2) / 2;;
		temp_height = get_height(node);
		is_taken = check_free(host, tmpx, node->y_coord, node->width,
				      temp_height, node->number, TRUE);
		contr = 1;
		while ( (is_taken) && (contr <= 4) )
		{
			gap = (node->x_coord - ((x1 + x2) / 2)) / (contr + 3);
			tmpx += gap;	contr ++;
			is_taken = check_free(host, tmpx, node->y_coord, node->width,
					      temp_height, node->number, TRUE);
		}
		if (! is_taken)
			node->x_coord = tmpx;
	}

	if ( ((y1 > node->y_coord) && (y2 > node->y_coord)) || ((y1 < node->y_coord) && (y2 < node->y_coord)) )
	{
		tmpy = (y1 + y2) / 2;;
		temp_height = get_height(node);
		is_taken = check_free(host, node->x_coord, tmpy, node->width,
				      temp_height, node->number, TRUE);
		contr = 1;
		while ( (is_taken) && (contr <= 4) )
		{
			gap = (node->y_coord - ((y1 + y2) / 2)) / (contr + 3);
			tmpy += gap;	contr ++;
			is_taken = check_free(host, node->x_coord, tmpy, node->width,
					      temp_height, node->number, TRUE);
		}
		if (! is_taken)
			node->y_coord = tmpy;
	}
}


	/* replace the more_links_node with two connections to other more_links_nodes */

extern void adjust_some_node_2()
{
	struct hostentity * dummyhost1;		/* node to check */
	struct hostentity * dummyhost2;		/* other nodes */
	int links;				/* number of links for the node to check */
	long int x1, y1, x2, y2;		/* coordinates for the more_links_node a node-2-links is connected to */
	long int w1, h1, w2, h2;		/* sizes for the more_links_node a node-2-links is connected to */
	long int x1_center, y1_center, x2_center, y2_center;	/* midpoint of the 2 connecting more_links_node */
	char first[MaxEnt];			/* name of the first node connected to */


	dummyhost1 = host;

	while (dummyhost1 != NULL)
	{
		links = 0;

		dummyhost2 = host;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				links++;

			dummyhost2 = dummyhost2->next;
		}

		/* get more_link_nodes with only two links to other more_link_nodes */
		/* when possible (causing no overlap) change coordinates for these nodes */

		if (links == 2)
		{
			strcpy(first,"what_a_strange_name_for_a_node");
			dummyhost2 = host;
			while (dummyhost2 != NULL)
			{
				if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
				   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
				{
					if ((strcmp(first,"what_a_strange_name_for_a_node")) == 0)
					{
						strcpy(first,dummyhost2->hostname);
						x1 = dummyhost2->x_coord;	y1 = dummyhost2->y_coord;
						w1 = dummyhost2->width;
						h1 = get_height(dummyhost2);
					}
					else
					{
						x2 = dummyhost2->x_coord;	y2 = dummyhost2->y_coord;
						w2 = dummyhost2->width;
						h2 = get_height(dummyhost2);
					}
				}

				dummyhost2 = dummyhost2->next;
			}

			if ( ((x1 != dummyhost1->x_coord) && (x2 != dummyhost1->x_coord))
			  && ((y1 != dummyhost1->y_coord) && (y2 != dummyhost1->y_coord)) )
			{
				nearest_point(x1, x2, w1, w2, &x1_center, &x2_center);
				nearest_point(y1, y2, h1, h2, &y1_center, &y2_center);

#ifdef DEBUG
				fprintf(stderr,"adjust_some_node_2: %s at %ld %ld\n",
					dummyhost1->hostname, dummyhost1->x_coord, dummyhost1->y_coord);
#endif
				check_coordinates(x1_center, y1_center, x2_center, y2_center, dummyhost1);
#ifdef DEBUG
				fprintf(stderr,"adjust_some_node_2: %s is NOW at %ld %ld\n",
					dummyhost1->hostname, dummyhost1->x_coord, dummyhost1->y_coord);
#endif
			}
		}

		dummyhost1 = dummyhost1->next;
	}
}


	/* allign connecting nodes so the layout looks nice */

extern void allign_nodes()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;

	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		dummyhost2 = host;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
			{
				if ( ((dummyhost1->x_coord - dummyhost2->x_coord) <= (2 * DEFAULTheight))
				  && ((dummyhost1->x_coord - dummyhost2->x_coord) > 0) )
				{
					if (dummyhost1->x_coord > dummyhost2->x_coord)
					{
						dummyhost1->x_coord -= (dummyhost1->x_coord - dummyhost2->x_coord) / 2;
						dummyhost2->x_coord += (dummyhost1->x_coord - dummyhost2->x_coord) / 2;
        				}
        				else
        				{
						dummyhost1->x_coord += (dummyhost1->x_coord - dummyhost2->x_coord) / 2;
						dummyhost2->x_coord -= (dummyhost1->x_coord - dummyhost2->x_coord) / 2;
        				}
#ifdef DEBUG
					fprintf(stderr,"allign_nodes: x-coord. fixed for %s and %s (now at %ld and %ld)\n",
						dummyhost1->hostname, dummyhost2->hostname,
						dummyhost1->x_coord, dummyhost2->x_coord);
#endif
				}
				if ( ((dummyhost1->y_coord - dummyhost2->y_coord) <= (2 * DEFAULTheight))
				  && ((dummyhost1->y_coord - dummyhost2->y_coord) > 0) )
				{
					if (dummyhost1->y_coord > dummyhost2->y_coord)
					{
						dummyhost1->y_coord -= (dummyhost1->y_coord - dummyhost2->y_coord) / 2;
						dummyhost2->y_coord += (dummyhost1->y_coord - dummyhost2->y_coord) / 2;
					}
					else
					{
						dummyhost1->y_coord += (dummyhost1->y_coord - dummyhost2->y_coord) / 2;
						dummyhost2->y_coord -= (dummyhost1->y_coord - dummyhost2->y_coord) / 2;
					}
#ifdef DEBUG
					fprintf(stderr,"allign_nodes: y-coord. fixed for %s and %s (now at %ld and %ld)\n",
						dummyhost1->hostname, dummyhost2->hostname,
						dummyhost1->y_coord, dummyhost2->y_coord);
#endif
				}
			}

			dummyhost2 = dummyhost2->next;
		}

		dummyhost1 = dummyhost1->next;
	}
}
