/*******************************************************************************
 *
 *	read.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *
 *	This part reads the node structure, and the link structure.
 *	The nodestructure is being sorted on length of the nodename,
 *	and the ranknumber of the nodestructure is put into the linkstructure.
 *
 ******************************************************************************/

# include "layout.h"
# include <sys/types.h>


	/* print usage and exit */

extern void usage(programname)
char *programname;
{
	fprintf(stderr,"Usage : %s [-i <number of iterations>] [-h height] [-w width]\n",programname);
	fprintf(stderr,"[-c connectionname] [-f forcefactor] [-p | -P [filename]]\n");
	fprintf(stderr,"[-H nodename] [-a] [-l] [-n] [-r] [-s]\n");
	exit (-1);
}


/* this function builds the list with all nodes, with their coordinates and sizes */
/* or with their NEW coordinates and sizes (New == TRUE) */
/* or the nodes with zero or one links (New == TRUE) */


extern struct hostentity * addnodestructure(list,id,newname,x_coord,y_coord,width,height,link,rank,New)
struct hostentity *list;
char id[MaxEnt], newname[MaxEnt];
long int x_coord, y_coord;
long int width, height;
int link, rank;
int New;			/* make temp. structure with new coordinates or not */
{
	/* if the hostname doesn't exist in the list then add a new hostname, with his coordinates and sizes */

	if (list == NULL)
	{
		list = (struct hostentity *) malloc(sizeof(struct hostentity));

		/* check if there is enough memory available */

		if (list == NULL)
		{
			fprintf(stderr,"addnode: no more memory available !\n");
			exit(3);
		}

#ifdef DEBUG
		if (New)
			fprintf(stderr,"addnodestructure: adding %s to the NEW structure\n",newname);
		else
			fprintf(stderr,"addnodestructure: adding %s to the structure\n",newname);
#endif

		strcpy(list->id,id);
		strcpy(list->hostname,newname);
		list->x_coord = x_coord;
		list->y_coord = y_coord;
		if (width != 0)
			list->width = width;
		else
			list->width = strlen(newname) * 7;
		if (height != 0)
			list->height = height;
		else
			list->height = DEFAULTheight;
		list->link = link;
		list->number = rank;
		list->next = NULL;
		if ( ! New)
			number_of_nodes ++;
	}

	/* if the hostname already exists then print this and go on (New == FALSE) */
	/* or change coordinates (New == FALSE) */

	else if (strcmp(list->hostname,newname) == 0)
	{
		if (New)
		{
			fprintf(stderr,"addnode (new / zero or nul links): %s already exists\n",newname);
			list->x_coord = x_coord;
			list->y_coord = y_coord;
		}
		else
			fprintf(stderr,"addnode: %s already exists\n",newname);
	}

	/* go through the complete list until the end of the list
	   or until the hostname is found . Add the hostname with
	   his coordinates and sizes */

	else
		list->next = addnodestructure(list->next,id,newname,x_coord,y_coord,width,height,link,rank,New);

	return list;
}


/* this function builds the list with all links and both connecting sites */

extern struct linkentity * addlinkstructure(linklist,typename,first_site,second_site,rank1,rank2)
struct linkentity *linklist;
char typename[MaxEnt], first_site[MaxEnt], second_site[MaxEnt];
int rank1, rank2;
{
	/* if the connection doesn't exist in the linklist then add a new
	connection, with its connecting sites */

	if (linklist == NULL)
	{
		linklist = (struct linkentity *) malloc(sizeof(struct linkentity));

		/* check if there is enough memory available */

		if (linklist == NULL)
		{
			fprintf(stderr,"addlink: no more memory available !\n");
			exit(3);
		}

#ifdef DEBUG
		fprintf(stderr,"addlinkstructure: adding %s <-> %s\n",first_site,second_site);
#endif

		strcpy(linklist->linktype,typename);
		strcpy(linklist->id_one,first_site);
		strcpy(linklist->id_two,second_site);
		linklist->nr_one = rank1;
		linklist->nr_two = rank2;
		linklist->next = NULL;
	}

	/* if the connection already exists then print this and go on */

	else if ((strcmp(linklist->id_one,first_site) == 0)
	     &&	 (strcmp(linklist->id_two,second_site) == 0 ))
		fprintf(stderr,"addlink: %s - %s already exists\n",first_site,second_site);

	/* go through the complete linklist until the end of the linklist
	   or until an existing connection is found .
	   Add the connection with both sites */

	else
		linklist->next = addlinkstructure(linklist->next,typename,first_site,second_site,rank1,rank2);

	return linklist;
}


/* this function builds the list with all more_link_nodes and its height (with the height of the
   connecting one_link_nodes added to it) */

extern struct nodeheightentity * addheightstructure(heightlist, name, number, height)
struct nodeheightentity *heightlist;
char name[MaxEnt];
int number;
long int height;
{
	/* if the connection doesn't exist in the list then add a new */

	if (heightlist == NULL)
	{
		heightlist = (struct nodeheightentity *) malloc(sizeof(struct nodeheightentity));

		/* check if there is enough memory available */

		if (heightlist == NULL)
		{
			fprintf(stderr,"addheight: no more memory available !\n");
			exit(3);
		}

#ifdef DEBUG
		fprintf(stderr,"addheightstructure: adding %s (%ld)\n", name, height);
#endif

		strcpy(heightlist->hostname, name);
		heightlist->number = number;
		heightlist->totalheight = height;
		heightlist->next = NULL;
	}

	/* if the nodename already exists then print this and go on */

	else if (strcmp(heightlist->hostname, name) == 0)
		fprintf(stderr,"addheight: %s already exists\n", name);

	/* go through the complete list until the end of the list or until an existing
	   connection is found . Add the connection with its box-height */

	else
		heightlist->next = addheightstructure(heightlist->next, name, number, height);

	return heightlist;
}


	/* sort the nodestructure on length of the names */

extern void sort_the_nodestructure()
{
	struct hostentity * dummyhost1;
	struct hostentity * newhost;
	int length;

	newhost = NULL;

	for (length=1; length <= MaxEnt; length++)
	{
		dummyhost1 = host;
		while (dummyhost1 != NULL)
		{
			if ((strlen(dummyhost1->hostname)) == length)
				newhost = addnodestructure (newhost, dummyhost1->id,dummyhost1->hostname,
							    dummyhost1->x_coord, dummyhost1->y_coord,
							    dummyhost1->width, dummyhost1->height,
							    dummyhost1->link, dummyhost1->number, TRUE);
			dummyhost1 = dummyhost1->next;
		}
	}

	host = newhost;
}


	/* get the ranks of the linked nodes */

extern void rank_number_into_link()
{
	struct hostentity * dummyhost1;
	struct linkentity * dummylink1;

	dummylink1 = link;
	while (dummylink1 != NULL)
	{
		dummyhost1 = host;
		while (dummyhost1 != NULL)
		{
			if ((strcmp(dummyhost1->id,dummylink1->id_one)) == 0)
				dummylink1->nr_one = dummyhost1->number;
			else if ((strcmp(dummyhost1->id,dummylink1->id_two)) == 0)
				dummylink1->nr_two = dummyhost1->number;
			dummyhost1 = dummyhost1->next;
		}

#ifdef DEBUG
		fprintf(stderr,"rank_number_into_link: %s %s = %d %d\n", dummylink1->id_one,dummylink1->id_two,dummylink1->nr_one,dummylink1->nr_two);
#endif

		dummylink1 = dummylink1->next;
	}
}


	/* Determine the height needed for a node. When it has more than 1 link : get the
	   height of all nodes (with 1 link) it is connected to, and add this. */
	/* uses structures: host, host_more_to_one, host_one_link and host_zero_links.
	   these four structures contain all read-in nodes */

extern void determine_height()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	long int boxheight;


	dummyhost1 = host;
	while (dummyhost1 != NULL)
	{
		if (dummyhost1->link == 0)
			boxheight = dummyhost1->height * 2;
		else
		{
			boxheight = dummyhost1->height + DEFAULTheight + 5;

			/* add the height (+ node-node gap) for all connected one_link_nodes */

			dummyhost2 = host_one_link;
			while (dummyhost2 != NULL)
			{
				if (linked_or_not(dummyhost1->number, dummyhost2->number))
					boxheight += (dummyhost2->height + 5);
				dummyhost2 = dummyhost2->next;
			}
		}

		nodeheight = addheightstructure(nodeheight, dummyhost1->hostname, dummyhost1->number, boxheight);

		dummyhost1 = dummyhost1->next;
	}

	dummyhost1 = host_more_to_one;
	while (dummyhost1 != NULL)
	{
		if (dummyhost1->link == 0)
			boxheight = dummyhost1->height * 2;
		else
		{
			boxheight = dummyhost1->height + DEFAULTheight +5;

			/* add the height (+ node-node gap) for all connected one_link_nodes */

			dummyhost2 = host_one_link;
			while (dummyhost2 != NULL)
			{
				if (linked_or_not(dummyhost1->number, dummyhost2->number))
					boxheight += (dummyhost2->height + 5);
				dummyhost2 = dummyhost2->next;
			}
		}

		nodeheight = addheightstructure(nodeheight, dummyhost1->hostname, dummyhost1->number, boxheight);

		dummyhost1 = dummyhost1->next;
	}

	dummyhost1 = host_one_link;
	while (dummyhost1 != NULL)
	{
		boxheight = dummyhost1->height;

		nodeheight = addheightstructure(nodeheight, dummyhost1->hostname, dummyhost1->number, boxheight);

		dummyhost1 = dummyhost1->next;
	}

	dummyhost1 = host_zero_links;
	while (dummyhost1 != NULL)
	{
		boxheight = dummyhost1->height;

		nodeheight = addheightstructure(nodeheight, dummyhost1->hostname, dummyhost1->number, boxheight);

		dummyhost1 = dummyhost1->next;
	}
}
