/*******************************************************************************
 *
 *	special.c
 *
 *	part of the layout program.
 *
 *
 *	Written by Gerard Leurs, NIKHEF-H, Amsterdam.
 *	December 1991.
 *	email: a89@nikhef.nl
 *
 *	It tries to place the nodes with 2 links to other nodes in between
 *	these two. And it tries to place the node with only 1 link to another
 *	more_link_node close to this one.
 *
 ******************************************************************************/

# include "layout.h"


	/* place the node at a random place, so we can in a second overlap-correction-step
	   move it to a non-taken place */

static void random_place(x1, y1, x2, y2, new_x, new_y)
long int x1, y1, x2, y2;
long int *new_x, *new_y;
{
	long int UTC;
	long int x_rand, y_rand;


	time (&UTC);
	srandom( (UTC % 10000) );
	x_rand = random() % 100 + 1;
	y_rand = random() % 100 + 1;

#ifdef DEBUG
	fprintf(stderr,"random_place: generated x/y-factors are %ld and %ld\n", x_rand, y_rand);
#endif

	if (x1 <= x2)
		*new_x = x1 + (x_rand * (x2 - x1) / 100);
	else
		*new_x = x2 + (x_rand * (x1 - x2) / 100);

	if (y1 <= y2)
		*new_y = y1 + (y_rand * (y2 - y1) / 100);
	else
		*new_y = y2 + (y_rand * (y1 - y2) / 100);
}


	/* check in an area defined by the connecting more_linked_node,
	   if the new coordinates do NOT exactly match another */

static int matching_1(x1, y1, w1, h1, w2, h2, new_x, new_y)
long int x1, y1, w1, h1, w2, h2;
long int *new_x, *new_y;
{
	struct hostentity * dummyhost1;
	int Found = FALSE;
	int Match;
	int contr, tmp;


	/* first try 3 times right under the connecting node */

	contr = 1;	*new_x = x1;
	while ( (! Found) && (contr <= 3) )
	{
		Match = FALSE;	contr++;
		tmp = 10 - (contr * (DEFAULTheight + 5));
		if (tmp < h2)
			tmp = h2 + ((contr - 1) * (DEFAULTheight + 5));
		*new_y = y1 - tmp;

		dummyhost1 = host_more;
		while (dummyhost1 != NULL)
		{
			if ( (*new_x == dummyhost1->x_coord)
			  && (*new_y == dummyhost1->y_coord) )
				Match = TRUE;
			dummyhost1 = dummyhost1->next;
		}
		if (Match)
			Found = FALSE;
		else
			Found = TRUE;
	}

	/* then try left and right of the connecting node */

	if (! Found)
	{
		*new_x = x1 - w2 - 10;	*new_y = y1;
		dummyhost1 = host_more;
		while (dummyhost1 != NULL)
		{
			if ( (*new_x == dummyhost1->x_coord)
			  && (*new_y == dummyhost1->y_coord) )
				Match = TRUE;
			dummyhost1 = dummyhost1->next;
		}
		if (Match)
			Found = FALSE;
		else
			Found = TRUE;
        }

	if (! Found)
	{
		*new_x = x1 + w2 + 10;	*new_y = y1;
		dummyhost1 = host_more;
		while (dummyhost1 != NULL)
		{
			if ( (*new_x == dummyhost1->x_coord)
			  && (*new_y == dummyhost1->y_coord) )
				Match = TRUE;
			dummyhost1 = dummyhost1->next;
		}
		if (Match)
			Found = FALSE;
		else
			Found = TRUE;
	}

	return Found;
}


	/* place all nodes with 1 link to more_linked_nodes close to the coordinates of the
	   more_linked_one, it is connected to */

extern void place_node_1()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	long int x1, y1;		/* coordinates for the more_links_node */
	long int w1, h1, w2, h2;	/* effective height for the more_links_node and the 1-link-node */
	int Found;
	long int x_coord, y_coord;


	fprintf(stderr,"placing nodes with one connection to the nodes, with more connections\n");

	dummyhost1 = host_1;
	while (dummyhost1 != NULL)
	{
		w2 = dummyhost1->width;
		h2 = get_height(dummyhost1);

#ifdef DEBUG
		fprintf(stderr,"place_node_1: adding %s to the structure\n",dummyhost1->hostname);
#endif

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
			{
				x1 = dummyhost2->x_coord;	y1 = dummyhost2->y_coord;
				w1 = dummyhost2->width;
				h1 = get_height(dummyhost2);
			}

			dummyhost2 = dummyhost2->next;
		}

	/* place the node if there is no other node with exactly the same coordinates */

		Found = matching_1 (x1, y1, w1, h1, w2, h2, &x_coord, &y_coord);
		if (Found)
		{
			dummyhost1->x_coord = x_coord;	dummyhost1->y_coord = y_coord;

			host_more = addnodestructure(host_more, dummyhost1->id, dummyhost1->hostname,
						     dummyhost1->x_coord, dummyhost1->y_coord,
						     dummyhost1->width, dummyhost1->height,
						     dummyhost1->link, dummyhost1->number, TRUE);
		}
		else
		{
			fprintf(stderr,"place_1: %s cannot be placed in the WINDOW\n",dummyhost1->hostname);
			fprintf(stderr,"Using random placement in area defined by (x y) %ld %ld and %ld %ld\n",
				x1, y1, x1, (y1 - 100));
			random_place(x1, y1, x1, (y1 - 100), &x_coord, &y_coord);
		}

		dummyhost1 = dummyhost1->next;
	}
}


	/* check in an area defined by the 2 connecting more_linked_nodes,
	   if the new coordinates do NOT exactly match another */

static int matching_2(x1, y1, x2, y2, new_x, new_y)
long int x1, y1, x2, y2;		/* coordinates for the connecting nodes */
long int *new_x, *new_y;		/* free coordinates for the node_to_add*/
{
	struct hostentity * dummyhost2;
	int Found = FALSE;
	int Match = FALSE;
	int Tmp;
	int contr;
	long int min_x, max_x, min_y, max_y;


	/* try halfway coordinates of node1 and node2 */

	if (! Found)
	{
		*new_x = (x1 + x2) / 2;		*new_y = (y1 + y2) / 2;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( (*new_x == dummyhost2->x_coord) && (*new_y == dummyhost2->y_coord) )
				Match = TRUE;
			dummyhost2 = dummyhost2->next;
		}
		if (! Match)
			Found = TRUE;
	}

	/* if exactly matching coordinates are found:
	   try halfway x (of node1 and node2), but y of node1, resp. node2 */

	if (! Found)
	{
		*new_x = (x1 + x2) / 2;		*new_y = y1;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( (*new_x == dummyhost2->x_coord) && (*new_y == dummyhost2->y_coord) )
				Match = TRUE;
			dummyhost2 = dummyhost2->next;
		}
		if (! Match)
			Found = TRUE;
	}

	if (! Found)
	{
		*new_x = (x1 + x2) / 2;		*new_y = y2;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( (*new_x == dummyhost2->x_coord) && (*new_y == dummyhost2->y_coord) )
				Match = TRUE;
			dummyhost2 = dummyhost2->next;
		}
		if (! Match)
			Found = TRUE;
	}

	/* if exactly matching coordinates are found:
	   try halfway y (of node1 and node2), but x of node1, resp. node2 */

	if (! Found)
	{
		*new_x = x1;		*new_y = (y1 + y2) / 2;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( (*new_x == dummyhost2->x_coord) && (*new_y == dummyhost2->y_coord) )
				Match = TRUE;
			dummyhost2 = dummyhost2->next;
		}
		if (! Match)
			Found = TRUE;
	}

	if (! Found)
	{
		*new_x = x2;		*new_y = (y1 + y2) / 2;

		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( (*new_x == dummyhost2->x_coord) && (*new_y == dummyhost2->y_coord) )
				Match = TRUE;
			dummyhost2 = dummyhost2->next;
		}
		if (! Match)
			Found = TRUE;
	}

	/* the most obvious places are taken.
	   try some spots close to "halfway node1 and node2" */

	if (! Found)
	{
		*new_x = (x1 + x2) / 2;		*new_y = (y1 + y2) / 2;

		contr = 1;	Match = FALSE;
		while ( (! Match) && (contr <= 3) )
		{
			contr++;

			dummyhost2 = host_more;
			while (dummyhost2 != NULL)
			{
				if ( (*new_x == dummyhost2->x_coord)
				  && (*new_y == dummyhost2->y_coord) )
					Match = TRUE;
				dummyhost2 = dummyhost2->next;
			}
			if (Match)
			{
				Tmp = TRUE;
				get_min_max_coord(host_more, &min_x, &max_x, TRUE);
				get_min_max_coord(host_more, &min_y, &max_y, FALSE);
				if ((max_x - min_x) >= (max_y - min_y))
					*new_y += 25;
				else
					*new_x += 25;
			}
			else
			{
				Found = TRUE;	contr = 4;
			}
			if (Tmp)
				Match = FALSE;
		}
	}

	if (! Found)
	{
		*new_x = (x1 + x2) / 2;		*new_y = (y1 + y2) / 2;

		contr = 1;	Match = FALSE;
		while ( (! Match) && (contr <= 3) )
		{
			contr++;

			dummyhost2 = host_more;
			while (dummyhost2 != NULL)
			{
				if ( (*new_x == dummyhost2->x_coord)
				  && (*new_y == dummyhost2->y_coord) )
					Match = TRUE;
				dummyhost2 = dummyhost2->next;
			}
			if (Match)
			{
				Tmp = TRUE;
				get_min_max_coord(host_more, &min_x, &max_x, TRUE);
				get_min_max_coord(host_more, &min_y, &max_y, FALSE);
				if ((max_x - min_x) >= (max_y - min_y))
					*new_y -= 25;
				else
					*new_x -= 25;
			}
			else
			{
				Found = TRUE;	contr = 4;
			}
			if (Tmp)
				Match = FALSE;
		}
	}

	return Found;
}


	/* place all nodes with 2 links to more_linked_nodes halfway the coordinates of the
	   2 more_linked_ones */

extern void place_node_2()
{
	struct hostentity * dummyhost1;
	struct hostentity * dummyhost2;
	long int x1, y1, x2, y2;	/* coordinates for the more_links_node a node-2-links is connected to */
	long int w1, h1, w2, h2;	/* sizes for the more_links_node a node-2-links is connected to */
	long int x1_center, y1_center, x2_center, y2_center;		/* midpoint of the 2 connecting more_links_node */
	char first[MaxEnt];		/* name of the first node connected to */
	int Found;
	long int x_coord, y_coord;


	fprintf(stderr,"placing nodes with two connections to other nodes\n");

	dummyhost1 = host_2;
	while (dummyhost1 != NULL)
	{
		strcpy(first,"what_a_strange_name_for_a_node");
		dummyhost2 = host_more;
		while (dummyhost2 != NULL)
		{
			if ( ((strcmp(dummyhost1->hostname,dummyhost2->hostname)) != 0)
			   && (linked_or_not(dummyhost1->number,dummyhost2->number)) )
			{
				if ((strcmp(first,"what_a_strange_name_for_a_node")) == 0)
				{
					strcpy(first,dummyhost2->hostname);
					x1 = dummyhost2->x_coord;	y1 = dummyhost2->y_coord;
					w1 = dummyhost2->width;
					h1 = get_height(dummyhost2);
					if (dummyhost2->link == 0)
						h1 -= dummyhost2->height;
				}
				else
				{
					x2 = dummyhost2->x_coord;	y2 = dummyhost2->y_coord;
					w2 = dummyhost2->width;
					h2 = get_height(dummyhost2);
					if (dummyhost2->link == 0)
						h2 -= dummyhost2->height;
				}
			}

			dummyhost2 = dummyhost2->next;
		}

		nearest_point(x1, x2, w1, w2, &x1_center, &x2_center);
		nearest_point(y1, y2, h1, h2, &y1_center, &y2_center);

	/* place the node if there is no other node with exactly the same coordinates */

		Found = matching_2 (x1_center , y1_center, x2_center, y2_center, &x_coord, &y_coord);
		if (Found)
		{
			dummyhost1->x_coord = x_coord;	dummyhost1->y_coord = y_coord;

			host_more = addnodestructure(host_more, dummyhost1->id, dummyhost1->hostname,
						     dummyhost1->x_coord, dummyhost1->y_coord,
						     dummyhost1->width, dummyhost1->height,
						     dummyhost1->link, dummyhost1->number, TRUE);
		}
		else
		{
			fprintf(stderr,"place_2: %s cannot be placed in the WINDOW\n",dummyhost1->hostname);
			fprintf(stderr,"Using random placement in area defined by (x y) %ld l%d and %ld %ld\n",
				x1_center, y1_center, x2_center, y2_center);
			random_place(x1_center, y1_center, x2_center, y2_center, &x_coord, &y_coord);
		}
#ifdef DEBUG
		fprintf(stderr,"place_node_2: %s place at %ld %ld\n", dummyhost1->hostname,
			dummyhost1->x_coord, dummyhost1->y_coord);
#endif

		dummyhost1 = dummyhost1->next;
	}
}
