/******************************************************************/
/******************************************************************/
/*  $Header: /nfs/bilbo/bilbo/user/a38/monet/bin/xmonet/RCS/map.c,v 1.2 1992/04/15 11:18:27 a38 Exp a38 $  */
/******************************************************************/
/*  This is part of the user interface and handles much of the    */
/*  X windows stuff.  A map area is the collection of X wigdets   */
/*  used to XmonetDisplay a map on the screen.                          */
/******************************************************************/

#include "xmonet.h"
#include <monet/msh/Script.h>
#include <monet/map/File.h>
#include <monet/bitmap/Icons.h>
#include <monet/db/Db.h>  /*  get_titleFormat  */

#define MAP_MARGIN_SIZE 5

/******************************************************************/
/*  Globals  */
/******************************************************************/

static ListWidget MapsListWidget = NULL;
static char ** ListWidgetArray = NULL;
static int ListWidgetNum = 0;

static void  set_list_widget ()
{
  char ** list;
  MonetMap m;
  int n, longest = 0;

  if (!MapsListWidget)
    return;

  n = 0;
  FORALLMAPS(m) {
    ++n;
  }
  list = (char **) MonetMalloc ((n+1) * sizeof (char *));
  n = 0;
  FORALLMAPS(m) {
    int len = MonetStrlen (Map_title (m));

    list[n] = MonetMalloc (len + 20);
    if (Map_classesOnly (m)) {
      (void) sprintf (list[n], "%-03.3d *class-map*", n+1);
    } else if (len > 0 && m->entity && Map_is_loaded(m)) {
      if (m->saved) {
	(void) sprintf (list[n], "%-03.3d Saved    %s", n+1, Map_title (m));
      } else {
	(void) sprintf (list[n], "%-03.3d Modified %s", n+1, Map_title (m));
      }
    } else {
      (void) sprintf (list[n], "%-03.3d *unloaded*", n+1);
    }
    len = MonetStrlen (list[n]);
    if (len > longest)
      longest = len;
    ++n;
  }
  list[n] = 0;

  /*  Checking for equality is probably less expensive than drawing
      the stuff and besides it gets rid of an irritating flash.  */
  if (! MonetEqStringOfStrings (ListWidgetArray, list, ListWidgetNum, n)) {
    /*  Longest has to be in pixels unfortunately  */
    XawListChange ((Widget ) MapsListWidget, list, n, 0 /*longest*/, True);
    MonetFreeStringOfStrings (ListWidgetArray, ListWidgetNum);
    ListWidgetNum = n;
    ListWidgetArray = list;
  } else {
    MonetFreeStringOfStrings (list, n);
  }
}

static void free_map (map)
     MonetMap map;
{
  if (map->clip_region) {
    XDestroyRegion (map->clip_region);
    map->clip_region = NULL;
  }
  if (map->mapWidget) {
    XtDestroyWidget (map->mapWidget);
    map->mapWidget = NULL;
  }
  set_list_widget ();
}

/***************************************************************************
  Set map widgets and set icon name and window name of shell ancestor
  **************************************************************************/

static void set_widget (map, widget)
     MonetMap map;
     Widget widget;
{
  ShellWidget shell;
  MonetBool toplevel;

  if (shell = XmonetShellAncestor (widget, &toplevel)) {
    Window win;
    if (win = XtWindow (widget)) {
      char * name;
      Arg tmp_arg[2];

      if (toplevel) {
	/*  If this is the main widget then use the name of the current map
	    as the label  */
	MonetMap current;
	if (current = MapCurrent_get ()) {
	  name = Map_nice_name (current);
	} else {
	  name = "Xmonet";
	}
	MONETDEBUG(31, ( "Is toplevel"));
      } else {
	/*  This is a popup map, use the name of the map as a label  */
	name = Map_nice_name (map);
      }
      MONETDEBUG(31, ( "Name [%s]", name));
      XtSetArg(tmp_arg[0], XtNiconName, name);
      XtSetArg(tmp_arg[1], XtNtitle, name);
      XtSetValues((Widget) shell, tmp_arg, 2);
    }
  }
  map -> mapWidget = widget;
}

/*****************************************************************************/
/*  This is called when the current is being changed  */
/*****************************************************************************/

static void current_map (previous, map, swap)
     MonetMap  previous, map;
     MonetBool swap;
{
  if (previous && map != previous && previous->mapWidget) {
    if (swap) {
      Widget w;
      MONETDEBUG(1, ( "Swapping map windows [%s] [%s]",
		  Map_title (previous), Map_title (map)));
      w = map->mapWidget;
      set_widget (map, previous->mapWidget);
      set_widget (previous, w);
      if (w) {
	previous -> scale_by = 0.0;
	Map_request_redraw (previous);
      }
      map -> scale_by = 0.0;
      Map_request_redraw (map);
    }
  }
  set_list_widget ();
}

/*****************************************************************************/
/*  Before doing rubber banding this should be called to handle any expose   */
/*  events that have been caused by other commands in the action  */
/*****************************************************************************/

static MonetBool xt_expose ()
{
  XEvent event;
  while (XtAppPending(XmonetAppCon)) {
    XtAppPeekEvent(XmonetAppCon, &event);
    if (event.type == Expose) {
      XtAppNextEvent(XmonetAppCon, &event);
      XtDispatchEvent(&event);
      return True;
    } else {
      break;
    }
  }
  return False;
}


/**********************************************************************/
/*  Get position of cursor in a widget  */
/*  !!! fixforscale */
/**********************************************************************/

static MonetBool create_new_drawing_at_cursor (entity_class, title)
     MonetEntity entity_class;
     String title;
{
  Widget widget;
  Window junk1, junk2;
  int root_x, root_y, win_y, win_x;
  unsigned int junkM;
  Position xloc, yloc, x,y;
  Dimension width, height;
  MonetMap map;
  Drawing drawing;

  map = Map_root (MapCurrent_get());
  if (map == NULL) {
    MonetPrintfError ("You must select a map first");
    return False;
  } 
  widget = Map_widget (map);

  if (widget == NULL)
    return False;

  /* The width will not be correct unless it is realized. */
  if (!XtIsRealized(widget))
    XtRealizeWidget(widget);
  
  if (XQueryPointer(XtDisplay(widget), XtWindow(widget), &junk1, &junk2, 
		    &root_x, &root_y, &win_x, &win_y, &junkM) == False) {
    MonetPrintfError ("Could not find location of mouse pointer");
    return False;
  }
  x = win_x;
  y = win_y;
  (void) Map_get_marked (&map, &x, &y, &xloc, &yloc, &width, &height);
  Map_adjust_for_grid (map, x, y, &x, &y);

  if (Map_can_insert_drawing (map, entity_class) == False) {
    MonetPrintfError ("Cannot insert instances of class [%s] in map [%s]",
		MonetEntity_title (entity_class), Map_title (map));
  } else if (! MonetEntity_is_title_valid (entity_class, title)) {
    MonetPrintfError
      ("Title [%s] does not match the pattern [%s] for class [%s]",
       title, get_titleFormat (entity_class), MonetEntity_title (entity_class));
  } else if (drawing = Drawing_create (title, entity_class, map, x,y, 0,0)) {
    free_selection (map);
    MonetPrintfInform ("New drawing [%s] created, now position it",
		 Drawing_title (drawing));
    Drawing_set_current (drawing);
    return True;
  } else {
    MonetPrintfError ("Could not create drawing");
  }
  return False;
}

/*****************************************************************************/
/*  Takes a widget and finds the associated map area */
/*****************************************************************************/

static MonetMap widget2map(w)
     Widget w;
{
  MonetMap m;

  FORALLMAPS (m)
    if (w == m->mapWidget)
      return m;
  MonetSysError ("Map not found from widget %d", w);
  return NULL;
}

/*****************************************************************************/
/*  These routines are called just before and just after a drawing is to be
    drawn */
/*****************************************************************************/

static MonetBool begin_draw (map)
     MonetMap map;
{
  Widget w;

  if (w = Map_widget (map))
    return start_draw_X11 (w, map->x_centre, map->y_centre,
			   map->x_scale, map->y_scale, (Region) None);
  return False;
}

static void end_draw (map)
     MonetMap map;
{

}

/*****************************************************************************/
/*  Page outline  */
/*****************************************************************************/

static void draw_page_outline (w,h)
     Dimension w,h;
{
  if (Monet_debug () > 10) {
    Draw_line(shade_grey, MAP_MARGIN_SIZE, h, w, h);
    Draw_line(shade_grey, w, MAP_MARGIN_SIZE, w, h);
  }
}

/*****************************************************************************/
/* Adjust size of map area to be just large enough for the objects in it     */
/*  This routine is called every time the size of a map is        */
/*  changed.                                                      */
/*  Needs to redraw entire map.  This may send two expose events, one for    */
/*  resize and one for redrawing, this should be change to one only !!!      */
/*****************************************************************************/

/**********************************************************************/

/**********************************************************************/

static Position fix_position (position, widget_dimension, map_dimension)
     int position;
     int widget_dimension, map_dimension;
{
  if (widget_dimension >= map_dimension) {
    return 0;
  } else if (position < widget_dimension - map_dimension) {
    return widget_dimension - map_dimension;
  } else if (position > 0) {
    return 0;
  }
  return position;
}

static void Map_changed_handler (map, w,h, redraw)
     MonetMap map;
     Dimension w, h;
     MonetBool redraw;
{
  Widget widget = map->mapWidget;

  if (widget && redraw) {
    Dimension ww, hw, wa, ha;
    Position x,y;

    Scale2ActualSize (map->x_scale, map->y_scale, w, h,& wa, & ha);
    Scale2ActualSize (map->x_scale, map->y_scale, x, y,
		      (Dimension *) & x, (Dimension *)& y);

    Widget_get_dimensions (map->mapWidget, &x,&y,&ww,&hw);
    if (w > 0 && h > 0) {
      float w_ratio, h_ratio, by;
      if (w < ww) {
	w_ratio = 1.0;
      } else {
	w_ratio  = (float) ww / (float) w;
      }
      if (h < hw) {
	h_ratio = 1.0;
      } else {
	h_ratio = (float) hw / (float) h;
      }
      if (w_ratio < h_ratio) {
	by = w_ratio;
      } else {
	by = h_ratio;
      }
      if (map->scale_by < 0.001) {
	map -> x_scale  = by;
	map -> y_scale  = by;
	map -> scale_by = by;
	map -> x_centre = 0;
	map -> y_centre = 0;
      }
    }

    /*       wa = ww / wa;
	     ha = hw / ha;
	     */

    map->x_centre  = fix_position (map->x_centre, (int) ww, (int) wa);
    map->y_centre  = fix_position (map->y_centre, (int) hw, (int) ha);

    MONETDEBUG(2, ( "Width: Widget %d %d, Map %d %d", ww, hw, wa, ha));

    MONETDEBUG(2, ( "  X,Y position %d %d, scale %f %f",
		map->x_centre, map->y_centre,
		map->x_scale, map->y_scale));
    XClearArea (XmonetDisplay, XtWindow(widget), 0,0 ,0,0, True);
  }
}

#ifdef _old_resizing_stuff
/*
if (widget) {
  XtWidgetGeometry request, result;
  Dimension w_popup, h_popup;
    Position x,y, old_w = w, old_h = h;

    Widget_get_dimensions (widget, &x,&y,&w_popup,&h_popup);
    if (w<w_popup)
      w = w_popup;
    if (h<h_popup)
      h = h_popup;
    if (w > 0 && h > 0) {
      float w_ratio, h_ratio, by;
      w_ratio  = (float) w_popup / (float) w;
      h_ratio = (float) h_popup / (float) h;
      if (w_ratio < h_ratio) {
	 by = w_ratio;
       } else {
	 by = h_ratio;
       }
      if (map->scale_by < 0.001) {
	map -> x_scale  = by;
	map -> y_scale  = by;
	map -> scale_by = by;
	map -> x_centre = 0;
	map -> y_centre = 0;
      }
      x = map -> x_centre;
      y = map -> y_centre;

      Scale2ActualSize (map->x_scale, map->y_scale, w, h, & w, & h);
      Scale2ActualSize (map->x_scale, map->y_scale, x, y, & x, & y);
      w += MAP_MARGIN_SIZE;
      h += MAP_MARGIN_SIZE;

      if (w > MAXDIMENSION) {
	w = MAXDIMENSION;
      }
      if (h > MAXDIMENSION) {
	h = MAXDIMENSION;
      }

      MONETDEBUG(2, ( "X,Y position %d %d", x, y));
      MONETDEBUG(2, ( "Adjusting [%d,%d] to [%d,%d]", old_w, old_h, w, h));
      request.x = x;
      request.y = y;
      request.width  = (Dimension) w;
      request.height = (Dimension) h;
      request.request_mode = CWWidth | CWHeight | CWX | CWY;
 -- Try to resize, if can't get exact values, just try with
	 -- best possible. 
      (void) XmonetMakeGeometryRequest(widget,&request,&result);
     }
    win  = XtWindow (widget);
    while (XCheckWindowEvent (XmonetDisplay, win, Expose, &event)) {
  Remove any expose events already in the queue, no  
  need to repeat ourselves.  
    }
    if (redraw) {
      XClearArea (XmonetDisplay, XtWindow(widget), 0,0 ,0,0, True);
    } else {
      draw_page_outline (old_w,old_h);
      draw_page_outline (w,h);
    }
  }
   } */
#endif


/**********************************************************************/

/**********************************************************************/

static MonetBool shift_xy (map, shift_x, shift_y)
     MonetMap map;
     Position shift_x, shift_y;
{
  Position xl, yl, w, h;
  if (map->mapWidget == NULL) {
  } else if (Map_corners (map, &xl,&yl, &w,&h, False)) {
/*    Dimension wa, ha;
    Scale2ActualSize (map->x_scale, map->y_scale, w, h, & wa, & ha); */

    map->x_centre += (w * shift_x) / 2;
    map->y_centre += (h * shift_y) / 2;
    MONETDEBUG(2, ( "Shift %d %d, map x,y %d %d",
		shift_x, shift_y,
		map->x_centre, map->y_centre));

    Map_request_redraw (map);
    return True;
    }
  return False;
}

/******************************************************************/
/*  Configure and Redraw events event  */
/******************************************************************/

/******************************************************************/
/*  Part of the map has been exposed  */
/*  All map redraws should send an expose event and not try to */
/*  redraw the window syncronously  */
/******************************************************************/

MONETCALLBACK(do_Map_redraw)
     XRectangle rectangle;
     MonetMap map;
     XEvent * event;
     Widget widget;

     if (! (event = XmonetCurrentEvent)) {
       MonetPrintfError ("No current event, must call from MonetACT");
       RETURNSC;
     }

     if (! (widget =XmonetCurrentWidget)) {
       MonetPrintfError ("No current widget, must call from MonetACT");
       RETURNSC;
     }

     if (!(map = widget2map (XmonetCurrentWidget))) {
       RETURNSC;
     }

     if (! isDrawingAllowed (map)) {
       /*  The map should be redrawn in its entirety later  */
       RETURNSC;
     }
    
      if (event->type == Expose) {
       if (map->clip_region == NULL) {
	 map->clip_region = XCreateRegion ();
       }
       /*  Could replace this lot with the one statement :
	   XtAddExposureToRegion (event, map->clip_region); */
       /*       for (;;) {  */
       /*  Search through the event list for all Expose  */
       /*  events.  Best deal with 'everything' at once. */
       /*  Merge the events into a clip-region before    */
       /*  redrawing the map.                            */
       /*  !!! May not work. */
       rectangle.x = (Position) event->xexpose.x;
       rectangle.y = (Position) event->xexpose.y;
       rectangle.width  = (Dimension) event->xexpose.width;
       rectangle.height = (Dimension) event->xexpose.height;
       XUnionRectWithRegion (&rectangle, map->clip_region, map->clip_region);
	 
       XClearArea (XmonetDisplay, XtWindow(widget),
		   rectangle.x, rectangle.y,
		   rectangle.width, rectangle.height,
		   False);

       if (event->xexpose.count == 0) {
	 Xmonet_wait_cursor (NULL);

	 if (start_draw_X11 (widget,
			     map->x_centre, map->y_centre,
			     map->x_scale, map->y_scale,
			     map->clip_region)) {
	   Position xl, yl, xr, yr;
	   Map_draw (map);
	   if (Map_corners (map, &xl,&yl, &xr,&yr, False)) {
	     draw_page_outline (xr, yr);
	   }
	 }
	 XDestroyRegion (map->clip_region);
	 map->clip_region = NULL;
	 Xmonet_normal_cursor (NULL);
       }
     } else if (event->type == ConfigureNotify) {
       XCreateWindowEvent * conf = (XCreateWindowEvent *) event;
       map->scale_by = 0.0;
       (void) shift_xy (map, 0, 0);
     } else {
       /* -- This is not an expose event, so redraw the whole lot */
       /* -- This may be inefficient, but it should be more robust */
       Map_request_redraw (map);
     }
MONETEND

/*****************************************************************/
/*  Resize a drawing marked by the cursor  */
/*****************************************************************/

MONETCALLBACK(do_MapResizeDrawing)
     Position xloc, yloc, x,y;
     Dimension width, height;
     Drawing s;
     MonetMap cursor_map, map;
     XEvent * event;
     Widget widget;

     if (! (event = XmonetCurrentEvent)) {
       MonetPrintfError ("No current event, must call from MonetACT");
       RETURNSC;
     }

     if (! (widget =XmonetCurrentWidget)) {
       MonetPrintfError ("No current widget, must call from MonetACT");
       RETURNSC;
     }

     if (!(map = widget2map (XmonetCurrentWidget))) {
       MonetSysError ("map is null");
       RETURNSC;
     }

     x = event->xbutton.x;
     y = event->xbutton.y;
     
     cursor_map = map;
     if (map->drawnas == drawnas_map &&
	 (s = Map_get_marked
	  (&cursor_map, &x, &y, &xloc, &yloc, &width, &height))) {
       MONETDEBUG(2, ( "Get marked [%s]", Drawing_title (s)));
       if (Drawing_isResizable (s)) {
	 Drawing_undraw (s);
	 (void) xt_expose ();
	 if (Xio_rubberband_makebox (widget,  map->grid_x, map->grid_y,
				     &xloc, &yloc, &width, &height,
				     Drawing_isElipse (s),
				     event)) {
	   
	   Position xl, yl, xr, yr;

	   Drawing_redraw (s);
	   Map_actual2map (&map, &xloc, &yloc, &width, &height);
	   /*		
	     if (drawing_map != map) {
	     MonetPrintfError ("Cannot resize out of the map");
	     } else if (
	       xloc < xl || yloc < yl ||
	       xloc+width > xr || yloc+height > yr) {
	     MonetPrintfError ("Can't resize bigger than map");
	     } else
	     */
	   {
	     Drawing_backup (s);
	     Map_insert_drawing (map, s, NULSHAPE, NULSTR,
				 xloc, yloc, width, height);
	     Map_corners (map, &xl, &yl, &xr, &yr, False);
	   }
	 } else
	   Drawing_redraw (s);
       } else {
	 MonetPrintfError ("[%s] is not resizable", Drawing_title(s));
       }
     }
MONETEND

MONETCALLBACK(do_MapSelectDrawing)
     Position xloc, yloc;
     Dimension width, height;
     MonetMap  map;
     XEvent * event;
     Widget widget;

     if (! (event = XmonetCurrentEvent)) {
       MonetPrintfError ("No current event, must call from MonetACT");
       RETURNSC;
     }

     if (! (widget =XmonetCurrentWidget)) {
       MonetPrintfError ("No current widget, must call from MonetACT");
       RETURNSC;
     }

     if (!(map = widget2map (XmonetCurrentWidget))) {
       MonetSysError ("map is null");
       RETURNSC;
     }

     xloc = event->xbutton.x;
     yloc = event->xbutton.y;
     width = 0;
     height = 0;

     (void) xt_expose ();
     (void) Xio_rubberband_makebox (widget, 0, 0,
				    &xloc, &yloc, &width, &height,
				    False,
				    event);
     (void) Map_select_in_box (map, xloc, yloc, width, height);
MONETEND

/*****************************************************************/
/*  Drag one or more drawings across the map  */
/*****************************************************************/

static Drawing SingleSelected;
static MonetMap MoveMap;

#define dragable(s) (Drawing_mapOf(s)==MoveMap && Drawing_isMoveable(s))


/*  !!! fixforscale */

static void drag_selected (x0, y0, x,y)
     Position x,y, x0, y0;
{
  Drawing d;
  MonetMap map;

  map = Map_root(MapCurrent_get());
  (void) Map_get_marked (&map, &x0, &y0,  NULL, NULL, NULL, NULL);
  map = Map_root(MapCurrent_get());
  (void) Map_get_marked (&map, &x, &y,  NULL, NULL, NULL, NULL);
  FOR_SELECTIONS (map, d)
    if (dragable(d))
      Drawing_outline (d, map, X(d)+x-x0,Y(d)+y-y0);
}

static void draw_selected (x0, y0, x,y)
     Position x,y, x0, y0;
{
  Drawing d;
  MonetMap map;

  map = Map_root(MapCurrent_get());
  (void) Map_get_marked (&map, &x0, &y0,  NULL, NULL, NULL, NULL);
  map = Map_root(MapCurrent_get());
  (void) Map_get_marked (&map, &x, &y, NULL, NULL, NULL, NULL);

  /*  Backup positions before doing anything  */
  Map_backup_selected (map);

  /*  Undraw and mark the interfaces as non-movable and drawable
      when redrawing and positioning the attached systems. */
  FOR_SELECTIONS (MoveMap, d)
    if (dragable(d) && Drawing_isInterface (d)) {
      Drawing_undraw (d);
   /*      Draw_connecting_lines (d, False); Delete them  */
      MonetConnectingLine_enable (d, False);
    }

  /*  Draw the non-interfaces at the new positions  */
  FOR_SELECTIONS (MoveMap, d)
    if (dragable(d) && ! Drawing_isInterface (d)) {
      Drawing_undraw (d);
      Drawing_position (d, X(d)+x-x0, Y(d)+y-y0);
      Drawing_prepare (d);
      Drawing_redraw (d);
    }

  /*  Do interfaces last because the positions of the connecting lines
      depend on the NEW positions of the connected systems.  */
  FOR_SELECTIONS (MoveMap, d)
    if (dragable(d) && Drawing_isInterface (d)) {
      Drawing_position (d, X(d)+x-x0, Y(d)+y-y0);
      Draw_connecting_lines (d, False);
      MonetConnectingLine_enable (d, True);
      Drawing_prepare (d);
      Drawing_redraw (d);
    }
}

static void drag_single_selected (x0, y0, x,y)
     Position x,y, x0, y0;
{
  MonetMap map;

  map = MapCurrent_get();
  (void) Map_get_marked (&map, &x0, &y0,  NULL, NULL, NULL, NULL);
  map = MapCurrent_get();
  (void) Map_get_marked (&map, &x, &y,  NULL, NULL, NULL, NULL);

  Drawing_outline (SingleSelected, map,
		   X(SingleSelected)+x-x0, Y(SingleSelected)+y-y0);
}

/*  !!! fixforscale */

static void draw_single_selected (x0, y0, x,y)
     Position x,y, x0, y0;
{
  MonetMap map;

  map = MapCurrent_get();
  (void) Map_get_marked (&map, &x0, &y0,  NULL, NULL, NULL, NULL);
  map = MapCurrent_get();
  (void) Map_get_marked (&map, &x, &y,  NULL, NULL, NULL, NULL);

  Drawing_backup (SingleSelected);

  Map_insert_drawing (map, SingleSelected, NULSHAPE, NULSTR,
		      X(SingleSelected)+x-x0, Y(SingleSelected)+y-y0,
		      (Dimension) 0, (Dimension) 0);
}

MONETCALLBACK(do_MapMoveDrawing)
     Drawing s;
     Position xloc, yloc;
     MonetMap map;
     XEvent * event;
     Widget widget;

     if (! (event = XmonetCurrentEvent)) {
       MonetPrintfError ("No current event, must call from MonetACT");
       RETURNSC;
     }

     if (! (widget =XmonetCurrentWidget)) {
       MonetPrintfError ("No current widget, must call from MonetACT");
       RETURNSC;
     }

     if (!(map = widget2map (XmonetCurrentWidget))) {
       MonetSysError ("map is null");
       RETURNSC;
     }

     if (map->drawnas != drawnas_map) {
       RETURNSC;
     }

     xloc = event->xbutton.x;
     yloc = event->xbutton.y;

     MoveMap = map;

     (void) xt_expose ();
     (void) start_draw_X11 (Map_widget(map),
			    map->x_centre, map->y_centre,
			    map->x_scale, map->y_scale,
			    (Region) None);

     if ((s = Map_get_marked (&map, &xloc, &yloc, NULL, NULL, NULL, NULL)) &&
	 !(s->submap_visible && current_Selection (map))) {
       if (Drawing_isMoveable (s)) {
	 if (s->label) {
	   MonetPrintfInform ("Moving [%s][%s]",
			      Drawing_title (s), s->label);
	 } else {
	   MonetPrintfInform ("Moving [%s]", Drawing_title (s));
	 }
	 SingleSelected = s;
	 Xio_rubberband_drag (widget, map->grid_x, map->grid_y,
			      drag_single_selected,
			      draw_single_selected,
			      event);
       } else {
	 MonetPrintfError ("System [%s] is not moveable", Drawing_title (s));
       }
     } else if (numberOf_Selection (map) == 0) {
       MonetPrintfWarning ("Nothing selected");
     } else {
       /*  Drag all selected systems  */
       Xio_rubberband_drag (widget, map->grid_x, map->grid_y,
			    drag_selected, draw_selected, event);
     }
     Draw_end_draw ();
MONETEND

/*****************************************************************/
/*  Get name of drawing pointed to by the x cursor  */
/*****************************************************************/

MONETCALLBACK(do_x_get_marked_drawing)
     MonetMap map;
     if (!(map = MapCurrent_get())) {
       MonetPrintfError ("No current Map");
     } else if (XmonetCurrentEvent) {
       Drawing s;
       Position xloc, yloc;

       xloc = XmonetCurrentX;
       yloc = XmonetCurrentY;
       if (s = Map_get_marked (&map, &xloc, &yloc,
			       NULL, NULL, NULL, NULL)) {
	 Drawing_set_current (s);
	 result = Drawing_title (s);
       } else {
	 result = NULL;
       }
     } else {
       MonetPrintfError ("No current event, must call from MonetACT");
       result = NULL;
     }
MONETEND

/******************************************************************/
/*  The cursor has moved into the map area  */
/******************************************************************/

MONETCALLBACK(do_x_new_map)
     MonetMap map;
     if (XmonetCurrentWidget) {
       if (!(map = widget2map (XmonetCurrentWidget))) {
	 MonetSysError ("map is null");
       } else {
	 MapCurrent_set (map, False);
       }
     } else {
       MonetSysError ("No current widget");
     }
MONETEND

MONETCALLBACK(do_create_map)
     if (XmonetCurrentWidget) {
       MonetMap map;
       MONETDEBUG(5, ( "Creating map for widget"));
       if (map = Map_create (NULL)) {
	 map -> mapWidget = XmonetCurrentWidget;
	 /*       MapCurrent_set (map, False); */
       } else {
	 MonetSysError ("Map not created");
       }
     } else {
       MonetSysError ("No current widget");
     }
MONETEND

MONETCALLBACK(do_current_map_widget)
     Widget widget;
     MonetMap map;
     if ( ! (map = MapCurrent_get())) {
       result = NULL;
     } else if (widget = Map_widget (map)) {
        result = WcWidgetToFullName (widget);
     } else {
       result = NULL;
     }
MONETEND

/**********************************************************************/
/*  Quit xmonet  */
/**********************************************************************/

static MonetMap SaveMap;
static void quit_mapper ()
{
  Map_save (SaveMap);
}

MONETCALLBACK(do_xmonet_quit)
     if (AreAllMapsSaved ()) {
     } else {
       FORALLMAPS(SaveMap) {
	 if (Map_is_loaded(SaveMap) &&
	     (! SaveMap->saved) && !Map_classesOnly (SaveMap)) {
	   char buf [1024];
	   (void) sprintf (buf, "Save map [%s]?", Map_title (SaveMap));
	   Monet_confirm (quit_mapper, buf);
	 }
       }
     }
     MonetPrintfInform ("Goodbye.");
     Monet_finish ();
     (void) exit(0);
MONETEND

/**********************************************************************/
/*          Widget Callback routines                                  */
/**********************************************************************/

static MonetEntity get_class (title)
     char ** title;
{
  MonetEntity entity_class;
  Drawing class_drawing;
  char * sclass, * sinstance;

  MonetEntity_class_instance_parts (*title, &sclass, &sinstance);
  if (sinstance) {
    if (entity_class = String2MonetEntityClass (sclass)) {
      *title = sinstance;
    } else {
      MonetPrintfError ("[%s] is not the name of a class", sclass);
    }
  } else if (class_drawing = Drawing_current_class ()) {
    entity_class = ENTITY(class_drawing);
  } else {
    entity_class = NULL;
    MonetPrintfError ("You must select a Class from the class map");
  }
  return entity_class;
}

MONETCALLBACK(do_register_maps_list)
     MapsListWidget = (ListWidget) Xmonet_sub_widget
       (argv[1],listWidgetClass,"a list widget");
MONETEND

MONETCALLBACK(do_colours)
     MonetMap map;

     FORALLMAPS(map) {
       Widget w;
       if (w = (map -> mapWidget)) {
	 if (argv[1]) {
	   WcSetValueFromString (w, "foreground",  argv[1]);
	   WcSetValueFromString (w, "borderColor",  argv[1]);
	   WcSetValueFromString (XtParent (w), "foreground", argv[1]);
	 }
	 if (argv[2]) {
	   WcSetValueFromString (w, "background",  argv[2]);
	 }
	 if (argc == 4 && argv[3]) {
	   WcSetValueFromString (w, "backingPixmap",  argv[3]);
	 }
       }
     }
MONETEND

MONETCALLBACK(do_add_system)
     String title;
     MonetEntity entity_class;

     MonetPrintfInform ("New/Create Drawing");
     if ((title = MonetTidyString (argv[1])) == NULL || *title == NULL) {
       MonetPrintfError ("You must specify the name of the Entity");
     } else if (entity_class = get_class (&title)) {
       (void) create_new_drawing_at_cursor (entity_class, title);
     } else {
       MonetPrintfError ("You must select a Class");
     }
MONETEND

MONETCALLBACK(do_zoom_map)
     Position xl, yl;
     Dimension w, h;
     MonetMap map;

     if (! (map = MapCurrent_get())) {
     } else if (map->mapWidget == NULL) {
     } else if (Map_corners (map, &xl,&yl,
			     (Position *) &w,
			     (Position *) &h, True)) {
       Position shift_x, shift_y;
       if (argc == 2) {
	 float by;
	 by = (float) atof (argv[1]);
	 if (by == 0.0) {
	   MonetPrintfError ("Must give a value > than zero");
	   RETURNSC;
	 }
	 MONETDEBUG(10, ("do_zoom_map: argv[1] = [%s], by = %f", argv[1], by));
	 map -> x_scale  *= by;
	 map -> y_scale  *= by;
	 map -> scale_by *= by;
       } else {
	 map -> scale_by = 0.0;
       }
       if (XmonetCurrentWidget) {
	 shift_x = XmonetCurrentX - w/2;
	 shift_y = XmonetCurrentY - h/2;
       }
       (void) shift_xy (map, 0, 0);
       MONETDEBUG(10, ( "Scale now [%f,%f]", map -> x_scale,  map -> y_scale));
     }
MONETEND

MONETCALLBACK(do_shift)
     MonetMap map;

     if (! (map = MapCurrent_get())) {
     } else {
       Position shift_x, shift_y;
       shift_x = MonetString2int (argv[1]);
       shift_y = MonetString2int (argv[2]);
       (void) shift_xy (map, shift_x, shift_y);
     }
MONETEND

/******************************************************************/
/*  Initialise and finish off                                     */
/******************************************************************/

extern void MapCB_init ()
{
  /*  -- Init map library, this also initialises the entity database  */
  MonetMap_init (Map_changed_handler, begin_draw, end_draw, current_map,
		 free_map);

  (void) MonetMember_module_class ("xmonet");
  MonetMember_create
    ("quit", do_xmonet_quit,
     "", "Quit xmonet");
  MonetMember_create
    ("colours", do_colours,
     "foreground background ?backingpixmap",
     "Sets colours and backing pixmap of all map widgets");
  MonetMember_create
    ("new-map", do_x_new_map,
     "",
     "Used when changing maps to another widget");
  MonetMember_create
    ("map-create", do_create_map,
     "", "Create a map and its widget");
  MonetMember_create
    ("new-drawing", do_add_system,
     "name",
     "Creates a new drawing on the current map, under the cursor. \
     A class may be selected.");
  MonetMember_create
    ("zoom",	do_zoom_map,
     "?scaling",
     "Zoom in or out, none means returns to default size");
  MonetMember_create
    ("currentmap-widget",	do_current_map_widget,
     "",
     "Returns the name of the widget of the current map. false if none.");
  MonetMember_create
    ("marked",	do_x_get_marked_drawing,
     "",
     "Returns drawing pointed to by the X cursor and sets current-drawing. \
     False if none pointed to but does not reset current drawing");
  MonetMember_create
    ("move-drawing",	do_MapMoveDrawing,
     "",
     "Pick up drawing pointed to by the mouse and move elsewhere");
  MonetMember_create
    ("select-drawing",	do_MapSelectDrawing,
     "",
     "Select drawing pointed to or enclosed in rubber band");
  MonetMember_create
    ("resize-drawing",	do_MapResizeDrawing,
     "",
     "Resize pointed to drawing using a rubberband");
  MonetMember_create
    ("redraw-map-widget",	do_Map_redraw,
     "",
     "Redraw map given the current widget");
  MonetMember_create
    ("configure-map-widget",	do_Map_redraw,
     "",
     "Handle configure event");
  MonetMember_create
    ("shift",	do_shift,
     "x y",
     "Shift map left and right up and down");
  MonetMember_create
    ("register-maps-list",	do_register_maps_list,
     "listwidget",
     "This list widget is used for listing loaded maps");

  /*  -- Create widget tree below XmonetTopWidget shell using Xrm database */
  WcWidgetCreation ( XmonetTopWidget );

  /*  -- Realize the widget tree */
  XtRealizeWidget ( XmonetTopWidget );

  /* this ensures that all requests are sent */
  /* without this line, linking with cc means */
  /* that the requests are not all sent */
  XFlush(XtDisplay(XmonetTopWidget));

  /*  -- Set up the icon pixmap */
  {
    Pixmap icon;
    Arg tmp_arg[2];
    icon = XCreateBitmapFromData(XmonetDisplay, XtWindow(XmonetTopWidget),
				 monet_bits, monet_width,
				 monet_height);
    XtSetArg(tmp_arg[0], XtNiconPixmap, icon);
    XtSetValues(XmonetTopWidget, tmp_arg, ONE);
  }
/*  (void) XMonetSetWidgetPixmap
    ("XMonet*print_map_button", True,
     psprint_bits, psprint_width, psprint_height);
*/
/*  printf ("%d %u %x\n", sizeof (Dimension), MAXDIMENSION, MAXDIMENSION); */
}

extern void MapCB_finish ()
{
  MonetMap_finish ();
}

/*
#define NUM_ACTIONS 3
  static XtActionsRec actions[NUM_ACTIONS] =
    {
      {"MapSelectDrawingACT",	MapSelectDrawingACT},
      {"MapResizeDrawingACT",	MapResizeDrawingACT},
      {"MapRedrawACT",		Map_redrawACT},
    };

  (void) XtAppAddActions(XmonetAppCon, (XtActionList) actions,
			 (Cardinal) NUM_ACTIONS);
*/
/******************************************************************/
/******************************************************************/
/*  $Header: /nfs/bilbo/bilbo/user/a38/monet/bin/xmonet/RCS/map.c,v 1.2 1992/04/15 11:18:27 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/


