/***************************************************************/
/***************************************************************/
/*  $Header: access.c,v 1.2 91/10/01 16:07:02 a38 Exp $  */
/***************************************************************/
/*  Declaration and Access operations for entity attributes.   */
/***************************************************************/
/*  Since all attributes are dealt with in roughly the same way*/
/*  the basic access operation on them is defined by a macro.  */
/*  Now this is pretty difficult to write, let alone debug, so */
/*  a better way should be found.                              */
/***************************************************************/

#include "Db.h"
#include <monet/msh/Script.h>
#include "entityP.h"

/***************************************************************/
/*  Accessing entity lists */
/***************************************************************/
/*
#define NEQ_ENTITY_LIST(e,v) (e != v)
#define SET_ENTITY_LIST(e,v) {e=v;}
#define GET_ENTITY_LIST(o) (MONETGETPOINTER(o, id))
ACCESS(access_entity_list, GET_ENTITY_LIST,
       MonetEntityList2String, String2MonetEntityList,
       MonetEntityList2String, String2MonetEntityList,
       MonetEntityList, NEQ_ENTITY_LIST, SET_ENTITY_LIST, NULL);
*/
/***************************************************************/
/*  Accessing entities  */
/***************************************************************/

#define NEQ_ENTITY(e,v) (e != v)
#define SET_ENTITY(e,v) {e=v;}
#define GET_ENTITY(o) (MONETGETPOINTER(o, id))
ACCESS(access_entity, GET_ENTITY,
       MonetEntity_title, String2MonetEntity,
       MonetEntity_title, String2MonetEntity,
       MonetEntity, NEQ_ENTITY, SET_ENTITY, NULL);

/***************************************************************/
/*  Accessing strings */
/***************************************************************/

#define NEQ_STRING(c,v) (c != v)
#define SET_STRING(s,v) { MonetStr_derefer (s); s = v; }
#define GET_STRING(o) MONETGETMONETSTR(o, id)
ACCESS(access_string, GET_STRING,
       MonetStr2String, MonetStr_refer,
       MonetStr2String, MonetStr_refer,
       MonetStr, NEQ_STRING, SET_STRING, 0);

/***************************************************************/
/*  Accessing title  */
/***************************************************************/

#define NEQ_TITLE(c,v) (c != v)
#define SET_TITLE(s,v) { MonetStr_derefer (s); s = v; }
#define GET_TITLE(o) MONETGETMONETSTR(o, id)
ACCESS(access_title, GET_TITLE,
       MonetStr2String, MonetStr_refer,
       MonetStr2String, MonetStr_refer,
       MonetStr, NEQ_TITLE, SET_TITLE, 0);

/***************************************************************/
/*  Accessing SHAPEs  */
/***************************************************************/

static MonetShape access_String2MonetShape (string)
     String string;
{
  MonetShape shape;
  if (! (shape = String2MonetShape (string))) {
    MonetPrintfError ("Shape [%s] is unknown", string);
  }
  return shape;
}

#define NEQ_SHAPE(c,v) (c != v)
#define SET_SHAPE(c,v) { c = v; }
#define GET_SHAPE(o) (MONETGETPOINTER(o, id))
ACCESS(access_shape, GET_SHAPE,
       MonetShape2String, access_String2MonetShape,
       MonetShape2String, access_String2MonetShape,
       MonetShape, NEQ_SHAPE, SET_SHAPE, 0);

/***************************************************************/
/*  Accessing counters  */
/***************************************************************/

#define NEQ_COUNTER(c,v) (c != v)
#define SET_COUNTER(c,v) { c = v; }
#define GET_COUNTER(o) (MONETGETINTEGER(o, id))
ACCESS(access_counter, GET_COUNTER,
       unsigned2string, String2unsigned,
       unsigned2string, String2unsigned,
       unsigned, NEQ_COUNTER, SET_COUNTER, 0);

/***************************************************************/
/* For accessing the 'anymap' flag  */
/***************************************************************/

#define NEQ_BOOLEAN(c,v) (c != v)
#define SET_BOOLEAN(c,v) { c = v; }
#define GET_BOOLEAN(o)  ( MONETGETBOOLEAN(o, id))
ACCESS(access_boolean, GET_BOOLEAN,
       MonetBool2String, String2MonetBool,
       MonetBool2String, String2MonetBool,
       MonetBool, NEQ_BOOLEAN, SET_BOOLEAN, 0);

/***************************************************************/
/*  Init and finish  */
/***************************************************************/

extern void monet_init_access ()
{
  MonetType title, shape, boolean, counter, string;
  shape =  MonetType_create
    ("Shape", False, access_shape, NULL,
     "Shape used when drawn on map");
  string = MonetType_create
    ("String", False, access_string, NULL,
     "general text");
  counter = MonetType_create
    ("Counter", False, access_counter, NULL,
     "32 bit unsigned, suitable for SNMP");
  (void) MonetType_create
    ("MonetEntity", False, access_entity, NULL,
     "An entity");
  title =  MonetType_create
    ("Title", True, access_title, NULL,
     "The title of an entity (domain name)");
  boolean=  MonetType_create
    ("Boolean", False, access_boolean, NULL,
     "True/False");

  (void) MonetAttribute_create ("title",  MonetAttribute_TITLE_NUM,
				title, "Title of the entity");
  (void) MonetAttribute_create ("shape",  MonetAttribute_SHAPE_NUM,
				shape,
				"The shape used to represent the entity");
  (void) MonetAttribute_create ("pingid", MonetAttribute_PINGID_NUM,
				counter, "");
  (void) MonetAttribute_create ("area", MonetAttribute_AREA_NUM,
				string,
				"Some indicator of where the system belongs");
  monet_init_components ();
}

extern void monet_finish_access ()
{
  monet_finish_components ();
}

/***************************************************************/
/***************************************************************/
/*  $Header: access.c,v 1.2 91/10/01 16:07:02 a38 Exp $  */
/***************************************************************/
/***************************************************************/

