/****************************************************************************/
/****************************************************************************/
/*  $Header: component.c,v 1.2 91/10/01 16:07:08 a38 Exp $  */
/****************************************************************************/
/****************************************************************************/

#include "Db.h"

/*****************************************************************************/

/*****************************************************************************/

extern void load_component (super, sub)
	char * super, * sub;
{
	MonetEntity o_super, o_sub;

	if ((o_super = String2MonetEntity (super)) == NULL) {
		MonetPrintfError ("[%s] does not exist", super);
		return;
	}
	if ((o_sub   = String2MonetEntity (sub)) == NULL) {
   		MonetPrintfError ("[%s] does not exist", sub);
		return;
	}
	if (isSubcomponent (o_super, o_sub)) {
		MonetPrintfError ("Already subcomponent");
		return;
	}
	make_subcomponent (o_super, o_sub);
}


extern void make_subcomponent (super, sub)
	MonetEntity super, sub;
{
	if (super == sub)
		return;
	if ( ! MonetEntityList_isMember (super->subcomponents, sub))
		MonetEntityList_insert (super->subcomponents, sub);
	if ( ! MonetEntityList_isMember (sub->supercomponents, super))
		MonetEntityList_insert (sub->supercomponents, super);
	/* MASK(super) |= StatusComponentMask; !!! */
}

extern void unmake_subcomponent (super, sub)
	MonetEntity super, sub;
{
	MonetEntityList_extract (super->subcomponents, sub);
	MonetEntityList_extract (sub->supercomponents, super);
	/* !!! MASK(super) |= StatusComponentMask; */
}

/*  Removes all of an objects supercomponents and sucomponents */
extern void free_components (sys)
     MonetEntity sys;
{
  MonetEntity s;
  FOR_ENTITYLIST (s, sys->subcomponents) {
    MonetEntityList_extract (s->supercomponents, sys);
  } END_FORLIST;
  FOR_ENTITYLIST (s, sys->supercomponents) {
    MonetEntityList_extract (s->subcomponents, sys);
    MonetEntityList_reset (sys->subcomponents);
    MonetEntityList_reset (sys->supercomponents);
  } END_FORLIST;
}


extern MonetBool isSubcomponent (super, sub)
	MonetEntity super, sub;
{
	return MonetEntityList_isMember (super->subcomponents, sub);
}

extern MonetBool isSupercomponent (super, sub)
	MonetEntity super, sub;
{
	if (super == sub)
		return False;
	/* !! Should we allow cycles? !!! */
	return MonetEntityList_isMember (sub->supercomponents, super);
}

/*****************************************************************/
/*  Access component attribute */
/*****************************************************************/

static String access_component (entity, id, op, format, null, list)
     MonetEntity entity;
     unsigned id;
     AttributeOperation op;
     AttributeFormat format;
     MonetBool null;
     String list;
{
  MonetEntity s;
  static char buffer[1024] = { 0 }; /* !!! */
  String place = buffer;

  MonetBool first;

  if (op == attribute_init) {
    entity->supercomponents	= MonetEntityList_create ();
    entity->subcomponents	= MonetEntityList_create ();
    return NULL;
  } else if (op == attribute_free) {
    if (entity->supercomponents && entity->subcomponents) {
      free_components (entity);
      MonetEntityList_free (MonetEntity_supercomponents(entity));
      MonetEntityList_free (MonetEntity_subcomponents(entity));
      entity -> subcomponents = NULL;
      entity -> supercomponents = NULL;
    }
    return NULL;
  } else if (op == attribute_unset) {
    free_components (entity);
    return NULL;
  } else if (op == attribute_set) {
    MonetEntity subcomp;
    char sub[1024]; /* !!! */
    String next;
    next = list;
    while (next) {
      if (!MonetCopyNextWord (&next, sub, sizeof(sub)))
	break;
      if (subcomp = MonetEntity_find_or_create (sub)) 
	make_subcomponent (entity, subcomp);
      for (;*next != NULL && *next != CONTINUATION_CHAR; next++);
      for (;*next != NULL && *next == CONTINUATION_CHAR; next++);
    }
    return "";
  }
  if (MonetEntityList_isEmpty (entity->subcomponents))
    return NULL;
	first = True;
  FOR_ENTITYLIST (s, entity->subcomponents) {
    if (first)
      first = False;
    else
      place = MonetStringCopyToEnd (place, " $ ");
    place = MonetStringCopyToEnd (place, MonetEntity_title (s));
  } END_FORLIST;
  return buffer;
}

/*****************************************************************************/
/*   Initiate and free  */
/*****************************************************************************/

extern void monet_init_components ()
{
  MonetType component;
  component = MonetType_create
    ("Component", True, access_component, NULL,
     "List of component entities of an entity");
  (void) MonetAttribute_create ("components",   MonetAttribute_COMPONENT_NUM,
				component, "");
}

extern void monet_finish_components ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/*  $Header: component.c,v 1.2 91/10/01 16:07:08 a38 Exp $  */
/*****************************************************************************/
/*****************************************************************************/

