/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*  Managing enumerated types, converting to and from strings  */
/*****************************************************************************/

#include "entityP.h"
#include <monet/db/Enum.h>

/*****************************************************************************/
/*****************************************************************************/

#define ENUM_DEFAULT 0

struct monet_enum {
  String *names;
  unsigned num;
  MonetType type;
  MonetEnum next;
};

static MonetEnum List;

/*****************************************************************************/
/*  Converts from a string to an enumerated value  */
/*****************************************************************************/

static unsigned String2Enum (s)
	String s;
{
  MonetEnum en = (MonetEnum) Monet_type_being_accessed_data ();
  return MonetEnum_String2unsigned (en, s);
}

static String Enum2String (c)
	unsigned c;
{
  MonetEnum en = (MonetEnum) Monet_type_being_accessed_data ();
  return MonetEnum_unsigned2String (en, c);
}

/***************************************************************/
/*  Accessing enumerated types  */
/***************************************************************/

#define NEQ_COUNTER(c,v) (c != v)
#define SET_COUNTER(c,v) { c = v; }
#define GET_COUNTER(o) (MONETGETINTEGER(o, id))
ACCESS(access_enum, GET_COUNTER,
       Enum2String, String2Enum,
       Enum2String, String2Enum,
       unsigned, NEQ_COUNTER, SET_COUNTER, 0);

/*****************************************************************************/
/*  Create and free */
/*****************************************************************************/

extern MonetEnum MonetEnum_create (name, num, names)
     String name, *names;
     unsigned num;
{
  MonetEnum en;

  if (num == 0) {
    MonetSysError ("MonetEnum_create: must be at least one value");
    return NULL;
  } else if (en = String2MonetEnum (name)) {
    /* already exists */
    if (num == en->num) {
      while (num) {
	-- num;
	if (NEQ (en->names[num], names[num]))
	  return NULL;
      }
      return en;  /*  Nothing changed so just return the current one  */
    }
    return NULL;
  }
  en = MonetNew (struct monet_enum);
  en->type = MonetType_create
    (name, False, access_enum, (caddr_t) en, "An enumerated type");
  if (en->type == NULL) {
    MonetFree (en);
    return NULL;
  } else {
    en -> num = num;
    en -> names = (String *) MonetMalloc (sizeof (String) * num);
    while (num) {
      -- num;
      en->names[num] = MonetNewString (names[num]);
    }
    LIST_ADD (en, List, next);
  }
  return en;
}
  
extern void MonetEnum_free (en)
     MonetEnum en;
{
  while (en->num > 0) {
    -- en->num;
    MonetFree (en->names[en->num]);
  }
  MonetType_free (en->type);
  MonetFree (en->names);
  MonetFree (en);
  LIST_DEL (MonetEnum, en, List, next);
}

/*****************************************************************************/
/* Enum look up  */
/*****************************************************************************/

extern MonetType MonetEnum2MonetType (en)
     MonetEnum en;
{
  if (en) {
    return en->type;
  } else {
    MonetSysError ("NULL enum");
  }
  return NULL;
}
extern MonetEnum MonetType2MonetEnum (type)
     MonetType type;
{
  MonetEnum en;
  FORALL (en, List, next)
    if (en->type == type)
      return en;
  return NULL;
}

extern void MonetEnum_report ()
{
  MonetEnum en;
  MonetPrintfnl ("Enumerated Types");
  FORALL (en, List, next) {
    unsigned n;
    MonetPrintf ("enum %s   ", MonetEnum2String (en));
    for (n=0; n<en->num; ++n)
      MonetPrintf (" %s", en->names[n]);
    MonetPrintfnl ("");
  }
}

extern String MonetEnum2String (en)
     MonetEnum en;
{
  return MonetType_name (en->type);
}

extern  MonetEnum String2MonetEnum (string)
     String string;
{
  MonetEnum en;
  FORALL (en, List, next)
    if (EQS(MonetEnum2String (en), string))
      return en;
  return NULL;
}

extern String MonetEnum_unsigned2String (en, num)
     MonetEnum en;
     unsigned num;
{
  if (num > en -> num) {
    num = ENUM_DEFAULT;
  }
  return en->names [num];
}

extern unsigned MonetEnum_String2unsigned (en, string)
     MonetEnum en;
     String string;
{
  unsigned n;
  for (n = 0; n < en->num; ++n)
    if (EQS(string, en->names [n]))
      return n;
  MonetPrintfWarning ("No such value [%s] for enumerated type [%s]",
		      string, MonetEnum2String (en));
  return ENUM_DEFAULT;
}

/*****************************************************************************/
/*  Init  */
/*****************************************************************************/

extern void MonetEnum_init ()
{
  List = NULL;
}

extern void MonetEnum_finish ()
{
  while (List)
    MonetEnum_free (List);
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
