/***********************************************************************/
/***********************************************************************/
/*  $Header$                                                           */
/***********************************************************************/
/*  Give class specific information on entitys */
/***********************************************************************/

#include "entityP.h"

/***************************************************************/
/*  Reports some info on interface classes  */
/***************************************************************/

static void MonetEntity_report_interfaceClass (o)
     MonetEntity o;
{
  MonetEntity fromclass = get_fromClass (o);
  MonetEntity toclass   = get_toClass (o);

  if (fromclass == NULL && toclass == NULL) {
    MonetPrintfnl	("  Can connect to entitys of any class");
  } else {
    if (fromclass == NULL)
      MonetPrintfnl ("  One connection to entitys of any class");
    else
      MonetPrintfnl ("  One connection to entitys of class [%s]",
		      MonetEntity_title (fromclass));
    if (toclass == NULL)
      MonetPrintfnl ("  Other connection to entitys of any class");
    else
      MonetPrintfnl ("  Other connection to entitys of class [%s]",
		      MonetEntity_title (toclass));
  }
  if (isBendingInterface (o))
    MonetPrintfnl ("  Connections can be bend");
  else
    MonetPrintfnl ("  Connections cannot be be bent");
}

/***************************************************************/
/*  Reports some info on interfaces  */
/***************************************************************/

static void report_interface (o)
     MonetEntity o;
{
  MonetEntity fromclass = get_fromClass (o);
  MonetEntity toclass   = get_toClass (o);
  MonetEntity from      = from_interface (o);
  MonetEntity to        = to_interface (o);

  if (fromclass == NULL && toclass == NULL) {
    MonetPrintfnl	("  Can connect to entitys of any class");
  } else {
    if (fromclass == NULL)
      MonetPrintfnl	("  One connection to entitys of any class");
    else
      MonetPrintfnl	("  One connection to entitys of class [%s]",
			 MonetEntity_title (fromclass));
    if (toclass == NULL)
      MonetPrintfnl	("  Other connection to entitys of any class");
    else
      MonetPrintfnl	("  Other connection to entitys of class [%s]",
			 MonetEntity_title (toclass));
  }
  if (to != NULL)
    MonetPrintfnl	("  Connected to [%s]", MonetEntity_title(to));
  if (from != NULL)
    MonetPrintfnl	("  Connected to [%s]", MonetEntity_title(from));
  if (isBendingInterface (o))
    MonetPrintfnl ("  Connections can bend");
  else
    MonetPrintfnl ("  Connections cannot be bent");
}

/***********************************************************************/
/* Report underlying system types */
/***********************************************************************/

static MonetEntity underlyingClass (o)
	MonetEntity o;
{
  MonetEntity c;
  for (c = MonetEntity_classOf (o); o; c = MonetEntity_classOf (c))
    if (isUnderlying (c))
      return c;
  return NULL;
}

static char * report_underlyingStuff (entity)
	MonetEntity entity;
{
#define MAXUNDER 128
	MonetEntity entity_class[MAXUNDER];
	MonetEntity o, c;
	unsigned found = 0, i;

	MonetPrintf ("  Underlying protocols/systems: [");
	FOR_ENTITYLIST (o, MonetEntity_subcomponents (entity)) {
		c = underlyingClass (o);
		if (c == NULL)
			continue;
		for (i = 0; i<found; i++)
			if (c == entity_class[i])
				goto out;
		entity_class[found] = c;
		MonetPrintf (" %s", TITLE(c));
		found ++;
		if (found == MAXUNDER) {
			MonetSysError ("Max underlying [%d] exceeded", MAXUNDER);
			return NULL;
		}
		out:;
	} END_FORLIST;
	if (found == 0)
		MonetPrintfnl ("None found]");
	else
		MonetPrintfnl ("]");
	return NULL;
}

/***********************************************************************/
/*  Give a status listing on a class  */
/***********************************************************************/

static void report_class (o)
     MonetEntity o;
{


  if (!MonetEntity_isClass (o)) {
    MonetSysError ("[%s] is not a class", MonetEntity_title (o));
  }
  
  MonetPrintfnl ("Reporting on [%s] ...", MonetEntity_title(o));
  MonetPrintfnl ("Reporting on [%s]", MonetEntity_title(o));
  MonetPrintfnl ("  %d subclasses",
		 MonetEntityList_sizeOf (MonetEntity_subclasses(o)));
  if (isUnderlying (o))
    MonetPrintfnl ("  Instances can will be reported as underlying !!!");
  else
    MonetPrintfnl ("  Instances will not be reported as underlying !!!");

  MonetPrintfnl ("Pattern for titles is [%s]", get_titleFormat (o));

  MonetPrintfnl ("  %d instances",
		 MonetEntityList_sizeOf (MonetEntity_instances(o)));
  if (MonetEntity_isInterfaceClass (o)) {
    MonetPrintfnl ("  Is interface class");
    MonetEntity_report_interfaceClass (o);
   }
}

/***************************************************************/
/*  Status Report */
/***************************************************************/

extern void MonetEntity_status_report (o)
     MonetEntity o;
{
  unsigned n;

  if (MonetEntity_isClass (o)) {
    report_class (o);
    return;
  }
  MonetPrintfnl ("Reporting on [%s] ...", MonetEntity_title(o));
  MonetPrintfnl ("%s", MonetEntity2Report (o, NULL, True, False, False));

  n = MonetEntityList_sizeOf (MonetEntity_subcomponents(o));
  if (n == 0)
    MonetPrintfnl ("  No subcomponents");
  else
    MonetPrintfnl ("  %d subcomponents", n);
  
  n = MonetEntityList_sizeOf (MonetEntity_supercomponents(o));
  if (n == 0)
    MonetPrintfnl ("  Not the subcomponent of any other entity");
  else if (n == 1)
    MonetPrintfnl ("  Subcomponent of only one other entity");
  else
     MonetPrintfnl ("  Subcomponent of %d entitys", n);
  
  n = MonetEntityList_sizeOf (MonetEntity_interfaces(o));
   if (n == 0)
      MonetPrintfnl ("  No interfaces");
    else if (n == 1)
      MonetPrintfnl ("  One interface", n);
    else
      MonetPrintfnl ("  %d interfaces", n);

  if (PINGID(o) == 0)
    MonetPrintfnl ("  Not being pinged");
  else
    MonetPrintfnl ("  Ping request [%d]", PINGID(o));

  if (MonetEntity_isInterface (o)) {
    MonetPrintfnl	("  Is an interface");
    report_interface (o);
  }
  (void) report_underlyingStuff (o);
}

/***********************************************************************/
/***********************************************************************/
/*  $Header$                                                           */
/***********************************************************************/
/***********************************************************************/
