/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/* $Log$ */
/***************************************************************/

#include "entityP.h"
#include <sys/param.h>  /* For MAXPATHLEN */
#include <monet/msh/Script.h>

static String	 MonetEntity_database_file;

/***************************************************************/
/* Process one line of input */
/***************************************************************/

extern MonetBool MonetEntity_load_line (entity_ref, line)
     MonetEntity * entity_ref;
     char * line;
{
  MonetAttribute id;
  String end_word, last_char;

  line = MonetNextWord (line);

  if (line == NULL || *line == NULL || *line == '\n' || *line == COMMENT_CHAR)
    return True;

  end_word = MonetEndOfWord (line);
  last_char = end_word - 1;

  if (*last_char == '=') {
    MonetEntity entity; 
    if (entity = *entity_ref) {
      *last_char = '\0';
      if ((id = String2MonetAttribute (line)) && entity) {
	char * data;
	data = MonetStripEndWhiteSpace (end_word);
	MonetEntity_set_attribute(entity, id, data);
      } else {
	MonetPrintfError ("No such attribute [%s]", line);
	return False;
      }
    } else {
      return False;
    }
  } else if (isdigit (*line)) {
    unsigned i = String2unsigned (line);
    if ( ! (*entity_ref = PingId2MonetEntity (i)))
      return False;
  } else {
    *end_word = '\0';
    if ( ! (*entity_ref = MonetEntity_find_or_create (line)))
      return False;
  }
  return True;
}

/***************************************************************/
/*  Load the entity database                                   */
/***************************************************************/

extern MonetBool MonetEntity_load_db (filename)
     String filename;
{
  FILE *f;
  char buffer_space[1024], *buffer = buffer_space;
  MonetEntity entity = NULL;

  if (filename == NULL)
    filename = MonetEntity_database_file;
  if ((f = MonetFopenFile (filename, "r")) == NULL) {
    return False;
  }
  MonetPrintfInform ("Loading entity database from file [%s]", filename);
  while (fgets(buffer, sizeof(buffer_space), f)) {
    if (MonetException_code ()) {
      /* SIG QUIT  possibly */
      break;
    } else if ( ! MonetEntity_load_line (&entity, buffer)) {
    }
    buffer = buffer_space;
  }
  fclose (f);
  return True;
}

/***************************************************************/
/*  Save a list of entitys                                     */
/*  Need to save connections after systems, not really tru now  */
/*  Does not save classes, this is hack for just now !!! */
/*  Only writes things if they have attributes set.  */
/***************************************************************/

extern MonetBool MonetEntity_save (f)
     FILE * f;
{
  MonetEntity s;

  FORALL (s, Entities, next)
    if (!MonetEntity_isClass (s)) {
      String report;
      if (report = MonetEntity2Report (s, NULL, True, False, True)) {
	(void) fputs (report, f);
	(void) fputc ('\n', f);
      }
    }
  return True;
}

/***************************************************************/
/*  Save the whole database  */
/*  Must create tempfiles in current directory  */
/*  because you can't link across devices.  */
/*  Save to temp file first then link to saved file to avoid */
/*  problems with crashes.  */
/***************************************************************/

extern MonetBool MonetEntity_save_db (filename)
     String filename;
{
  FILE *f;
  int fd;
  String tempname1 = "./tmpedbXXXXXX";
  char  tempname2 [MAXPATHLEN];

  if ( filename == NULL) 
    filename = MonetEntity_database_file;

  filename = MonetFilename (filename);
  if ( ! (fd = mkstemp (tempname1))) {
    MonetPrintfError ("Sorry, can't open temp file [%s]", tempname1);
    return False;
  }
  f = fdopen (fd, "w");
  
  (void) sprintf (tempname2, "%s_XXXXXX", filename);
  (void) mktemp (tempname2);
  (void) fprintf (f, "%c Saved by [%s] on %s\n",
		  COMMENT_CHAR, Monet_program_name (), MonetPrettyTimeStamp ());
  MonetEntity_save (f);

  fclose (f);

  if (link (filename, tempname2) != 0) {
    MonetPrintfError ("Could not link [%s] to [%s], errno %d",
		filename, tempname2, errno);
  } else if (unlink (filename) != 0) {
    MonetPrintfError ("Could not unlink [%s], errno %d", filename, errno);
    (void) unlink (tempname2);
  } else if (link (tempname1, filename) != 0) {
    MonetPrintfError ("Could not link [%s] to [%s], errno %d",
		tempname1, filename, errno);
    if (link (tempname2, filename) != 0) {
      MonetPrintfError ("Lost original, could not link [%s] to [%s], errno %d",
		  tempname2, filename, errno);
    } else {
      MonetPrintfInform ("Restored original");
    }
  } else if (unlink (tempname1) != 0) {
    MonetPrintfWarning ("Could not unlink [%s], errno %d", tempname1, errno);
  } else {
    MonetPrintfInform ("Saved entity database to file [%s]", filename);
    MonetPrintfInform ("Backed up in [%s]", tempname2);
    return True;
  }
  return False;
}

/***************************************************************/
/*  Init and finish  */
/***************************************************************/

extern void MonetDbParser_init ()
{
  MonetEntity_database_file = NULL;
  MonetMember_create_write ( "file", &MonetEntity_database_file,
			    "Cache/default MonetEntity Database file");
  MonetEntity_database_file   = MonetNewString ("monet.edb");
  MonetConfig_create ("-edb",  "edb->file", NULL,
		      "entity database file");
}

extern void MonetDbParser_finish ()
{
}

/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/***************************************************************/
