/***************************************************************/
/***************************************************************/
/*  $Header: report.c,v 1.1 91/10/07 15:18:36 a38 Exp $  */
/***************************************************************/
/***************************************************************/

#include "entityP.h"

typedef struct report_attribute * ReportAttribute;
typedef struct report_attribute {
  MonetAttribute attribute;
  unsigned field_width;
};

struct monet_report_format {
  String header;
  unsigned number;
  ReportAttribute attributes;
};

/***************************************************************/
/*  Takes a list of strings and converts these into a format   */
/*  list.   The list of strings is ::= {[width] <attribute>}+  */
/*  Field width is optional  */
/***************************************************************/

extern MonetReportFormat MonetReportFormat_create (argc, argv)
     unsigned argc;
     String * argv;
{
  MonetReportFormat format;
  ReportAttribute list;
  MonetAttribute at;
  unsigned place, space, width = 0;
  String header = NULL, header_end = NULL;
  unsigned header_len = 0, header_space = 0, i;

  for (space = argc, i=0; i<argc; ++i)
    if (isdigit(*argv[i]))
      -- space;

  if (space == 0) {
    MonetPrintfError ("No attributes defined for format");
    return NULL;
  }
  list = (ReportAttribute) MonetMalloc
    (sizeof (struct report_attribute) * space);
  for (place = 0, i=0; i<argc; ++i) {
    String header_start;
    if (isdigit(*argv[i])) {
      width = (unsigned) atoi (argv[i]);
    } else if (at = String2MonetAttribute (argv[i])) {
      unsigned at_len;
      at_len = MonetStrlen (argv[i]);
      list[place].attribute = at;
      if (width <= at_len)
	width = at_len + 1;
      list[place].field_width = width;
      if (header_space <= header_len + width) {
	header_space = header_len + width + 1;
	if (header)
	  header = MonetRealloc (header, header_space);
	else
 	  header = MonetMalloc (header_space);
	header_end = header + header_len;
      }
      header_start = header_end;
      header_end = MonetStringCopyFieldToEnd (header_start, argv[i], width);
      header_len += width;
      if (islower (*header_start)) {
	*header_start = toupper (*header_start);
      }
      ++place;
      width = 0;
    } else {
      MonetPrintfError ("No such attribute [%s]", argv[i]);
      MonetFree (header);
      MonetFree (list);
      return NULL;
    }
  }
  format = MonetNew (struct monet_report_format);
  format -> number = space;
  format -> attributes = list;
  format -> header = header;
  MONETDEBUG(30, ( "Report heading: [%s]", header));
  return format;
}

/***************************************************************/
/*  Free  */
/***************************************************************/

extern void MonetReportFormat_free (format)
     MonetReportFormat format;
{
  if (format) {
    MonetFree (format->attributes);
    MonetFree (format->header);
    MonetFree (format);
  }
}


/***************************************************************/
/*  Return report header  */
/***************************************************************/

extern String MonetReportFormat_header (format)
     MonetReportFormat format;
{
  if (format)
    return format->header;
  if (Monet_report_format)
    return Monet_report_format->header;
  return "";
}

/***************************************************************/
/*  Simple status report, for logs  */
/***************************************************************/

extern String MonetReportFormat_simple (format, entity)
     MonetReportFormat format;
     MonetEntity entity;
{
  String string;
  string = MonetReportFormat_pretty (format, entity);
  return string;
}

/***************************************************************/
/*  Human readable status report, for lists  */
/***************************************************************/

extern String MonetReportFormat_pretty (format, entity)
     MonetReportFormat format;
     MonetEntity entity;
{
  static char buffer [8192] = { 0 }; /* !!! */
  unsigned i;
  String f;

  if (! format) {
    if (!(format = Monet_report_format)) {
      return NULL;
    }
  }
  f = buffer;
  for (i=0; i<format->number; ++i) {
    MonetAttribute id;
    String value;
    id = format->attributes[i].attribute;
    if (value = MonetEntity_get_attribute_full (entity, id)) {
      String c;
      unsigned width;
      width = format->attributes[i].field_width;
      if (*value != ' ') {
	*f = ' ';
	++f;
	--width;
      }
      for (c=value; *c!=NULL; ++c, --width) {
	*f = *c; ++f;
      }
      for (;width>0; --width) {
	*f = ' ';
	++f;
      }
    }
  }
  *f = '\0';
  return buffer;
}

/***************************************************************/
/***************************************************************/
/*  $Header: report.c,v 1.1 91/10/07 15:18:36 a38 Exp $  */
/***************************************************************/
/***************************************************************/

