/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

#include "requestP.h"
#include <monet/msh/Script.h>
#include <monet/msh/NiceSocket.h>
#include <monet/msh/Channel.h>
#include "Redirect.h"
#include <monet/msh/Event.h>
#include <errno.h>
#include <sys/ioctl.h>		/* For FIONBIO  */
#include <sys/param.h>  /*  For MAXHOSTNAMELEN  */

/**********************************************************/
/*  Definitions  */
/**********************************************************/

static RequestConnection DummyRequestConnection;
static RequestConnection RequestConnectionList;
static void free_request_connection PR((RequestConnection connection));
static int Port;

/***************************************************************************/
/*  Send message  */
/***************************************************************************/
/*
static void send_message (conn, message)
     RequestConnection conn;
     char * message;
{
  if (conn && conn->channel && Channel_send_record (conn->channel, message));
*/
extern int  MonetRequest_port()
{
  if (Port > 0)
    return Port;
  return 0;
}

/**********************************************************/
/*  Convert a MonetRequest into a text string */
/**********************************************************/

static char *  request2text (request)
     MonetRequest request;
{
  static char buffer [1024];
  RequestConnection connection = request -> connection;
  MonetEntity entity = request -> entity;

  (void) sprintf (buffer, "%3d:%-4lu", connection->fid, request->id);
  (void) sprintf (buffer, "%s %s", buffer, MonetEntity_title (entity));

  return buffer;
}

/**********************************************************/
/*  Given a connection and a MonetRequest number, return the MonetRequest */
/**********************************************************/

extern MonetRequest id2MonetRequest (conn, id)
     RequestConnection conn;
     RequestId id;
{
  MonetRequest r;
  FORALL(r, conn->requests, connection_next)
    if (r->id == id)
      return r;
  return NULL;
}

/**********************************************************/
/*  Return changed results  */
/**********************************************************/

extern void MonetRequest_return_result (request, all)
     MonetRequest request;
     MonetBool all;
{
  RequestConnection c;

  c = request -> connection;
  if (c -> fid > 0) {
    char *header, * result, buffer [32];
    MonetBool compact;

    if (c->format == attribute_ascii) {
      header = NULL;
      compact = False;
    } else {
      header = buffer;
      (void) sprintf (header, "%ld", request->id);
      compact = True;
    }
    result = MonetEntity2Report (request->entity, header, all, compact, True);

    if (result && *result) {
      (void) Channel_send_record (c->channel, result);
      MONETDEBUG(1, ("Sending to Client [%d] [%s]", c->fid, result));
    }
  }
}

/****************************************************************************/
/*  Return all the results to the clients */
/****************************************************************************/

extern void MonetEntity_return_result (entity, all)
     MonetEntity entity;
     MonetBool all;
{
  MonetRequest r;

  FORALL (r, entity->requests, entity_next) {
    MonetRequest_return_result (r, all);
  }
}

/**********************************************************/
/*  Free connection  */
/**********************************************************/

static void free_request_connection (conn)
     RequestConnection conn;
{
  while (conn->requests != NULL)
    MonetRequest_free (conn->requests);

  LIST_DEL (RequestConnection, conn, RequestConnectionList, next);
  MonetFree (conn->source);
  MonetFree (conn);
}

/**********************************************************/
/*  Called when Disconnected  */
/**********************************************************/

static void disconnect (chan)
     Channel chan;
{
  RequestConnection conn = (RequestConnection) Channel_data (chan);
  MONETDEBUG(1, (
	      "The client seems to have disconnected, closing connection [%d]",
	      conn->fid));
  free_request_connection (conn);
}

/**********************************************************/
/*  Create/Free new connections  */
/**********************************************************/

static RequestConnection create_request_connection (fid, source)
     int fid;
     char * source;
{
  RequestConnection conn;


  conn = MonetNew (struct connection);
  if (conn == NULL) {
    MonetSysError ("Could not allocate space for new connection");
    return NULL;
  }
  conn -> source = MonetNewString (source);
  conn -> requests = NULL;
  conn -> fid = fid;
  conn -> format = attribute_compact_ascii;
  conn -> loading_entity = NULL;
  conn -> next = RequestConnectionList;
  RequestConnectionList = conn;
  conn -> channel = Channel_create (fid, False, parse_message,
				    disconnect, (caddr_t) conn);
  MONETDEBUG(1, ("The created new connection [%d]", conn->fid));
  return conn;
}

/**********************************************************/
/*   Find a connection, given the fid, Obsolete ? !!!   */
/**********************************************************/

static RequestConnection fid2connection (fid)
     int fid;
{
  RequestConnection c;

  FORALL(c, RequestConnectionList, next)
    if (c->fid == fid)
      return c;
  MonetSysError ("No connection with fid %d", fid);

  return NULL;
}

/**********************************************************/
/*  Find or create a entity to associate with a new MonetRequest  */
/**********************************************************/

extern MonetRequest MonetRequest_create (conn, name, id)
     String name;
     RequestConnection conn;
     RequestId id;
{
  MonetRequest request;
  MonetEntity entity;

  if (name == NULSTR) {
    MonetPrintfError ("Invalid MonetRequest id [%d]", id);
  } else if (id == 0) {
    MonetPrintfError ("Invalid MonetRequest id [%d]", id);
  } else if (id2MonetRequest (conn, id) != NULL) {
    MonetPrintfError ("Request %d:%d already exists", conn->fid, id);
  } else if ((request = MonetNew (struct monet_request)) == NULL) {
    MonetPrintfError ("Could not allocate space for new request");
  } else if ( !(entity = MonetEntity_find_or_create (name))) {
    /*  Find entity in list or create it  */
    /*  if it has a ping instruction then create a event for it */
    MonetPrintfError ("Could not create entity [%s]", name);
    MonetFree (request);
  } else if (! MonetEntity_isClass (entity)) {
    RedirectMonetEntity_add (entity);  /*  This handles the probing  */
    request -> entity = entity;
    request -> id = id;
    LIST_ADD (request, conn->requests, connection_next);
    request -> connection = conn;
    LIST_ADD (request, entity->requests, entity_next);
    /* -- Send current status to client straight away  */
    MonetRequest_return_result (request, True);
    return request;
  } else {
    MonetPrintfWarning ("Request for class [%s] not created", name);
  }
  return NULL;
}

/****************************************************************************/
/*  Create a request on the default connection  */
/****************************************************************************/

extern MonetRequest MonetRequest_default_create (title)
     char * title;
{
  static RequestId id = 1;

  if (MonetRequest_create (DummyRequestConnection, title, id)) {
    ++id;
  }
}

/****************************************************************************/
/*  Lists current requests  */
/****************************************************************************/

MONETCALLBACK (do_help_requests)
     RequestConnection c;
     MonetRequest r;
     unsigned total_requests = 0, connections = 0;
     char * pattern = argv[1];

     FORALL(c, RequestConnectionList, next) {
       unsigned requests = 0;
       ++connections;
       MonetPrintfnl ("Connection %d [%s]", c->fid, c->source);
       FORALL(r, c->requests, connection_next) {
	 char * title = MonetEntity_title (r->entity);
	 if (pattern && MonetMatchWildcard (title, pattern)) {
	   MonetPrintfnl (" %d [%s]", r->id, title);
	 }
	 ++requests;
       }
       total_requests += requests;
       MonetPrintPrettyCount ("", "request", requests);
     }
     MonetPrintPrettyCount ("Total of", "request", total_requests);
     MonetPrintPrettyCount ("Total of", "connection", connections);
     if (Port > 0) {
       MonetPrintfnl ("Listening for requests on port %d", Port);
     } else {
       MonetPrintfnl ("Not listening for requests");
     }
MONETEND

/****************************************************************************/
/*  Lists current requests for an entity  */
/****************************************************************************/

MONETCALLBACK(do_help_entity_requests)
     if (entity) {
       MonetRequest r;
       for (r = entity->requests; r; r= r->entity_next) {
	 MonetPrintfnl (" %d [%s] [%s]",
			r->id, MonetEntity_title (r->entity),
			r->connection->source);
       }
     } else {
       MonetPrintfError ("No entity given");
     }
MONETEND

/**********************************************************/
/*  Free a MonetRequest  */
/**********************************************************/

extern void MonetRequest_free (request)
	MonetRequest request;
{
  MonetEntity entity = request->entity;
  RequestConnection connection = request->connection;

  /* remove MonetRequest from connections list */
  LIST_DEL(MonetRequest, request, connection->requests, connection_next);
  /* remove MonetRequest from entity list */
  LIST_DEL(MonetRequest, request, entity->requests, entity_next);
  MonetFree (request);

  /* If entity has no more requests delete it */
  if (entity->requests == NULL && entity->drawing == NULL) {
    RedirectMonetEntity_delete (entity);
  }
}

/**********************************************************/
/*  Free all the MonetRequest of an entity  */
/**********************************************************/

extern void MonetRequest_free_entity (entity)
     MonetEntity entity;
{
  MonetRequest request;
  FORALL(request, entity->requests, entity_next)
    MonetRequest_free (request);
}

/**********************************************************/
/*  Report state of all requests  */
/**********************************************************/

extern void MonetRequest_report_all ()
{
  RequestConnection c;
  MonetRequest r;


  FORALL (c, RequestConnectionList, next) {
    FORALL (r, c->requests, connection_next) {
      MonetPrintfnl ("%s", request2text (r));
    }
  }
}

/**********************************************************/
/*  Remove a default request  */
/**********************************************************/

MONETCALLBACK(do_delete_request)
     RequestId id;
     MonetRequest request;

     id = MonetString2long (argv[1]);
     if (request = id2MonetRequest (DummyRequestConnection, id)) {
       MonetRequest_free (request);
     } else {
       MonetPrintfError ("No such request to delete [%d]", id);
     }
MONETEND

/**********************************************************/
/*  Create a request on the default connection */
/**********************************************************/

MONETCALLBACK(do_request)
     (void) MonetRequest_default_create (argv[1]);
MONETEND

/***************************************************************************/
/*  Handles a new connection MonetRequest from a client  */
/***************************************************************************/

static void HandleAccept (fid, address, data)
     int fid;
     char * address;
     caddr_t data;
{
  (void) create_request_connection (fid, address);
}

/**********************************************************/
/*  Listen for requests on such and such a port  */
/**********************************************************/

MONETCALLBACK(do_listen)
     int port, fid;

     port = (int) MonetString2long (argv[1]);
     if (Port > 0) {
       Channel_free_server (Port);
       Port = 0;
     }
     if (port <= 0) {
       MonetPrintfInform ("Stopped listening for requests");
     } else {
       int fid;

       fid = Channel_create_server(port,
				   HandleAccept,
				   NULL,
				   NULL);
       Port = port;
       if (fid >= 0)
	 result = Monetint2String (fid);
       else
	 result = NULSTR;
     }
MONETEND

/*****************************************************************/
/*  Treat the list of requests as an attribute.  This way the    */
/*  basic entity stuff does not need know anything about them.   */
/*****************************************************************/

static String access_request_list (entity, id, op, format, null, node_name)
     MonetEntity entity;
     unsigned id;
     MonetBool null;
     AttributeOperation op;
     AttributeFormat format;
     String node_name;
{
  if (op == attribute_init) {
    entity->requests	= NULL;
  } else if (op == attribute_free || op == attribute_unset) {
    MonetRequest_free_entity (entity);
    entity->requests	= NULL;
  } else if (op == attribute_set) {
  }
  return NULL;
}

/**********************************************************/
/*  Initiate and finish  */
/**********************************************************/

extern void MonetRequest_init ()
{
  MonetType request;

  DummyRequestConnection = NULL;
  RequestConnectionList = NULL;
  Port = 0;

  /*  -- Create a special connection  */
  DummyRequestConnection = create_request_connection (-1, "default");

  (void) MonetMember_module_class ("request");

  MonetMember_create ("help",  do_help_requests,
		      "?pattern",
		      "Lists current requests");
  MonetMember_create ("create",  do_request,
		      "entity",
		      "Creates a probe request. \
This is put on the default connection -1");
  MonetMember_create ("delete",  do_delete_request,
		      "id",
		      "Deletes a probe request, id is a unique number. \
		      This is removed from the default connection -1");
  MonetMember_create ("listen",  do_listen,
		      "port-number",
		      "Listens for requests on specified port \
		      if port number <= 0 then stops listening");
  request = MonetType_create
    ("RequestList", True, access_request_list, NULL,
     "List of requests for entity");
  (void) MonetAttribute_create ("requests",   MonetAttribute_REQUEST_NUM,
				request, "");

  (void) MonetMember_module_class ("Top");
  MonetMember_create ( "help-requests",		do_help_entity_requests,
		      "",
		      "Lists all of an entities requests.");
}

extern void MonetRequest_finish ()
{
  RequestConnection c;
  FORALL (c, RequestConnectionList, next) {
    Channel_shutdown (c->channel);
    free_request_connection (c);
  }
  if (Port > 0)
    Channel_free_server (Port);
  RequestConnectionList = NULL;
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/

