#ifndef MONET_DB_REQUEST_PRIVATE_H
#define MONET_DB_REQUEST_PRIVATE_H

/**********************************************************/
/*  Private Datastructures */
/**********************************************************/
/*                                                        */
/*    connections 1:N requests N:1 MonetEntitys N:1 redirects  */
/*                                                        */
/**********************************************************/
/*                                                        */
/*  connection0  ->  request0 -> request1                 */
/*       |            ^           ^                       */
/*       |            |           |                       */
/*       |            |           +-------------->  MonetEntity0  */
/*       |            |                              |    */
/*       |            |                              v    */
/*       |            +-------------------------->  MonetEntity1  */
/*       |                       +---------------->  |    */
/*       |                       |                   |    */
/*       v                       v                   |    */
/*  connection1  ->  request2 -> request3            |    */
/*                    ^          ^                   |    */
/*                    |          |                   v    */
/*                    |          +--------------->  MonetEntity3  */
/*                    |                              |    */
/*                    |                              v    */
/*                    +-------------------------->  MonetEntity4  */
/*                                                        */
/**********************************************************/

#include "entityP.h"
#include <monet/msh/Channel.h>

/***************************************************************************/
/*  RequestConnections  */
/***************************************************************************/

typedef struct connection * RequestConnection;

/* typedef struct monet_request * MonetRequest; */
typedef unsigned long int RequestId;

#define BUFFERLENGTH  4096
#define MAX_CLIENTS 9

struct connection {
  int fid;   /* Indicates on which connection the request came,
		-1 is through input window */
  char * source; /* Name of source, other end of connection  */
  MonetRequest requests;
  Channel channel;
  AttributeFormat format;
  MonetEntity  loading_entity;
  RequestConnection next;
};

struct monet_request {
  RequestConnection connection;
  RequestId id;
  MonetRequest connection_next;
  MonetRequest entity_next;
  MonetEntity entity;
};

/***************************************************************************/
/*  Function Prototypes  */
/***************************************************************************/

extern void MonetRequest_init
  PR((void));
extern void MonetRequest_finish
  PR((void));
extern void MonetRequest_return_result
  PR((MonetRequest, MonetBool all));
extern void MonetEntity_return_result
  PR((MonetEntity, MonetBool all));
extern void MonetRequest_report_all
  PR((void));
extern void MonetRequest_free_entity
  PR((MonetEntity));
extern void MonetRequest_free
  PR((MonetRequest));
extern MonetRequest MonetRequest_create
  PR((RequestConnection, String title,RequestId));
extern MonetRequest MonetRequest_default_create
  PR((String title));
extern MonetBool parse_message
  PR((Channel, String));
extern MonetRequest id2MonetRequest
  PR((RequestConnection, RequestId));
extern int MonetRequest_port
  PR((void));

#endif 

