/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/*  Parse incoming instructions */
/**********************************************************/

#include "requestP.h"

static MonetBool 	CloseRequestConnection;

/**********************************************************/
/*  Parse/create new MonetRequest */
/**********************************************************/

static MonetBool parse_MonetRequest (conn, text)
     RequestConnection conn;
     String text;
{
  RequestId id = 0;
  char title [512]; /* !!! */
  int cc;
	
  cc = sscanf (text, "request%lu %s", &id, title);
  if (cc <= 0) {
    return False;
  } else if (cc != 2) {
    MonetPrintfError ("Bad Request [%s]", text);
  } else {
    MonetRequest_create (conn, title, id);
  }
  return True;
}

/**********************************************************/
/*  Parse/add a redirect message, for certain requests    */
/*  always use another monitor.                           */
/**********************************************************/

static MonetBool parse_redirect (conn, text)
     String text;
     RequestConnection conn;
{
  int i, cc;
  char host[512], regexp[512];
  unsigned int port;
  MonetRedirect rule;

  cc = sscanf (text, "redirect %[^ \t\n+]+%u %[^\n]", host, &port, regexp);
  for (i=MonetStrlen(regexp)-1; i>=0 && SPACEORTAB(regexp[i]) ; i--);
  regexp[i+1] = NULL;

  if (cc < 1) {
    return False;
  } else if (cc != 3) {
    MonetPrintfError ("Invalid redirect instruction [%s]", text);
  } else if (rule = create_redirect (host, port, regexp)) {
    MONETDEBUG(1, ("Will redirect [%s] to [%s+%u]", regexp, host, port));
    Redirect_say_hello (rule);
  } else {
    MonetPrintfError ("Cannot redirect [%s] to [%s+%u]", regexp, host, port);
  }
  return True;
}

/**********************************************************/
/*  Parse/delete a MonetRequest  */
/**********************************************************/

static MonetBool parse_delete (conn, text)
     String text;
     RequestConnection conn;
{
  int cc;
  RequestId id;
  MonetRequest request;

  cc = sscanf (text, "delete%lu", &id);
  if (cc != 1)
    return False;

  request = id2MonetRequest (conn, id);
  if (request == NULL)
    MonetPrintfError ("Request %d:%d does not exists",
		      conn->fid, id);
  else
    MonetRequest_free (request);

  return True;
}
/**********************************************************/
/*  Parse/select full information on a MonetRequest  */
/**********************************************************/

static MonetBool parse_select (conn, text)
	String text;
	RequestConnection conn;
{
  int cc;
  RequestId id;
  MonetRequest request;

  cc = sscanf (text, "select%lu", &id);
  if (cc != 1)
    return False;

  request = id2MonetRequest (conn, id);
  if (request == NULL)
    MonetPrintfError ("Request %d:%d does not exists", conn->fid, id);
  else
    MonetRequest_return_result (request, True);
  return True;
}

/**********************************************************/
/*  Parse/clear all requests on a connection  */
/**********************************************************/

static MonetBool parse_clear (conn, text)
     RequestConnection conn;
     String text;
{
  if (strncmp ("clear", text, 5) != 0)
    return False;

  while (conn->requests != NULL)
    MonetRequest_free (conn->requests);

  return True;
}

/**********************************************************/
/*  Parse/clear all redirects      */
/**********************************************************/

static MonetBool parse_clear_redirects (conn, text)
	RequestConnection conn;
	String text;
{
  if (MonetStringPrefixEqual ("clear_redirects", text)) {
    clear_redirects ();
    return True;
  }
  return False;
}

/**********************************************************/
/*  Parse format  */
/**********************************************************/

static MonetBool parse_ascii (conn, text)
	RequestConnection conn;
	String text;
{
	if (strncmp ("ascii", text, 5) != 0)
		return False;

	conn->format = attribute_ascii;
	return True;
}

static MonetBool parse_compact (conn, text)
	RequestConnection conn;
	String text;
{
	if (strncmp ("compact", text, 6) != 0)
		return False;

	conn->format = attribute_compact_ascii;
	return True;
}

static MonetBool scan (message, id)
	String *message;
	unsigned *id;
{
	String s = *message;

	while (!isdigit(*s)) {
		if (*s == NULL)
			return False;
		s++;
	}
	for (*id = 0; isdigit(*s); s++)
		*id = (*id) * 10 + (unsigned) (*s - '0');
	*message = s;
	return True;
}

static MonetBool parse_set_compact (conn, text)
     RequestConnection conn;
     String text;
{
  if (strncmp ("set", text, 6) != 0)
    return False;
#ifdef ABCD
  text += 4;
  while (scan (&text, &id)) {
    if (*text == MonitorEndCode || *text == '\n')
      continue;
    do {
      FORALL (r, conn->requests, connection_next) {
	MonetEntity entity = r->entity;
	if (PINGID(entity) == id) {
	  if (! MonetEntity_decodeCompactAscii (entity, &text))
	    MonetPrintfError ("Couldn't parse status [%s]", text);
	  
	}
      }
    } while (*text == MonitorSepCode);
    for (; *text != MonitorEndCode && *text != NULL; text++);
  }
#endif
  return True;
}

/**********************************************************/
/*  Parse/say hello  */
/**********************************************************/

static MonetBool parse_hello (conn, text)
     RequestConnection conn;
     String text;
{
  char host [512];
  int port, pid, cc;

  cc = sscanf (text, "hello %s %u %u", host, &port, &pid);
  if (cc < 1)
    return False;

  if (cc == 1) {
  } else if (cc != 3) {
    return False;
  } else if (MonetRedirect_check_cycle (host, port, pid)) {
    MonetPrintfError ("Attempt to redirect to local agent [%s+%d], pid [%u]",
		      host, port, pid);
    CloseRequestConnection = True;
    return True;
  }
  MONETDEBUG(1, ("The Client Says Hello")); 
  
  return True;
}

/**********************************************************/
/*  Parse/shutdown */
/**********************************************************/

static MonetBool parse_shutdown (conn, text)
	RequestConnection conn;
	String text;
{
	if (strncmp ("shutdown", text, 8) != 0)
		return False;

	/*  !!! Shutdown */
	return True;
}

/**********************************************************/
/*  Parse/close connection */
/**********************************************************/

static MonetBool parse_close (conn, text)
	RequestConnection conn;
	String text;
{
	if (strncmp ("close", text, 5) != 0)
		return False;
	MONETDEBUG(1, ("Requesting close connection [%d]", conn->fid));
	CloseRequestConnection = True;
	return True;
}

/**********************************************************/
/*  Parse incoming messages  */
/**********************************************************/

extern MonetBool parse_message (chan, request)
     Channel chan;
     String request;
{
  RequestConnection conn = (RequestConnection) Channel_data (chan);
  static MonetBool loading = False;

  CloseRequestConnection = False;
  MONETDEBUG(1, ("Message on connection [%d], [%s]", conn->fid, request));
  if (conn == (RequestConnection) 0) {
    MonetSysError ("RequestConnection is NULL");
    return False;
  } else if (parse_MonetRequest  (conn, request)) {
  } else if (parse_select   (conn, request)) {
  } else if (parse_delete   (conn, request)) {
  } else if (parse_hello    (conn, request)) {
  } else if (parse_redirect (conn, request)) {
  } else if (parse_clear_redirects (conn, request)) {
  } else if (parse_clear    (conn, request)) {
  } else if (parse_shutdown (conn, request)) {
  } else if (parse_ascii    (conn, request)) {
  } else if (parse_compact (conn, request)) {
  } else if (parse_close (conn, request)) {
  } else if (parse_set_compact (conn, request)) {
  } else if (MonetEntity_load_line (&(conn->loading_entity), request)) {
    loading = True;
  } else {
    MonetPrintfError ("Could not parse message [%s]", request);
  }
  if (CloseRequestConnection)
    return False;
  if (! loading)
    conn->loading_entity = NULL;
  return True;
}

/**********************************************************/
/**********************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/request.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/**********************************************************/
/**********************************************************/
