/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "entityP.h"
#include <monet/msh/Script.h>
#include <monet/db/Enum.h>

MonetReportFormat Monet_report_format = NULL;

/***************************************************************************/
/*  Do for everything in the database  */
/***************************************************************************/

MONETCALLBACK(do_for_all_edb)
     MonetEntity e;
     String condition = argv[1], action = argv[2];
     FORALL(e, Entities, next) {
       char *names[3], *values[3];
       names[0]  = "entity";
       names[1]  = NULL;
       values[0] = MonetEntity_title (e);
       values[1] = NULL;
       if (MonetshExecute_defined (condition, NULL, names, values, False)) {
	 values[0] = MonetEntity_title (e);
	 if (action) {
	   result =
	     MonetshExecute_defined (action, NULL, names, values, False);
	 } else {
	   MonetPrintfnl ("%s", values[0]);
	 }
       }
     }
MONETEND

/***************************************************************************/
/*  Create an enumerated type  */
/***************************************************************************/

MONETCALLBACK(do_enum)
     if (MonetEnum_create (argv[1], argc-2, argv+2)) {
     } else {
       MonetPrintfError ("Failed to create enum [%s]", argv[1]);
     }
MONETEND

MONETCALLBACK(do_help_enum)
     MonetEnum_report ();
MONETEND

/*****************************************************************************/
/*  Load and save the database  */
/*****************************************************************************/

MONETCALLBACK(do_load_db)
     String name = argv[1];
     if (name && * name) {
       MonetEntity_load_db (name);
     } else
       MonetEntity_load_db (NULL);
MONETEND

MONETCALLBACK(do_save_db)
     String name = argv[1];
     if (name && * name) {
       MonetEntity_save_db (name);
     } else
       MonetEntity_save_db (NULL);
MONETEND

MONETCALLBACK(do_save_db_remote)
     MonetPrintfError ("Not implemented yet");
MONETEND

/*****************************************************************************/
/*  Set default reports format  */
/*****************************************************************************/

MONETCALLBACK(do_report_format)
     MonetReportFormat format;
     if (format = MonetReportFormat_create (argc-1, argv+1)) {
       if (Monet_report_format) {
	 MonetReportFormat_free (Monet_report_format);
       }
       Monet_report_format = format;
     } else {
       MonetPrintfError ("Report format not set");
     }
MONETEND

MONETCALLBACK(do_create)
     MonetEntity new_entity;
     if (new_entity = MonetEntity_find_or_create (argv[1])) {
       result = MonetEntity_title (new_entity);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_has_requests)
     if (entity) {
       if (entity->requests == NULL && entity->drawing == NULL)
	 result = NULL;
     } else {
       MonetPrintfError ("No entity given");
     }
MONETEND

MONETCALLBACK(do_is_redirected)
     if (! isRedirected (entity)) {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_is_attribute)
     if ( ! String2MonetAttribute (argv[1])) {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_neighbours)
     result = "0";
     if (MonetEntity_isNode (entity)) {
       result = Monetint2String (MonetEntityList_sizeOf(entity->interfaces));
     } else if (MonetEntity_isInterface (entity)) {
       if (FROM(entity)) {
	 if (TO(entity)) {
	   return "2";
	 }
	 return "1";
       }
     }
MONETEND

/*****************************************************************************/
/*  Initialise and finish  */
/*****************************************************************************/

extern void init_db_script ()
{
  (void) MonetMember_module_class ("edb");

  MonetMember_create ("create",  do_create,
		      "name",
		      "Finds an existing or creates a new entity");
  MonetMember_create ("enum",  do_enum,
		      "name values ...",
		      "Creates an enumerated type. Default value comes first. \
		     The values are not checked for uniqueness. (help-enum)");
  MonetMember_create ("help-enum",  do_help_enum,
		      "",
		      "Lists all the defined enumerated types");
  MonetMember_create ("default-report-format",  do_report_format,
		      "?width attribute ...",
		      "Set default format for listing reports");
  MonetMember_create ( "for",		do_for_all_edb,
		      "condition ?action",
		      "Perform action for every entity in the database, \
that meets the condition.");
  MonetMember_create ( "load",	do_load_db,
		      "?filename",
		      "load entity database");
  MonetMember_create ( "save",	do_save_db,
		      "?filename",
		      "save entity database");
  MonetMember_create ( "put",		do_save_db_remote,
		      "",
		      "send entity database to monetd");
  (void) MonetMember_module_class ("Top");
  MonetMember_create ( "has-requests",		do_has_requests,
		      "",
		      "Returns false if the entity has no requests \
or drawings associated with it.");
  MonetMember_create ( "is-redirected",		do_is_redirected,
		      "",
		      "True if the system is currently redirected");
  MonetMember_create ( "is-attribute",		do_is_attribute,
		      "name",
		      "True if 'name' is the name of an attribute");
  MonetMember_create ("num-neighbours",  do_neighbours,
		      "",
		      "Returns number of neigbours an entity has");
}

extern void finish_db_script ()
{
  MonetReportFormat_free (Monet_report_format);
}


/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
