/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/***************************************************************/
          
#include "entityP.h"
#include <monet/msh/StringTable.h>
#include <monet/msh/Script.h>
#include <monet/draw/Draw.h>

/******************************************************************/
/*  Call monetsh routine when an attribute changes  */
/******************************************************************/

typedef struct change_callback * ChangeCallback;
struct change_callback {
  MonetAttribute attribute;
  char * command;
  MonetBool any;
  ChangeCallback next;
};
static ChangeCallback ChangeCallbacks = NULL;

/******************************************************************/
/*  */
/******************************************************************/

static void ChangeCallback_clear ()
{
  ChangeCallback c;
  FORALL (c, ChangeCallbacks, next) {
    MonetFree (c->command);
    MonetFree (c);
  }
  ChangeCallbacks = NULL;
}

MONETCALLBACK(do_clear)
     ChangeCallback_clear ();
MONETEND

static ChangeCallback ChangeCallback_create (attribute, command)
     char * attribute, * command;
{
  ChangeCallback change = NULL;
  MonetAttribute at;
  if (at = String2MonetAttribute (attribute)) {
    ChangeCallback c;
    FORALL (c, ChangeCallbacks, next) {
      if (c->attribute == at) {
	MonetFree (c->command);
	break;
      }
    }
    if (c) {
      change = c;
    } else {
      change = MonetNew (struct change_callback);
      LIST_ADD (change, ChangeCallbacks, next);
    }
    change -> attribute = at;
    change -> command = MonetNewString (command);
  } else {
    MonetPrintfError ("No such attribute [%s]", attribute);
  }
  return change;
}

MONETCALLBACK(do_new_any)
     ChangeCallback c;
     if (c = ChangeCallback_create (argv[1], argv[2])) {
       c -> any = True;
     }
MONETEND

MONETCALLBACK(do_new_null)
     ChangeCallback c;
     if (c = ChangeCallback_create (argv[1], argv[2])) {
       c -> any = False;
     }
MONETEND

MONETCALLBACK(do_help)
     ChangeCallback c;
     if (! ChangeCallbacks) {
       MonetPrintfnl ("No triggers registered");
     } else {
       FORALL (c, ChangeCallbacks, next) {
	 if (c->any)
	   MonetPrintfnl ("any   %s %s",
			  MonetAttribute2String (c->attribute), c->command);
	 else
	   MonetPrintfnl ("non-null  %s %s",
			  MonetAttribute2String (c->attribute), c->command);
       }
     }
MONETEND

/***************************************************************/
/*  Trigger routine to be called when attributes change        */
/***************************************************************/

extern void MonetEntity_attribute_triggers (entity)
     MonetEntity entity;
{
  ChangeCallback c;
  FORALL (c, ChangeCallbacks, next) {
    unsigned number;
    number = MonetAttribute2AttributeNumber (c -> attribute);
    if (MonetMask_ISSET(number, &MASK(entity))) {
      if (c -> any == False &&
	  MonetEntity_null_attribute (entity, c->attribute)) {
	/* Don't call when changed to null */
      } else {
	(void) MonetshEntityExecute (entity, c -> command, NULL, False);
      }
    }
  }
}

/***************************************************************/
/*  Init and Finish                                            */
/***************************************************************/

extern void MonetDb_trigger_init ()
{
  (void) MonetMember_module_class ("Trigger");
  MonetMember_create ( "create-non-null",	do_new_null,
		      "attribute command",
		      "Register a new attribute trigger, to be called \
		      when 'attribute' is set to a non null value");
  MonetMember_create ( "create-any",		do_new_any,
		      "attribute command",
		      "Register a new attribute trigger, these \
		      are called when an attribute changes its value");
  MonetMember_create ( "clear",  do_clear,
		      "",
		      "Clears all registered attribute triggers");
  MonetMember_create ( "help",  do_help,
		      "",
		      "Lists all registered attribute triggers");
}

extern void MonetDb_trigger_finish ()
{
  
  ChangeCallback_clear ();
}

/***************************************************************/
/***************************************************************/
/* $Header$ */
/***************************************************************/
/***************************************************************/

