/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/lib.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

#include "drawP.h"
#include <monet/msh/Script.h>

static DrawLibrary Draw;
typedef struct monet_draw_box * MonetDrawBox;

struct monet_draw_box {
  Position x_offset, y_offset;
  Dimension out_w, out_h;
  float x_scale, y_scale;
  MonetDrawBox next;
};

static MonetDrawBox BoxStack = NULL;

extern MonetShade MonetShade_inverse (shade)
     MonetShade shade;
{
  switch (shade) {
  case shade_0:
    return shade_5;
  case shade_1:
    return shade_4;
  case shade_2:
    return shade_3;
  case shade_3:
    return shade_2;
  case shade_4:
    return shade_1;
  case shade_5:
    return shade_0;
  default:
    MonetSysError ("[shade_inverse] Unknown shade [%d]", shade);
  }
  return shade_0;
}

/******************************************************************/
/* Adjust position depending on scale of map */
/******************************************************************/

#define SCALE(a, offset, factor) (offset + a * factor)
#define DESCALE(a, offset, factor) ((-offset + a) / factor)
#define x_adjust(x) SCALE(x, BoxStack->x_offset, BoxStack->x_scale)
#define y_adjust(y) SCALE(y, BoxStack->y_offset, BoxStack->y_scale)
#define x_readjust(x) DESCALE(x, BoxStack->x_offset, BoxStack->x_scale)
#define y_readjust(y) DESCALE(y, BoxStack->y_offset, BoxStack->y_scale)

#define w_adjust(w) SCALE(w, 0, BoxStack->x_scale)
#define h_adjust(h) SCALE(h, 0, BoxStack->y_scale)

/*****************************************************************/
/*  Adjust dimensions and check if inside drawing region */
/*****************************************************************/

static MonetBool adjust (x, y, w,h)
     Position *x,*y;
     Dimension *w,*h;
{
  *x = x_adjust(*x);
  *y = y_adjust(*y);
  *w = w_adjust(*w);
  *h = h_adjust(*h);
  return Draw->in_region (*x, *y, *w, *h);
}

static MonetBool adjust_line (xl, yl, xr, yr)
     Position *xl,*yl, *xr,*yr;
{
  Position x,y;
  Dimension w,h;
  *xl = x_adjust(*xl);
  *yl = y_adjust(*yl);
  *xr = x_adjust(*xr);
  *yr = y_adjust(*yr);

  if (*xl<*xr) {
    x = *xl;
    w = *xr-*xl;
  } else {
    x = *xr;
    w = *xl-*xr;
  }
  if (*yl<*yr) {
    y = *yl;
    h = *yr-*yl;
  } else {
    y = *yr;
    h = *yl-*yr;
  }
  return Draw->in_region (x, y, w, h);
}

/******************************************************/

extern void Draw_percentage_line (shade,  numer, denom, xl, yl, xr, yr)
	MonetShade shade;
	unsigned numer, denom;
	Position      xl, yl, xr, yr;
{
  if (adjust_line (&xl, &yl, &xr, &yr)) {
    if (numer == 0)
      Draw->draw_line (shade, xl, yl, xr, yr);
    else if (numer >= denom || denom == 0)
      Draw->draw_percentage_line (shade,  xl,yl, xl, yl, xr, yr);
    else {
      Position x,y;
      if (xl>xr) {
	x =  (xl-xr)*numer;
	x = x/denom + xr;
      } else {
	x =  (xr-xl)*numer;
	x = x/denom + xl;
      }
      if (yl>yr) {
	y =  (yl-yr)*numer;
	y = y/denom + yr;
      } else {
	y = (yr-yl)*numer;
	y = y/denom + yl;
      }
      Draw->draw_percentage_line (shade, x, y, xr, yr, xl, yl);
    }
  }
}

/*****************************************************************
  Only draw lines that are longer than 1
  *****************************************************************/

extern MonetBool Draw_significant_line (shade,  xl, yl, xr, yr)
	Position        xl, yl, xr, yr;
	MonetShade shade;
{
  if (adjust_line (&xl, &yl, &xr, &yr)) {
    if (abs (xl - xr) > 1 || abs (yl - yr) > 1 ) {
      Draw->draw_line (shade,  xl, yl, xr, yr);
      return True;
    }
  }
  return False;
}

extern void Draw_line (shade,  xl, yl, xr, yr)
	Position        xl, yl, xr, yr;
	MonetShade shade;
{
  if (adjust_line (&xl, &yl, &xr, &yr)) {
    Draw->draw_line (shade,  xl, yl, xr, yr);
  }
}

extern void Draw_fill_elipse (shade, x, y, width, height)
     MonetShade shade;
     Position        x, y;
     Dimension       width, height;
{
  if (adjust (&x, &y, &width, &height))
    Draw->fill_elipse (shade, x, y, width, height);
}

extern void Draw_elipse (shade, x, y, width, height)
     MonetShade shade;
     Position        x, y;
     Dimension       width, height;
{
  if (adjust (&x, &y, &width, &height))
    Draw->draw_elipse (shade, x, y, width, height);
}


extern void Draw_rectangle (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  if (adjust (&x, &y, &width, &height))
    Draw->draw_rectangle (shade, x, y, width, height);
}

extern void Draw_roundedRectangle (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  if (adjust (&x, &y, &width, &height))
    Draw->draw_roundedRectangle (shade, x, y, width, height);
}

extern void Draw_box_elipse (border, inside, x, y, width, height, o_inner)
     MonetShade border, inside;
     Position        x, y, o_inner;
     Dimension       width, height;
{
  Position inner;

  if (! adjust (&x, &y, &width, &height)) {
    return;
  }

  inner = x_adjust(o_inner);

  if (inside == border || o_inner == 0) {
    if (inside != shade_background) {
      Draw->fill_elipse (inside, x, y, width, height);
    }
  } else if (o_inner == 1) {
    if (inside != shade_background && width > 2 && height > 2) {
      Draw->fill_elipse (inside, x+1, y+1, width-2, height-2);
    }
    if (border != shade_background) {
      Draw->draw_elipse (border, x, y, width, height);
    }
  } else if (o_inner == 2) {
    if (inside != shade_background && width > 4 && height > 4) {
      Draw->fill_elipse (inside, x+2, y+2, width-4, height-4);
    }
    if (border != shade_background) {
      Draw->draw_elipse (border, x, y, width, height);
      if (width > 2 && height > 2) {
	Draw->draw_elipse (border, x+1, y+1, width-2, height-2);
      }
    }
  } else if (width > 0 && height > 0 && inner <= width) {
    int i2 = inner + inner;
    Draw->fill_elipse (inside, x, y, width, inner);
    if (i2 < width && i2 < height) {
      Draw->fill_elipse (inside, x, y+inner, inner, height - i2);
      if (i2 < width) {
	Draw->fill_elipse (inside, x+width-inner, y+inner,
			      inner, height-i2);
	if (i2 < height) {
	  Draw->fill_elipse (inside, x, y+height-inner, width, inner);
	}
      }
    }
  }
}

extern void Draw_box (border, inside, x, y, width, height, o_inner)
     MonetShade border, inside;
     Position        x, y, o_inner;
     Dimension       width, height;
{
  Position inner;

  if (! adjust (&x, &y, &width, &height)) {
    return;
  }

  inner = x_adjust(o_inner);

  if (inside == border || o_inner == 0) {
    if (inside != shade_background) {
      Draw->fill_rectangle (inside, x, y, width, height);
    }
  } else if (o_inner == 1) {
    if (inside != shade_background && width > 2 && height > 2) {
      Draw->fill_rectangle (inside, x+1, y+1, width-2, height-2);
    }
    if (border != shade_background) {
      Draw->draw_rectangle (border, x, y, width, height);
    }
  } else if (o_inner == 2) {
    if (inside != shade_background && width > 4 && height > 4) {
      Draw->fill_rectangle (inside, x+2, y+2, width-4, height-4);
    }
    if (border != shade_background) {
      Draw->draw_rectangle (border, x, y, width, height);
      if (width > 2 && height > 2) {
	Draw->draw_rectangle (border, x+1, y+1, width-2, height-2);
      }
    }
  } else if (width > 0 && height > 0 && inner <= width) {
    int i2 = inner + inner;
    Draw->fill_rectangle (inside, x, y, width, inner);
    if (i2 < width && i2 < height) {
      Draw->fill_rectangle (inside, x, y+inner, inner, height - i2);
      if (i2 < width) {
	Draw->fill_rectangle (inside, x+width-inner, y+inner,
			      inner, height-i2);
	if (i2 < height) {
	  Draw->fill_rectangle (inside, x, y+height-inner, width, inner);
	}
      }
    }
  }
}


extern void Draw_fill_rectangle (shade, x, y, width, height)
     MonetShade shade;
     Position        x, y;
     Dimension       width, height;
{
  if (adjust (&x, &y, &width, &height)) {
    if (width > 0 && height > 0) {
      Draw->fill_rectangle (shade, x, y, width, height);
    }
  }
}

extern void Draw_fill_circle (shade, x, y, diameter)
     MonetShade shade;
     Position        x, y;
     Dimension       diameter;
{
  Dimension width, height;

  width = diameter;
  height = diameter;
  if (adjust (&x, &y, &width, &height)) {
    if (width > 0 && height > 0) {
      Draw->fill_elipse (shade, x, y, width, height);
    }
  }
}

extern void Draw_circle (shade, x, y, diameter)
     MonetShade shade;
     Position        x, y;
     Dimension       diameter;
{
  Dimension width, height;

  width = diameter;
  height = diameter;
  if (adjust (&x, &y, &width, &height)) {
    Draw->fill_elipse (shade, x, y, width, height);
  }
}

extern MonetBool Draw_in_region (x,y,width,height)
     Position x,y;
     Dimension width,height;
{
  return adjust (&x, &y, &width, &height);
}

extern void Draw_string (shade, x, y, width, height, string, len)
     MonetShade shade;
     Position        x, y;
     Dimension width, height;
     String string;
     unsigned len;
{
  if (len > 0 && adjust (&x, &y, &width, &height)) {
    MONETDEBUG(4, ( "draw_string, %d %d %d %d %u", x,y,width,height, len));
    if (width > 0 && height > 0) {
      Draw->draw_string (shade, x, y, width, height, string, len);
    }
  }
}

extern void Draw_boxed_string (shade, x, y, width, height, string, len)
     MonetShade shade;
     Position        x, y;
     Dimension width, height;
     String string;
     unsigned len;
{
  if (len > 0 && adjust (&x, &y, &width, &height)) {
    MONETDEBUG(4, ( "draw_string, %d %d %d %d", x,y,width,height));
    if (width > 0 && height > 0) {
      Draw->draw_boxed_string (shade, x, y, width, height, string, len);
    }
  }
}

extern void  MonetDraw_adjust_for_scale (x, y, width, height)
     Position  *x, *y;
     Dimension *width, *height;
{
  *x = x_adjust(*x);
  *y = y_adjust(*y);
  *width = w_adjust(*width);
  *height = h_adjust(*height);
}

extern void  MonetDraw_adjust_wh_for_scale (width, height)
     Dimension *width, *height;
{
  *width = w_adjust(*width);
  *height = h_adjust(*height);
}

extern void  MonetDraw_adjust_xy_for_scale (x, y)
     Position  *x, *y;
{
  *x = x_adjust(*x);
  *y = y_adjust(*y);
}

/***************************************************************************/
/*  For scaling, set the sides of the universe  */
/***************************************************************************/

extern void Draw_set_box (x,y,in_w, in_h, out_w, out_h)
     Position x,y;
     Dimension in_w, in_h, out_w, out_h;
{
  MonetDrawBox box;

  box = MonetNew (struct monet_draw_box);
  box -> next = BoxStack;

  box->x_offset = x_adjust(x);
  box->y_offset = y_adjust(y);
  box -> out_w = out_w;
  box -> out_h = out_h;
  if (in_w && in_h) {
    float w, h, use;
    w = (float) out_w / (float) in_w;
    h = (float) out_h / (float) in_h;
    if (w < h) {
      use = w;
    } else {
      use = h;
    }
    box->x_scale = BoxStack->x_scale * use;
    box->y_scale = BoxStack->y_scale * use;
      
    Draw->set_scale (BoxStack->x_scale, BoxStack->y_scale);
  } else {
    MonetSysError ("zero inside box %d %d", in_w, in_h);
  }
  BoxStack = box;
}

extern void Draw_reset_box ()
{
  MonetFree (BoxStack);
  BoxStack = BoxStack->next;
  Draw->set_scale (BoxStack->x_scale, BoxStack->y_scale);
}

extern void Draw_set_reverse ()
{
	Draw->set_reverse ();
}

extern MonetBool Draw_is_xor ()
{
  return Draw->is_xor;
}

/*****************************************************************/
/*  Start up drawing batch  */
/*****************************************************************/

extern void Draw_start_draw (lib, x,y,xscale, yscale)
     DrawLibrary lib;
     Position x,y;
     float xscale, yscale;
{
  Draw = lib;
  BoxStack->x_offset = x;
  BoxStack->y_offset = y;
  BoxStack->out_w = 0;
  BoxStack->out_h = 0;
  BoxStack->x_scale = xscale;
  BoxStack->y_scale = yscale;
  Draw->set_scale (xscale, yscale);
}

extern void Draw_end_draw ()
{
  Draw->end_draw ();
  if (BoxStack->next) {
    MonetSysError ("Box stack not empty !!!");
  }
}

/******************************************************************/
/*  Init and finish  */
/******************************************************************/

extern void MonetDraw_init ()
{
  MonetLibrary_begin (monetlib, "draw", "Drawing Shapes");
  BoxStack = MonetNew (struct monet_draw_box);
  BoxStack->next = NULL;
  Draw_init_text();
  Draw_init_postscript();

  MonetShape_init ();
  MonetLibrary_done ();
}

extern void MonetDraw_finish ()
{
  MonetShape_finish ();

  Draw_finish_postscript();
  Draw_finish_text();
  MonetFree (BoxStack);
}

/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/lib.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

