/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/ps.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

#include <monet/msh/General.h>
#include <monet/draw/Draw.h>
#include <monet/msh/Script.h>

/******************************************************************/
/*  Globals  */
/******************************************************************/

static DrawLibrary DrawPSLib;

static FILE * DumpFile = NULL;
static float ReverseMask = 0;

static char *   Maps_postscript_prolog;
static float    ScaleX, ScaleY;

/******************************************************************/
/*  Send message to dump file  */
/******************************************************************/

static void command  (name, shade)
     char * name;
     MonetShade shade;
{
  float value;

  switch (shade) {
  case shade_0:
    value = 1.0;
    break;
  case shade_1:
    value = 0.75;
    break;
  case shade_2:
    value = 0.5;
    break;
  case shade_3:
    value = 0.5;
    break;
  case shade_4:
    value = 0.25;
    break;
  case shade_5:
    value = 0.0;
    break;
  default:
    value = 0.5;
    MonetSysError ("Unknown shade [%d]", shade);
    break;
  }
  if (ReverseMask >= value)
    value = ReverseMask - value;
  else
    value = value - ReverseMask;
  fprintf (DumpFile, " %g M%s\n", value, name);
}

static void four_out (name, shade, x,y,w,h)
     char * name;
     MonetShade shade;
     int x,y,w,h;
{
  fprintf (DumpFile, " %d %d %d %d ", w,h,x,y);
  command (name, shade);
}

static void four_value_out (name, x,y,w,h)
     char * name;
     int x,y,w,h;
{
  fprintf (DumpFile, "%s %d %d %d %d\n", name, w,h,x,y);
}

static void comment_out (a)
     char * a;
{
  fprintf (DumpFile, "%%%s\n", a);
}

static void def_out (name, value)
     char * name, * value;
{
  fprintf (DumpFile, "/M%s   (%s) def", name, value);
  fprintf (DumpFile, "\n");
}

static void line_out (s)
     char * s;
{
  fprintf (DumpFile, "%s\n", s);
}

static void value_out (s, value)
     char * s, *value;
{
  fprintf (DumpFile, "%s %s\n", s, value);
}

/******************************************************************/
/*  Copy the prologue file to the output                          */
/******************************************************************/

static MonetBool out_prologue (name)
     char * name;
{
  char * filename;
  char data [1024]; /* !!! */
  FILE * fid;

  filename = (char *) MonetFilename (Maps_postscript_prolog);

  comment_out (" --- Page Format --- ");
  def_out ("format", name);
  comment_out (" ------------------- ");

  fid = fopen (filename, "r");
  if (fid == NULL) {
    MonetPrintfWarning ("Cannot open postscript prologue file [%s]", filename);
    return False;
  }

  while (fgets(data, sizeof(data), fid) != NULL) {
    fputs (data, DumpFile);
  }

  fclose (fid);
  return True;
}

/******************************************************************/
/* Before drawing anything on a particular map, this routine must */
/* be called */
/******************************************************************/

extern char * Draw_start_postscript (prologue_name, title, filename,
				      author, contact, source,
				      xl, xr, yl, yr)
     char * title, *prologue_name, *filename;
     char * author, *contact, *source;
     Position xl,xr, yl,yr;
{
  ScaleX = 0.0;
  ScaleY = 0.0;
  filename = MonetMakePathname ("/tmp/", filename, ".ps");
  DumpFile = fopen (filename, "w");
  if (DumpFile == NULL) {
    MonetPrintfError ("Could not open file [%s]", filename);
    return NULSTR;
  } else {
    MonetPrintfInform ("Dumping 'postscript' format map to file [%s]",
		       filename);
  }
  ReverseMask = 0.0;

  value_out ("%! ", Monet_software_title ());
  if (prologue_name == NULL) {
    fprintf (DumpFile, "%s %d %d %d %d\n", "%%BoundingBox:", xl, yr, xr, yl);
    comment_out ("This picture has no headers or footers");
    comment_out ("and is for inclusion in other documents");
    prologue_name = "A_.PS";
  } else {
    comment_out ("This picture has headers and footers");
    fprintf (DumpFile, "%s %d %d %d %d\n", "%%BoundingBox:", 18, 23, 570, 819);
  }
  value_out ("%%Creator: ", author);
  value_out ("%%Title: ", title);
  value_out ("%%CreationDate: ", (char *) MonetPrettyTimeStamp ());
  line_out  ("%%Pages: 1");
  line_out  ("%%DocumentFonts: Times-Roman Helvetica");
  line_out  ("%%EndComments");

  comment_out ("Parameters for the prologues");
  def_out ("timestamp", MonetPrettyTimeStamp ());
  def_out ("title", title);
  def_out ("author", author);
  def_out ("contact", contact);
  def_out ("source", source);
  def_out ("softwaretitle", Monet_software_title ());
  def_out ("softwaresource", Monet_software_source ());
  out_prologue (prologue_name);
  
  fprintf (DumpFile, "%d %d %d %d Mscalemap\n", xl,yl, xr,yr);
  comment_out ("%%Page: 0 1");

  Draw_start_draw (DrawPSLib, 0,0, 1.0, 1.0);
  return filename;
}

static void end_draw ()
{
  line_out ("showpage\n");
  fclose (DumpFile);
}


/* !!! Should be draw_fraction line */
static void draw_percentage_line (shade, x,y, xl, yl, xr, yr)
     Position        xl, yl, xr, yr, x,y;
     MonetShade shade;
{
  fprintf (DumpFile, " %d %d %d %d %d %d ", xl,yl,x,y,xr,yr);
  command ("PerLin", shade);
}

static void 
draw_line (shade,  xl, yl, xr, yr)
     Position        xl, yl, xr, yr;
     MonetShade shade;
{
  four_out ("line", shade,(int) xl,(int)yl, (int)xr,(int)yr);
}

static void 
fill_elipse (shade, x, y, width, height)
     MonetShade shade;
     Position        x, y;
     Dimension       width, height;
{
  four_out ("Felipse", shade, x,y, width,height);
}

static void 
draw_elipse (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  four_out ("Delipse", shade, x,y, width,height);
}


static void draw_rectangle (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  four_out ("DRec", shade, x,y, width,height);
}

static void draw_roundedRectangle (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  four_out ("DRec", shade, x,y, width,height);
}

static void fill_roundedRectangle (shade, x, y, width, height)
	MonetShade shade;
	Position        x, y;
	Dimension       width, height;
{
  four_out ("FRec", shade, x,y, width,height);
}

static void fill_rectangle (shade, x, y, width, height)
     MonetShade shade;
     Position        x, y;
     Dimension       width, height;
{
  if (shade != shade_background) {
    four_out ("FRec", shade, x,y, width,height);
  }
}

static void draw_string (shade, x, y, w,h, string, len)
     MonetShade shade;
     Position        x, y;
     String string;
     Dimension       w,h;
     unsigned len;
{
  String buf;

  buf = (String) MonetMalloc (len+1);
  (void) strncpy (buf, string, len);
  buf[len] = NULL;
  fprintf (DumpFile, " (%s) ", buf);
  four_out ("St", shade, x,y, w,h);
  MonetFree (buf);
}

static void draw_boxed_string (shade, x, y, w,h, string, len)
     MonetShade shade;
     Position        x, y;
     String string;
     Dimension       w,h;
     unsigned len;
{
  String buf;

  buf = (String) MonetMalloc (len+1);
  (void) strncpy (buf, string, len);
  buf[len] = NULL;
  fprintf (DumpFile, " (%s) ", buf);
  four_out ("BSt", shade, x,y, w,h);
  MonetFree (buf);
}

static void set_scale (x,y)
     float x,y;
{
  /*  Scale text, set only if changed for efficiency reasons  */
  /*  Should never do a direct compare on floats but I think it */
  /*  is the best thing in this situation.  */
  if (ScaleX != x && ScaleY != y) {
    fprintf (DumpFile, " %g %g MTS\n", x,y);
    ScaleX = x;
    ScaleY = y;
  }
}

static void set_reverse ()
{
  if (ReverseMask > 0.5)
    ReverseMask = 0.0;
  else
    ReverseMask = 1.0;
}

static MonetBool in_region (x,y,w,h)
     Position x,y;
     Dimension w,h;
{
  return True;
}

/*****************************************************************/
/* Init and finish drawing of entitys */
/*****************************************************************/

extern void Draw_init_postscript ()
{
  NEWDRAWLIB(DrawPSLib, False);

  (void) MonetMember_module_class ("postscript");
  MonetMember_create_write ( "prolog", &Maps_postscript_prolog,
			    "Postscript prologue for maps");
  Maps_postscript_prolog = MonetNewString ("prolog.PS");
  MonetConfig_create ("-ps",  "postscript->prolog", NULL,
		      "Prolog file for postscript output");
}

extern void 
Draw_finish_postscript()
{
  MonetFree (DrawPSLib);
  DrawPSLib = NULL;
}


/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/Draw/RCS/ps.c,v 1.1 1991/03/26 10:52:24 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/



