/***************************************************************/
/***************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/attach.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************/
/*  Seems to do everything I want it to, might be nice if */
/*  it was more inteligent and tried to avoid other boxes */
/*  Would be nice f it drew an absolute 45 degree line rather */
/*  than another angle. */
/*  Could do partial adjustments better */
/*  A totally straight line is better than two at right angles */
/*  !!! Bug:  Does not properly clip lines to edges of boxes */
/******************************************************************/

#include <monet/map/Drawing.h>
#include <monet/map/Lines.h>

/***************************************************************/
/***************************************************************/

/* !!! These should be elsewhere, in configure */
#define Config_x_attachMargin 2
#define Config_y_attachMargin 2

/***************************************************************/
/* Return the difference between big and small or 0 if big <small */
/***************************************************************/

#define diffCard(big,small) (big<small?0:big-small)

/***************************************************************
  Takes a line that would connect two rectangles but one is
  actuall an elipse, so this would extend it.
  ***************************************************************/

#define SQR(a) ((a) * (a))
#define SQRTX(r,y) (r - (MonetSqrt ((unsigned) SQR(r) - (unsigned) SQR(y))))
#define INC(r,y,ry,rh)\
  (SQRTX(r , (((y) - ry) - rh) * r / rh))

static void elipse_extend_line (x,y, rx,ry, drw,drh)
     Position  *x,*y;
     Position rx, ry;
     Dimension drw, drh;
{
  float rh, rw;

  rh = (float) drh / 2.0;
  rw = (float) drw / 2.0;

  MONETDEBUG(10, ( "x,y  %d %d   rx,ry  %d %d  rw,rh %f %f",
	      *x,*y, rx,ry, rw,rh));

  if (*x <= rx) {
    *x += (Position) INC(rw, *y, ry, rh);
  } else if (*x >= rx+drw) {
    *x -= (Position) INC(rw, *y, ry, rh);
  }
  if (*y <= ry) {
    *y += (Position) INC(rh, *x, rx, rw);
  } else if (*y >= ry+drh) {
    *y -= (Position) INC(rh, *x, rx, rw);
  }
  MONETDEBUG(10, ( "x,y %d %d", *x, *y));
}

/***************************************************************
  Given two lines, find two points one on each line (xr1,xr2)
  such that a line drawn between these.
  ***************************************************************/

static void nearest_point_of_2_lines (x1, x2, w1,w2, xr1, xr2)
     Position        *xr1, *xr2, x1, x2;
     Dimension       w1, w2;
{
  /* If not overlapping */ 
  if (x1+w1 <= x2) {
    *xr1 = x1+w1;
    *xr2 = x2;
  } else if (x1 >= x2+w2) {
    *xr1 = x1;
    *xr2 = x2+w2;
    /*  Else overlapping, use mid point of overlapping region */
  } else if (x1 <= x2) {
    if (x1+w1 <= x2+w2)
        *xr1 = *xr2 = x2 + (w1 - x2 + x1)/2;
    else
      *xr1 = *xr2 = x1 + w1/2;
  } else if (x1 > x2) {
    if (x1+w1 > x2+w2)
      *xr1 = *xr2 = x1 + (w2 - x1 + x2)/2;
    else
      *xr1 = *xr2 = x1 + w1/2;
  }
}

static void nearest_point_of_2_elipses (x1, x2, w1,w2, xr1, xr2)
     Position        *xr1, *xr2, x1, x2;
     Dimension       w1, w2;
{
  nearest_point_of_2_lines (x1, x2, w1,w2, xr1, xr2);
}

static void nearest_point_of_line_and_elipse (xl, xe, wl,we, xrl, xre)
     Position         xl, xe, *xrl, *xre;
     Dimension       wl, we;
{
  nearest_point_of_2_lines (xl, xe, wl,we, xrl, xre);
}

/************************************************************************/
/* Adjust so that straight lines never hit very end of box */
/* Pretty ad hoc */
/************************************************************************/

static void adjust_for_edge (draw, x_attach, y_attach, x2, y2, w2, h2)
     Drawing draw;
     int x_attach, y_attach;
     Position   *x2, *y2;
     Dimension  *w2, *h2;
{
  *x2 = X(draw);
  *y2 = Y(draw);
  *w2 = WIDTH(draw);
  *h2 = HEIGHT(draw);

  if (*w2 < x_attach * 2) {
    *x2 = *x2 + *w2 / 2;
    *w2 = 0;
  } else {
    *x2 += x_attach;
    *w2 -= x_attach * 2;
  }
  if (*h2 < y_attach * 2) {
    *y2 = *y2 + *h2 / 2;
    *h2 = 0;
  } else {
    *y2 += y_attach;
    *h2 -= y_attach * 2;
  }
}

/************************************************************************/
/* Choose attachment points */
/* Chooses the 'best' line between two drawings */
/************************************************************************/

static void choose_attachment (from, to, xl, yl, xr, yr,
			       straight, xstraight, length)
     Drawing        from, to;
     Position       *xl, *yl, *xr, *yr;
     MonetBool        *straight, *xstraight;
     Dimension      *length;
{
  Monet_choose_attachment (Config_x_attachMargin, Config_x_attachMargin,
			   from, to, xl, yl, xr, yr,
			   straight, xstraight, length);
}

extern void Monet_choose_attachment (x_attach, y_attach,
				     from, to, xl, yl, xr, yr,
				     straight, xstraight, length)
     int x_attach, y_attach;
     Drawing        from, to;
     Position       *xl, *yl, *xr, *yr;
     MonetBool        *straight, *xstraight;
     Dimension      *length;
{
   Position        x1, y1, x2, y2;
   Dimension       w1, w2, h1, h2;
   MonetBool from_elipse, to_elipse;

   if (Drawing_mapOf (from) != Drawing_mapOf (to)) {
     /*  They are not on the same map, 'to' is on the parent map */
   }

   adjust_for_edge (from, (int) x_attach, (int) y_attach, &x1,&y1, &w1, &h1);
   adjust_for_edge (to, (int) x_attach, (int) y_attach, &x2, &y2, &w2, &h2);

   from_elipse = Drawing_isElipse (from);
   to_elipse   = Drawing_isElipse (to);

   nearest_point_of_2_lines (x1,x2, w1,w2, xl, xr);
   nearest_point_of_2_lines (y1,y2, h1,h2, yl, yr);

   /*   Work out if the line is straight and if so parallel to which axis  */
   if (*xr == *xl) {
     if (*yr == *yl) {
       *length = 0;  /* !!! Undefined */
       *straight = False;
     } else {
       *xstraight = True;
       *length = abs (yl - yr);
       *straight = True;
     }
   } else if (*yr == *yl) {
     *length = abs (xl - xr);
     *xstraight = False;
     *straight = True;
   } else {
     *length = 0;  /* !!! Undefined */
     *straight = False;
   }

   /* If one side or the other is an elipse then extend the line
      so that it touches the side of the elipse.
      */
   if (from_elipse) {
     elipse_extend_line (xl, yl, x1, y1, w1, h1);
   }
   if (to_elipse) {
     elipse_extend_line (xr, yr, x2, y2, w2, h2);
   }

}

/***************************************************************
  Finds a 'good' place to position drawing 'mid', somewhere between
   'sys1' and 'sys2'.  If the current position is 'better' it will
   return the current position.

   if the new position is on a straight line choose that, else
   if the previous place had straight lines between at least one of the
   systems do not use the new value.
   ***************************************************************/


extern void choose_midpoint_attachment (mid, sys1,sys2, x, y)
     Drawing mid, sys1, sys2;
     Position *x, *y;
{
  Position xl,yl, xr,yr;
  MonetBool     straight,     straight1,     straight2, isx1, isx2, isx;
  Dimension len, len1, len2;
  static int rand = -2;

  if (rand == 2) {
    rand = -2;
  } else {
    ++ rand;
  }
  choose_attachment (mid, sys1, &xl,&yl, &xr,&yr, &straight1, &isx1, &len1);
  choose_attachment (mid, sys2, &xl,&yl, &xr,&yr, &straight2, &isx2, &len2);
  choose_attachment (sys1, sys2, &xl,&yl, &xr,&yr, &straight, &isx,  &len);
  if ((!straight || (straight && len > len1 + len2))
      && straight1 && straight2) {
    MONETDEBUG(10, ("Keep position [%s]", Drawing_title(mid)));
    *x = X(mid);
    *y = Y(mid);
  } else if (!straight && straight1) {
    MONETDEBUG(10, ( "Alter position sys2 [%s]", Drawing_title(mid)));
    /* Should straighten connection to sys2 */
    if (isx1) { /* Alter the y */
      *y = Y(sys2) + HEIGHT(sys2)/2 + rand;
      *x = X(mid);
    } else { /* Alter the x */
      *x = X(sys2) + WIDTH(sys2)/2 + rand;
      *y = Y(mid);
    }
  } else if (!straight && straight2) {
    MONETDEBUG(10, ( "Alter position sys1 [%s]", Drawing_title(mid)));
    /* Should straighten connection to sys1 */
    if (isx2) {
      *y = Y(sys1) + HEIGHT(sys1)/2 + rand;
      *x = X(mid);
    } else {
      *x = X(sys1) + WIDTH(sys1)/2 + rand;
      *y = Y(mid);
    }
  } else if (straight1 && straight2) {
    if (isx1 && isx2) {
      *x = X(mid);
      *y = (yl + yr)/2 + rand;
    } else if ((!isx1) && (!isx2)) {
      *y = Y(mid);
      *x = (xl + xr)/2 + rand;
    } else {
      *x = X(mid);
      *y = Y(mid);
    }
  } else {
    /* MonetPrintfInform ("New position [%s]", Drawing_title(mid)); !!! */
    *x = (xl + xr)/2 + rand;
    *y = (yl + yr)/2 + rand;
  }
}

/***************************************************************/
/*  Give position half way between systems  */
/***************************************************************/

extern void choose_midpoint_position (sys1, sys2, x,y)
     Drawing sys1, sys2;
     Position *x, *y;
{
  Position  xl,yl, xr,yr;
  MonetBool   straight, isx;
  Dimension len;
  
  choose_attachment (sys1, sys2, &xl,&yl, &xr,&yr, &straight, &isx,  &len);
  /* MonetPrintfInform ("New position [%s]", Drawing_title(mid)); !!! */
  *x = (xl + xr)/2;
  *y = (yl + yr)/2;
}

/***************************************************************/
/***************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/attach.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************/
/***************************************************************/

