/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/draw.c,v 1.1 1991/07/22 11:37:05 a38 Exp a38 $  */
/******************************************************************/
/*  Does all the drawing of maps, should really be split into two */
/*  parts, one for drawing the maps, and the other for actually   */
/*  the drawings.                                                 */
/******************************************************************/

#include "mapP.h"
#include <monet/map/Interface.h>
#include <monet/msh/Script.h>
#include <monet/draw/Shape.h>

/************************************************************************/
/*  Globals  */
/************************************************************************/

static void draw_map PR((MonetMap map));

static MonetBool isHidden (o)
	Drawing o;
{
  MonetEntity e = ENTITY(o);

  if (MonetEntity_isClass (e))
    return False;
  return False;
}

/************************************************************************/
/*  Set sizes before drawing a map  */
/************************************************************************/

static MonetMap map_set (map)
     MonetMap map;
{/*
  Position xl, yl, xr, yr;

  while (map) {
    if (map->mapWidget)
      break;
    else if (map->parent) {
      Map_corners (map, &xl, &yl, &xr, &yr, False);
      Draw_set_box (X(map->parent)+1, Y(map->parent)+1,
		    (Dimension) xr, (Dimension) yr,
		    WIDTH(map->parent)-2, HEIGHT(map->parent)-2);
      map = Drawing_mapOf (map->parent);
    } else
      break;
  }*/
  return map;
}

static void map_reset (map)
     MonetMap map;
{/*
  Position xl, yl, xr, yr;

  while (map) {
    if (map->mapWidget)
      break;
    else if (map->parent) {
      Map_corners (map, &xl, &yl, &xr, &yr, False);
      Draw_reset_box ();
      map = Drawing_mapOf (map->parent);
    } else
      break;
  }*/
}

/************************************************************************/
/*  Draw a submap  */
/************************************************************************/

static void draw_submap (drawing)
     Drawing drawing;
{
  Position xl, yl, xr, yr;
  Map_corners (drawing->submap, &xl, &yl, &xr, &yr, False);
  Draw_set_box (X(drawing)+1, Y(drawing)+1,
		(Dimension) xr, (Dimension) yr,
		WIDTH(drawing)-2, HEIGHT(drawing)-2);
  Draw_set_reverse ();
  draw_map(drawing->submap);
  Draw_set_reverse ();
  Draw_reset_box ();
}

/************************************************************************/
/*  Recursize procedure for drawing maps  */
/************************************************************************/

static void draw_map (map)
     MonetMap map;
{
  Drawing drawing;

  if (map->background_shape) {
    MonetShape_draw (map->background_shape,
		     1, 1, 1000, 1000,
		     MONETSTATE_NO_BORDER & MONETSTATE_NO_LABEL,
		     False, NULSTR, 0, NULSTR);
  }
  FOR_DRAWINGLIST(drawing, Map_elements (map)) {
    if (drawing->submap_visible && drawing->submap) {
      draw_submap (drawing);
    } else if (MonetEntity_isInterface(ENTITY(drawing))) {
      MonetConnectingLine_draw (drawing->to, drawing);
      MonetConnectingLine_draw (drawing->from, drawing);
    }
    Draw_system (drawing);
  } END_FORLIST;
}

/************************************************************************/
/*  Draws just the outline of a drawing, properly scaled  */
/************************************************************************/

extern void Drawing_outline (drawing, insert_map, x,y)
     Drawing          drawing;
     Position x,y;
     MonetMap insert_map;
{
  (void) map_set (insert_map);
  if (Drawing_isElipse (drawing)) {
    Draw_elipse(shade_dark, x, y, WIDTH(drawing), HEIGHT(drawing));
  } else {
    Draw_rectangle(shade_dark, x, y, WIDTH(drawing), HEIGHT(drawing));
  }
  map_reset (insert_map);
}

/************************************************************************/
/*  Draw the systems on the screen                                      */
/************************************************************************/

extern void Draw_system(drawing)
     Drawing          drawing;
{
  char * label;
  if ( ! isHidden (drawing)) {
    MonetShapeState state = drawing->state;
    if (! Drawing_mapOf (drawing) -> draw_shape)
      state |= MONETSTATE_NO_PICTURE;
    if (LABEL(drawing)) {
      label = LABEL(drawing);
    } else {
      label = TITLE(ENTITY(drawing));
    }
    MonetShape_draw (drawing->shape,
		X(drawing), Y(drawing), WIDTH(drawing), HEIGHT(drawing),
		state,
		Drawing_isClass (drawing),
		label, drawing->labellen, LABEL2(drawing));
  }
}

/************************************************************************/
/* Actually draw the map                                                */
/* Draws a line between two systems. May later allow different line     */
/* styles. Very non-optimal method employed, need not do so much        */
/* calcutlation each time. Attempts to be clever with lines and draw a  */
/* straight one when it can                                             */
/* !!! May later also try 45 degree lines if nothing better available   */
/************************************************************************/

extern void Map_draw(map)
     MonetMap          map;
{
  Drawing          s;

  switch (map->drawnas) {
  case drawnas_list:
    if (!Map_classesOnly(map)) {
      Dimension w, h;
      char * heading;

      if (map->entity) {
	heading = MonetReportFormat_pretty (map->report_format,
					    map->entity);
      } else {
	heading = "Not loaded";
      }
      w = DrawCharWidth  * MonetStrlen (heading);
      h = DrawCharHeight;
      Draw_box (shade_dark, shade_background, 19, 12, w+6,h+6, 3);
      Draw_string (shade_dark, 22, 15 + h, w, h,
		   heading, MonetStrlen (heading));
    }
    FOR_DRAWINGLIST(s, Map_elements (map)) {
      Drawing_list_draw (s);
    } END_FORLIST;
    break;
  default:
  case drawnas_map:
    if (Map_classesOnly(map)) {
      MonetMap_draw_classes (map);
    } else {
      draw_map (map);
    }
    break;
  }
  Draw_end_draw ();
}

/***************************************************************/
/*  Draw a drawing, drawing connecting lines in moved  */
/***************************************************************/

extern void Drawing_draw (drawing, moved, undraw)
     Drawing drawing;
     MonetBool moved, undraw;
{
  static Undraw = True;
  MonetBool r;
  MonetMap map;

  map = Drawing_mapOf(drawing);

  switch (map->drawnas) {
  case drawnas_list:
    Drawing_list_draw (drawing);
    break;
  case drawnas_map:
    if (Drawing_isInterface(drawing))
      moved = True;
    if ( ! moved) {
      r = drawing->redraw_connectors;
      drawing->redraw_connectors = False;
    } else if (drawing->redraw_connectors == False) {
      drawing->redraw_connectors = True;
    }

    if (drawing->submap_visible && drawing->submap) {
      draw_submap (drawing);
    }
    Draw_connecting_lines (drawing, undraw);
    Draw_system(drawing);
    /*
      map_set (map));
      map_reset (map);
      */
    if ( ! moved)
      drawing->redraw_connectors = r;
    break;
  }
}

/************************************************************************/
/*  Init and Finish  */
/************************************************************************/

extern void init_drawMaps ()
{
}

extern void finish_drawMaps ()
{
}

/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/draw.c,v 1.1 1991/07/22 11:37:05 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

