/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/label.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/* $Log: label.c,v $
 * Revision 1.1  1991/06/11  11:26:10  a38
 * Initial revision
 *  */
/*****************************************************************************/
/* Sets up and Returns an drawings label */
/* !!! Not really implemented yet */
/*****************************************************************************/

#include <monet/map/Drawing.h>
#include <ctype.h>

#define SETLABEL(d, label, len) {LABEL(d) = (label); d->labellen = len;}

/*****************************************************************************/
/*  Set a default domain for a map.  This way labels are kept short, because */
/*  the default is not displayed.  */
/*****************************************************************************/

extern void Map_set_domain (map, domain)
     MonetMap map;
     String domain;
{
  if (NEQ(domain, map->default_domain)) {
    if (map->default_domain)
      MonetFree (map->default_domain);
    map->default_domain = MonetNewString (domain);
    map->default_domain_len = MonetStrlen (map->default_domain);
    Map_request_changed_redraw (map);
  }
}

/*****************************************************************************/
/*  Reset the label of an drawing to the default */
/*****************************************************************************/

extern void Drawing_set_label (d, label)
     Drawing d;
     char * label;
{
  if (label && *label && !EQS(label, d->given_label)) {
    MonetFree (d->given_label);
    if (EQS(label, TITLE (ENTITY(d)))) {
      d->given_label = NULL;
    } else {
      d->given_label = MonetNewString (label);
    }
    if (LABEL(d) != NULL) {
      MonetFree (LABEL(d));
      SETLABEL(d, NULL, 0);
    }
  }
}

extern String MonetLabelState2String (state)
     MonetLabelState state;
{
  switch (state) {
  default:
    return "short";
  case label_abbrev:
    return "short";
  case label_none:
    return "none";
  case label_full:
    return "full";
  }
}

/*****************************************************************************/
/*  Toggle the primary label  between one of its states  */
/*****************************************************************************/

extern void Drawing_toggle_label (drawing)
     Drawing drawing;
{
  if (drawing->primary_label == label_last)
    drawing->primary_label = label_first;
  else
    drawing->primary_label = (MonetLabelState)
      ((unsigned int) drawing->primary_label + 1);
}

extern void Drawing_reset_labels (drawing)
     Drawing drawing;
{
  drawing -> primary_label = label_short;
  drawing -> secondary_label = label_attribute;
}

/***************************************************************/
/*  Get label for drawing, when map is drawn as a list  */
/***************************************************************/

extern void Drawing_reset_list_label (drawing)
     Drawing drawing;
{
  String label;
  char buffer [1024]; /* !!! */
  MonetMap map = Drawing_mapOf (drawing);
  MonetEntity entity = ENTITY(drawing);

  if (LISTBOX(drawing).label != NULL)
    MonetFree (LISTBOX(drawing).label);

  if (Drawing_isClass (drawing)) {
    label = (char *) MonetEntity_title (entity);
  } else {
    (void) sprintf (buffer, "%s",
		    MonetReportFormat_pretty (map->report_format, entity));
    label = buffer;
  }
  LISTBOX(drawing).label  = MonetNewString   (label);
  LISTBOX(drawing).width  = DrawCharWidth  * MonetStrlen (label) + 5;
  LISTBOX(drawing).height = DrawCharHeight + 5;
}


/*****************************************************************************/
/*  Set and Return primary label  */
/*****************************************************************************/

extern char * Drawing_primary_label (drawing)
     Drawing drawing;
{
  String label;
  unsigned old_len, len;
  MonetMap map = Drawing_mapOf (drawing);
  char buffer [64];
  MonetBool domain = True;
  MonetEntity entity;

  if ( !drawing) {
    MonetSysError ("Drawing is null");
    return NULL;
  }
  entity = ENTITY(drawing);
  switch (drawing->primary_label) {
  default:
    if (drawing->given_label) {
      label = drawing->given_label;
    } else {
      label = (char *) TITLE(entity);
    }
    break;
  case label_attribute: {
    char * value, * at;
    if (map->primary_label) {
      value = MonetEntity_get_attribute_full (entity, map->primary_label);
      at = MonetAttribute2String (map->primary_label);
      domain = False;
    } else {
      value = (char *) TITLE(ENTITY(drawing));
      at = "";
    }
    if (value == NULL)
      value = "";
    (void) sprintf (buffer, "%s: %s", at, value);
    label = buffer;
  }
    break;
  case label_none:
    label = "";
    break;
  case label_full:
    label = Drawing_title (drawing);
    break;
  }
  old_len = MonetStrlen (LABEL(drawing));
  len     = MonetStrlen (label);
  if (domain && len >= map->default_domain_len) {
    String place;
    place = label + (len - map->default_domain_len);
    if (EQS(place, map->default_domain)) {
      len -= map->default_domain_len;
    }
  }
  if (LABEL(drawing)){
    if (len > old_len) {
      /* -- Not enough space so reallocate */
      MonetFree (LABEL(drawing));
      LABEL(drawing) = MonetMalloc (len+1);
    } else {
      /* -- already enough space allocated */
    }
  } else {
    SETLABEL(drawing, MonetMalloc (len+1), len);
  }

  (void) strncpy (LABEL(drawing), label, (int) len);
  drawing->labellen = len;
  LABEL(drawing)[len] = '\0';

  return (LABEL(drawing));
}

/*****************************************************************************/
/*  Set the secondary label  */
/*****************************************************************************/

extern char * Drawing_secondary_label (drawing)
     Drawing drawing;
{
  MonetMap map;
  char * name;

  map = Drawing_mapOf(drawing);

  if (map->secondary_label) {
    long value, num;
    num = MonetAttribute2AttributeNumber (map->secondary_label);
    value = (long) MONETGETINTEGER(ENTITY(drawing), num);
    if (value != 0) {
      name = Monetlong2MegaString (value);
    } else {
      name = NULL;
    }
  } else {
    name = NULL;
  }
  if (EQS (LABEL2(drawing), name)) {
    /*  They are the same so don't change  */
  } else {
    if (LABEL2(drawing)) {
      MonetFree (LABEL2(drawing));
    }
    LABEL2(drawing) = MonetNewString(name);
  }
  return LABEL2(drawing);
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/label.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/*****************************************************************************/
/*****************************************************************************/

