/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/drawing.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/******************************************************************/
/*  Tasks
    draw connectors and work out there actual positions.
    find out what should be connected to what.
    draw connections of a drawing and reposition interfaces.

    To decide what to link to the logical connections of the underlying
    entities are used.  If a drawing does not exist on the same map as
    an interface and they are connected then a connecting line may be
    drawn to map reference.
    */
/******************************************************************/

#include <monet/map/Drawing.h>

struct monet_connecting_line {
  MonetShade shade;
  /*  How much of the line to make thick, how to scale the lines  */
  unsigned numerator, denominator;
  Drawing system;
  Position xr, yr, xl, yl;
  MonetBool draw_sys;		/* Draw line at end too */
  Position sys_xr, sys_yr, sys_xl, sys_yl;
  MonetBool do_draw;
};

static MonetBool interface_not_drawable (interface)
     Drawing interface;
{
  if (interface->from && ! interface->from->do_draw)
    return True;

  if (interface->to && ! interface->to->do_draw)
    return True;

  return False;
}

/***************************************************************/
/*  Work out the position of interfaces on the map */
/***************************************************************/

extern MonetBool get_newInterfacePosition (map, interface, x, y)
     Drawing interface;
     MonetMap map;
     Position *x, *y;
{
  Drawing from , to;

  from = interface->from->system;
  to   = interface->to->system;

  if (from == NULL || to == NULL) {
    /* !!!  Big fuck up with loading interfaces must use value > than 0 */
    *x = 0;
    *y = 0;
    return False;
  }
  choose_midpoint_attachment (interface, from, to, x, y);
  return True;
}

/************************************************************************/
/*  Draw connection between interface and systems                       */
/*  !!! replace this with draw connecting line                          */
/************************************************************************/

static MonetBool prepare_connector (interface, line)
     Drawing          interface;
     MonetConnectingLine line;
{
  Drawing system;
  if ((system = line->system) && line->do_draw) {
    MonetBool straight, xstraight;
    Dimension length;
    MonetShade shade;
    Position xl, yl, xr, yr;

    Monet_choose_attachment(0, 0, interface, system, &xl, &yl, &xr, &yr,
			    &straight, &xstraight, &length);
    if (Drawing_isSelected (interface))
      shade = shade_heavy;
    else
      shade = shade_dark;
    
    line->shade = shade;
    line->xl = xl;
    line->xr = xr;
    line->yr = yr;
    line->yl = yl;

    if (line->draw_sys = Drawing_isMinimal (system)) {
      Dimension w = X(system) + WIDTH(system);
      Dimension h = Y(system) + HEIGHT(system);
      if (xr == X(system)) {
	line->sys_xl = X(system);
	line->sys_xr = X(system);
	line->sys_yr = h;
	line->sys_yl = Y(system);
      } else if (yr == Y(system)) {
	line->sys_xl = X(system);
	line->sys_xr = w;
	line->sys_yr = Y(system);
	line->sys_yl = Y(system);
      } else if (xr == w) {
	line->sys_xl = w;
	line->sys_xr = w;
	line->sys_yr = Y(system);
	line->sys_yl = h;
      } else  {
	line->sys_xl = X(system);
	line->sys_xr = w;
	line->sys_yr = h;
	line->sys_yl = h;
      }
    }
    return True;
  }
  return False;
}

static void fix_interface_pos (map, interface)
     MonetMap map;
     Drawing interface;
{
  Position x, y;
  if (get_newInterfacePosition  (map, interface, &x, &y)) {
    X(interface) = x;
    Y(interface) = y;
    (void) prepare_connector (interface, interface->to);
    (void) prepare_connector (interface, interface->from);
  }
}

/***************************************************************/

/***************************************************************/

static void detach (interface, line)
     Drawing interface;
     MonetConnectingLine line;
{
  if (line->system) {
    if (line->system->submap &&
	interface->from->system != interface->to->system) {
      DrawingList_extract (line->system->map_interfaces, interface);
    }
    line -> system = NULL;
  }
}

static MonetBool attach (interface, line, drawing, remove)
     Drawing interface, drawing;
     MonetConnectingLine line;
     MonetBool remove;
{
  if (line->system != drawing) {
    if (remove || ! drawing) {
      MonetMap map = Drawing_mapOf(interface);
      if (Drawing_begin (map)) {
	MONETDEBUG(3, ( "DRAW LINE from Attach"));
	MonetConnectingLine_draw (line, interface);
	Drawing_end (map);
      }
    }
    if (line->system) {
      if (line->system->submap &&
	  interface->from->system != interface->to->system) {
	DrawingList_extract (line->system->map_interfaces, interface);
      }
    }
    line -> system = drawing;
    if (drawing && drawing->submap && 
	interface->from->system != interface->to->system) {
      DrawingList_insert (drawing->map_interfaces, interface);
    }
    (void) prepare_connector (interface, line);
    return True;
  }
  return False;
}

/************************************************************************/
/*  Draw connection between interface and systems                       */
/*  !!! replace this with draw connecting line                          */
/************************************************************************/

extern void MonetConnectingLine_draw (line, interface)
     Drawing          interface;
     MonetConnectingLine line;
{
  Position        xl, yl, xr, yr;

  if (!line || line->system == (Drawing) NULL || ! line -> do_draw)
    return;
  MONETDEBUG(3, ( "DRAW LINE"));
  xl = line->xl;
  yl = line->yl;
  xr = line->xr;
  yr = line->yr;
  if (line->denominator > 0) {
    Draw_percentage_line(line->shade, line->numerator, line->denominator,
			 xl, yl, xr, yr);
  } else {
    Draw_line(line->shade, xl, yl, xr, yr);
  }
  if (line->draw_sys) {
    Draw_line(line->shade,
	      line->sys_xl, line->sys_yl, line->sys_xr, line->sys_yr);
  }
}

/***************************************************************/
/* Lines can be drawn to with linear or logarthmic scaling     */
/***************************************************************/

extern void MonetConnectingLine_set (line, num, den)
     MonetConnectingLine line;
     int num, den;
{
  if (den == 0 && num > 0) {
    /*  Logarthmic scaling  */
    /*  denominator = 32 (bits),  numerator = log2(num)  */
    line->numerator = MonetLog2 (num);
    line->denominator = 32;
  } else {
    /* linear scaling  */
    line->numerator = num;
    line->denominator = den;
  }
}

/***************************************************************/
/*  Prepare all the connections of a drawing for display       */
/*  If changed should return True, !!! in fact it always       */
/*  returns true.  And sets redraw connectors to true too.     */
/***************************************************************/

static MonetBool fix_submap_interface (interface, redraw)
     Drawing interface;
     MonetBool redraw;
{
  MonetEntity entity;
  MonetMap map = Drawing_mapOf(interface);
  MonetBool change = False;

  entity = ENTITY(interface);

  if (interface->to->system == NULL || interface->to->system->submap) {
    Drawing o;
    if (o = get_submap_drawing (to_interface (entity), map)) {
      if (o->submap) {
	if (attach (interface, interface->to, o, False)) {
	  change = True;
	  if (redraw) {
	    MONETDEBUG(3, ( "DRAW LINE from fix"));
	    MonetConnectingLine_draw (interface->to, interface);
	  }
	}
      }
    } else {
      detach (interface, interface->to);
    }
  }
  if (interface->from->system == NULL || interface->from->system->submap) {
    Drawing o;
    if (o = get_submap_drawing (from_interface (entity), map)) {
      if (o->submap) {
	if (attach (interface, interface->from, o, False)) {
	  change = True;
	  if (redraw) {
	    MONETDEBUG(3, ( "DRAW LINE from fix"));
	    MonetConnectingLine_draw (interface->from, interface);
	  }
	}
      }
    } else {
      detach (interface, interface->from);
    }
  }
  return change;
}

/***************************************************************/

/***************************************************************/

extern void Drawing_remove_connections (drawing, map)
     Drawing drawing;
     MonetMap map;
{
  MonetEntity entity = ENTITY(drawing);

  if (MonetEntity_isInterface(entity)) {
    detach (drawing, drawing->from);
    detach (drawing, drawing->to);
  } else if (drawing->submap) {
    Drawing interface;
    FOR_DRAWINGLIST(interface, drawing->map_interfaces) {
      if (interface->from->system == drawing) {
	detach (interface, interface->from);
      }
      if (interface->to->system == drawing) {
	detach (interface, interface->to);
      }
      interface->redraw_connectors = True;
      
      /* Try to see if there is another map that can be used  */
      (void) fix_submap_interface (interface, True);
    }    END_FORLIST;
  } else {
    MonetEntity inf;
    FOR_ENTITYLIST(inf, MonetEntity_interfaces (entity)) {
      Drawing          interface;
      if (interface = get_drawing (inf, map)) {
	if (from_interface(inf) == entity) {
	  detach (interface, interface->from);
	}
	if (to_interface (inf) == entity) {
	  detach (interface, interface->to);
	}
	interface->redraw_connectors = True;
	/*  See if we can connect to a map instead  */
	(void) fix_submap_interface (interface, True);
      } else {
	/* The interface, is not on the same map as 'entity'.
	   check other maps.  */
	Drawing d;
	FORALL (d, inf->drawing, next) {
	  MonetMap map_d;
	  if (d -> from -> system && d -> to -> system) {
	    continue;
	  }
	  map_d = Drawing_mapOf(d);
	  if (map_d != map) {
	    (void) fix_submap_interface (d, True);
	  }
	}
      } END_FORLIST;
    }
  }
}

/***************************************************************/
/*  Set up a systems connecting liness */
/***************************************************************/

static Drawing get_draw (entity, map)
     MonetEntity entity;
     MonetMap map;
{
  Drawing d;
  if (d = get_drawing (entity, map))
    return d;
  return get_submap_drawing (entity, map);
}

extern MonetBool Prepare_connecting_lines (drawing)
     Drawing drawing;
{
  MonetBool changed;
  MonetEntity entity;
  MonetMap map = Drawing_mapOf(drawing);

  entity = ENTITY(drawing);
  changed = drawing->redraw_connectors;

  if (MonetEntity_isInterface(entity)) {
    Drawing interface = drawing, o;

    o = get_draw (to_interface (entity), map);
    (void) attach (interface, interface->to, o, False);
    (void) prepare_connector (interface, interface->to);

    o = get_draw (from_interface (entity), map);
    (void) attach (interface, interface->from, o, False);
    (void) prepare_connector (interface, interface->from);

    changed = True;
  } else if (drawing->submap) {
    Drawing interface;
    FOR_DRAWINGLIST(interface, Map_elements (map)) {
      if (Drawing_isInterface (interface)) {
	if (! fix_submap_interface (interface, False)) {
	  if (interface->from->system == drawing) {
	    changed |= prepare_connector (interface, interface->from);
	  }
	  if (interface->to->system == drawing) {
	    changed |= prepare_connector (interface, interface->to);
	  }
	}
      }
    } END_FORLIST;
    /*
      FOR_DRAWINGLIST(interface, drawing->map_interfaces) {
      if (interface->from->system == drawing) {
      changed |= prepare_connector (interface, interface->from);
      } else {
      changed |= prepare_connector (interface, interface->to);
      }
      } END_FORLIST;
      */
  } else {
    MonetEntity inf;
    FOR_ENTITYLIST(inf, MonetEntity_interfaces (entity)) {
      Drawing          interface;
	
      if (interface = get_drawing (inf, map)) {
	if (from_interface(inf) == entity) {
	  changed |= attach (interface, interface->from, drawing, True);
	}
	if (to_interface(inf) == entity) {
	  changed |= attach (interface, interface->to, drawing, True);
	}
      } else {
	/* The interface, inf, is not on the same map as 'entity'.
	   check other maps.  */
	Drawing d;
	FORALL (d, inf->drawing, next) {
	  if (Drawing_mapOf(d) != map) {
	    (void) fix_submap_interface (d, False);
	  }
	}
      }
    } END_FORLIST;
  }
  drawing->redraw_connectors = changed;
  return changed;
}

/*****************************************************************/
/*  Reposition an interface and draw it  */
/*****************************************************************/

static void	setup_and_draw_interface (drawing, interface, undraw)
     Drawing drawing, interface;
     MonetBool undraw;
{
  MonetMap map;
  Position x,y;

  if (interface_not_drawable (interface))
    return;

  map = Drawing_mapOf(drawing);

/*  MONETDEBUG(3, ( "DRAW LINE from setup")); */


  if (! undraw && get_newInterfacePosition (map, interface, &x, &y)) {
    /* -- Remove previous line  */
    if (interface->from->system == drawing) {
      MonetConnectingLine_draw (interface->to, interface);
    }
    if (interface->to->system == drawing) {
      MonetConnectingLine_draw (interface->from, interface);
    }
    Draw_system(interface);
    X(interface) = x;
    Y(interface) = y;
    Draw_system(interface);
    (void) prepare_connector (interface, interface->from);
    (void) prepare_connector (interface, interface->to);
    MonetConnectingLine_draw (interface->from, interface);
    MonetConnectingLine_draw (interface->to, interface);
  } else if (interface->from->system == drawing) {
    (void) prepare_connector (interface, interface->from);
    MonetConnectingLine_draw (interface->from, interface);
  } else {
    (void) prepare_connector (interface, interface->to);
    MonetConnectingLine_draw (interface->to, interface);
  }
}

/************************************************************************/
/*  Actually draw the connecting lines  */
/************************************************************************/

extern void Draw_connecting_lines (drawing, undraw)
     Drawing          drawing;
     MonetBool undraw;
{
  MonetMap map;

  map = Drawing_mapOf(drawing);

  if (drawing->redraw_connectors) {
    drawing->redraw_connectors = False;
    if (Drawing_isInterface(drawing)) {
      Position x,y;

      if (interface_not_drawable (drawing))
	return;

      if (!undraw && get_newInterfacePosition  (map, drawing, &x, &y)) {
	X(drawing) = x;
	Y(drawing) = y;
      }

      Prepare_connecting_lines (drawing);
      MONETDEBUG(3, ( "DRAW LINE from draw"));

      MonetConnectingLine_draw (drawing->to, drawing);
      MonetConnectingLine_draw (drawing->from, drawing);
    } else if (drawing->map_interfaces) {
      Drawing interface;
      FOR_DRAWINGLIST(interface, drawing->map_interfaces) {
	setup_and_draw_interface (drawing, interface, undraw);
      } END_FORLIST;
    } else {
      MonetEntity e;
      FOR_ENTITYLIST(e, MonetEntity_interfaces (ENTITY(drawing))) {
	Drawing          interface;
	if (interface = get_drawing (e, Drawing_mapOf(drawing))) {
	  setup_and_draw_interface (drawing, interface, undraw);
	}
      }  END_FORLIST;
    }
  }
}

/************************************************************************/
/*  Create line  */
/************************************************************************/

extern MonetConnectingLine MonetConnectingLine_create ()
{
  MonetConnectingLine line;
  line = MonetNew (struct monet_connecting_line);
  line->shade = shade_dark;
  line->numerator = 0;
  line->denominator = 0;
  line->system = NULL;
  line->xr = 0;
  line->xl = 0;
  line->yr = 0;
  line->yl = 0;
  line -> do_draw = True;
  return line;
}

/************************************************************************/
/*  Disable drawing of an interfaces lines */
/************************************************************************/

extern void MonetConnectingLine_enable (interface, enable)
     Drawing interface;
     MonetBool enable;
{
  if (interface->from) {
    interface->from->do_draw = enable;
  }
  if (interface->to) {
    interface->to->do_draw = enable;
  }
}

/************************************************************************/
/*  Free line  */
/************************************************************************/

extern void MonetConnectingLine_free (interface)
     Drawing interface;
{
  detach (interface, interface->from);
  detach (interface, interface->to);
  MonetFree (interface->from);
  MonetFree (interface->to);
}

/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/drawing.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/******************************************************************/
/******************************************************************/

