/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/map.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/* $Log: map.c,v $
 * Revision 1.1  1991/06/11  11:26:10  a38
 * Initial revision
 *  */
/*****************************************************************************/
/*  For maps.  This part tries to be totally independent of X windows, it    */
/*  defines only the purely structural aspects.   Each map may have an       */
/*  X widget, this is mentioned here but only as an opaque type so porting   */
/*  should in principle be a huge problem, all the same ...  */
/*  See bin/xmonet/map.c for X stuff  */
/*****************************************************************************/

#include "mapP.h"
#include <monet/map/Interface.h>
#include <monet/msh/Script.h>

/*****************************************************************************/
/* Globals */
/*****************************************************************************/

static MonetMap AllMaps = NULL;
static MapCurrentCallback CurrentCallback = NULL;
static MapFreeCallback FreeCallback;

/******************************************************************
  Get name of map
  ******************************************************************/

extern char * Map_nice_name (map)
     MonetMap map;
{
  char * result;
  if (map) {
    if (Map_is_loaded(map))
      result = Map_title (map);
    else if (Map_classesOnly(map))
      result = "Class Map";
    else
      result = "Unloaded Map";
  } else {
    result = "None";
  }
  return result;
}

/*****************************************************************
  Temporarily Switch off drawing
  *****************************************************************/

extern void	Map_disable_set_changed		(map)
     MonetMap map;
{
  map->can_change = False;
  map->drawing_allowed = False;
}

extern void	Map_enable_set_changed		(map)
     MonetMap map;
{
  map->can_change = True;
  map->drawing_allowed = True;
}

/******************************************************************/
/*  The boundaries of the map have changed  */
/******************************************************************/

extern void Map_set_resized (map)
     MonetMap map;
{
  if (map->can_change) {
    map -> saved = False;
    map -> scale_by = 0.0;
    map->xl = 0;
    map->xr = 0;
    map->yl = 0;
    map->yr = 0;
    Map_request_redraw (map);
  }
}

/******************************************************************/
/*  The map will need to be saved  */
/******************************************************************/

extern void Map_set_changed (map)
	MonetMap map;
{
  if (map->can_change) {
    map -> saved = False;
  }
}

/*****************************************************************************/
/*  Class maps changed  */
/*****************************************************************************/
/*
extern void MonetMap_changed()
{
  MonetMap         m;
  FORALLMAPS (m)
    if (Map_classesOnly(m))
      Map_request_redraw (m);
}
*/

/*****************************************************************************/
/*  Return a string giving scaled lines  */
/*****************************************************************************/

extern char * MonetMap_scale_lines (map)
     MonetMap map;
{
  if (map->scale_from_line && map->scale_to_line) {
    static char buf [512]; /* !!! */
    (void) sprintf (buf, "%s %u %s %u",
		    MonetAttribute2String (map->scale_from_line),
		    map->scale_from_by,
		    MonetAttribute2String (map->scale_to_line),
		    map->scale_to_by);
    return buf;
  }
  return NULL;
}

/*****************************************************************************/
/*  Given name or number find map  */
/*****************************************************************************/

extern MonetMap MonetString2Map (s)
     char * s;
{
  MonetMap          m;
  int num;

  num = MonetString2int (s);
  if (num > 0) {
    unsigned n = 0;
    FORALLMAPS (m) {
      ++n;
      if (n == num) {
	return m;
      }
    }
  } else {
    FORALLMAPS (m) {
      if (Map_is_loaded(m) && EQS(Map_title (m), s)) {
	return m;
      }
    }
  }
  return NULL;
}

/*****************************************************************
  Check if a map can be freed and if so do so  
  *****************************************************************/

static MonetMap check_map_free (map)
     MonetMap map;
{
  if (map->entity == NULL && map->mapWidget == NULL) {
    Map_free (map);
    return NULL;
  }
  return map;
}

/*****************************************************************
  Set current map  
  We want to be able to change the top map. If swap is set then 
  use the widget of the previous top map, and give the previous 
  map the new maps widget.  
  When loading, change the widgets.  When loading into a popup  
  widget ...
  *****************************************************************/

extern MonetMap MapCurrent_get ()
{
  return AllMaps;
}

extern void MapCurrent_set (map, swap)
     MonetMap map;
     MonetBool swap;
{
  if (map) {
    MonetMap old;
    old = AllMaps;
    if (map != old) {
      LIST_DEL(MonetMap, map, AllMaps, next);
      LIST_ADD(map, AllMaps, next);
      MONETDEBUG(2, ( "Current Map changed to [%s], from [%s], swap windows %d",
		  Map_title (old),
		  Map_title (AllMaps),
		  swap));
    }
    if (CurrentCallback) {
      CurrentCallback (old, map, swap);
    }
    if (map != old) {
      (void) check_map_free (old);
    }
  }
}

extern void  MapCurrent_swap ()
{
  if (AllMaps && AllMaps -> next) {
    MapCurrent_set (AllMaps->next, True);
  }
}

extern void  MapCurrent_set_next ()
{
  MonetMap m;
  if (AllMaps->mapWidget) {
    FORALLMAPS(m) {
      if (m->mapWidget == NULL) {
	MapCurrent_set (m, False);
	break;
      }
    }
  }
}

/******************************************************************/
/*  Set position for grid */
/******************************************************************/

extern void Map_set_grid (map, x,y)
	MonetMap map;
	unsigned x, y;
{
	map->grid_x = x;
	map->grid_y = y;
}

/******************************************************************/
/*  Adjust position for grid */
/******************************************************************/

extern void Map_adjust_for_grid (map, x,y, newx,newy)
	MonetMap map;
	Position x,y, *newx, *newy;
{
	*newx = (x >> map->grid_x) << map->grid_x;
	*newy = (y >> map->grid_y) << map->grid_y;
}

/*****************************************************************************/
/*  Return the map widget  */
/*****************************************************************************/

extern Widget Map_widget (map)
     MonetMap map;
{
  return map->mapWidget;
}

/*****************************************************************************/
/*  Return the root map  */
/*****************************************************************************/

extern MonetMap Map_root (map)
     MonetMap map;
{
  return map;
}

/*****************************************************************************/
/*  Background file with graphics commands  */
/*****************************************************************************/

extern void Map_set_background_shape	(map, shape)
     MonetMap map;
     MonetShape shape;
{
  map->background_shape = shape;
}

extern MonetShape Map_get_background_shape (map)
	MonetMap map;
{
  return map->background_shape;
}

extern void Map_set_background_colour	(map, colourname)
	MonetMap map;
	String colourname;
{
	if (map->background_colour)
		MonetFree (map->background_colour);
	map->background_colour = MonetNewString (colourname);
}

extern String Map_get_background_colour		(map)
	MonetMap map;
{
	return map->background_colour;
}

extern String Map_get_domain (map)
	MonetMap map;
{
	return map -> default_domain;
}

/*****************************************************************************/
/*  Traverse list of maps  */
/*****************************************************************************/

extern MonetMap Map_first ()
{
	MonetMap map;
	map = AllMaps;
	return map;
}

extern MonetMap Map_next (current)
	MonetMap current;
{
	/*  assume current != NULL  */
	return current->next;
}

/*****************************************************************************/
/*  Return False if there are maps still to be saved.                        */
/*****************************************************************************/

extern MonetBool AreAllMapsSaved()
{
  MonetMap          m;
  FORALLMAPS (m)
    if ( ! Map_classesOnly (m) && m->saved == False)
      return False;
  return True;
}

/*****************************************************************************/
/*  Draw a new drawing on a map that has just been placed there              */
/*  ie for map areas drawn as lists, redraw the entire map                   */
/*****************************************************************************/

static void Drawing_draw_new (s)
     Drawing s;
{
  MonetMap map = Drawing_mapOf(s);
  
  if (Map_classesOnly(map)) {
    Map_request_redraw (map);
  } else {
    switch (map->drawnas) {
    case drawnas_list:
      Map_request_redraw (map);
      break;
    case drawnas_map:
      Drawing_redraw(s);
      break;
    }
  }
}

/*****************************************************************************/
/*  UnDraw an old drawing on a map that has just been removed from there     */
/*****************************************************************************/

static void Drawing_draw_old (s)
     Drawing s;
{
  MonetMap map = Drawing_mapOf(s);
  if (Map_classesOnly(map)) {
    Map_request_redraw (map);
  } else {
    switch (map->drawnas) {
    case drawnas_list:
      Map_request_redraw (map);
      break;
    case drawnas_map:
      Drawing_undraw(s);
      break;
    }
  }
}

/*****************************************************************************/
/* Tests wether or not a particular class of drawing can be placed in a      */
/* particular map. */
/*****************************************************************************/

extern MonetBool Map_can_insert_drawing (map, entity_class)
     MonetEntity entity_class;
     MonetMap map;
{
  if (map->entity == NULL) {
    return False;
  }
  return True;
}

/*****************************************************************************/
/* This is the move system instruction, to be called whenever the position of
 * a system is changed */
/*****************************************************************************/

extern MonetBool Map_insert_drawing (map, drawing, shape, label, x, y, w, h)
     Drawing drawing;
     MonetMap map;
     MonetShape shape;
     char * label;
     Position x,y;
     Dimension w,h;
{
  /**************************************************************************/
  /*  Check parameters  */
  /**************************************************************************/

  if (drawing == NULL) {
    MonetSysError ("Can't insert NULL drawing into a map");
    return False;
  }
  if (map == NULL) {
    MonetSysError ("NULL map");
    return False;
  }
  if (Drawing_classOf(map) == NULL) {
    MonetSysError ("Cannot have a map with a NULL class");
    return False;
  }
  
  /* This is a little silly, ought to have a special routine for classes !!! */
  if (Map_classesOnly(map)) {
    if (! MonetEntity_isClass (ENTITY(drawing))) {
      MonetSysError ("Cannot put an instance [%s] into class map",
		 Drawing_title (drawing));
      return False;
    }
  } else if (Drawing_isClass (drawing)) {
    MonetSysError ("Cannot put a class [%s] into a map other than class_map",
	       Drawing_title (drawing));
    return False;
  }
  
  if (Map_can_insert_drawing (map, Drawing_classOf (drawing)) == False) {
    String map_title = MonetNewString (Map_title (map));
    MonetPrintfError ("Cannot insert [%s] in map [%s]",
		      Drawing_title (drawing), map_title);
    MonetFree (map_title);
    return False;
  }

  /********************************************************************/
  /*  Set label and shape  */
  /********************************************************************/

  if (label)
    Drawing_set_label (drawing, label);
  if (shape)
    MonetEntity_set_shape (ENTITY(drawing), shape);

  /********************************************************************/
  /* Deal with map part first */
  /********************************************************************/

  if (Drawing_mapOf(drawing) == map) {
    Drawing_undraw(drawing);
  } else {
    if (Drawing_mapOf(drawing)) {
      Map_extract_drawing (Drawing_mapOf(drawing), drawing);
    }
    Drawing_mapOf(drawing) = map;
    DrawingList_insert(Map_elements (map), drawing);
  }
  
  /********************************************************************/
  /*  Now work out where the drawing goes on the map,  x,y,w,h        */
  /********************************************************************/

  if (x<=0 || y<= 0) {
    if (Drawing_isInterface (drawing) &&
	get_newInterfacePosition  (map, drawing, &x, &y)) {
    } else {
      if (x<=0)
	x = map->unplaced_x;
      if (y<=0)
	y = map->unplaced_y += HEIGHT(drawing) + 10;
    }
  }
  /*  This also says the map has been changed  */
  Drawing_dimension (drawing, x,y,w,h);

  /**************************************************************************/
  /*  Now reflect the changes on the map  */
  /**************************************************************************/

  Drawing_prepare(drawing);
  Drawing_draw_new (drawing);

  /***************************************************************************/
  /* !!!	(void) position_interfaces (drawing);  Should work */
  /*	straighten_acute_interfaces (drawing);   !!! */
  /***************************************************************************/

  return True;
}

/*****************************************************************************/
/*  Extract a drawing from a map */
/*****************************************************************************/

extern void Map_extract_drawing (map, drawing)
     MonetMap map;
     Drawing          drawing;
{
  if (drawing == NULL)
    return;

  if (map == Drawing_mapOf(drawing)) {
    Map_adjust_corners (map, drawing, False);
    Drawing_draw_old (drawing);
    DrawingList_extract(Map_elements (map), drawing);
    Map_set_changed (map);
    Drawing_mapOf(drawing) = NULL;
    Drawing_remove_connections (drawing, map);
  } else {
    MonetSysError ("map does not correspond to argument");
  }
}

/*****************************************************************************/
/*  Free/Extract all drawings of a map */
/*****************************************************************************/

extern void Map_extract_all_drawings (map)
     MonetMap map;
{
  Drawing d;
  MonetBool change;

  if (change = isDrawingAllowed (map))
    Map_disable_set_changed (map);
  while (d=DrawingList_first (map->elements)) {
    Drawing_free (d);
  }
  if (change)
    Map_enable_set_changed (map);
  Map_set_changed (map);
  Map_request_redraw (map);
}

/***************************************************************/
/*  Title of a map  */
/***************************************************************/

extern String Map_title (map)
     MonetMap map;
{
  if (map && map->entity)
    return MonetEntity_title (map->entity);
  return NULL;
}

static void map_discard (map)
     MonetMap map;
{
  Map_extract_all_drawings (map);
  map->entity = NULL;
  map->saved = True;
  MapCurrent_set_next ();
}

/* !!! Should free the entity if no one else is referencing it */
/* !!! Should put a reference counter into entity structure */

extern MonetMap Map_discard (map)
     MonetMap map;
{
  MonetBool change;

  if (! map || ! check_map_free (map)) {
    return NULL;
  } else {
    MonetPrintfInform ("Discarding MonetMap");
    if (change = isDrawingAllowed (map)) {
      Map_disable_set_changed (map);
    }

    map_discard (map);
    if (change) {
      Map_enable_set_changed (map);
    }
    Map_set_changed (map);
    Map_request_redraw (map);
  }
  return map;
}

extern void Map_free (map)
     MonetMap map;
{
  MonetBackup_free (map->backup);
  map_discard (map);
  if (FreeCallback)
    FreeCallback (map);
  LIST_DEL(MonetMap, map, AllMaps, next);
  /* DrawingList_reset (map->elements);  --  should already be empty  */
  DrawingList_free (map->elements);
  DrawingList_free (map->selected);
  MonetReportFormat_free (map->report_format);
  MonetFree (map->background_shape);
  MonetFree (map->background_colour);
  MonetFree (map);
  MapCurrent_set (AllMaps, False);
}

static MonetMap basic_create ()
{
  MonetMap map;

  if (map = MonetNew(struct monet_map)) {
    LIST_ADD(map, AllMaps, next);
    /* -- map now == AllMaps, AllMaps-> next is previous current map */
    map -> backup = NULL;
    map -> primary_label = NULL;
    map -> secondary_label = NULL;
    map -> scale_from_line = NULL;
    map -> scale_to_by = NULL;
    map -> scale_from_by = NULL;
    map -> scale_to_line = NULL;
    map -> marked = False;
    map -> drawnas = drawnas_map;
    map -> entity = NULL;
    map -> unplaced_x = 20;
    map -> unplaced_y = 20;
    map -> xl = 0;
    map -> xr = 0;
    map -> yl = 0;
    map -> yr = 0;
    map -> class_map = False;
    map -> elements = DrawingList_create ();
    map -> selected = DrawingList_create ();
    map -> saved = True;  /* !!! Hack, because reset in position_map */
    map -> drawing_allowed = True;
    map -> can_change = True;
    map -> grid_x = 3;
    map -> grid_y = 3;
    map -> default_domain = NULL;
    map -> report_format = NULL;
    map -> default_domain_len = 0;
    map -> background_shape = NULL;
    map -> shape_width = 1000;
    map -> shape_height = 1000;
    map -> background_colour = NULL;
    map -> connectors = ConnectorState_default ();
    /*  !!!  These fields are available to systems using X  */
    /*  !!!  Not referenced in this module.  */
    map -> mapWidget = NULL;
    map -> clip_region = NULL;
    map -> x_centre = 0;
    map -> y_centre = 0;
    map -> x_scale = 1.0;
    map -> y_scale = 1.0;
    map -> scale_by = 1.0;
    map -> draw_shape = True;
    map -> net_border = MonetNetBorderDefault;
    return map;
  } else {
    MonetSysError ("Couldn't malloc space for new map area");
  }
  return False;
}

extern MonetMap Map_create_class ()
{
  MonetMap map;
  if (map = basic_create ()) {
    map -> class_map = True;
    if (CurrentCallback) {
      CurrentCallback (map->next, map, True);
    }
  }
  return map;
}

extern MonetMap Map_create (entity)
     MonetEntity entity;
{
  MonetMap map;
  if(map = basic_create ()) {
    map->entity = entity;
    if (CurrentCallback) {
      /* Must not try to set widget with somethign else, since
	 there might already be a widget ready to be set.  */
      CurrentCallback (map->next, map, /* swap */ False);
    }
  }
  return map;
}

/***********************************************************/
/*  Initialise / finish off */
/***********************************************************/

extern void MonetMap_init (change, begin_draw, end_draw, current_map, free_map)
     MapChangedCallback change;
     DrawingBeginCallback begin_draw;
     DrawingEndCallback end_draw;
     MapCurrentCallback current_map;
     MapFreeCallback free_map;
{
  MonetMap_resize_init (change, begin_draw, end_draw);
  /*  Initialise the entity database as a client */
  MonetDb_init (False, Drawing_change_trigger);

  MonetLibrary_begin (monetlib, "map", "Network Map");

  FreeCallback = free_map;
  CurrentCallback = current_map;
  AllMaps = NULL;
  map_server_init ();
  MapScript_init ();
  MonetDrawing_init ();
  init_drawMaps ();
  MapDrawClass_init ();
  MapDrawList_init ();
  monet_map_change_init ();
  monet_map_file_init ();
  MonetMap_interfaces_init ();
  MapScript_file_init ();
  MapScript_components_init ();
  MapScript_edit_init ();
  MapScript_map_init ();
  MapScript_drawing_init ();
  MapScript_select_init ();
  MonetMap_undo_init ();

  MonetLibrary_done ();
}

extern void MonetMap_finish ()
{
  MonetMap m;

  MonetMap_undo_finish ();
  MapScript_select_finish ();
  MapScript_drawing_finish ();
  MapScript_map_finish ();
  MapScript_edit_finish ();
  MapScript_components_finish ();
  MapScript_file_finish ();
  MonetMap_interfaces_finish ();
  monet_map_file_finish ();
  monet_map_change_finish ();
  MapDrawList_finish ();
  MapDrawClass_finish ();
  finish_drawMaps();
  MonetDrawing_finish ();
  MapScript_finish ();
  FORALLMAPS (m)
    Map_free (m);
  map_server_finish ();
  MonetDb_finish ();
  MonetMap_resize_finish ();
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/map.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/*****************************************************************************/
/*****************************************************************************/

