/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/map/Drawing.h>
#include <monet/msh/Channel.h>
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>
#include "mapP.h"

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

static MonetBool at_least_one (map)
     MonetMap map;
{
  if (numberOf_Selection (map) == 0) {
    MonetPrintfError ("You must select at least one system");
    return False;
  }
  return True;
}

static MonetEntity exactly_one_entity (map)
     MonetMap map;
{
  Drawing current;

  if (numberOf_Selection (map) != 1) {
    MonetPrintfError ("You must select exactly one system");
    return NULL;
  }
  current = current_Selection (map);
  free_selection (map);
  return ENTITY (current);
}

/*****************************************************************************/
/*  Select or report componented of an object */
/*****************************************************************************/

static void select_components (map, super, report)
     MonetMap map;
     MonetBool super, report;
{
  MonetEntity sys, s;
  MonetEntityList list;

  if (sys = exactly_one_entity (map)) {
    if (super) {
      MonetPrintfInform ("Selecting Supercomponents of [%s]",
			 MonetEntity_title (sys));
      list = MonetEntity_supercomponents (sys);
    } else {
      MonetPrintfInform ("Selecting Subcomponents of [%s]",
			 MonetEntity_title (sys));
      list = MonetEntity_subcomponents (sys);
    }
    FOR_ENTITYLIST (s, list) {
      if (report)
	MonetPrintfnl ("  %s", MonetEntity_title (s));
      else
	MonetEntity_select_drawings (s);
    } END_FORLIST;
  }
}

/*********************************************************************/
/* Super and Sub Components */
/*********************************************************************/

MONETCALLBACK(do_MakeSubcomponent)
     MonetEntity sub, super;
     char super_title [2048];  /* !!! */
     Drawing drawing_super;

     ISLOADED;
     if (!at_least_one (CurrentMap)) {
       RETURNSC;
     }
     drawing_super = Drawing_get_current ();
     if (drawing_super == NULL)  {
       MonetPrintfError ("No supercomponent selected");
       RETURNSC;
     }
     super = ENTITY(drawing_super);
     strcpy (super_title, MonetEntity_title(super));

     FOR_SELECTED_ENTITY (CurrentMap, sub) {
       if (isSubcomponent (super, sub)) {
	 MonetPrintfError ("[%s] is already a subcomponent of [%s]",
			   MonetEntity_title(sub), super_title);
       } else {
	 MonetPrintfInform ("Making [%s] a subcomponent of [%s]",
			    MonetEntity_title(sub), super_title);
	 make_subcomponent (super, sub);
       }
     } END_SELECTED_ENTITY
       free_selection (CurrentMap);
MONETEND

MONETCALLBACK(do_UnMakeSubcomponent)
     MonetEntity super, sub;
     char super_title[2048];  /* !!! */
     Drawing drawing_super;
     
     ISLOADED;
     if (!at_least_one (CurrentMap)) {
       RETURNSC;
     }

     drawing_super = Drawing_get_current ();
	if (drawing_super == NULL)  {
		MonetPrintfError ("No supercomponent selected");
		RETURNSC;
	}
	super = ENTITY(drawing_super);

	strcpy (super_title, MonetEntity_title(super));
	FOR_SELECTED_ENTITY (CurrentMap, sub)
		if (isSubcomponent (super, sub)) {
			unmake_subcomponent (super, sub);
			MonetPrintfInform ("[%s] is no longer a subcomponent of [%s]",
				MonetEntity_title(sub), super_title);
		} else {
			MonetPrintfError ("[%s] is not a subcomponent of [%s]",
				MonetEntity_title(sub), super_title);
		}
	END_SELECTED_ENTITY	
	free_selection (CurrentMap);
MONETEND

MONETCALLBACK(do_SelectSubcomponents)
     ISLOADED;
     select_components (CurrentMap, False, False);
MONETEND

MONETCALLBACK(do_SelectSupercomponents)
     ISLOADED;
     select_components (CurrentMap, True, False);
MONETEND

MONETCALLBACK(do_ReportSupercomponents)
     ISLOADED;
     select_components (CurrentMap, True, True);
MONETEND

MONETCALLBACK(do_ReportSubcomponents)
     ISLOADED;
     select_components (CurrentMap, False, True);
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_components_init ()
{
  (void) MonetMember_module_class ("component");
  MonetMember_create ("select-sub",		do_SelectSubcomponents,
		      "",
		      "Select subcomponents of selected list");
  MonetMember_create ("report-sub", 		do_ReportSubcomponents,
		      "",
		      "Report on subcomponents of selected list");
  MonetMember_create ("select-super",	 do_SelectSupercomponents,
		      "",
		      "Select supercomponents of selected list");
  MonetMember_create ("report-super",	 do_ReportSupercomponents,
		      "",
		      "Report on supercomponents of selected list");
  MonetMember_create ("make-sub",		do_MakeSubcomponent,
		      "",
	      "Make selected entities subcomponents of current drawing");
  MonetMember_create ("unmake-sub",		do_UnMakeSubcomponent,
		      "",
		      "Removes selected entities from subcomponent list, \
		      Must first select a super component");
}

extern void MapScript_components_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
