/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "mapP.h"
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

/*****************************************************************************/
/*  Select routines */
/*****************************************************************************/

MONETCALLBACK(do_for_in_map)
     Map_parse_select (argc -1, argv+1, (DoDrawingCallback) 0);
MONETEND

/***************************************************************************/
/*  Straighten selected Drawings  */
/***************************************************************************/

static void straighten_do_command (draw)
     Drawing draw;
{
  if (position_interfaces (draw) == False) {
    MonetPrintfError ("Position interfaces failed [%s]",
		      Drawing_title (draw));
  }
}

MONETCALLBACK(do_straighten)
     Map_parse_select (argc-1, argv+1, straighten_do_command);
MONETEND

/*****************************************************************************/
/* Set label */
/*****************************************************************************/

static void abbrev_do_command (d)
	Drawing d;
{
  Drawing_set_label (d, MonetShortLabel (TITLE (ENTITY(d))));
  MONETDEBUG(1, ( "Set label [%s]", d->label));
}

MONETCALLBACK(do_abbrev)
     Map_parse_select_redraw (argc-1, argv+1, NULTEST, abbrev_do_command);
MONETEND

MONETCALLBACK(do_label)
     MonetEntity e;
     ISLOADED;
     if (e = String2MonetEntity (argv[1])) {
       Drawing d;
       if (d = get_drawing (e, MapCurrent_get ())) {
	 Drawing_unmoved_undraw (d);
	 Drawing_set_label (d, argv[2]);
	 Drawing_prepare (d);
	 Drawing_unmoved_redraw (d);
       } else {
	 MonetPrintfError ("[%s] is not on this map", argv[1]);
       }
     } else {
       MonetPrintfError ("No such entity [%s]", argv[1]);
     }
MONETEND

/************************************************************/
/*  Toggle labels  */
/************************************************************/

MONETCALLBACK(do_toggle_primary_label)
     Map_parse_select_redraw (argc-1, argv+1, NULTEST, Drawing_toggle_label);
MONETEND

/************************************************************/
/*  Toggle labels  */
/************************************************************/

MONETCALLBACK(do_reset_primary_label)
     Map_parse_select_redraw (argc-1, argv+1, NULTEST, Drawing_reset_labels);
MONETEND

/************************************************************/
/*  Reset size  */
/************************************************************/

static void reset_size_do_command (d)
	Drawing d;
{
  WIDTH(d) = 0;
  HEIGHT(d) = 0;
}

MONETCALLBACK(do_reset_size)
     Map_parse_select_moved_redraw
       (argc-1, argv+1, NULTEST, reset_size_do_command);
MONETEND

/************************************************************/
/*  Get Isolated  */
/************************************************************/

MONETCALLBACK(do_isolated)
     Map_parse_select_redraw (argc-1, argv+1, is_isolated, select_no_draw);
MONETEND

/************************************************************/
/*  Explore  */
/************************************************************/

static void explore_do_command (d)
	Drawing d;
{
  MonetPrintfInform ("Exploring from [%s]", Drawing_title (d));
  (void) explore_connectedSystems (d);
}
MONETCALLBACK(do_explore_ip_host)
     Map_parse_select (argc-1, argv+1, explore_do_command);
MONETEND

/***************************************************************************/
/*  List names of specified Entities  */
/***************************************************************************/

static void list_do_command (d)
	Drawing d;
{
	MonetPrintfnl ("%s", Drawing_title (d));
}

MONETCALLBACK(do_list)
     Map_parse_select (argc-1, argv+1, list_do_command);
MONETEND

/***************************************************************************/
/*  Count specified Entities  */
/***************************************************************************/

static unsigned Count;
static void count_do_command (d)
	Drawing d;
{
	++Count;
}

MONETCALLBACK(do_count)
	static char count[16];
	Count = 0;
	Map_parse_select (argc-1, argv+1, count_do_command);
	(void) sprintf (count, "%u", Count);
	result = count;
MONETEND

/***************************************************************************/
/*  Full Information on specified Entities  */
/***************************************************************************/

static void info_do_command (o)
     Drawing o;
{
  MonetPrintfnl (MonetEntity2Report(ENTITY(o), NULL, True, False, False));
}

MONETCALLBACK(do_info)
	Map_parse_select (argc-1, argv+1, info_do_command);
MONETEND

/***************************************************************************/
/* list geometry of drawings in map  */
/***************************************************************************/

static void list_geometry_do_command (d)
     Drawing d;
{
  MonetPrintfnl ("%s\t%s\t%d %d %u %u",
		 Drawing_title (d), LABEL(d), 
X(d), Y(d), WIDTH(d), HEIGHT(d));
}

MONETCALLBACK(do_list_geometry)
	Map_parse_select (argc-1, argv+1, list_geometry_do_command);
MONETEND

/***************************************************************************/
/*  One line report on each specified Entities  */
/***************************************************************************/

static MonetReportFormat Format;
static void report_do_command (o)
	Drawing o;
{
  MonetPrintfnl ("%s", MonetReportFormat_pretty (Format, ENTITY(o)));
}

MONETCALLBACK(do_report)
     ISLOADEDORCLASS;
     Format = CurrentMap->report_format;
     MonetPrintfnl ("%s", MonetReportFormat_header (Format));
     Map_parse_select (argc-1, argv+1, report_do_command);
MONETEND

/***************************************************************************/
/*  Add to selection list all specified Entities  */
/***************************************************************************/

MONETCALLBACK(do_select)
     ISLOADED;
     Map_parse_select_redraw (argc-1, argv+1, NULTEST, select_no_draw);
MONETEND

/***************************************************************************/
/*  Current drawing  */
/***************************************************************************/

MONETCALLBACK(do_current_drawing)
     Drawing d;
     if (d = Drawing_get_current ()) {
       result = Drawing_title (d);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_help)
     Drawing_dump ();
MONETEND

/***************************************************************************/
/*  Visible  */
/***************************************************************************/

static MonetBool visible_test (d)
     Drawing d;
{
  return (d -> submap && d -> submap_visible);
}

static void visible_do_command (d)
     Drawing d;
{
  d -> submap_visible = True;
}

MONETCALLBACK(do_submap_visible)
     Map_parse_select_redraw (argc-1, argv+1,
			      visible_test, visible_do_command);
MONETEND

static MonetBool invisible_test (d)
     Drawing d;
{
  return (d -> submap && !(d -> submap_visible));
}

static void invisible_do_command (d)
     Drawing d;
{
  d -> submap_visible = False;
}

MONETCALLBACK(do_submap_invisible)
     Map_parse_select_redraw (argc-1, argv+1,
			      invisible_test, invisible_do_command);
MONETEND

/**************************************************************************
  Get the neighbouring system given an interface
  **************************************************************************/

static Drawing neighbour (the_map, entity, interface)
     MonetMap the_map;
     MonetEntity entity;
     MonetEntity interface;
{
  MonetEntity  other, from, to;

  from = from_interface (interface);
  to   = to_interface (interface);
  if (from == entity) {
    other = to;
  } else if (to == entity) {
    other = from;
  } else {
    return NULL;
  }
  return get_drawing (other, the_map);
}

/**************************************************************************
  Position hangers on of selected Drawings
  **************************************************************************/

static void position_neighbours (draw)
     Drawing draw;
{
  MonetEntity interface, entity = ENTITY(draw);
  MonetMap map;
  Position x, y_above, y_below, extend_below = 0, extend_above = 0;
  unsigned sep = 10, ext = 5;

  if (MonetEntity_isNode (entity)) {
    x = X(draw);
    y_below = Y(draw) + HEIGHT (draw) + sep;
    y_above = Y(draw) - sep;

    map = Drawing_mapOf(draw);
    /*  Run through all the entities interfaces  */
    FOR_ENTITYLIST (interface, entity->interfaces) {
      Drawing other;
      if (other = neighbour (map, entity, interface)) {
	/*  Found a neighbouring system */
	/*  The neighbouring system is drawn on this map  */
	MonetEntity ointerface;
	Drawing f1 = NULL, f2 = NULL;
	int neighbours = 0;

	FOR_ENTITYLIST (ointerface, ENTITY(other)->interfaces) {
	  Drawing f;
	  if (f = neighbour (map, ENTITY(other), ointerface)) {
	    ++ neighbours;
	    if (! f1) {
	      f1 = f;
	    } else {
	      f2 = f;
	    }
	  }
	} END_FORLIST;
	if (neighbours == 1) {
	  Position y, extend;
	  Drawing i;
	  i = get_drawing (interface, map);

	  /*  The neighbouring system has exactly one neighbour  */
	  if (Y(other) + HEIGHT(other)/2 > Y(draw) + HEIGHT(draw)/2) {
	    y = y_below;
	    y_below += HEIGHT (other) + sep;
	    if (i) {
	      extend_above += ext;
	      extend = extend_above;
	    }
	  } else {
	    y_above -= (HEIGHT (other) + sep);
	    y = y_above;
	    if (i) {
	      extend_below += ext;
	      extend = extend_below;
	    }
	  }
	  Map_insert_drawing (map, other, NULSHAPE, NULSTR, x, y, 0,0);
	  if (i) {
	    Map_insert_drawing (map, i, NULSHAPE, NULSTR,
				(Position) (x+WIDTH(other)+extend),
				(Position) (y+HEIGHT(other)/2), 0,0);
	  }
	  position_interfaces (other);
	} else if (neighbours == 2) {
	  Position px, py;
	  choose_midpoint_attachment (other, f1, f2, &px, &py);
	  Map_insert_drawing (map, other, NULSHAPE, NULSTR, px,py, 0,0);
	  position_interfaces (other);
	}
      }
    } END_FORLIST;
  }
}

MONETCALLBACK(do_position_neighbours)
     Map_parse_select (argc-1, argv+1, position_neighbours);
MONETEND

/***************************************************************************/
/* list connections geometry of drawings in current map  */
/***************************************************************************/

static void list_links_do_command (draw)
     Drawing draw;
{
  MonetEntity entity = ENTITY(draw);
  if (MonetEntity_isInterface (entity)) {
    Drawing from, to;
    if ((from = Drawing_from_interface(draw)) &&
	(to = Drawing_to_interface(draw))) {
      MonetPrintfnl ("link\t%s %s",
		     Drawing_title (from),
		     Drawing_title (to));
    } else {
      MONETDEBUG(11, ( "links [%s]: ", MonetEntity_title (entity)));
    }
  }
}

MONETCALLBACK(do_list_links)
     Map_parse_select (argc-1, argv+1, list_links_do_command);
MONETEND

/***************************************************************************/
/*  Query routines  */
/***************************************************************************/

MONETCALLBACK(do_x)
     MonetEntity  e;
     Drawing d;

     ISLOADED;
     result = NULL;
     if (e = String2MonetEntity (argv[1])) {
       if (d = get_drawing (e, CurrentMap))
	 result = Monetint2String (X(d));
     }
MONETEND

MONETCALLBACK(do_y)
     MonetEntity  e;
     Drawing d;

     ISLOADED;
     result = NULL;
     if (e = String2MonetEntity (argv[1])) {
       if (d = get_drawing (e, CurrentMap))
	 result = Monetint2String (Y(d));
     }
MONETEND

MONETCALLBACK(do_w)
     MonetEntity  e;
     Drawing d;

     ISLOADED;
     result = NULL;
     if (e = String2MonetEntity (argv[1])) {
       if (d = get_drawing (e, CurrentMap))
	 result = Monetint2String (WIDTH(d));
     }
MONETEND

MONETCALLBACK(do_h)
     MonetEntity  e;
     Drawing d;

     ISLOADED;
     result = NULL;
     if (e = String2MonetEntity (argv[1])) {
       if (d = get_drawing (e, CurrentMap))
	 result = Monetint2String (HEIGHT(d));
     }
MONETEND

MONETCALLBACK(do_is_placed)
     MonetEntity  e;

     ISLOADED;
     if ((e = String2MonetEntity (argv[1])) && get_drawing (e, CurrentMap)) {
       /*  -- It exists and is on the map  */
     } else {
       result = NULSTR;
     }
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_drawing_init ()
{
  (void) MonetMember_module_class ("drawing");

  /***********************/

  MonetMember_create ( "x-pos",		do_x,
		      "drawing",
		      "Returns x position of drawing on current map. \
Fails if does not exist on map");

  MonetMember_create ( "y-pos",		do_y,
		      "drawing",
		      "Returns y position of drawing on current map. \
Fails if does not exist on map");

  MonetMember_create ( "w-pos",		do_w,
		      "drawing",
		      "Returns width of drawing on current map. \
Fails if does not exist on map");

  MonetMember_create ( "h-pos",		do_h,
		      "drawing",
		      "Returns height of drawing on current map. \
Fails if does not exist on map");

  MonetMember_create ( "is-placed",		do_is_placed,
		      "drawing",
		      "Succeeds if drawing is on current map");

  /***********************/

  MonetMember_create ( "abbrev-label",		do_abbrev,
		      "selection_pattern ...",
		      "Tries to abbreviate label names");
  MonetMember_create ( "submap-visible",	do_submap_visible,
		      "selection_pattern ...",
		      "Sets submap visible");
  MonetMember_create ( "submap-invisible",	do_submap_invisible,
		      "selection_pattern ...",
		      "Sets submap invisible");
  MonetMember_create ( "label",		do_label,
		      "drawing label",
		      "Sets the label of a drawing");
  MonetMember_create ( "explore-ip-host",		do_explore_ip_host,
		      "selection_pattern ...",
		      "network auto-discovery for ip hosts");
  MonetMember_create ( "isolated",		do_isolated,
		      "selection_pattern ...",
		      "Selects all systems with no connections");
  MonetMember_create ( "count",		do_count,
		      "selection_pattern ...",
		      "Count number of selected entities. \
takes a selection pattern as an argument");
  MonetMember_create ( "list",		do_list,
		      "selection_pattern ...",
		      "list names of entities. \
takes a selection pattern as an argument");
  MonetMember_create ( "info",		do_info,
		      "selection_pattern ...",
		      "long report on entities");
  MonetMember_create ( "for",		do_for_in_map,
		      "selection_pattern ...",
		      "Performs command on all selected drawings");
  MonetMember_create ( "list-geometry",		do_list_geometry,
		      "selection_pattern ...",
		     "lists the geometry of every drawing on the current map. \
format: name label x y width height");
  MonetMember_create ( "report",		do_report,
		      "selection_pattern ...",
		      "short report on entities");

  MonetMember_create ( "select",		do_select,
		      "?selection_pattern ...",
		      "Adds all the specified drawings to the selection list");
  MonetMember_create ( "current",		do_current_drawing,
		      "",
		      "return name of current drawing");
  MonetMember_create ( "straight",	do_straighten,
		      "selection_pattern ...",
		      "straighten interfaces");
  MonetMember_create ( "position-neighbours",  do_position_neighbours,
		      "selection_pattern ...",
		      "Positions neighbours with only one link. \
Those with two are positioned between its neighbours");
  MonetMember_create ( "list-links",		do_list_links,
		      "?selection_pattern ...",
		      "lists the linked systems in the current map. \
Any interfaces with two connections have the connections printed.");
  MonetMember_create ( "reset-size",	do_reset_size,
		      "selection_pattern ...",
		      "reset the width and height of the drawings to default");
  MonetMember_create ("toggle-primary-label", do_toggle_primary_label,
		      "selection_pattern ...",
		      "Toggle the way the primary label is displayed.");
  MonetMember_create ("reset-primary-label", do_reset_primary_label,
		      "selection_pattern ...",
	 "Reset the way the primary label is displayed to the default.");

  (void) MonetMember_module_class ("memory");
  MonetMember_create ( "drawing", 	do_help,
		      "",
		      "some internal info about drawings");

}

extern void MapScript_drawing_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
