/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/map/Drawing.h>
#include <monet/msh/Channel.h>
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>
#include "mapP.h"

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

/*****************************************************************************/
/*  Get currently selected class */
/*****************************************************************************/

static MonetEntity get_type ()
{
  Drawing type;

  if ((type = Drawing_current_class ()) == NULL) {
    MonetPrintfError ("You must select a Class");
    return NULL;
  }
  return ENTITY(type);
}


/*****************************************************************************/
/*  Get informations  */
/*****************************************************************************/

MONETCALLBACK(do_SelectInstances)
     MonetEntity entity_class, s;
     
     if (entity_class = get_type ()) {
       MonetPrintfInform ("Selecting Instances of [%s]",
			  MonetEntity_title (entity_class));
       FOR_ENTITYLIST (s, MonetEntity_instances (entity_class)) {
	 MonetEntity_select_drawings (s);
       } END_FORLIST;
     }
MONETEND

MONETCALLBACK(do_ReportInstances)
	MonetEntity entity_class, s;

	if ((entity_class = get_type ()) == NULL)
		RETURNSC;
	MonetPrintfnl ("Instances of [%s]", MonetEntity_title (entity_class));
	MonetPrintfnl ("Instances of [%s]", MonetEntity_title (entity_class));
	FOR_ENTITYLIST (s, MonetEntity_instances (entity_class)) {
	  MonetPrintfnl ("  %s", MonetEntity_title (s));
	} END_FORLIST;
MONETEND

MONETCALLBACK(do_ReportClass)
     MonetEntity type;
	
     if ((type = get_type ()) == NULL)
     RETURNSC;
     MonetEntity_status_report (type);
MONETEND

/***************************************************************/
/*  Create an interface between two systems, create a unique */
/*  name for it */
/***************************************************************/

static Drawing make_anonymousInterface (entity_class, from, to)
     Drawing from, to;
     MonetEntity entity_class;
{
  Drawing interface;

  static long int Linkid = 0;
  char title [26];
  Position x,y;

  if (Drawing_mapOf(from) != Drawing_mapOf(to)) {
    MonetPrintfError
      ("Cannot create an interface between objects on different maps");
    return NULL;
  }
  if (!can_interface (entity_class, Drawing_classOf(from)) ||
      !can_interface (entity_class, Drawing_classOf(to))) {
    MonetPrintfError ("Can't create an interface between these two objects");
    return NULL;
  }
  /* !!!!!! This is not the way to do it */
  (void) sprintf (title, "link%ld.%d.%d", Linkid, (int) from, (int) to);
  Linkid ++;
  choose_midpoint_position (from, to, &x, &y);
  if ((interface = Drawing_create (title, entity_class,
			     Drawing_mapOf(from), x,y, 0,0))
      == NULL) {
    MonetSysError ("Anonymous interface already exists");
    return NULL;
  }
  MonetEntity_make_interface (ENTITY(interface), ENTITY(from));
  MonetEntity_make_interface (ENTITY(interface), ENTITY(to));
  return interface;
}

/*****************************************************************************/
/*  Join and detatch nodes  */
/*****************************************************************************/

MONETCALLBACK(do_join)
     MonetEntity sys1 = NULL, sys2 = NULL, interface = NULL;
     Drawing d1, d2, d;
     MonetEntity entity_class;
     MonetMap map = NULL;

     ISLOADED;

     FOR_SELECTIONS (CurrentMap, d) {
       MonetMap map_drawing;
       MonetEntity sys = d->entity;
       map_drawing = Drawing_mapOf (d);
       if (Map_root (map_drawing) != CurrentMap)
	 continue;
       if (map) {
	 if (map_drawing != map) {
	   MonetPrintfError ("All selections must be on same map");
	   RETURNSC;
	 }
       } else
	 map = map_drawing;
       if (MonetEntity_isInterface (sys)) {
	 if (interface) {
	   MonetPrintfError ("Only one Interface required");
	   RETURNSC;
	 } else {
	   interface = sys;
	 }
       } else if (sys1) {
	 if (sys2) {
	   MonetPrintfError ("No more than 2 systems required");
	   RETURNSC;
	 } else {
	   d2 = d;
	   sys2 = sys;
	 }
       } else {
	 d1 = d;
	 sys1 = sys;
       }
     }

     free_selection (CurrentMap);

     if (sys1 && sys2) {
       if (Drawing_isMap (d1) || Drawing_isMap(d2)) {
	 MonetPrintfError ("Cannot interface to a Map");
       } else if (are_interfaced ((sys1), (sys2))) {
	 MonetPrintfError ("These systems are already interfaced");
       } else if (interface) {
	 /* !!! Should check both operations are possible before trying */
	 /* !!! either. */
	 MonetEntity_make_interface (interface, sys2);
	 MonetEntity_make_interface (interface, sys1);
       } else {
	 if ( ! (entity_class = get_type ()))
	   entity_class = MonetEntity_choose_interface (sys1,sys2);
	 if (entity_class && MonetEntity_isInterfaceClass (entity_class)) {
	   (void) make_anonymousInterface (entity_class, d1, d2);
	 } else {
	   MonetPrintfError ("Must first select an interface class");
	 }
       }
     } else if (interface && sys1) {
       MonetEntity_make_interface (interface, sys1);
     } else {
       MonetPrintfError ("Must make more than one selection");
     }
MONETEND

MONETCALLBACK(do_detach)
     MonetEntity sys1, sys2;
     Drawing d1, d2;

     ISLOADED;
     MonetPrintfInform ("Detach");

     d1 = current_Selection (CurrentMap);
     if ( ! d1) {
       MonetPrintfError ("Nothing selected");
       RETURNSC;
     }
     sys1 = ENTITY(d1);
     d2 = second_selection (CurrentMap);
     if ( ! d2) {
       if (MonetEntity_isInterface (sys1)) {
	 if (sys2 = from_interface (sys1))
	   MonetEntity_unmake_interface (sys1, sys2);
	 if (sys2 = to_interface (sys1))
	   MonetEntity_unmake_interface (sys1, sys2);
	 RETURNSC;
       } else {
	 MonetPrintfError ("Must select an interface and a system");
	 RETURNSC;
       }
     }
     sys2 = ENTITY(d2);

     if (are_interfaced (sys1, sys2) == False) {
       MonetPrintfError ("These systems aren't interfaced");
     } else if (MonetEntity_isInterface (sys1)) {
       MonetEntity_unmake_interface (sys1, sys2);
     } else if (MonetEntity_isInterface (sys2)) {
       MonetEntity_unmake_interface (sys2, sys1);
     } else {
       MonetPrintfInform ("Sorry, you have to delete the interface box");
     }
     free_selection (CurrentMap);
MONETEND

/****************************************************************/
/*  Delete drawings  */
/****************************************************************/

static MonetMap DeleteMap;

static void delete_many ()
{
  Drawing sys;

  Map_backup_selected (DeleteMap);
  sys= current_Selection (DeleteMap);
  while (sys != NULL) {
    MonetPrintfInform ("Freeing drawing of [%s]", Drawing_title (sys));
    Drawing_free (sys);
    sys = current_Selection (DeleteMap);
  }
}

MONETCALLBACK(do_delete)
     Drawing sys;
     int num;

     ISLOADED;
     DeleteMap = MapCurrent_get ();
     num = numberOf_Selection (CurrentMap);
     if (num == 1) {
       sys = current_Selection (CurrentMap);
       Drawing_backup (sys);
       MonetPrintfInform ("Freeing Drawing [%s]", Drawing_title (sys));
       Drawing_free (sys);
     } else if (num > 1) {
       Monet_confirm (delete_many,
		    "You really want to Delete all Selected Objects?");
     } else {
       MonetPrintfError ("Nothing selected for deletion");
     }
     free_selection (CurrentMap);
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_edit_init ()
{
  (void) MonetMember_module_class ("drawing");

  MonetMember_create ( "detach",		do_detach,
		      "",
		      "detach joined entities");
  MonetMember_create ( "join",		do_join,
		      "",
		      "Link drawings together.");
  MonetMember_create ( "delete",		do_delete,
		      "",
		      "Removes all drawings in the selection list");
  MonetMember_create ("select-instances",	 do_SelectInstances,
		      "",
		      "select the instances of a class");
  MonetMember_create ("report-instances",	 do_ReportInstances,
		      "",
		      "report on the instances of a class");
  MonetMember_create ("report-class",		do_ReportClass,
		      "",
		      "status report on a class");
}

extern void MapScript_edit_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
