/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/map/Drawing.h>
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>
#include <monet/map/File.h>
#include "mapP.h"

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

/***************************************************************************/
/*  Save  */
/***************************************************************************/

MONETCALLBACK(do_save)
     ISLOADED;
     (void) Map_save (MapCurrent_get());
MONETEND

MONETCALLBACK(do_write)
     ISLOADED;
     (void) Map_write (MapCurrent_get(), argv[1]);
MONETEND

MONETCALLBACK(do_saveall)
     MonetMap          m;
     FORALLMAPS(m) {
       if (m->saved == False)
	 (void) Map_save (m);
     }
MONETEND

/***************************************************************************/
/*  Save  */
/*****************************************************************************/

MONETCALLBACK(do_load)
     if (Map_load (NULMAP, argv[1])) {
       result = Map_title (MapCurrent_get ());
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_class_map)
     MonetMap map = MapCurrent_get ();
     MonetBool change;
     if (map && Map_classesOnly (map)) {
       RETURNSC;
     }
     if (change = isDrawingAllowed (map)) {
       Map_disable_set_changed (map);
     }
     if (! map || map->entity || Map_is_saved (map)) {
       map = Map_create_class ();
     }
     map -> drawnas = drawnas_list;
     Map_reset_corners (map, True);
     if (change) {
       Map_enable_set_changed (map);
     }
     Map_set_changed (map);
     map->saved = True;
     /*  Map_request_redraw (map); */
MONETEND

MONETCALLBACK(do_make_map)
     if (Map_make (NULMAP, argv[1])) {
       result = Map_title (MapCurrent_get ());
     } else {
       result = NULL;
     }
MONETEND

/*****************************************************************************/
/*  This stuff is used in map files  */
/*****************************************************************************/

/*****************************************************************************
  This function is an optimisation of the following monetsh code

  function link interface from-sys to-sys {
    $interface -> set from $from-sys  to $to-sys
  }

  It makes a link between two systems via an interface.
****************************************************************************/

MONETCALLBACK(do_link)
     MonetEntity from, to, interface;

     if (interface = String2MonetEntity (argv[1])) {
       if (from = MonetEntity_find_or_create (argv[2])) {
	 if (to = MonetEntity_find_or_create (argv[3])) {
	   MonetEntity_make_links (interface, from, to);
	 } else {
	   MonetPrintfError ("Could not create [%s]", argv[3]);
	 }
       } else {
	 MonetPrintfError ("Could not create [%s]", argv[2]);
       }
     } else {
       MonetPrintfError ("Interface [%s] does not exist", argv[1]);
     }
MONETEND

/*****************************************************************************/
/*  Insert a drawing in a map  */
/*****************************************************************************/

MONETCALLBACK(do_insert_drawing)
     int x,y, w,h;
     Drawing drawing;
     MonetEntity e;
     String title, label;
     MonetShape shape;
     int n;

     ISLOADED;

     title = argv[1];
     if (argc == 6) {
       label = NULL;
       shape = NULL;
       n = 2;
     } else if (argc == 7) {
       label = argv[2];
       shape = NULL;
       n = 3;
     } else /* argc == 8 */{
       label = argv[2];
       shape = String2MonetShape (argv[3]);
       n = 4;
     }
     x = MonetString2int (argv[n]);
     y = MonetString2int (argv[n+1]);
     w = MonetString2int (argv[n+2]);
     h = MonetString2int (argv[n+3]);

     if (e = MonetEntity_find_or_create (title)) {
       if (drawing = get_drawing (e, CurrentMap)) {
	 Map_insert_drawing (CurrentMap, drawing, shape, label,
			     (Position) x, (Position) y,
			     (Dimension) w, (Dimension) h);
       } else if (drawing = Drawing_create_from_entity
		  (e, CurrentMap, shape, label, (Position) x, (Position) y,
		   (Dimension) w, (Dimension) h)) {
       } else {
	 result = NULL;
	 RETURNSC;
       }
     } else {
       result = NULL;
     }
MONETEND

/*****************************************************************************/
/*  Map Attributes  */
/*****************************************************************************/

MONETCALLBACK(do_net_border)
     ISLOADED;
     if (argc>1) {
       MonetNetBorder border = String2MonetNetBorder (argv[1]);
       if (border != CurrentMap->net_border) {
	 CurrentMap->net_border = border;
	 Map_request_changed_redraw (CurrentMap);
       }
     }
     result = MonetNetBorder2String (CurrentMap->net_border);
MONETEND

MONETCALLBACK(do_background_shape)
     ISLOADED;
     if (argc > 1) {
       Map_set_background_shape (CurrentMap, String2MonetShape (argv[1]));
       if (argc == 4) {
	 CurrentMap->shape_width  = MonetString2int (argv[2]);
	 CurrentMap->shape_height = MonetString2int (argv[3]);
       }
       Map_request_redraw (CurrentMap);
     }
     result = MonetShape2String (CurrentMap->background_shape);
MONETEND

MONETCALLBACK(do_domain)
     ISLOADED;
     Map_set_domain (CurrentMap, argv[1]);
     Map_request_redraw (CurrentMap);
MONETEND

MONETCALLBACK(do_drawnas)
     ISLOADEDORCLASS;
     if (argc > 1) {
       CurrentMap->drawnas = string2drawnas (argv[1]);
       Map_reset_corners (CurrentMap, True);
       Map_request_redraw (CurrentMap);
     }
     result = drawnas2string (CurrentMap->drawnas);
MONETEND

MONETCALLBACK(do_grid)
     ISLOADED;
     CurrentMap->grid_x = MonetString2int (argv[1]);
     CurrentMap->grid_y = MonetString2int (argv[2]);
MONETEND

MONETCALLBACK(do_connectors)
     ISLOADED;

     CurrentMap -> connectors = String2ConnectorState (argv[1]);
     Map_request_changed_redraw (CurrentMap);
MONETEND

MONETCALLBACK(do_secondary_label)
     MonetAttribute  attribute;
     ISLOADEDORCLASS;
     if (attribute = String2MonetAttribute (argv[1])) {
       if (CurrentMap -> secondary_label != attribute) {
	 CurrentMap -> secondary_label = attribute;
	 Map_request_changed_redraw (CurrentMap);
       }
     } else {
       MonetPrintfError ("No such attribute [%s]", argv[1]);
     }
MONETEND

MONETCALLBACK(do_no_secondary_label)
     ISLOADEDORCLASS;
     if (CurrentMap -> secondary_label != NULL) {
       CurrentMap -> secondary_label = NULL;
       Map_request_changed_redraw (CurrentMap);
     }
MONETEND

MONETCALLBACK(do_primary_label)
     MonetAttribute  attribute;
     ISLOADEDORCLASS;
     if (attribute = String2MonetAttribute (argv[1])) {
       if (CurrentMap -> primary_label != attribute) {
	 CurrentMap -> primary_label = attribute;
	 Map_request_changed_redraw (CurrentMap);
       }
     } else {
       MonetPrintfError ("No such attribute [%s]", argv[1]);
     }
MONETEND

MONETCALLBACK(do_no_primary_label)
     ISLOADEDORCLASS;
     if (CurrentMap -> primary_label != NULL) {
       CurrentMap -> primary_label = NULL;
       Map_request_changed_redraw (CurrentMap);
     }
MONETEND

/*****************************************************************************/
/*  Map Directory  */
/*****************************************************************************/

MONETCALLBACK(do_local_dir)
	(void) Monet_list_dir (Monet_map_directory (NULSTR), argv[1]);
MONETEND

/*****************************************************************************/
/*  Register drawing/status attributes  */
/*****************************************************************************/

MONETCALLBACK(do_scale_lines)
     MonetAttribute  attribute;
 
     ISLOADED;
     if (attribute = String2MonetAttribute (argv[1])) {
       CurrentMap->scale_to_line = attribute;
       if (EQS (argv[2], "log")) {
	 CurrentMap->scale_to_by = 0;
       } else {
	 CurrentMap->scale_to_by = MonetString2int(argv[2]);
       }
     } else {
       MonetPrintfError ("No such attribute [%s]", argv[2]);
     }
     if (attribute = String2MonetAttribute (argv[3])) {
       CurrentMap->scale_from_line = attribute;
       if (EQS (argv[4], "log")) {
	 CurrentMap->scale_from_by = 0;
       } else {
	 CurrentMap->scale_from_by = MonetString2int(argv[4]);
       }
     } else {
       MonetPrintfError ("No such attribute [%s]", argv[3]);
     }
     Map_request_changed_redraw (CurrentMap);
MONETEND

MONETCALLBACK(do_no_scale_lines)
     ISLOADED;
     CurrentMap->scale_from_line = NULL;
     CurrentMap->scale_from_by = 0;
     CurrentMap->scale_to_line = NULL;
     CurrentMap->scale_to_by = 0;
     Map_request_changed_redraw (CurrentMap);
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_file_init ()
{
  (void) MonetMember_module_class ("map");
  MonetMember_create ( "save",	do_save,
		      "",
		      "save current map");
  MonetMember_create ( "write",	do_write,
		      "file",
		      "write current map to a file");
  MonetMember_create ( "save-all",	do_saveall,
		      "",
		      "save all maps");
  MonetMember_create ( "load",	do_load,
		      "filename",
		      "load map file");
  MonetMember_create ( "make",	do_make_map,
		      "?name",
		      "make a new map, does nto try to load from file");
  MonetMember_create ( "ls",		do_local_dir,
		      "?pattern",
	      "list maps in directory on local machine matching <pattern>");
  MonetMember_create ("class-map",	 do_class_map,
		      "",
		      "Make current map a class map, \
if map already loaded, then this is discarded");
  MonetMember_create ( "draw",		do_insert_drawing,
		      "drawing ?label ?shape x y width height",
		      "Insert/create drawing in a map at particular position. \
If 7 arguments shape is not set");
  MonetMember_create ( "link",		do_link,
		      "interface from to",
		      "Links entities 'from' and 'to' with 'interface'");
  MonetMember_create ( "domain",  do_domain,
		      "string",
		      "Sets the default domain for the current map");
  MonetMember_create ( "backing",  do_background_shape,
		      "?string ?width ?height",
		      "Sets/returns the backing shape for current map \
		      this would be something like a picture of a map");
  MonetMember_create ( "net-border",  do_net_border,
		      "?box|round|minimal",
		      "Sets/returns flag for current map, this indicates if \
		      networks are shown as elipses, instead of rectangles");
  MonetMember_create ( "drawnas",  do_drawnas,
		      "?listing|map",
		      "Sets how map is drawn as, as a list or as a map. \
Returns new and/or current value");
  MonetMember_create ( "grid",  do_grid,
		      "x y",
		      "Sets the grid for the current map");
  MonetMember_create
    ("scale-lines", do_scale_lines,
     "from-attribute scale-factor to-attribute scale-factor",
     "Register the attribute used to scale interface lines.  Scale-factor is \
a number of linear scaling, or 'log' or 0 for logarithmic.");
  MonetMember_create ("no-scale-lines", do_no_scale_lines,
		      "",
		      "No scaling of lines to/from interfaces");
  MonetMember_create ( "connectors",  do_connectors,
		      "big|small|tiny",
		      "Sets the way conenctors are drawn on the current map");
  MonetMember_create
    ("secondary-label", do_secondary_label,
     "attribute",
     "Register the attribute that will be displayed as secondary label");
  MonetMember_create
    ("no-secondary-label", do_no_secondary_label,
     "",
     "No secondary label registered");
  MonetMember_create
    ("primary-label", do_primary_label,
     "attribute",
     "Register the attribute that will be displayed as primary label");
  MonetMember_create
    ("no-primary-label", do_no_primary_label,
     "",
     "No primary label registered");
}

extern void MapScript_file_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
