/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "mapP.h"
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>

extern long random
  PR((void));

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

MONETCALLBACK(do_redraw_all_maps)
	MonetMap_redraw_all ();
MONETEND

MONETCALLBACK(do_redraw)
     /* -- There are two redraws , Map_request_changed_redraw and
	-- Map_request_redraw.
	-- The former can always be used in place of the latter but is more
	-- expensive, it is used here so that the user doesn't have to choose
	-- which they want.
	*/

     Map_request_changed_redraw (MapCurrent_get ());
MONETEND

MONETCALLBACK(do_random_layout)
     Position xl,yl,xr,yr;
     Drawing d;
     Dimension w,h;
     MonetBool change;

     ISLOADED;

     if (argv[1]) {
       srandom (MonetString2int (argv[1]));
     }
     if (argc == 4) {
       w = MonetString2int (argv[2]);
       h = MonetString2int (argv[3]);
     } else if (Map_corners (CurrentMap, &xl,&yl, &xr,&yr, False)) {
       w = xr-xl;
       h = yr-yl;
     } else {
       MonetPrintfWarning ("Map [%s] is empty", Map_title (CurrentMap));
       RETURNSC;
     }
     Map_backup (CurrentMap);


     if (change = isDrawingAllowed (CurrentMap)) {
       Map_disable_set_changed (CurrentMap);
     }
     FOR_DRAWINGLIST (d, Map_elements (CurrentMap)) {
       Position x,y;
       x = (Position) random () % (w - WIDTH(d) - 100)+50;
       y = (Position) random () % (h - HEIGHT(d) - 100)+50;
       Drawing_position (d, x, y);
     } END_FORLIST;
     if (change) {
       Map_enable_set_changed (CurrentMap);
     }
     Map_set_resized (CurrentMap);
MONETEND

/*****************************************************************************/
/*  Change positions of everything on the current map  */
/*****************************************************************************/

MONETCALLBACK(do_redimension)
     Position xl,yl,xr,yr;
     Drawing d;
     MonetBool change;
     int xmul, ymul, wmul, hmul, divisor, xoffset, yoffset;
     ISLOADED;

     if (Map_corners (CurrentMap, &xl,&yl, &xr,&yr, False) == False) {
       MonetPrintfWarning ("Map [%s] is empty", Map_title (CurrentMap));
       RETURNSC;
     }
     xoffset = MonetString2int (argv[1]);
     yoffset = MonetString2int (argv[2]);
     xmul    = MonetString2int (argv[3]);
     ymul    = MonetString2int (argv[4]);
     wmul    = MonetString2int (argv[5]);
     hmul    = MonetString2int (argv[6]);
     divisor = MonetString2int (argv[7]);

     if (divisor == 0) {
       MonetPrintfError ("Divisor is 0");
       RETURNSC;
     }
     Map_backup (CurrentMap);
     if (change = isDrawingAllowed (CurrentMap)) {
       Map_disable_set_changed (CurrentMap);
     }
     FOR_DRAWINGLIST (d, Map_elements (CurrentMap)) {
       Drawing_dimension (d,
			  xoffset + (X(d) * xmul) / divisor,
			  yoffset + (Y(d) * ymul) / divisor,
			  (WIDTH(d)  * wmul) / divisor,
			  (HEIGHT(d) * hmul) / divisor);
     } END_FORLIST;
     FOR_DRAWINGLIST (d, Map_elements (CurrentMap)) {
       Drawing_prepare (d);
     } END_FORLIST;

     if (change) {
       Map_enable_set_changed (CurrentMap);
     }
     Map_set_resized (CurrentMap);
MONETEND

/*****************************************************************************/
/*  Force resize check on position of corners  */
/*****************************************************************************/

MONETCALLBACK(do_force_corners)
     Position xl,yl,xr,yr;
     ISLOADEDORCLASS;

     MonetPrintfnl ("This is a bit of a debugging function");
     if (Map_corners (CurrentMap, &xl,&yl, &xr,&yr, False) == False) {
       MonetPrintfWarning ("Map [%s] is empty", Map_title (CurrentMap));
     } else {
       MonetPrintfnl ("Dimension before forcing [%u,%u]", xr, yr);
       if (Map_corners (CurrentMap, &xl,&yl, &xr,&yr, True) == False)
	 MonetPrintfWarning ("Map [%s] is empty",
			     Map_title (CurrentMap));
       else
	 MonetPrintfnl ("Dimension after forcing [%u,%u]", xr, yr);
     }
MONETEND

MONETCALLBACK(do_adjust_corners)
     MonetMap map;
     ISLOADEDORCLASS;
     map = CurrentMap;
     Map_reset_corners (CurrentMap, True);
     MonetPrintfInform ("Adjusting to [%d,%d]",  map->xr, map->yr);
MONETEND

/************************************************************/
/*  Line up non connected nodes  */
/************************************************************/

static int rows, row;
static Position lx, ly, wide;
static Dimension maxh, width = 0;

static void line_up_do_command (d)
	Drawing d;
{
  Drawing_position (d, lx, ly);
  lx += WIDTH(d) + 20;
  if (maxh < HEIGHT(d))
    maxh = HEIGHT(d);

  wide += WIDTH(d);
  if (row == 0 || wide >= width) {
    row = rows;
    ly = maxh + 20;
  } else {
    --row;
  }
}

MONETCALLBACK(do_line_up)
     Drawing d;
     Position xl,yl,xr,yr;
     MonetBool change;

     ISLOADED;

     if (Map_corners (CurrentMap, &xl,&yl, &xr,&yr, False) == False) {
       MonetPrintfWarning ("Map [%s] is empty", Map_title (CurrentMap));
       RETURNSC;
     }

     FOR_DRAWINGLIST (d, Map_elements (CurrentMap)) {
       width += WIDTH (d) + 20;
     } END_FORLIST;

     wide = 0;
     rows = xr % width + 1;
     lx = (Position) MonetString2int (argv[1]);
     ly = (Position) MonetString2int (argv[2]);
     maxh = 0;
     row = rows;

     Map_backup (CurrentMap);
     if (change = isDrawingAllowed (CurrentMap)) {
       Map_disable_set_changed (CurrentMap);
     }
     Map_parse_select (argc-3, argv+3, line_up_do_command);
     if (change) {
       Map_enable_set_changed (CurrentMap);
     }
     Map_set_resized (CurrentMap);
MONETEND

/*****************************************************************************/
/*  Promote Maps  */
/*****************************************************************************/
/*
MONETCALLBACK (do_promote_map)

MONETEND
*/
/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_map_init ()
{
  (void) MonetMember_module_class ("map");

  MonetMember_create ("force-corners",		do_force_corners,
		      "",
		      "Forces a recheck on the corners of a map. \
This really should be done automatically.");
  MonetMember_create ("adjust-corners",		do_adjust_corners,
		      "",
		      "Forces a recheck on the corners of a map. \
This really should be done automatically.");
  MonetMember_create ( "random-layout",		do_random_layout,
		      "?seed ?width ?height",
		      "Put all drawings on current map at a random position");
  MonetMember_create ( "redimension",		do_redimension,
		      "xoffset yoffset xmul ymul wmul hmul divisor",
		      "Alters positions of everything on the current map");
  MonetMember_create ( "redraw",		do_redraw,
		      "",
		      "redraw current map");
  MonetMember_create ( "redraw-all",	do_redraw_all_maps,
		      "",
		      "redraw all maps");
/*
  MonetMember_create ( "promote",		do_promote_map,
		      "",
		      "promote common drawings in maps. \
Looks in the sub maps of the current map for ...");
*/
  MonetMember_create ( "line-up",		do_line_up,
		      "x y selection_pattern ...",
		      "Lines up selected nodes at the position given");
}

extern void MapScript_map_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
