/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/query.c,v 1.1 1991/07/30 11:10:25 a38 Exp a38 $  */
/* $Log: query.c,v $
 * Revision 1.1  1991/07/30  11:10:25  a38
 * Initial revision
 *
 * Revision 1.1  1991/06/11  11:26:10  a38
 * Initial revision
 *  */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/map/Drawing.h>
#include <monet/map/Draw.h>
#include <monet/msh/Script.h>

/*****************************************************************************/

/*****************************************************************************/

static String ForCommand;
static MonetBool Redraw, Moved, BatchDraw;

static void select_in_map (map, id, pattern, test, command, op, selected)
     MonetMap map;
     MonetAttribute id;
     String pattern;
     DoTestRedrawCallback test;
     DoDrawingCallback command;
     char op;
     MonetBool selected;
{	
  Drawing d;
  MonetBool change, r = Redraw;

  if (BatchDraw) {
    if (change = isDrawingAllowed (map)) {
      Map_disable_set_changed (map);
      Redraw = False;
    }
  } else {
    change = False;
  }

  if (Redraw) {
    Redraw = Drawing_begin (map);
  }
  FOR_DRAWINGLIST (d, Map_elements (map)) {
    char * value;
    MonetEntity entity = ENTITY(d);

    if (selected && !Drawing_isSelected (d))
      continue;

    if (id) {
      value = MonetEntity_get_attribute (entity, id);
    } else {
      value = Drawing_title (d);
    }

    if (MonetMatchWildcard (value, pattern)) {
      if (op == '#')
	continue;
      if ((!test) || test (d)) {
	if (Redraw) {
	  Drawing_draw (d, Moved, True);
	  Map_disable_set_changed (map);
	}
	if ((!ForCommand) ||
	    MonetshEntityExecute (entity, ForCommand, NULL, False))
	  if (command)
	    command (d);
	if (Redraw) {
	  Map_enable_set_changed (map);
	  Drawing_prepare (d);
	  Drawing_draw (d, Moved, False);
	}
      }
    }
  } END_FORLIST;
  if (Redraw)
    Drawing_end (map);
  if (change) {
    Map_enable_set_changed (map);
    Map_request_redraw (map);
  }
  Redraw = r;
}

/*****************************************************************************/
/*  Select a set of drawings from a map and call a procedure to process    */
/*  them.  */
/***************************************************************************/

static void parse_select (argc, arg, test, command)
     int argc;
     String *arg;
     DoTestRedrawCallback test;
     DoDrawingCallback command;
{
  char attribute[1024], patbuf[1024];
  MonetAttribute id;
  String map_pattern = NULL, pattern;
  unsigned place;
  char op = '=';
  MonetBool selected = False;
  MonetMap map;

  if (argc > 0) {
    ForCommand = arg[argc-1];
    if (ForCommand && *ForCommand == '{') {
      argc --;
    } else {
      ForCommand = NULL;
    }
  } else {
    ForCommand = NULL;
  }
  for (place=0;place < argc; ++place) {
    int cc;
    if (arg[place]) {
      cc = sscanf (arg[place], "%[^ \t\n=#]%c%s", attribute, &op, patbuf);
    } else {
      cc = 0;
    }
    if (cc == 3) {
      if (id = String2MonetAttribute (attribute)) {
      } else if (MonetStringsEqual ("map", attribute)) {
	map_pattern = patbuf;
	continue;
      } else if (MonetStringsEqual ("selected", attribute)) {
	selected = True;
      } else {
	MonetPrintfError ("No such attribute [%s]", attribute);
	return;
      }
      pattern = patbuf;
    } else {
      id = NULL;
      op = '=';
      pattern = arg[place];
    }
    if (!pattern || ! *pattern) {
      MonetPrintfWarning ("Null pattern given");
    } else if (map_pattern) {
      MonetMap          m;
      FORALLMAPS (m)
	if (Map_is_loaded(m) &&
	    MonetMatchWildcard (Map_title (m), map_pattern))
	  select_in_map (m, id, pattern, test, command, op, selected);
    } else if (map = MapCurrent_get ()) {
      if (Map_classesOnly(map) || Map_is_loaded(map))
	select_in_map (map, id, pattern, test, command, op, selected);
      else {
	MonetPrintfError ("Current map is not loaded");
	break;
      }
    } else {
      MonetPrintfError ("No current map selected");
      break;
    }
  }
}

static void parse_select_redraw (argc, arg, test, command)
     int argc;
     String *arg;
     DoTestRedrawCallback test;
     DoDrawingCallback command;
{
  MonetBool r = Redraw;
  Redraw = True;
  if (argc == 1 && (EQS(arg[0], "*"))) {
    BatchDraw = True;
  } else {
    BatchDraw = False;
  }
  parse_select (argc, arg, test, command);
  Redraw = r;
}

extern void Map_parse_select_redraw (argc, arg, test, command)
     int argc;
     String *arg;
     DoTestRedrawCallback test;
     DoDrawingCallback command;
{
  MonetBool m = Moved;
  Moved = False;
  parse_select_redraw (argc, arg, test, command);
  Moved = m;
}

extern void Map_parse_select_moved_redraw (argc, arg, test, command)
     int argc;
     String *arg;
     DoTestRedrawCallback test;
     DoDrawingCallback command;
{
  MonetBool m = Moved;
  Moved = True;
  parse_select_redraw (argc, arg, test, command);
  Moved = m;
}

extern void Map_parse_select (argc, arg, command)
     int argc;
     String *arg;
     DoDrawingCallback command;
{
  MonetBool r = Redraw, m = Moved;
  BatchDraw = False;
  Redraw = False;
  Moved = False;
  parse_select (argc, arg, NULTEST, command);
  Moved = m;
  Redraw = r;
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/query.c,v 1.1 1991/07/30 11:10:25 a38 Exp a38 $  */
/*****************************************************************************/
/*****************************************************************************/
