/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/map.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/* $Log: map.c,v $
 * Revision 1.1  1991/06/11  11:26:10  a38
 * Initial revision
 *  */
/*****************************************************************************/
/*  Handles changes in the dimensions of the map.

    The client program must supply a routine to be called when a map
    changes its dimensions, may be NULL.

    Handles a map being changed, switches on and off drawing.

    Whenever the position of a drawing changes, use Map_adjust_corners,
    this checks to see if anything

    Works out the prositions of a maps corners.  When a drawing is moved
    its is checked to see if the borders of the map may have been changed.
    If smaller it just resets the corners to say that they are unkown.
    The routine Map_corners returns the border of the map, if the border
    is known it just returns them otherwise it has to scan every drawing
    to find out what the boundaries are.

    
    */
/*****************************************************************************/

#include "mapP.h"
#include <monet/map/Interface.h>
#include <monet/msh/Script.h>

/*****************************************************************************/
/* Globals */
/*****************************************************************************/

static void set_unplaced (map)
     MonetMap map;
{
  if (map->unplaced_y > map->yr) {
    map->unplaced_x += 100;
    map->unplaced_y = 10;
  }
  if (map->unplaced_x > map->xr) {
    map->unplaced_x = map->xr;
  }
}

/***********************************************************/
/*  Find the extreme corners of a map, ie the left and right
    most positions of the drawings on it.  Returns true if
    the corners have changed  */
/***********************************************************/

extern MonetBool Map_check_corners (map, xl,yl, xr,yr, force_check)
     MonetMap map;
     Position *xl, *yl, *xr, *yr;
     MonetBool force_check;
{
  Drawing s;
  MonetBool areset, change;

  *xl = map->xl;
  *xr = map->xr;
  *yl = map->yl;
  *yr = map->yr;

  areset = ARESET_DIMENSIONS(map);

  if (!force_check && areset)
    return False;

  *xl = 0;
  *xr = 0;
  *yl = 0;
  *yr = 0;
  FOR_DRAWINGLIST (s, Map_elements (map)) {
    Position x,y;
    Dimension w,h;

    switch (map->drawnas) {
    case drawnas_list:
      x = LISTBOX(s).x;
      y = LISTBOX(s).y;
      w = LISTBOX(s).width;
      h = LISTBOX(s).height;
      break;
    case drawnas_map:
      x = X(s);
      y = Y(s);
      w = WIDTH(s);
      h = HEIGHT(s);
      break;
    }
    if (x < *xl)
      *xl = x;
    if (x+w > *xr)
      *xr = x+w;
    if (y < *yl)
      *yl = y;
    if (y+h > *yr)
      *yr = y+h;
  } END_FORLIST;
  change =
    (*xl != map->xl || *yl != map->yl || *xr != map->xr || *yr != map->yr);
  
  if (!areset && !change) {
    /*  No children  */
    *xl = 1;
    *yl = 1;
  } else {
    map->xl = *xl;
    map->xr = *xr;
    map->yl = *yl;
    map->yr = *yr;
  }
  set_unplaced (map);
  return change;
}

/***********************************************************/
/*  Map changed or resized */
/***********************************************************/

static MapChangedCallback Change_callback;
static DrawingBeginCallback Begin_draw;
static DrawingEndCallback End_draw;

/******************************************************************/
/*  Should use this to redraw X11 maps.  This clears the whole    */
/*  then sends an Expose event.  Does the redraw Asyncronously.   */
/*  A map may have only one widget, but many drawings referencing */
/*  these are all redrawn.                                        */
/******************************************************************/

extern void Map_request_redraw (map)
     MonetMap map;
{
  if (Change_callback && isDrawingAllowed (map)) {
    Position xl, yl, xr, yr;
    Dimension w, h;
    (void) Map_check_corners (map, &xl,&yl, &xr,&yr, False);
    w = xr;
    h = yr;
    if (map->drawnas == drawnas_list) {
      MonetMapList_setup (map);
      Map_reset_corners (map, True);
      /* Map_set_resized (map); */
    }
    if (map->mapWidget)
      Change_callback (map, w, h, True);
    if (map->entity) {
      Drawing d;
      FORALL(d, map->entity->drawing, next) {
	Drawing_unmoved_undraw (d);
	Drawing_unmoved_redraw (d);
      }
    }
  }
}

/******************************************************************/
/*  Request a redraw but calls Drawing_prepare for each drawing   */
/******************************************************************/

extern void Map_request_changed_redraw (map)
     MonetMap map;
{
  Drawing d;
  MonetBool change;

  if (change = isDrawingAllowed (map))
    Map_disable_set_changed (map);

  FOR_DRAWINGLIST (d, Map_elements (map)) {
    d->redraw_connectors = True;
    Drawing_prepare (d);
    d->redraw_connectors = True;
  } END_FORLIST;

  if (change) {
    Map_enable_set_changed (map);
    Map_request_redraw (map);
  }
}

/*****************************************************************************/
/* Redraw all the maps  */
/*****************************************************************************/

extern void MonetMap_redraw_all()
{
  MonetMap         m;
  FORALLMAPS (m)
    Map_request_changed_redraw (m);
}

/***********************************************************/
/* Check the placement of a drawing and adjust the corners */
/* of the map if necessary.  */
/***********************************************************/

static MonetBool adjust_corners (map, drawing, adding)
     MonetMap map;
     Drawing drawing;
     MonetBool adding;
{
  Position x,y;
  Dimension w,h;
  MonetBool change;

  if (map != Drawing_mapOf (drawing)) {
    MonetSysError ("map doesn't match");
    return False;
  }
  if ( ! ARESET_DIMENSIONS(map))
    return False;

  change = False;
  switch (map->drawnas) {
  case drawnas_list:
    x = LISTBOX(drawing).x;
    y = LISTBOX(drawing).y;
    w = LISTBOX(drawing).width;
    h = LISTBOX(drawing).height;
    break;
  case drawnas_map:
    x = X(drawing);
    y = Y(drawing);
    w = WIDTH(drawing);
    h = HEIGHT(drawing);
    break;
  }
  /*  Resize the map area  */
  if (adding) {
    if (x < map->xl)	{ change = True; map->xl = x; }
    if (x+w > map->xr)	{
      change = True; map->xr = x+w;
    }
    if (y < map->yl)	{ change = True; map->yl = y; }
    if (y+h > map->yr)	{
      change = True; map->yr = y+h;
    }
  } else {
    /* -- if we are removing this extreme node */
    /* -- then invalidate map size  */
    if (x <= map->xl)   { change = True; }
    if (x+w >= map->xr) { change = True; }
    if (y <= map->yl)   { change = True; }
    if (y+h >= map->yr) { change = True; }
    if (change) {
      map->xl = 0;
      map->xr = 0;
      map->yl = 0;
      map->yr = 0; 
    }
  }
  set_unplaced (map);

  return change;
}

/***********************************************************/
/* Check the placement of a drawing and adjust the corners */
/* of the map if necessary.  */
/***********************************************************/

extern void Map_adjust_corners (map, drawing, adding)
     MonetMap map;
     Drawing drawing;
     MonetBool adding;
{
  if (adjust_corners (map, drawing, adding))
    Map_set_resized (map);
}

/***********************************************************/
/*  Find the extreme corners of a map, ie the left and right */
/*  most positions of the drawings on it.  Returns false if  */
/*  the map is empty, else true.   */
/***********************************************************/

extern MonetBool Map_reset_corners (map,force_check)
     MonetMap map;
     MonetBool force_check;
{
  Position xl, yl, xr, yr;
  return Map_corners(map, &xl,&yl, &xr,&yr, force_check);
}

extern MonetBool Map_corners (map, xl,yl, xr,yr, force_check)
     MonetMap map;
     Position *xl, *yl, *xr, *yr;
     MonetBool force_check;
{
  if (Map_check_corners (map, xl,yl, xr,yr, force_check)) {
    Map_set_resized (map);
    return True;
  }
  return False;
}

/************************************************************************/
/*  Set the dimensions of an drawing, but do not try to draw anything!! */
/*  !!! Should do bound checking here                                   */
/************************************************************************/

extern MonetBool Drawing_dimension (o, x,y, w,h)
     Drawing o;
     Position x,y;
     Dimension w,h;
{
  MonetBool can_resize, xc, yc, wc, hc;  /*  Changes  */

  can_resize = Drawing_isResizable (o);
  xc = (x > 0 && X(o)      != x);
  yc = (y > 0 && Y(o)      != y);
  wc = (can_resize && w > 0 && WIDTH(o)  != w);
  hc = (can_resize && h > 0 && HEIGHT(o) != h);

  if (wc || hc || xc || yc) {
    MonetMap map;
    map = Drawing_mapOf(o);
    Map_set_changed (map);

    (void) adjust_corners (map, o, False);
    if (xc)
      X(o) = x;
    if (yc)
      Y(o) = y;
    if (wc)
      WIDTH(o)  = w;
    if (hc)
      HEIGHT(o) = h;

    if (X(o) < 0) {
      X(o) = 0;
    }
    if (X(o) < 0) {
      Y(o) = 0;
    }

    /*  See if the map dimensions are affected by the changes  */
    if ( adjust_corners (map, o, True)) {
      /*
	Change_callback (map, w, h, True);
	*/
    }

    if (wc || hc) {
      DRAWING(o)->resized = True;
      Drawing_prepare (o);
    }
    
/*    if (change)
      Map_set_resized (map);
*/
  } else {
    if (X(o) < 0) {
      X(o) = 0;
    }
    if (X(o) < 0) {
      Y(o) = 0;
    }
  }
  return True;
}

/************************************************************************/
/*  Position an drawing on the map */
/*  !!! Should undraw and redraw system */
/************************************************************************/

extern MonetBool Drawing_position (o, x,y)
     Drawing o;
     Position x,y;
{
  return Drawing_dimension (o, x,y, 0,0);
}

/******************************************************************/
/*  Undraw a system before making a change to its status          */
/******************************************************************/

extern MonetBool Drawing_begin (map)
     MonetMap map;
{
  if (isDrawingAllowed (map) && Begin_draw && Begin_draw (map)) {
    return True;
  }
  return False;
}

extern void Drawing_end (map)
     MonetMap map;
{
  if (End_draw) {
    Draw_end_draw ();
    End_draw (map);
  }
}

extern void Drawing_changed_redraw (d, moved)
     Drawing d;
     MonetBool moved;
{
  MonetMap map;
  MonetBool change;
  map = Drawing_mapOf(d);
  if (change = Drawing_begin (map)) {
    Drawing_draw (d, moved, True);
  }
  Drawing_prepare (d);
  if (change) {
    Drawing_draw (d, moved, False);
    Drawing_end (map);
  }
}

static void draw (s, moved, undraw)
     Drawing    s;
     MonetBool	moved, undraw;
{
  MonetMap map;
  map = Drawing_mapOf(s);
  if (Drawing_begin (map)) {
    Drawing_draw (s, moved, undraw);
    Drawing_end (map);
  }
}

/******************************************************************/
/*  Undraw a system a system before making changes to its state    */
/******************************************************************/

extern void Drawing_undraw(s)
     Drawing          s;
{
  draw (s, True, True);
}


extern void Drawing_redraw(s)
     Drawing          s;
{
  draw (s, True, False);
}

/******************************************************************/
/*  Redraw a system a system after making changes to its state    */
/******************************************************************/

static MonetMap Begun;

extern void Drawing_unmoved_undraw (s)
	Drawing          s;
{
  Begun = Drawing_mapOf(s);
  if (Drawing_begin (Begun)) {
    Drawing_draw (s, False, True);
  } else {
    Begun = NULL;
  }
}

extern void Drawing_unmoved_redraw (s)
	Drawing          s;
{
  if (Begun) {
    Drawing_draw (s, False, False);
    Drawing_end (Begun);
  }
}

/*****************************************************************************/
/*  Initialise  */
/*****************************************************************************/

extern void MonetMap_resize_init (change, begin_draw, end_draw)
     MapChangedCallback change;
     DrawingBeginCallback begin_draw;
     DrawingEndCallback end_draw;
{
  Change_callback = change;
  Begin_draw = begin_draw;
  End_draw = end_draw;
}

/****************************************************************************
  Finish off
  ****************************************************************************/

extern void MonetMap_resize_finish ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/map.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/*****************************************************************************/
/*****************************************************************************/

