/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/saveload.c,v 1.1 1991/07/22 11:36:41 a38 Exp a38 $  */
/******************************************************************/
/* This module is concerned with accessing map files from a local */
/*  directory  */
/******************************************************************/

#include "mapP.h"
#include <monet/map/Interface.h>
#include <monet/map/File.h>
#include <monet/msh/Script.h>

#include <sys/dir.h>

/***************************************************************/
/*  Convert between String names and Enum values.              */
/***************************************************************/

struct {
	String          name;
	DrawnAs		drawn;
}               DrawnTable[] =
{
	{ "map",  drawnas_map },
	{ "listing", drawnas_list },
	{ NULL, drawnas_map }
};

extern String drawnas2string (d)
	DrawnAs d;
{
	int i;
	for (i=0; DrawnTable[i].name != NULL; i++)
		if (DrawnTable[i].drawn == d)
			return DrawnTable[i].name;
	return "MonetSysError";
}

extern DrawnAs string2drawnas (s)
	String s;
{
	int i;
	for (i = 0; DrawnTable[i].name != NULL; i++)
		if (EQS(s, DrawnTable[i].name))
			return DrawnTable[i].drawn;
	return drawnas_map;
}

extern MonetNetBorder String2MonetNetBorder (string)
     char * string;
{
  if (EQS(string, "round"))
    return netb_round;
  if (EQS(string, "box"))
    return netb_box;
  if (EQS(string, "minimal"))
    return netb_minimal;
  return MonetNetBorderDefault;
}

extern String MonetNetBorder2String (net)
  MonetNetBorder net;
{
  switch (net) {
  case netb_round:
    return  "round";
  default:
  case netb_box:
    return "box";
  case netb_minimal:
    return "minimal";
  }
  /* NOTREACHED */
}

/*****************************************************************************/
/*  Convert a map into a string, this format is suitable for transfer or     */
/*  saving.  Return NULL on error, caller must free the result */
/*****************************************************************************/

#define MAPATTRIBUTE(name, value)  \
	if (value && * value) { \
		(void) sprintf(result+len, "mapset %s ", name); \
		len += strlen (result+len); \
		(void) MonetString2PrintString (result+len, value, 256); \
		len += strlen (result+len); \
		(void) sprintf(result+len, "\n"); \
		len += strlen (result+len); \
	}

#define MAPCOMMAND(name, v)  \
  { char * value = v; \
	if (value && * value) { \
		(void) sprintf(result+len, "%s ", name); \
		len += strlen (result+len); \
		(void) MonetStringCopyToEnd (result+len, value); \
		len += strlen (result+len); \
		(void) sprintf(result+len, "\n"); \
		len += strlen (result+len); \
	} \
	}

#define MAPPRCOMMAND(name, v)  \
  { char * value = v; \
	if (value && * value) { \
		(void) sprintf(result+len, "%s ", name); \
		len += strlen (result+len); \
		(void) MonetString2PrintString (result+len, value, 256); \
		len += strlen (result+len); \
		(void) sprintf(result+len, "\n"); \
		len += strlen (result+len); \
	} \
	}

static unsigned save_map_elements (map, in_result, len, available)
     MonetMap map;
     String *in_result;
     unsigned len;
     unsigned available ;
{
  Drawing d;
  String result;

  result = *in_result;

  MAPCOMMAND("map->primary-label", MonetAttribute2String (map->primary_label));
  MAPCOMMAND("map->secondary-label",
	     MonetAttribute2String (map->secondary_label));
  MAPCOMMAND("map->scale-lines", MonetMap_scale_lines (map));
  MAPCOMMAND("map->drawnas", drawnas2string(map->drawnas));
  MAPCOMMAND("map->net-border", MonetNetBorder2String (map->net_border));
  MAPPRCOMMAND("map->domain", Map_get_domain (map));
  if (map->background_shape)
    MAPCOMMAND("map->backing",
	       MonetShape2String (map->background_shape));
  MAPCOMMAND("map->connectors", ConnectorState2String (map->connectors));

  FOR_DRAWINGLIST(d, Map_elements (map)) {
    (void) Drawing_write (d, &result, &len, &available);
  } END_FORLIST;

  result[len] = NULL;
  *in_result = result;
  return len;
}

static unsigned save_map_links (map, in_result, len, available)
     MonetMap map;
     String *in_result;
     unsigned len;
     unsigned available ;
{
  Drawing d;
  String result;

  result = *in_result;
  (void) sprintf ((String) result+len, "\n# Topology Information\n");
  len += strlen ((String) result+len);
  FOR_DRAWINGLIST(d, Map_elements (map)) {
    MonetEntity from, to, e;
    e = ENTITY(d);
    if (MonetEntity_isInterface (e)) {
      from = from_interface (e);
      to =   to_interface (e);
      if (from || to) {
	char * interface_name;
	if (available <= len + MAXMAPLINELEN) {
	  available = (unsigned) (len) + MAPSPACEALLOCATION;
	  /*  !!! No recovery  */
	  result = MonetRealloc (result, available);
	}
	interface_name = MonetNewString (MonetEntity_title (e));
	(void) sprintf ((String) result+len, "link \"%s\" \"%s\" \"%s\"\n",
			interface_name,
			MonetEntity_title (from),
			MonetEntity_title (to));
	len += strlen ((String) result+len);
	MonetFree (interface_name);
      }
    }
  } END_FORLIST;
  result[len] = NULL;
  *in_result = result;
  return len;
}

extern String Map2String (map)
     MonetMap map;
{
  char * result;
  unsigned len = 0;
  unsigned available ;

  result = NULL;
  len = 0;
  available = MAPSPACEALLOCATION;
  if (result = MonetRealloc (result, available)) {
    (void) sprintf(result+len, "# %s [%s]\n", MonetPrettyTimeStamp (),
		   Map_title (map));
    len += strlen (result+len);
    (void) sprintf(result+len, "# This is a monetsh script\n");
    len += strlen (result+len);
    (void) sprintf ((String) result+len,"map->loading ");
    len += strlen ((String) result+len);
    (void) MonetString2PrintString (result+len, Map_title (map), 256);
    len += strlen ((String) result+len);
    result[len] = '\n';
    ++len;
    
    len = save_map_elements (map, &result, len, available);
    len = save_map_links (map, &result, len, available);
  }
  return result;
}

/*****************************************************************************/
/*  Load map  */
/*****************************************************************************/

static MonetBool map_load (map, filename)
	MonetMap map;
	String filename;
{
  MonetBool okay, change;


  if (! filename || ! *filename)
    filename = Map_title(map);
  filename = Map_filename (filename);

  MONETDEBUG(2, ( "Loading map [%s] from file [%s]",
		 Map_title(map), filename));

  if (change = isDrawingAllowed (map))
    Map_disable_set_changed (map);
  MapCurrent_set (map, False);
  if (okay = Script_source_file (filename, "", "Couldn't load map")) {
    MonetPrintfInform ("Loaded map [%s]", Map_title (map));
    Map_free_backup (map);
    if (change) {
      Map_enable_set_changed (map);
    }
    Map_set_resized (map);
    map->saved = True;
  } else if (change) {
    Map_enable_set_changed (map);
  }
  return okay;
}

/*****************************************************************************/
/*****************************************************************************/

extern MonetMap Map_get_map_for_entity (entity)
     MonetEntity entity;
{
  MonetMap m;
  FORALLMAPS (m)
    if (m->entity && m->entity == entity)
      return m;
  return NULL;
}

/***************************************************************/
/*  Given an entity get a map  */
/***************************************************************/

extern MonetMap MapEntity_make (entity)
     MonetEntity entity;
{
  MonetMap map, cm;

  if (map = Map_get_map_for_entity (entity)) {
    return map;
  }
  if (entity -> drawing) {
    Drawing d;
    FORALL(d, entity->drawing, next)
      if (d->submap) {
	return d->submap;
      }
  }
  cm = MapCurrent_get();
  if (map = Map_create (entity)) {
    MONETDEBUG(2, ( "Making map reference [%s]",
		MonetEntity_title (map->entity)));
    (void) map_load (map, NULSTR);
  }
  MapCurrent_set(cm, False);
  return map;
}

/***************************************************************/
/* Load a map, should be able to load from a file, or remotely */
/***************************************************************/

extern MonetMap Map_make (map, filename)
	MonetMap map;
	String filename;
{
  MonetEntity entity_class, entity;
  String title, name;
  MonetBool isclass;

  name = Map_filename2name (filename);
 
 if (entity = parse_title (name, &title, &entity_class, &isclass)) {
    MonetPrintfInform ("MonetEntity found [%s]", MonetEntity_title (entity));
  } else if (entity_class == NULL || title == NULL) {
    MonetPrintfError("Can't parse title [%s]", name);
    return NULL;
  } else if (isclass) {
    MonetPrintfError("Can't create map from class [%s]", name);
    return NULL;
  } else if ( ! MonetEntity_isMapClass (entity_class)) {
    MonetPrintfError ("[%s] is not a map class",
		      MonetEntity_title (entity_class));
    return NULL;
  } else if (!(entity = MonetEntity_create_instance (entity_class, title))) {
    MonetPrintfError ("Could not create entity [%s:%s]",
		MonetEntity_title (entity_class), title);
    return NULL;
  }

  if (entity && (map = Map_get_map_for_entity (entity))) {
    /*  Check to see if this map file is already loaded  */
    /*  Currently disallows this, could arrange for all  */
    /*  changes made to one copy are simultaneously made */
    /*  to the other.  */
    MonetPrintfInform ("MonetMap [%s] is already loaded",
		       MonetEntity_title (entity));
  } else {
    if (map == NULL) {
      map = MapCurrent_get ();
    }

    if (! map || Map_is_loaded(map) || Map_classesOnly (map)) {
      MonetMap previous = MapCurrent_get();
      map = Map_create (entity);
      MapCurrent_set (previous, False);
    } else if (map->entity == NULL) {
      map->entity = entity;
    }
  }
  MapCurrent_set (map, True);
  MonetPrintfInform ("Current map is now [%s]", Map_title (map));

  return map;
}

/***************************************************************/
/* Load a map, should be able to load from a file, or remotely */
/***************************************************************/

extern MonetBool Map_load (map, filename)
	MonetMap map;
	String filename;
{
  if (map = Map_make (map, filename)) {
    return map_load (map, filename);
   }
  return False;
}

/***************************************************************/
/*  Write a map to a file */
/***************************************************************/

static int write_map (map, file)
     MonetMap map;
     char * file;
{
  FILE * f;
  char * result;
  int okay;

  if (Map_classesOnly(map)) {
    MonetPrintfError ("Can't write/save class map");
    return 0;
  }
  if ( ! (result = Map2String (map))) {
    MonetPrintfError ("Could not create string from map [%s]",
		      Map_title (map));
    okay = 0;
  } else if (f = Map_fopen_save (file, Map_title(map))) {
    if (fputs (result, f) == 0) {
      okay = 1;
    } else {
      fflush (f);
      okay = 2;
    }
    fclose(f);
    MonetFree (result);
  } else {
    okay = 0;
  }
  return okay;
}

extern MonetBool Map_write (map, file)
     MonetMap map;
     char * file;
{
  switch (write_map (map, file)) {
  default:
    break;
  case 1:
    MonetPrintfError ("Error while writing map [%s]", Map_title (map));
    break;
  case 2:
    MonetPrintfInform ("Wrote map [%s] to file [%s]",
		       Drawing_title (map), file);
    return True;
  }
  return False;
}

/***************************************************************/
/*  Save a map  */
/***************************************************************/

extern MonetBool Map_save (map)
     MonetMap map;
{
  switch (write_map (map, NULSTR)) {
  default:
    break;
  case 1:
    MonetPrintfError ("Error while saving map [%s]", Map_title (map));
    break;
  case 2:
    MonetPrintfInform ("Saved map [%s]", Drawing_title (map));
    map->saved = True;
    return True;
  }
  return False;
}


/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/saveload.c,v 1.1 1991/07/22 11:36:41 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

