/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*  This module implements a set of standard operations on maps, that can    */
/*  be called from a monet script (or command line).  It is built upon the   */
/*  script library in monet/lib/general which supports loops, conditionals,  */
/*  pipes etc.                                                               */
/*  A lot of this stuff can now be implemented using monetsh alone, thus     */
/*  a lot is pretty redundent  */
/*****************************************************************************/

#include "mapP.h"
#include <monet/msh/Script.h>
#include <monet/map/Interface.h>
#include <monet/map/File.h>

/***************************************************************************/
/*  Definitions  */
/***************************************************************************/

static String   Maps_author  = NULL	;
static String   Maps_contact = NULL	;
static String   Maps_source  = NULL	;

/******************************************************************/
/*  Draw a text map  */
/******************************************************************/

extern void Map_draw_text (map)
     MonetMap map;
{
  if (map == NULL) {
    MonetSysError ("Map  is NULL");
  } else if (Draw_start_text (Map_title (map))) {
    Map_draw (map);
  }
}

/******************************************************************/
/*  Draw a postscript Map  */
/******************************************************************/

extern String Map_draw_postscript (map, prologue, file,
				   author, contact, source)
     MonetMap map;
     String prologue, file, author, contact, source;
{
  String title;
  Position xl,xr, yl,yr;

  if (map == NULL) {
    MonetSysError ("Map Area is NULL");
  } else {
    title = Map_title (map);
    if (Map_corners (map, &xl,&yl, &xr,&yr, True) == False) {
      MonetPrintfWarning ("Map [%s] is empty", title);
    } else {
      if (file == NULL)
	file = title;
      if (file = Draw_start_postscript (prologue, title, file,
				 author, contact, source,
				 xl,xr, yl, yr)) {
	Map_draw (map);
	return file;
      }
    }
  }
  return NULL;
}

/***************************************************************************/
/*  Position and set width and height of a drawing  */
/***************************************************************************/

MONETCALLBACK(do_current_map)
     MonetMap map;
     if (map = MapCurrent_get ()) {
       result = Map_nice_name (map);
     } else {
       result = "No current map";
     }
MONETEND

MONETCALLBACK(do_current_map_unloaded)
     MonetMap map;
     if (map = MapCurrent_get ()) {
       if (Map_is_loaded (map)) {
	 result = NULL;
       } else if (Map_classesOnly(map)) {
	 result = NULL;
       } else {
	 result = "";
       }
     } else {
       result = NULL;
     }
MONETEND


MONETCALLBACK(do_num_drawings)
     MonetMap map;

     if (map = MonetString2Map (argv[1])) {
       result = Monetint2String (DrawingList_sizeOf(map->elements));
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_free_map)
     MonetMap map;
     if (map = MonetString2Map (argv[1])) {
       if (DrawingList_sizeOf(map->elements) > 0) {
	 MonetSysError("Can't free a map with drawings on it");
       } else {
	 Map_free (map);
       }
     } else {
       MonetPrintfError ("No such map [%s]", argv[1]);
     }
MONETEND

MONETCALLBACK(do_change_map)
     MonetMap m;

     if (argc == 1) {
       MapCurrent_swap ();
       RETURNSC;
     }
     m = MonetString2Map (argv[1]);
     if (m) {
       if (Map_is_loaded(m)) {
	 MonetPrintfInform ("Current map changed to [%s]", Map_title (m));
       } else if (Map_classesOnly(m)) {
	 MonetPrintfInform ("New map is a class map");
       } else {
	 MonetPrintfInform ("New map is unloaded");
       }
       MapCurrent_set (m, True);
     } else {
       MonetPrintfError ("Could not find map [%s]", argv[1]);
     }
MONETEND

static void discard_map ()
{
  (void) Map_discard (MapCurrent_get ());
}

MONETCALLBACK(do_discard_map)
     MonetMap map;

     if (argc == 1) {
       if (map = MapCurrent_get ()) {
       } else {
	 MonetPrintfError ("No current map selected");
       }
     } else if (! (map = MonetString2Map (argv[1]))) {
       MonetPrintfError ("Could not find map [%s]", argv[1]);
     }

     if (map) {
       if (map->entity) {
	 Drawing d;
	 /*  Check to see if it has drawings referencing it on other maps  */
	 FORALL(d, map->entity->drawing, next) {
	   if (d->map != map) {
	     MonetPrintfError ("Cannot discard [%s], referenced in other maps",
			       Map_title (map));
	     result = NULL;
	     RETURNSC;
	   }
	 }
       }
       if (map->saved) {
	 (void) Map_discard (map);
       } else {
	 Monet_confirm (discard_map,"Do you really want to discard this map?");
       }
     }
MONETEND

/*****************************************************************************/
/*  Print/dump maps  */
/*****************************************************************************/

MONETCALLBACK(do_dump_text)
     ISLOADEDORCLASS;
     (void) Map_draw_text (CurrentMap);
MONETEND

MONETCALLBACK(do_dump_ps)
     char * file, *prolog;
     ISLOADEDORCLASS;
     if (prolog = argv[1]) {
       file = argv[2];
     } else {
       file = NULL;
     }
     result = Map_draw_postscript (CurrentMap, prolog, file,
				   Maps_author, Maps_contact, Maps_source);
MONETEND

/*****************************************************************************/

/*****************************************************************************/

MONETCALLBACK(do_stat)
     MonetMap          m;
     unsigned n = 0;

     MonetPrintfnl ("Status Report");
     MonetPrintfnl ("Local maps in directory [%s]",
		    Monet_map_directory (NULSTR));
     FORALLMAPS (m) {
       ++n;
       MonetPrintf ("%u", n);
       if (Map_classesOnly (m)) {
	 MonetPrintf (" Class Map [%u classes]",
		       DrawingList_sizeOf (Map_elements (m)));
       } else if (Map_is_loaded(m)) {
	 if (m->saved)
	   MonetPrintf (" Saved [%s]", Map_title (m));
	 else
	   MonetPrintf (" Unsaved [%s]", Map_title (m));
	 MonetPrintf (" [%u drawings]",
		       DrawingList_sizeOf (Map_elements (m))); 
       } else {
	 MonetPrintf (" UnLoaded");
       }
       if (m == MapCurrent_get())
	 MonetPrintf (" [Current]");
       if (m -> mapWidget)
	 MonetPrintf (" [Widget]");
       MonetPrintfnl ("");
     }
     if (n == 0) {
       MonetPrintfnl ("No maps loaded just now");
     }
MONETEND

MONETCALLBACK(do_enable_drawing)
     ISLOADED;
     Map_enable_set_changed (CurrentMap);
MONETEND

MONETCALLBACK(do_disable_drawing)
     ISLOADED
     if (isDrawingAllowed (CurrentMap)) {
       Map_disable_set_changed (CurrentMap);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_map_dir)
     result = Monet_map_directory (argv[1]);
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void MapScript_init ()
{
  char * buffer;

  (void) MonetMember_module_class ("map");

  MonetMember_create ( "free",		do_free_map,
		      "map",
		      "free/delete map");
  MonetMember_create ( "dir",		do_map_dir,
		      "?value",
		      "Set or return current map directory");
  MonetMember_create ( "disable-drawing",		do_disable_drawing,
		      "",
		      "Disable drawing onto the maps");
  MonetMember_create ( "enable-drawing",		do_enable_drawing,
		      "",
		      "Enable drawing onto the maps");
  MonetMember_create ( "discard",		do_discard_map,
		      "?map",
		      "Discard map, no argument given then uses the top map");
  MonetMember_create ( "change",		do_change_map,
		      "?name",
		     "Change current map. If no arguments, swaps the top two");
  MonetMember_create ( "status",		do_stat,
		      "",
		      "A quick summary of the state of the program");
  MonetMember_create ( "list",		do_stat,
		      "",
		      "A quick summary of the state of the program");
  MonetMember_create ( "dump-ps",		do_dump_ps,
		      "?A4|A3 ?filename",
		      "dump map as postscript to file");
  MonetMember_create ( "dump-text",	do_dump_text,
		      "",
		      "dump map as text to file");
  MonetMember_create ( "current",		do_current_map,
		      "",
		      "return name of current map");
  MonetMember_create ( "num-drawings",		do_num_drawings,
		      "map",
		      "return number of drawings on map");
  MonetMember_create ( "current-map-unloaded",	 do_current_map_unloaded,
		      "",
		      "return true if there is a current map which is \
loaded and is not a class map");
  MonetConfig_create ("-maps",  "map->dir", NULL,
		      "Directory where maps are kept");

  /* -- Map author, contact and source of maps, this information
     -- is drawn on the postscript map output  */

  Maps_author = MonetNewString (Monet_user_id ());
  Maps_contact = MonetNewString (Monet_user_address (NULSTR));
  buffer = MonetMalloc (5 + MonetStrlen (Monet_host_name ()));
  (void) sprintf (buffer, "ftp %s", Monet_host_name ());
  Maps_source = buffer;


  MonetMember_create_write ( "author", &Maps_author,
			   "Author of the maps");
  MonetMember_create_write ( "contact", &Maps_contact,
			   "Who to contact about maps");
  MonetMember_create_write ( "data-source", &Maps_source,
			   "Where to get the maps from");
  MonetConfig_create ("-map-source",  "map->data-source", NULL, "");
  MonetConfig_create ("-map-author",  "map->author", NULL, "");
  MonetConfig_create ("-map-contact", "map->contact", NULL, "");
}

extern void MapScript_finish ()
{
  
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/map/RCS/script.c,v 1.1 1991/07/22 11:37:27 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
