/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/sizing.c,v 1.1 1991/07/16 11:08:23 a38 Exp a38 $  */
/******************************************************************/
/*  This is used for finding the correct sizes of drawings as they*/
/*  should appear on screen.  Because maps can be inside maps,    */
/*  drawings are not all drawn to the same scale, hence the need  */
/*  for this module.  */
/******************************************************************/

#include <monet/map/Drawing.h>

/******************************************************************/
/* Adjust position depending on scale of map */
/******************************************************************/

#define SCALE(a, offset, factor)   (Dimension) ((offset) + (a) * (factor))
#define DESCALE(a, offset, factor) (Dimension) (((-offset) + (a)) / (factor))

extern void Scale2ActualSize (fx, fy, x,y, newx, newy)
     float fx,fy;
     Dimension x,y, *newx, *newy;
{
  *newx =  SCALE(x,0,fx);
  *newy =  SCALE(y,0,fy);
}

static void ActualPosition2Scale (map, x,y, newx, newy)
     MonetMap map;
     Position x,y, *newx, *newy;
{
  *newx =  (Position) DESCALE(x,map->x_centre,map->x_scale);
  *newy =  (Position) DESCALE(y,map->y_centre,map->y_scale);
}

static void ActualSize2Scale (map, x,y, newx, newy)
     MonetMap map;
     Dimension x,y, *newx, *newy;
{
  *newx =  DESCALE(x,0,map->x_scale);
  *newy =  DESCALE(y,0,map->y_scale);
}

/*****************************************************************/
/*  Find the position of a drawing that is marked  */
/*  Returns position relative to result map */
/*  and dimensions on absolute map  */
/*****************************************************************/

static Drawing map_get_marked (map, x_map, y_map, x_win, y_win, w_win, h_win)
     MonetMap *map;
     Position *x_map, *y_map, *x_win, *y_win;
     Dimension *w_win, *h_win;
{
  Drawing s, d;

  FOR_DRAWINGLIST(s, Map_elements (*map)) {
    if (Drawing_is_marked (s, *x_map,*y_map)) {
      if (s->submap && s->submap_visible) {
	Position xl, yl, xr, yr;
	Map_corners (s->submap, &xl, &yl, &xr, &yr, False);
	if (xr && yr) {
	  float w, h, factor;
	  w = (float) (WIDTH(s)-2) / (float) xr;
	  h = (float) (HEIGHT(s)-2) / (float) yr;
	  if (w < h)
	    factor = w;
	  else
	    factor = h;
	  *x_map = (Position) (*x_map - X(s)-1) / factor;
	  *y_map = (Position) (*y_map - Y(s)-1) / factor;
	  *map = s->submap;
	  if (d = map_get_marked
	      (map, x_map,y_map, x_win, y_win, w_win, h_win)) {
	    if (w_win && h_win) {
	      *w_win *= factor;
	      *h_win *= factor;
	    }
	    if (x_win && y_win) {
	      *x_win = *x_win * factor + X(s) + 1;
	      *y_win = *y_win * factor + Y(s) + 1;
	    }
	    return d;
	  }
	}
      }
      if (w_win && h_win) {
	*w_win = WIDTH(s);
	*h_win = HEIGHT(s);
      }
      if (x_win && y_win) {
	*x_win = X(s);
	*y_win = Y(s);
      }
      return s;
    }
  } END_FORLIST;

  return NULL;
}
/*****************************************************************/
/*  Takes window coordinates and returns a map they correspond   */
/*  with the coordinates for that map.  */
/*****************************************************************/

static Drawing map_actual2map (map, x_map, y_map, w_map, h_map)
     MonetMap *map;
     Position *x_map, *y_map;
     Dimension *w_map, *h_map;
{
  Drawing s, d;
  MonetMap original_map = *map;
  
  FOR_DRAWINGLIST(s, Map_elements (*map)) {
    if (Drawing_is_marked (s, *x_map,*y_map)) {
      if (s->submap && s->submap_visible) {
	Position xl, yl, xr, yr;
	Map_corners (s->submap, &xl, &yl, &xr, &yr, False);
	if (xr && yr) {
	  float w, h, factor;
	  w = (float) (WIDTH(s)-2) / (float) xr;
	  h = (float) (HEIGHT(s)-2) / (float) yr;
	  if (w < h)
	    factor = w;
	  else
	    factor = h;
	  *x_map = (*x_map - X(s)-1) / factor;
	  *y_map = (*y_map - Y(s)-1) / factor;
	  *map = s->submap;
	  *w_map /= factor;
	  *h_map /= factor;
	  
	  if (d = map_actual2map (map, x_map,y_map, w_map, h_map)) {
	    if (d->submap && d->submap_visible)
	      *map = original_map;
	    return d;
	  }
	}
      }
      return s ;
    }
  } END_FORLIST;
  return NULL;
}

/*****************************************************************/
/*  Select a drawing or if none is marked lets the user create a */
/*  rectangle, every drawing inside will then be marked.         */
/*  Will select the resized drawing.                             */
/*****************************************************************/

static MonetBool map_select_in_box (map, xloc, yloc, width, height)
     Position xloc, yloc;
     Dimension width, height;
     MonetMap map;
{
  Drawing s, sub;
  unsigned num = 0, e;
  MonetBool change = False, result = True;

  e = DrawingList_sizeOf(map->elements);
  FOR_DRAWINGLIST(s, map->elements) {
    if (Drawing_in_box (s, xloc, yloc, width, height)) {
      sub = s;
      ++num;
    }
  } END_FORLIST;
  /*  --  this is a hack, the idea is that if we are redrawing a lot
      --  then redraw the whole screen.  */
  if ((num == 0 && numberOf_Selection(map) > 30) ||
      (e > 1 && (num > 30 || e < 2 * num) && ! Map_classesOnly(map))) {
    if (change = isDrawingAllowed (map)) {
      Map_disable_set_changed (map);
    }
  }
  if (num == 1 && sub->submap && sub->submap_visible &&
      xloc > X(sub) && yloc > Y(sub) &&
      xloc + width < X(sub) + WIDTH(sub) &&
      yloc + height < Y(sub) + HEIGHT(sub))
    {
      Position xl, yl, xr, yr;
      s = sub;
      Map_corners (s->submap, &xl, &yl, &xr, &yr, False);
      if (xr && yr) {
	float w, h, factor;
	w = (float) (WIDTH(s)-2) / (float) xr;
	h = (float) (HEIGHT(s)-2) / (float) yr;
	xloc = xloc - X(s)-1;
	yloc = yloc - Y(s)-1;
	if (w < h)
	  factor = w;
	else
	  factor = h;
	xloc /= factor;
	yloc /= factor;
	width  /= factor;
	height /= factor;
	if (! map_select_in_box (sub->submap, xloc, yloc, width, height))
	  toggle_selection (sub);
      } else
	toggle_selection (sub);
    } else if (num > 0) {
      FOR_DRAWINGLIST(s, map->elements) {
	if (Drawing_in_box (s, xloc, yloc, width, height))
	  if (Drawing_isClass (s)) {
	    if (Drawing_current_class() == s)
	      unmark_class ();
	    else
	      mark_class (s);
	  } else
	    toggle_selection (s);
      } END_FORLIST;
    } else {
      free_selection (map);
      result = False;
    }
  if (change) {
    Map_enable_set_changed (map);
    Map_request_redraw (map);
  }
  return result;
}

/******************************************************************/
/*  external routines  */
/******************************************************************/

extern Drawing Map_get_marked (map, x_map, y_map, x_win, y_win, w_win, h_win)
     MonetMap *map;
     Position *x_map, *y_map, *x_win, *y_win;
     Dimension *w_win, *h_win;
{
  Drawing d;
  MonetMap m = *map;

  ActualPosition2Scale	(*map, *x_map, *y_map, x_map, y_map);
  d = map_get_marked	(map, x_map, y_map, x_win, y_win, w_win, h_win);
  if (x_win && y_win) {
    /* Scale to actual position */
    *x_win = SCALE(*x_win,0, m->x_scale);
    *y_win = SCALE(*y_win,0, m->y_scale);
  }
  if (w_win && h_win)
    /* Scale to actual dimension */
    Scale2ActualSize ((*map)->x_scale, (*map)->y_scale,
		      *w_win, *h_win, w_win, h_win);
  return d;
}

extern Drawing Map_actual2map (map, x_map, y_map, w_map, h_map)
     MonetMap *map;
     Position *x_map, *y_map;
     Dimension *w_map, *h_map;
{
  Drawing d;
  
  ActualPosition2Scale (*map, *x_map, *y_map, x_map, y_map);
  ActualSize2Scale (*map, *w_map, *h_map, w_map, h_map);
  d = map_actual2map (map, x_map, y_map, w_map, h_map);
  
  return d;
}

extern MonetBool Map_select_in_box (map, xloc, yloc, width, height)
     Position xloc, yloc;
     Dimension width, height;
     MonetMap map;
{
  ActualPosition2Scale (map, xloc, yloc, &xloc, &yloc);
  ActualSize2Scale (map, width, height, & width, & height);
  return map_select_in_box (map, xloc, yloc, width, height);
}

/******************************************************************/
/******************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/sizing.c,v 1.1 1991/07/16 11:08:23 a38 Exp a38 $  */
/******************************************************************/
/******************************************************************/

