/****************************************************************/
/****************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/interface.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/****************************************************************
  A primitive undo facility, drawings are simply 'saved' to a
  string in 'monetsh' script format, restoration simply
  involves executing the script.
  ****************************************************************/

#include <monet/map/Drawing.h>
#include <monet/map/Lines.h>
#include <monet/map/Interface.h>
#include "mapP.h"

/****************************************************************/
/*  Definitions  */
/****************************************************************/

struct monet_backup {
  int number;
  char * data;
  MonetBackup next;
};

/************************************************************************/
/*  Create and free drawings  */
/************************************************************************/

extern MonetBackup MonetBackup_create ()
{
  MonetBackup backup;
  if (backup = MonetNew (struct monet_backup)) {
    backup -> data = NULL;
    backup->number = 0;
  }
  return backup;
}

extern void MonetBackup_free (backup)
     MonetBackup backup;
{
  while (backup) {
    MonetFree (backup->data);
    MonetFree (backup);
    backup = backup->next;
  }
}

extern void Map_free_backup (map)
     MonetMap map;
{
  MonetBackup_free (map->backup);
  map -> backup = NULL;
}

/************************************************************************/
/*  Write the drawing description to a string  */
/************************************************************************/

extern unsigned Drawing_write (d, in_result, in_len, in_available)
     Drawing d;
     String *in_result;
     unsigned *in_len, *in_available;
{
  String result;
  MonetShape shape;
  Position        x, y;
  Dimension       width, height;
  MonetEntity entity;
  unsigned len, available;

  result = *in_result;
  len    = *in_len;
  available = *in_available;

  entity = ENTITY(d);
  x = X(d);
  y = Y(d);
  width = WIDTH(d);
  height = HEIGHT(d);

  if (available  <  len + MAXMAPLINELEN) {
    available = (unsigned) (len) + MAPSPACEALLOCATION;
    /*  !!! No recovery  */
    result = MonetRealloc (result, available);
  }
  (void) sprintf ((String) result+len, "draw ");
  len += strlen ((String) result+len);
  (void) MonetString2PrintString (result+len, Drawing_title (d), 256);
  len += strlen ((String) result+len);
  if (MonetEntity_isClass (entity)) {
    /* !!! Should never happen, should never try to save a class */
    shape = NULL;
  } else if (d->shape != MonetEntity_shapeOf (MonetEntity_classOf (entity))){
    shape = d->shape;
  } else {
    shape = NULL;
  }
  if (d->given_label) {
    result[len] = ' ';
    ++len;
    (void) MonetString2PrintString (result+len, d->given_label, 256);
    len += strlen ((String) result+len);
  } else if (shape) {
    (void) sprintf ((String) result+len, " \"\" ");
    len += strlen ((String) result+len);
  }
  if (shape) {
    result[len] = ' ';
    ++len;
      (void) MonetString2PrintString (result+len,
				      MonetShape2String (d->shape), 256);
    len += strlen ((String) result+len);
  }
  (void) sprintf ((String) result+len, " %d %d %u %u\n",x, y, width, height);
  len += strlen ((String) result+len);

  *in_result = result;
  *in_len = len;
  *in_available = available;

  return len;
}

/************************************************************************/
/*  MonetBackup coordinates  */
/************************************************************************/

static MonetBackup start_backup (map)
     MonetMap map;
{
  MonetBackup backup;

  backup = MonetBackup_create ();
  backup->next = map->backup;
  map -> backup = backup;
  return backup;
}

extern void Map_backup_selected (map)
     MonetMap map;
{
  Drawing d;
  MonetBackup backup;
  unsigned len = 0, available = 0;
  char * data = NULL;

  backup = start_backup (map);
  MonetPrintfInform
    ("Backing up selected drawings, Undo to restore positions");

  data = backup->data;
  FOR_SELECTIONS (map, d) {
    ++ backup->number;
    Drawing_write (d, &data, &len, &available);
  }
  backup->data = data;
  MONETDEBUG(2, ( "Backed up [%s]", map->backup->data));
}

extern void Drawing_backup (d)
     Drawing d;
{
  MonetMap map = Drawing_mapOf (d);
  MonetBackup backup;
  unsigned len = 0, available = 0;
  char * data;

  backup = start_backup (map);

  MonetPrintfInform
    ("Backing up selected drawings, Undo to restore positions");

  ++ backup->number;
  data = backup->data;
  Drawing_write (d, &data, &len, &available);
  backup->data = data;

  MONETDEBUG(2, ( "Backed up [%s]", map->backup));
}

extern void Map_backup (map)
     MonetMap map;
{
  Drawing d;
  MonetBackup backup;
  unsigned len = 0, available = 0;
  char * data;

  backup = start_backup (map);
  MonetPrintfInform ("Backing up Map, Undo to restore positions");

  data = backup->data;
  FOR_DRAWINGLIST (d, Map_elements (map)) {
    ++ backup->number;
    Drawing_write (d, &data, &len, &available);
  } END_FORLIST;
  backup->data = data;
  MONETDEBUG(2, ( "Backed up [%s]", map->backup));
}

extern void Map_restore (map)
     MonetMap map;
{
  if (map->backup) {
    MonetshExecute (map->backup->data, NULL, False);
    MonetFree (map->backup->data);
    MonetFree (map->backup);
    map->backup = map->backup->next;
/*    Map_request_redraw (map); */
  }
}

/****************************************************************/

/****************************************************************/

MONETCALLBACK(do_restore)
     ISLOADED;
     Map_restore (CurrentMap);
MONETEND

MONETCALLBACK(do_free_backup)
     ISLOADED;
     Map_free_backup(CurrentMap);
MONETEND

MONETCALLBACK(do_dump)
     FILE *f;
     char * file = argv[1];
     MonetBackup backup;
     ISLOADED;
     if (file == NULL) {
       f = stdout;
     } else {
       f = fopen (file, "w");
     }
     if (f) {
       FORALL(backup, CurrentMap->backup, next) {
	 fprintf (f, "%s", backup->data);
       }
       fclose (f);
     } else {
       MonetPrintfError ("Sorry could not open [%s]", file);
     }
MONETEND

/****************************************************************/
/*  Init and Finish  */
/****************************************************************/

extern void MonetMap_undo_init ()
{
  (void) MonetMember_module_class ("map");

/*
  MonetMember_create ("backup",		do_backup,
		      "pattern",
		      "Copies all selected drawings into backup buffer");
*/
  MonetMember_create ("undo",	do_restore,
		      "",
		      "Restores drawings in backup buffer");
  MonetMember_create ("dump-undo",	do_dump,
		      "?filename",
		      "Dumps all the recorded undo information into a file");
  MonetMember_create ("free-backup",	do_free_backup,
		      "",
		      "Frees all the backup information stored for a map");
}


extern void MonetMap_undo_finish ()
{
  
}

/****************************************************************/
/****************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/interface.c,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $ */
/****************************************************************/
/****************************************************************/


