/***************************************************************************/
/* $Header$ */
/***************************************************************************/

#ifndef MONET_CHANNEL_H
#define MONET_CHANNEL_H

/**************************************************************************
  Select Scheduler routines                                              
  *************************************************************************
  The following routines must be supplied by some external module, this  
  module will be responsible for listening to sockets using 'select' and 
  calling the supplied handler routines when a connection becomes pending
  !!! Needs to define an event loop routine  
  **************************************************************************/

typedef struct monet_channel * Channel;
#define NULL_CHANNEL (Channel) 0

/**************************************************************************
  Function types  
  **************************************************************************/

/* Returns false if it wants to end the connection */
typedef void (* MonetFdHandler) PR ((int, caddr_t));

typedef void (* MonetInputFdCreator)
     PR ((int, MonetFdHandler read, MonetFdHandler write, caddr_t));
typedef void (* MonetInputFdFreer)
     PR ((int));
typedef void (BeforeSelectHandler)
     PR ((int *, fd_set *readbits, fd_set *writebits, fd_set *other,
      struct timeval *timer));
typedef void (AfterSelectHandler)
     PR ((int, fd_set *readbits, fd_set *writebits, fd_set *other));

/**************************************************************************
  Some simple to use comms routines between to applications  
  Currently implemented for tcp/ip only 
  MonetInputFd_create and the creator routines can be be used to change  
  the read and write routines of an existing input fd.                   
  **************************************************************************/

extern void MonetInputFd_create
  PR((int, MonetFdHandler read, MonetFdHandler write, caddr_t));
extern void MonetInputFd_free
  PR ((int));

/* -- Channel handler returns false to disconnect */
typedef MonetBool	(* ChannelHandler)
     PR((Channel, String));
typedef void	(* ChannelDisconnect)
     PR((Channel));
typedef void	(* ChannelAcceptHandler)
     PR((int fd, char * address, caddr_t));
typedef void	(* ChannelAcceptDisconnect)
     PR((int fd, caddr_t));

extern void Channel_help
  PR((void));
extern void	Channel_shutdown
  PR((Channel));
extern void	Channel_free
  PR((Channel));
extern void Channel_free_server
  PR((unsigned port));
extern Channel	Channel_create
  PR((int fid, MonetBool interactive,
   ChannelHandler, ChannelDisconnect, caddr_t data));
extern Channel	Channel_create_client
  PR((unsigned server_port, String server_address, MonetBool interactive,
   ChannelHandler, ChannelDisconnect, caddr_t));
extern int	Channel_create_server
  PR((unsigned server_port, ChannelAcceptHandler, ChannelAcceptDisconnect,
      caddr_t data));
extern void	Channel_flush
  PR((Channel));
extern MonetBool	Channel_end_of_record
  PR((Channel));
extern MonetBool	Channel_send
  PR((Channel, String));
extern MonetBool Channel_send_record
  PR((Channel, String));
extern int	Channel2fid
  PR((Channel));
extern Channel	fid2Channel
  PR((int fid));
extern void	Channel_init
  PR((void));
extern void	Channel_finish
  PR((void));
extern caddr_t	Channel_data
  PR((Channel));
extern unsigned Channel_write_buffered
  PR((Channel));
extern void MonetSelectForever
  PR((BeforeSelectHandler *before, AfterSelectHandler *after));
extern void Monet_set_input_fd_routines
  PR((MonetInputFdCreator create, MonetInputFdFreer free));
extern void Monet_reset_input_fd_routines
  PR((void));


#endif

/***************************************************************************/
/* $Header$ */
/***************************************************************************/
