#ifndef MONET_SCRIPT_TREE_H
#define MONET_SCRIPT_TREE_H

/*****************************************************************************/
/*  C interface that has access to the parse tree itself  */
/*****************************************************************************/

typedef struct monet_tree * MonetTree;

typedef String (* ScriptEnvironmentCallback)
     PR((MonetEntity entity, String caller_name,
	 MonetEntity arg_entity,
	 int argc, MonetTree arg));

#define NULTREE ((MonetTree) NULL)

#define SCRIPTENVIRONMENT(name) \
/*ARGSUSED*/  \
  static String name (entity, caller_name, arg_entity, argc, arg) \
  MonetEntity entity, arg_entity; \
  String caller_name; \
  int argc; \
  MonetTree arg; \
{ String result = "";

/*****************************************************************************/
/*  Lexical tokens, could maybe better use lex                               */
/*****************************************************************************/

typedef enum {
	lex_null = 0,
	/* -- End of commands */
	lex_end = 1,
	/*  */
	lex_number = 2,
	lex_parenthesis = 3,
	lex_call_function = 4,
	lex_string = 5,
	lex_label = 6,
	lex_int = 7,
	lex_end_parenthesis = 8,
	lex_braces = 9,
	lex_end_braces = 10,
	lex_open_array = 11,
	lex_end_array = 12,
	/* -- Unary operators  */
	lex_not = 13,
	lex_var = 14,
	lex_bitwise_not = 15,
	lex_unary_minus = 16,
	lex_unary_plus  = 17,
	lex_inc_dec     = 18,
	lex_question    = 19,
	/* --- All the rest are binary operators */
	/* -- Sequence seperators */
	lex_sep         = 20,
	lex_pipe        = 21,
	lex_and         = 22,
	lex_or          = 23,
	/* -- Arithmetic */
	lex_eq_ne       = 24,
	lex_greater_less = 25,
	lex_sub          = 26,
	lex_add          = 27,
	lex_mul_div      = 28,
	lex_op4          = 29,
	lex_entity_name  = 30,
	lex_get_attribute = 31,
	/* -- redirect out put to file  */
	lex_pipe_out     = 32,
	lex_pipe_in      = 33,
	lex_pipe_append_out = 34,
	/* -- name of variable and fields */
	lex_dot          = 35,
	lex_assign       = 36,
	lex_pipe_append_in = 37,
	lex_shift_left   = 38,
	lex_shift_right  = 39,
	lex_bitwise_or   = 40,
	lex_bitwise_xor  = 41,
	lex_bitwise_and  = 42
	} MonetLex;

#define LEX_UNARY(op)  (((int)(op))<((int)lex_sep) \
			&& ((int)(op)) >= ((int)lex_not))
#define LEX_BINOP(op)  (((int) (op)) >= ((int) lex_sep))
#define LEX_OPEN(op) ((op) == lex_parenthesis || (op) == lex_braces)
#define LEX_NUM (((int) lex_bitwise_and) + 1)

/*****************************************************************************/
/*  The Parse Tree  */
/*****************************************************************************/

struct monet_tree {
#ifdef MonetDebug
  MonetBool debris;
#endif
  /*  The name of the node */
  MonetStr id;
#define LNAME(node) (MonetStr2String((node)->id))
  MonetTree next, arg;
  unsigned argc;
  MonetLex type;
};

/***********************************************************/
/*  Execute parsed script  */
/***********************************************************/

extern void MonetTree_assign
  PR((MonetEntity, MonetTree, char *));

extern String MonetTree_execute
  PR((MonetEntity entity, MonetTree exec, String * args));

extern String MonetTree_execute_qualified
  PR((MonetEntity entity, MonetEntity qualifier,
      MonetTree exec, String * args));

extern String MonetMember_execute
  PR((MonetEntity entity, MonetMember,
      MonetEntity arg_entity, MonetStr id, MonetTree args));

/**********************************************************
  Create environment
  **********************************************************/

extern MonetMember MonetMember_create_environment PR((
	String name, ScriptEnvironmentCallback callback,
	String syntax, String help));

#endif

/***********************************************************/
/***********************************************************/
/* $Header$ */
/***********************************************************/
/***********************************************************/
