/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*  Sets up a set of configuration parameters, mainly from the environement  */
/*  Expects MONETDIR, VER and SYS to be predefined                           */
/*****************************************************************************/

#include <monet/msh/General.h>
#include "monetP.h"
#include "scriptP.h"
#include <sys/dir.h>
#include <sys/stat.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/param.h> /* For MAXHOSTNAMELEN */
#include <netdb.h>
#include <pwd.h>

/****************************************************************************
  Globals
  ****************************************************************************/

static String   UserId;
static String   ProgramName;
static String   LocalHost;
static String   MonetDir;
static String   HomeDir;
static String   UserAddress;

/****************************************************************************

 ****************************************************************************/

extern String Monet_dir (new_value)
     char * new_value;
{
  if (new_value) {
    MonetFree(MonetDir);
    MonetDir = MonetNewString (new_value);
  }
  return MonetDir;
}

/*****************************************************************************/

extern String Monet_home_dir (new_value)
     char * new_value;
{
  if (new_value) {
    MonetFree (HomeDir);
    HomeDir = MonetNewString (new_value);
  }
  return HomeDir;
}

/*****************************************************************************/

extern String Monet_user_id ()
{
  return UserId;
}

/*****************************************************************************/

extern String Monet_program_name ()
{
  return ProgramName;
}

/*****************************************************************************/

extern String Monet_host_name ()
{
  return LocalHost;
}

/*****************************************************************************/

extern String Monet_user_address (new_value)
     char * new_value;
{
  if (new_value) {
    MonetFree (UserAddress);
    UserAddress = MonetNewString (new_value);
  }
  if (! UserAddress) {
    MonetSysError ("Could not allocate space for user address");
  }
  return UserAddress;
}

/****************************************************************************
  Some system access functions, maybe these should be 
  ****************************************************************************/

extern String Monet_software_source ()
{
  return "ftp nikhefh.nikhef.nl";
}

extern String Monet_software_title ()
{
  return "Monet";
}

extern String Monet_software_author ()
{
  return "James Barr";
}

extern String Monet_software_contact ()
{
  return "a38@nikhef.nl";
}

extern String Monet_software_version ()
{
  return VER;
}

extern String Monet_system_type ()
{
#ifdef SYS
  return SYS;  /*  System Type, must be passed to compiler  */
#else
# ifdef sun
   return "sun";
#else
# ifdef apollo
   return "apollo";
#else
# ifdef ultrix
   return "ultrix";
#else
# ifdef GOULD_NP1
   return "gould";
#else
/*#error  SYS not defined, should be defined in Monet.tmpl */
  return "unknown";
#endif
#endif
#endif
#endif
#endif
}

/*****************************************************************************/
/* Init and Finish */
/*****************************************************************************/

extern void monet_config_init (program_name, directory)
     char * program_name, * directory;
{
  char buffer [MAXHOSTNAMELEN+32];
  struct hostent *host;
  int uid;
  struct passwd *pass;
  String monet_dir, home_dir;

  ProgramName = program_name;

  if (monet_dir = getenv ("MONETDIR")) {
  } else {
    monet_dir = directory;
  }
  MonetDir = MonetNewString (monet_dir);

  if (home_dir = getenv ("HOME")) {
  } else {
    home_dir = "~";
  }
  HomeDir = MonetNewString (home_dir);

  /*  -- Set the local host name, to full domain name,
      -- gethostbyname will return full name with domain part
      -- if linked with Erik Wassenars modified resolv library
      -- otherwise there will be problems. */
  LocalHost = "unknown";
  if (gethostname (buffer, sizeof(buffer)) != 0) {
    MonetSysError ("Get Host Name Failed, don't know local host");
  } else if (host = gethostbyname (buffer)) {
    LocalHost = MonetNewString (host->h_name);
  } else {
    MonetSysError ("Get Host By Name Failed, don't know local host");
  }

  /* -- Set user id */
  uid = getuid ();
  pass = getpwuid (uid);
  if (pass == NULL) {
    MonetSysError ("Could not find password entry of uid [%d]", uid);
    UserId = MonetNewString ("unknown");
  } else {
    UserId = MonetNewString (pass->pw_name);
  }

  /*  -- Users address */
  UserAddress = (char *) MonetMalloc (MonetStrlen (UserId) +
			      MonetStrlen (LocalHost) + 2);
  (void) sprintf (UserAddress, "%s@%s", UserId, LocalHost);
}

extern void monet_config_finish ()
{
  MonetFree (UserId);
  MonetFree (LocalHost);
  MonetFree (HomeDir);
  MonetFree (MonetDir);
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
