
/****************************************************************************/
/*									    */
/* Author : 	James Barr & Alena Lacova, NIKHEF-H			    */
/* Version :	10-Jul-1991						    */
/*									    */
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/*  Maintain simple sequential lists of objects.  Can be used for any type  */
/*  of object since generic pointers 'caddr_t' are used.  Create a new      */
/*  header file for each new type of list.                                  */
/*  This should really be called ordered-set                                */
/****************************************************************************/

#include "General.h"
#include "OrdSet.h"

/****************************************************************************/
/*  Definitions  */
/****************************************************************************/

#define SIZE 16

struct monet_ordset {
	unsigned size, count, available;
	caddr_t * head;
};

/*****************************************************************************/
/*  Some help information  */
/*****************************************************************************/

static int SetMalloced = 0,  SetFreed = 0, SpaceMalloced = 0, SpaceFreed = 0;
extern void MonetOrdSet_help ()
{
  int s = sizeof (caddr_t);
  MonetPrintfnl ("-- OrdSet");
  MonetPrintfnl ("sizeof ordset     : %d", sizeof(struct monet_ordset));
  MonetPrintfnl ("Malloced          : %d", SetMalloced);
  MonetPrintfnl ("Freed             : %d", SetFreed);
  MonetPrintfnl ("Total space       : %d", SetMalloced - SetFreed);
  MonetPrintfnl ("Data Malloced     : %d", SpaceMalloced * s);
  MonetPrintfnl ("Data Freed        : %d", SpaceFreed * s);
  MonetPrintfnl ("Total data space  : %d\n", (SpaceMalloced - SpaceFreed) * s);
}

/****************************************************************************/

/****************************************************************************/

static MonetBool Find(head, o, size, position)
     caddr_t * head, o;
     unsigned size;
     unsigned * position;
{
  int min = 0, max = size, mid = max/2;
  if(head == NULL)
    return False;
  if (o==NULL) {
    (void) MonetSysError ("caddr_t is NULL\n");
    return False;
  }
  while(1)
    {
      if(o < head[mid])
	{
	  max = mid;
	  mid = (mid + min)/2;
	  if(max == mid)
	    {
	      *position = max;
	      break;
	    }
	}
      else if (o > head[mid])
	{
	  min = mid;
	  mid = (max + mid)/2;
	  if(min == mid)
	    {
	      *position = min + 1;
	      break;
	    }
	}
      else
	{
	  *position = mid;
	  return True;
	}
    }
  return False;
}
extern MonetBool MonetOrdSet_isMember (olist, o)
     MonetOrdSet olist;
     caddr_t o;
{
  unsigned  position ;
 	
  if (olist == (MonetOrdSet) 0) {
    MonetSysError ("MonetOrdSet_insert caddr_t (simple) list is NULL");
  } else if (o == NULL) {
    MonetSysError ("Attempt to insert NULL into MonetOrdSet");
  } else {
    return(Find(olist->head, o, olist->size, &position)) ;
  }
  return False;
}

extern MonetBool MonetOrdSet_extract (olist, o)
     MonetOrdSet olist;
     caddr_t o;
{
  unsigned  position ,i ;

  if (olist == NULL) {
    (void) MonetSysError ("list is null\n");
    return False;
  }

  if(Find(olist->head, o, olist->size, &position)) {
    olist->size --;
    if(olist->size == 0) {
      SpaceFreed += olist->available;
      MonetFree(olist->head);
      olist->head = NULL;
/*    } else if (olist->available > 8 && olist->available >= olist->size * 2) {
      caddr_t * new, *old;
      SpaceFreed += olist->available;
      old = olist->head;
      olist->available = olist->size + 2;
      new = (caddr_t *) MonetMalloc (olist->available);
      olist->head = new;
      for(i=0; i<position; i++)
	new[i] = old[i];
      for(i=position; i<olist->size; i++)
	new[i] = old[i+1];
      MonetFree (old);
      SpaceMalloced += olist->available;*/
    } else {
      for(i=position; i<olist->size; i++)
	olist->head[i] = olist->head[i+1];
    }
    if (olist->count >= position)
      -- (olist->count);
    return True;
  }
  return False;
}

extern MonetOrdSet MonetOrdSet_create ()
{
  MonetOrdSet ordset;

  ordset = MonetNew (struct monet_ordset);
  ++ SetMalloced;
  ordset -> size = 0;
  ordset -> count = 0;
  ordset -> available = 0;
  ordset -> head = (caddr_t *) 0;
  return ordset;
}

extern void MonetOrdSet_reset (ordset)
     MonetOrdSet ordset;
{
  if (ordset && ordset -> head) {
    SpaceFreed += ordset->available;
    MonetFree (ordset->head);
    ordset->head = (caddr_t *) 0;
    ordset->size = 0;
  }
}

extern unsigned MonetOrdSet_sizeof (list)
	MonetOrdSet list;
{
	return list->size;
}

extern MonetBool MonetOrdSet_isEmpty (list)
	MonetOrdSet list;
{
	return list->size == 0;
}

extern caddr_t MonetOrdSet_getN (list, n)
     MonetOrdSet list;
     unsigned n;
{
  if (list && n < list->size)
    return list->head[n];
  return NULL;
}

/* -- A kind of a hack the idea is that is that it gets entry 'ordset_pos'
   -- if not equal to 'previous', otherwise returns the next one.
   -- This is used during iteration, where the current list entry may be
   -- removed and the following ones shifted back a place.
   */
extern caddr_t MonetOrdSet_check_getN (list, ordset_pos, previous)
     MonetOrdSet list;
     unsigned *ordset_pos;
     caddr_t previous;
{
  caddr_t ordset_next;

  ordset_next = MonetOrdSet_getN (list, *ordset_pos);
  if (ordset_next == (caddr_t) previous) { 
    ++ (*ordset_pos);
    ordset_next  = MonetOrdSet_getN (list, *ordset_pos);
  }
  return ordset_next;
}

extern void MonetOrdSet_free (ordset)
	MonetOrdSet ordset;
{
  if (ordset == NULL)
    return;
  if (ordset->size != 0) {
    MonetSysError ("Freeing an object list before it is empty\n");
  }
  MonetFree (ordset -> head);
  SpaceFreed += ordset->available;
  ++ SetFreed;
  MonetFree (ordset);
}

extern unsigned  MonetOrdSet_sizeOf (olist)
	MonetOrdSet olist;
{
	return olist->size;
}

/* Guaranteed to handle null olist or null o properly */

extern void MonetOrdSet_insert(olist, o)
     MonetOrdSet olist;
     caddr_t o;
{
  caddr_t * newspace;
  unsigned position;
  int i;
 	
  if (olist == (MonetOrdSet) 0) {
    MonetSysError ("MonetOrdSet_insert caddr_t (simple) list is NULL");
    return;
  }
  if (o==NULL) {
    MonetSysError ("caddr_t is NULL\n");
    return;
  }
  if(olist->head == NULL) {
    int space;
    space = SIZE * sizeof(caddr_t);
    olist->head = (caddr_t *)MonetMalloc(space);
    SpaceMalloced += space;
    olist->available = SIZE;
    olist->head[0] = o;
    olist->size = 1;
    return;
  }

  if(Find(olist->head, o, olist->size, &position)) {
    MonetSysError ("There is such a member: %d [%s]\n",o, o);
    return;
  }

  olist->size ++;	

  if (olist->size > olist->available) {
    int space;
    olist->available += olist->available / 3 + 1;
    space = (olist->available) * sizeof(caddr_t);
    SpaceFreed += olist->available;
    newspace = (caddr_t *)MonetMalloc(space);
    SpaceMalloced += space;
    bcopy(olist->head, newspace, space);
    MonetFree(olist->head);
    olist->head = newspace;
  }
  
  for(i=olist->size-1; i>position; i--)
    {
      olist->head[i] = olist->head[i-1];
    }
  olist->head[i] = o;
  
  if (olist->count > position)
    ++ (olist->count);
  
/*	printf("&o = %d, o = %s, olist->head[%d] = %d\n",o, o, olist->size-1, olist->head);  */
  
}

extern caddr_t MonetOrdSet_first (l)
     MonetOrdSet l;
{
  if (l == NULL || l-> size == 0)
    return NULL;

  l -> count = 0;
  return l->head [0];
}


/* !!! This is so commonly used it is worth having a special routine */
extern caddr_t MonetOrdSet_second (l)
	MonetOrdSet l;
{
  if (l-> size < 2)
    return NULL;

  l -> count = 1;
  return l->head [1];
}


extern caddr_t MonetOrdSet_next (l)
	MonetOrdSet l;
{
  l->count++;
  if (l->size <= l->count)
    return (NULL);
  return l->head[l->count];
}

/****************************************************************************/
/****************************************************************************/
/*  $Header$  */
/****************************************************************************/
/****************************************************************************/

