/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*  Supports a hashed table of unique strings  */
/*  The entries can be searched for by name or uniquely access */
/*****************************************************************************/

#include "General.h"
#include "StrFifo.h"

/*****************************************************************************/
/*  Globals  */
/*****************************************************************************/

typedef struct monet_fifo_entry * Entry;

struct monet_fifo_entry {
  String string;
  int len;
  Entry next;
};

struct monet_string_fifo {
  char * popped_message;
  Entry top, bottom;
  unsigned size;
};

static int TabMalloced = 0,  TabFreed = 0;
static int EntMalloced = 0,  EntFreed = 0;
static int MesMalloced = 0,  MesFreed = 0;

extern void MonetStrFifo_help ()
{
  MonetPrintfnl ("-- Fifos");
  MonetPrintfnl ("sizeof fifo      : %d", sizeof(struct monet_string_fifo));
  MonetPrintfnl ("Malloced          : %d", TabMalloced);
  MonetPrintfnl ("Freed             : %d", TabFreed);
  MonetPrintfnl ("diff              : %d\n", TabMalloced - TabFreed);
  MonetPrintfnl ("-- Fifo Entries");
  MonetPrintfnl ("sizeof entry      : %d", sizeof(struct monet_fifo_entry));
  MonetPrintfnl ("Malloced          : %d", EntMalloced);
  MonetPrintfnl ("Freed             : %d", EntFreed);
  MonetPrintfnl ("diff              : %d\n", EntMalloced - EntFreed);
  MonetPrintfnl ("-- Fifo Entry Messages");
  MonetPrintfnl ("Malloced          : %d", MesMalloced);
  MonetPrintfnl ("Noted for Freeing : %d", MesFreed);
  MonetPrintfnl ("diff              : %d\n", MesMalloced - MesFreed);
}

extern unsigned MonetStrFifo_size (fifo)
     MonetStrFifo fifo;
{
  return fifo->size;
}

/*****************************************************************************/
/*  Create a new fifo  */
/*****************************************************************************/

extern MonetStrFifo MonetStrFifo_create ()
{
  MonetStrFifo fifo;

  fifo = MonetNew (struct monet_string_fifo);
  ++TabMalloced;
  fifo -> top = NULL;
  fifo -> bottom = NULL;
  fifo -> size = 0;
  fifo->popped_message = NULSTR;
  return fifo;
}

/*****************************************************************************/
/*  Free fifo  */
/*****************************************************************************/

extern void    MonetStrFifo_free (fifo)
     MonetStrFifo fifo;
{
  unsigned len;
  while (MonetStrFifo_pop (fifo, &len));

  ++ TabFreed;
  MonetFree (fifo);
}

/*****************************************************************************/
/*  access entries  */
/*****************************************************************************/

extern MonetBool    MonetStrFifo_push (fifo, str, len)
     MonetStrFifo fifo;
     char * str;
     unsigned len;
{
  Entry entry;
  if (str == NULL || *str == NULL || len == 0)
    return False;
  if (entry = MonetNew (struct monet_fifo_entry)) {
    ++ EntMalloced;
    ++fifo->size;
    entry->len = len;
    entry->string = MonetNewString (str);
    MesMalloced += len;
    entry->next = NULL;
    if (fifo->top) {
      fifo->bottom->next = entry;
      fifo->bottom = entry;
    } else {
      fifo->top = entry;
      fifo->bottom = entry;
    }
    return True;
  }
  return False;
}

extern MonetBool    MonetStrFifo_push_on_top (fifo, str, len)
     MonetStrFifo fifo;
     char * str;
     unsigned len;
{
  Entry entry;
  if (str == NULL || *str == NULL || len == 0) {
    /*  - empty message so ignore it  */
  } else if (entry = MonetNew (struct monet_fifo_entry)) {
    ++ EntMalloced;
    ++fifo->size;
    entry->len = len;
    entry->string = MonetNewString (str);
    MesMalloced += len;
    entry->next = fifo->top;
    if (! fifo->top) {
      fifo->bottom = entry;
    }
    fifo->top = entry;
    return True;
  }
  return False;
}

extern String    MonetStrFifo_pop (fifo, len)
     MonetStrFifo fifo;
     unsigned *len;
{
  Entry entry;

  MonetFree (fifo->popped_message);

  if (entry = fifo->top) {

    fifo->popped_message = entry->string;
    MesFreed += entry->len;

    if (fifo->bottom == fifo->top) {
      fifo->bottom = entry->next; /* -- entry->next == NULL */
    }
    fifo->top = entry->next;
    ++ EntFreed;
    *len = entry->len;
    MonetFree (entry);
    --fifo->size;
  } else {
    fifo->popped_message = NULSTR;
  }
  return fifo->popped_message;
}

extern String    MonetStrFifo_top (fifo, len)
     MonetStrFifo fifo;
     unsigned *len;
{
  if (fifo->top) {
    *len = fifo->top->len;
    return fifo->top->string;
  }
  return NULSTR;
}

/****************************************************************************
  Returns an array of pointers to each string in the array, this is the
  iovec'tor.
  ****************************************************************************/

#include <sys/types.h>
#include <sys/uio.h>

extern MonetBool MonetStrFifo_iovec (fifo, iov, iovcnt)
     MonetStrFifo fifo;
     struct iovec **iov;
     int *iovcnt;
{
  if (fifo->top) {
    if (*iov =
	(struct iovec *) MonetMalloc (sizeof (struct iovec) * fifo->size)) {
      unsigned i = 0;
      Entry entry;
      FORALL (entry, fifo->top, next) {
	(*iov)[i].iov_base = (caddr_t) entry->string;
	(*iov)[i].iov_len  = entry->len;
      }
      *iovcnt = fifo->size;
      return True;
    }
  }
  return False;
}

/****************************************************************************
  Init and finish
  ****************************************************************************/

extern void MonetStrFifo_init ()
{
}

extern void      MonetStrFifo_finish ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
