/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*  Supports a hashed table of unique strings  */
/*  The entries can be searched for by name or uniquely access */
/*****************************************************************************/

#include "General.h"
#include "StringTable.h"

/*****************************************************************************/
/*  Globals  */
/*****************************************************************************/

static MonetStringTable GlobalTable;


struct monet_table_entry {
  unsigned references, hash_place;
  caddr_t data;
  char * name;
  MonetStr next;
};

struct monet_string_table {
  unsigned size, stored;
  MonetStr * entries;
};

static int TabMalloced = 0,  TabFreed = 0;
static int EntMalloced = 0,  EntFreed = 0;
extern void MonetStr_help ()
{
  MonetPrintfnl ("-- String Tables");
  MonetPrintfnl ("sizeof table      : %d", sizeof(struct monet_string_table));
  MonetPrintfnl ("Malloced          : %d", TabMalloced);
  MonetPrintfnl ("Freed             : %d", TabFreed);
  MonetPrintfnl ("diff              : %d\n", TabMalloced - TabFreed);
  MonetPrintfnl ("-- String Table Entries");
  MonetPrintfnl ("sizeof entry      : %d", sizeof(struct monet_table_entry));
  MonetPrintfnl ("Malloced          : %d", EntMalloced);
  MonetPrintfnl ("Freed             : %d", EntFreed);
  MonetPrintfnl ("diff              : %d\n", EntMalloced - EntFreed);
}

/*****************************************************************************/
/*  Get name given string  */
/*****************************************************************************/

extern char * MonetStringTable_get (table, entry)
     MonetStringTable table;
     MonetStr entry;
{
  if (entry)
    return entry->name;
  return NULL;
}

/*****************************************************************************/
/*  Create a new table  */
/*****************************************************************************/

extern MonetStringTable MonetStringTable_create (size)
     unsigned size;
{
  MonetStringTable table;
  unsigned i;

  /*  Size used must be prime  */
  /*  These values take fairly arbitrarily from /usr/games/primes */
  if (size <= 13) {
    size = 13;
  } else if (size <= 31) {
    size = 31;
  } else if (size <= 211) {
    size = 211;
  } else if (size <= 509) {
    size = 509;
  } else if (size <= 1021) {
    size = 1021;
  } else {
    MonetPrintfWarning ("Hash table size bigger than 1021, using 2003");
    size = 2003;
  }
  table = MonetNew (struct monet_string_table);
  ++TabMalloced;
  table -> stored = 0;
  table -> size = size;
  table -> entries =
    (MonetStr *) MonetMalloc (sizeof (MonetStr) * size);
  for (i = 0; i < table->size; ++i)
    table->entries[i] = NULL;
  return table;
}

/*****************************************************************************/
/*  Free table  */
/*****************************************************************************/

extern void    MonetStringTable_free (table)
     MonetStringTable table ;
{
  unsigned i;
  for (i = 0; i < table->size; ++i) {
    MonetStr entry;
    FORALL(entry, table->entries[i], next) {
      ++ EntFreed;
      MonetFree (entry -> name);
      MonetFree (entry);
    }
  }
  MonetFree (table->entries);
  ++ TabFreed;
  MonetFree (table);
}

/*****************************************************************************/
/*  Hash function to find entry  */
/*  Adapted from Aho, Sethi, Ulman red dragon book  */
/*****************************************************************************/

static int hashpjw (s, prime)
     String s;
     unsigned prime;
{
  char * p;
  unsigned long  h = 0, g;  /* Must be 32 bits at least  */
  for (p = s; *p; ++p) {
    h = (h << 4) + (*p);
    if (g = h&0xf0000000) {
      h = h ^ (g >> 24);
      h = h ^ g;
    }
  }
  return h % prime;
}

/*****************************************************************************/
/*  Dump contents of table and some statistics about it  */
/*****************************************************************************/

static void dump_table (table)
     MonetStringTable table;
{
  int i;
  long max = 0, num = 0, hashes = 0, num_refers = 0, max_refers = 0;
  long total_len = 0, max_len = 0, freed =0;

  for (i = 0; i < table->size; ++i) {
    int depth = 0, refers, len;
    MonetStr entry;
    FORALL(entry, table->entries[i], next) {
      ++num;
      ++depth;
      if (entry->name == NULL) {
	++freed;
      }
      len = MonetStrlen (entry->name);
      total_len += len;
      if (len > max_len)
	max_len = len;
      refers = entry->references;
      if (refers > max_refers)
	max_refers = refers;
      num_refers += refers;
      MonetPrintf ("(%d -> [%d:%s#%d])  ",
	      i, entry->hash_place, entry->name, entry->references);
    }
    if(table->entries[i]) {
      ++hashes;
      if (depth > max)
	max = depth;
      MonetPrintf ("\n");
    }
  }
  MonetPrintfnl ("Max len    : %ld", max_len);
  MonetPrintfnl ("Total len  : %ld", total_len);
  MonetPrintfnl ("Mean len   : %f", (float) total_len / (float) num);
  MonetPrintfnl ("Max refers : %ld", max_refers);
  MonetPrintfnl ("Num refers : %ld", num_refers);
  MonetPrintfnl ("Mean refers: %f", (float) num_refers / (float) num);
  MonetPrintfnl ("Max depth  : %ld", max);
  MonetPrintfnl ("Num entries: %ld", num);
  MonetPrintfnl ("Num hashes : %ld", hashes);
  MonetPrintfnl ("Used slots : %d%%", (hashes * 100) / table->size);
  MonetPrintfnl ("Mean depth : %f", (float) num / (float) hashes);
  if (freed > 0) {
    MonetPrintfError ("%d freed entries on the table", freed);
  }
}

/*****************************************************************************/

/*****************************************************************************/

static MonetStr find_with_hash (table, place, string)
     MonetStringTable table;
     int place;
     char * string;
{
  MonetStr entry;
  FORALL(entry, table->entries[place], next)
    if (EQS(entry->name, string))
      return entry;
  return NULL;
}

static MonetStr find (table, string)
     MonetStringTable table;
     String string;
{
  int place;

  if (string == NULL)
    return NULL;
  place = hashpjw (string, table->size);
  return find_with_hash (table, place, string);
}

/*****************************************************************************/
/*  Lookup a string in the table and return associated data                  */
/*****************************************************************************/

extern caddr_t MonetStringTable_lookup (table, string)
     MonetStringTable table;
     String string;
{
  MonetStr entry;
  if (entry = find (table, string))
      return entry->data;
  return NULL;
}

/****************************************************************************/
/*  create  */
/****************************************************************************/

static MonetStr MonetStr_create (table, string, data, is_new)
     MonetStringTable table;
     String string;
     caddr_t data;
     MonetBool *is_new;
{
  MonetStr entry;
  int place;

  if (string == NULL)
    return NULL;

  place = hashpjw (string, table->size);
  if (entry = find_with_hash (table, place, string)) {
    *is_new = False;
    ++ entry -> references;
    return entry;
  }
  entry = MonetNew (struct monet_table_entry);
  ++  EntMalloced;
  ++ table -> stored;
  LIST_ADD(entry, table->entries[place], next);
  entry->name = MonetNewString (string);
  entry->hash_place = place;
  entry -> data = data;
  entry -> references = 1;
  *is_new = True;
  return entry;
}

static void MonetStr_free (table, entry)
     MonetStringTable table;
     MonetStr entry;
{
  LIST_DEL (MonetStr, entry, table->entries[entry->hash_place], next);
  MonetFree (entry->name);
  ++ EntFreed;
  MonetFree (entry);
}

/*****************************************************************************/
/*  Insert a new entry in the table, if it is not already there.  Fails      */
/*  if string with this name already in there.  Returns 0 if okay otherwise  */
/*  the data fro the existign entry.                                         */
/*****************************************************************************/

extern MonetStr MonetStringTable_insert_unique (table, string, data)
     MonetStringTable table;
     String string;
     caddr_t data;
{
  MonetBool is_new;
  MonetStr entry;

  if (entry = MonetStr_create (table, string, data, &is_new)) {
    if ( !is_new) {
      return entry;
    }
  }
  return 0;
}

/*****************************************************************************/
/*  Defererence an entry  */
/*****************************************************************************/

extern void    MonetStringTable_derefer (table, entry)
     MonetStringTable table;
     MonetStr entry;
{
  if (entry) {
    if (entry -> references == 0) {
      MonetSysError ("Attempt to dereference, when already 0 references %s",
		 entry -> name);
    } else if (entry -> references == 1) {
      entry -> references = 0;
      MonetStr_free (table, entry);
    } else {
      -- entry -> references;
    }
  }
}

/*****************************************************************************/
/*  Remove an entry from the table  */
/*****************************************************************************/

extern void    MonetStringTable_remove (table, string, data)
     MonetStringTable table;
     String string;
     caddr_t data;
{
  MonetStr entry;
  if (entry = find (table, string)) {
    MonetStr_free (table, entry);
  }
}

/*****************************************************************************/
/*  Rererence an entry  */
/*****************************************************************************/

extern MonetStr    MonetStringTable_refer (table, name, data)
     MonetStringTable table;
     char * name;
     caddr_t data;
{
  MonetBool is_new;
  MonetStr entry;

  if (entry = MonetStr_create (table, name, data, &is_new)) {
    return entry;
  }
  return NULL;
}

/*****************************************************************************/
/*  Global string lookup table  */
/*****************************************************************************/

extern void    MonetStr_derefer (str)
     MonetStr str;
{
  MonetStringTable_derefer (GlobalTable, str);
}

extern char *    MonetStr2String (str)
     MonetStr str;
{
  return MonetStringTable_get (GlobalTable, str);
}

extern MonetBool    MonetStrEQString (str, string)
     MonetStr str;
     char * string;
{
  if (str == find (GlobalTable, string))
    return True;
  return False;
}

extern MonetStr    String2MonetStr (string)
     char * string;
{
  return find (GlobalTable, string);
}


extern MonetStr    MonetStr_refer (string)
     char * string;
{
  if (string == "") {
    MONETDEBUG(3, ( "refer to empty string"));
  }
  return MonetStringTable_refer (GlobalTable, string, (caddr_t) NULL);
}

extern void MonetStr_dump ()
{
  dump_table (GlobalTable);
}

/****************************************************************************
  Init and finish
  ****************************************************************************/

extern void MonetStringTable_init ()
{
  GlobalTable = MonetStringTable_create (1021);
}

extern void      MonetStringTable_finish ()
{
  MonetStringTable_free (GlobalTable);
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
