/******************************************************************/
/******************************************************************/
/* $Header: attribute.c,v 1.3 91/10/01 16:07:06 a38 Exp $ */
/******************************************************************/
/*  Types/ Classes and their attributes  */
/*                  */
/*  Types are kept in a MonetStringTable for fast lookup  */
/*  !!! Stores the type name in the lookup table, here and for    */
/*  !!! classes also in the entity list, this is very inefficient */
/*  !!! Need to generalise this to become part of a proper types  */
/*  !!! system for monetscript  */
/******************************************************************/

#include <monet/msh/Entity.h>
#include <monet/msh/Script.h>
#include <monet/msh/StringTable.h>
#include "monetP.h"

static MonetStatusTrigger ChangeTrigger = NULL;
static MonetType Monet_type_being_accessed = NULL;
static MonetStringTable Table;

#define MonetMask_ZERO(p)      bzero((char *)(p), sizeof (*(p)))

#define TNAME(type) (MonetStr2String((type)->id))
#define ANAME(attr) (MonetStr2String((attr)->id))

/******************************************************************/
/*  Globals  */
/******************************************************************/

#define CHECK_ID(id,value) \
  if (id == NULL || id->type == NULL || (id->type->access == NULL)) { \
    MonetSysError ("Attribute id is not correct"); \
    return value; \
  }

/***************************************************************/
/*  For cycling through all the attributes  */
/***************************************************************/

#define FOR_ATTRIBUTES(id) \
{ int c; for (c=0; c < MONET_MAX_ATTRIBUTES; ++c) { \
    id = &(AttributeTable[c]); {

#define FOR_SET_ATTRIBUTES(id) \
{ int c; for (c=0; c < MONET_MAX_ATTRIBUTES; ++c) { \
    id = &(AttributeTable[c]); if (id->set) {

#define ENDFOR }}}

/******************************************************************/
/*  Types  */
/******************************************************************/

struct monet_type {
  MonetStr id;
  AttributeRoutine access;
  String help;
  MonetBool special;
  caddr_t data;
  MonetType  next;
};

struct monet_attribute {
  MonetBool set;  /*  Is this used or not  */
  String help;
  MonetStr id;
  unsigned number;  /*  Unique identifier  */
  MonetType type;
};

static struct monet_attribute AttributeTable[MONET_MAX_ATTRIBUTES];
static MonetType TypeList = NULL;

/******************************************************************/
/*  Data of currently access type  */
/******************************************************************/

extern caddr_t Monet_type_being_accessed_data ()
{
  if (Monet_type_being_accessed)
    return Monet_type_being_accessed->data;
  return NULL;
}

/******************************************************************/
/*  Masks  */
/******************************************************************/

static MonetBool MonetMask_ISEMPTY(mask)
     MonetMask * mask;
{
  int i;
  for (i=0; i<MonetWORDS; ++i)
    if (mask->bits[i] != 0)
      return False;
  return True;
}
/*
static MonetBool MonetMask_INTERSECT(p,q)
     MonetMask * p , * q;
{
  int i;
  for (i=0; i<MonetWORDS; ++i)
    if ((p)->bits[i] == (q)->bits[i])
      return True;
  return False;
}
*/

/******************************************************************/
/*  Handle changes to an entity, by triggering actions  */
/******************************************************************/

extern void MonetEntity_change_trigger (entity)
     MonetEntity entity;
{
  if (entity) {
    if (! MonetMask_ISEMPTY(&(entity->mask))) {
      if (ChangeTrigger) {
	ChangeTrigger (entity);
      }
      MonetMask_ZERO(&MASK(entity));
    }
  } else {
    MonetSysError ("Null passed to MonetEntity_change_trigger");
  }
}

/******************************************************************/
/*  Create a type  */
/******************************************************************/

extern MonetType MonetType_create (name, special, access, data, help)
     String name;
     MonetBool special;
     AttributeRoutine access;
     caddr_t data;
     String help;
{
  MonetType type;
  type = MonetNew (struct monet_type);

  if (MonetStringTable_insert_unique (Table, name, (caddr_t) type)) {
    MonetPrintfError ("Type/class already exists [%s]", name);
    MonetFree (type);
    return NULL;
  }
  type -> id = MonetStr_refer (name);
  type -> access = access;
  type -> special = special;
  type -> data = data;
  type -> help = MonetNewString (help);
  LIST_ADD (type, TypeList, next);
  return type;
}

extern void MonetType_free (type)
     MonetType type;
{
  MonetStringTable_remove (Table, TNAME(type), (caddr_t) type);
  LIST_DEL (MonetType, type, TypeList, next);
  MonetStr_derefer (type->id);
  MonetFree (type->help);
  MonetFree (type);
}

/******************************************************************/
/*  Finds the type given by 'name' or null if there is none       */
/******************************************************************/

extern MonetType String2MonetType (string)
     String string;
{
  return  (MonetType) MonetStringTable_lookup (Table, string);
}

/******************************************************************/
/*  Finds the type given by 'name' or null if there is none       */
/******************************************************************/

extern char * MonetType_name (type)
     MonetType type;
{
  return TNAME(type);
}

/******************************************************************/
/*  Finds a type given its name and access routine and returns    */
/*  the data acssociated with the type.                           */
/*  this routine is a hack that gives efficient lookup of classes */
/******************************************************************/

extern caddr_t MonetType_find_data (string, access)
     String string;
     AttributeRoutine access;
{
  MonetType type;
  if (type = (MonetType) MonetStringTable_lookup (Table, string))
    if (type->access == access)
      return type->data;
  return NULL;
}

/******************************************************************/
/*  Create an attribute  */
/******************************************************************/

extern MonetAttribute MonetAttribute_create (name, number, type, help)
     String name, help;
     MonetType type;
     unsigned number;
{
  MonetAttribute at;
  if (number > MONET_MAX_ATTRIBUTES) {
    MonetPrintfError ("Attribute Number %lu is greater than max %lu",
		number, MONET_MAX_ATTRIBUTES);
    return NULL;
  } else if (number == 0) {
    MonetPrintfError ("Attribute number must be greater than zero");
    return NULL;
  }
  at = &AttributeTable[number-1];
  if (at->set) {
    if (EQS(name, ANAME(at)) && at->type == type) {
      if (help && *help && at->help) {
	MonetFree (at->help);
	at->help   = MonetNewString (help);
      }
    } else {
      MonetPrintfError
	("Attribute [%lu] already defined as [%s %s], not [%s %s]",
	 (unsigned long) at->number, TNAME(at->type), ANAME(at),
	 TNAME(type), name);
      return NULL;
    }
  } else {
    at->set = True;
    at->id = MonetStr_refer (name);
    at->type   = type;
    at->help   = MonetNewString (help);
  }
  return at;
}

/******************************************************************/
/*  Display the types */
/******************************************************************/

extern void MonetType_report ()
{
  MonetType type;

  MonetPrintfnl ("Types");
  FORALL(type, TypeList, next)
    if (type->help == NULL)
      MonetPrintfnl ("%s\t", TNAME(type));
    else if (MonetStrlen (TNAME(type)) >= 8)
      MonetPrintfnl ("%s\t- %s", TNAME(type), type->help);
    else
      MonetPrintfnl ("%s\t\t- %s", TNAME(type), type->help);
}

/*****************************************************************
  null attribute
  ******************************************************************/

extern MonetBool MonetEntity_null_attribute (entity, attribute)
     MonetEntity entity;
     MonetAttribute attribute;
{
  if (MonetEntity_get_attribute (entity, attribute))
    return False;
  return True;
}

/******************************************************************/
/*  Convert to string  */
/******************************************************************/

extern String MonetAttribute2String (attribute)
     MonetAttribute attribute;
{
  if (attribute)
    return ANAME(attribute);
  return NULL;
/*
  static char buffer[1024];
  (void) sprintf (buffer, "%lu %s %s", attribute->number, 
  ANAME(attribute));
  return buffer;
*/
}

extern String MonetAttribute2FullName (attribute)
     MonetAttribute attribute;
{
  if (attribute) {
    static char buffer[1024];
    (void) sprintf (buffer, "%lu %s %s",
		    attribute->number,
		    TNAME(attribute->type),
		    ANAME(attribute));
    return buffer;
  }
  return NULL;
}

extern unsigned MonetAttribute2AttributeNumber (attribute)
     MonetAttribute attribute;
{
  return attribute->number;
}

extern MonetType MonetAttribute2MonetType (attribute)
     MonetAttribute attribute;
{
  if (attribute)
    return attribute->type;
  return NULL;
}

/*****************************************************************
  List defined attributes
  *****************************************************************/

extern void MonetAttribute_report ()
{
  MonetAttribute id;
  MonetPrintfnl ("Attributes");
  FOR_SET_ATTRIBUTES(id)
    if (id->help)
      MonetPrintfnl ("%s %s %lu\t- %s",
		      TNAME(id->type), ANAME(id), id->number, id->help);
    else
      MonetPrintfnl ("%s %s %lu",
		      TNAME(id->type), ANAME(id), id->number);
  ENDFOR
}

/******************************************************************/
/*  Encode status from compact ascii */
/******************************************************************/

extern MonetBool MonetEntity_decodeCompactAscii (entity, text)
     MonetEntity entity;
     String *text;
{
  String i;
  MonetAttribute id;

  while (SPACEORTAB(**text))
    text ++;

  if (**text == MonitorEndCode || **text == '\n' || **text == '\0')
    return True;

  for (i = *text;; ++i) {
    char * value, sep;
    unsigned code;

    while (*i == MonitorSepCode)
      ++i;

    code = atoi(i);
    i++;
    value = i;
    for (; *i != NULL && *i != MonitorSepCode && *i != MonitorEndCode; ++i);
    sep = *i;
    *i = '\0';
    FOR_SET_ATTRIBUTES (id)
      if (id -> number == code) {
	MonetMask_SET (id->number, &MASK(entity));
	Monet_type_being_accessed = id->type;
	if (id->type->access
	    (entity, id->number,
	     attribute_set, attribute_compact_ascii, True, value)) {
	  MonetMask_SET(id->number, &MASK(entity));
	}
      }
    ENDFOR;
    if (sep != MonitorSepCode)
      break;
  }
  *text = i;
  MonetEntity_change_trigger (entity);
  return True;
}

/******************************************************************/
/*  Init status  */
/******************************************************************/

extern void MonetStatusSet_create (entity)
     MonetEntity entity;
{
  MonetAttribute id;

  entity -> size_status_set  = 0;

  if ( ! entity) {
    MonetSysError ("MonetStatusSet_create: entity is null");
  } else {
    MonetMask_ZERO (&MASK(entity));
    bzero ((char *) entity -> status, sizeof (MonetStatusSet));
    FOR_ATTRIBUTES (id) {
      if (id->number == 0) {
	MonetSysError ("Zero id number [%s:%d]", ANAME(id), id->number, c);
      } else {
	entity -> status[id->number-1].pointer = NULL;
      }
      if (id->set) {
	Monet_type_being_accessed = id->type;
	(void) id->type->access (entity, id->number,
				 attribute_init, attribute_ascii,
				 True, 
				 NULSTR);
      }
    } ENDFOR
  }
  MonetMask_ZERO (&(entity->mask));
}

extern void MonetStatusSet_free (entity)
     MonetEntity entity;
{
  MonetAttribute id;

  if ( ! entity) {
    MonetSysError ("MonetStatusSet_free: entity is null");
  } else{ 
    FOR_SET_ATTRIBUTES (id) {
      Monet_type_being_accessed = id->type;
      (void) id->type->access (entity, id->number,
			       attribute_free, attribute_ascii, True, 
			       (String) NULL);
    } ENDFOR;
  }
}

/******************************************************************/
/*  Convert an attribute ID to text representation and vice versa */
/******************************************************************/

extern MonetAttribute String2MonetAttribute (s)
     String s;
{
  MonetAttribute id;
  unsigned number;

  if (s == NULL || *s == NULL)
    return NULL;
  if (number = String2unsigned(s)) {
    if (number <= MONET_MAX_ATTRIBUTES) {
      id = &AttributeTable[number-1];
      if (id->set)
	return id;
      else
	return NULL;
    }
  } else {
    FOR_SET_ATTRIBUTES(id)
      if (EQS(ANAME(id), s))
	return (id);
    ENDFOR;
  }
  return NULL;
}

/******************************************************************/
/* Delete Attribute, or rather reset it */
/******************************************************************/

extern void MonetEntity_delete_attribute (entity,id)
	MonetAttribute id;
	MonetEntity entity;
{
  CHECK_ID(id,);
  Monet_type_being_accessed = id->type;
  (void) id->type->access
    (entity, id->number, attribute_unset, attribute_ascii, True, NULL);
  MonetMask_SET(id->number, &MASK(entity));
  MonetEntity_change_trigger (entity);
}

/******************************************************************/
/*  Sets an attribute value  */
/******************************************************************/

extern void MonetEntity_set_attribute (entity, id, s)
     MonetEntity entity;
     MonetAttribute id;
     char * s;
{
  CHECK_ID(id,);
  Monet_type_being_accessed = id->type;
  if (id->type->access
    (entity, id->number, attribute_set, attribute_ascii, True, (String) s)) {
    MonetMask_SET(id->number, &MASK(entity));
    MonetEntity_change_trigger (entity);
  }
}

/******************************************************************/
/*  Set a list of attribute values  */
/******************************************************************/

extern void MonetEntity_set_attributes (entity, id, s)
     MonetEntity entity;
     MonetAttribute *id;
     char ** s;
{
  if (id && *id) {
    unsigned i;
    for (i = 0; id[i]; ++i) {
      Monet_type_being_accessed = id[i]->type;
      if (id[i]->type->access
	(entity, id[i]->number, attribute_set, attribute_ascii, True, s[i])) {
	MonetMask_SET(id[i]->number, &MASK(entity));
      }
    }
    MonetEntity_change_trigger (entity);
  }
}

/******************************************************************/
/*  Gets an attributes value using inheritance */
/*  Converts to string if not already string */
/*  Returns null if value is default  */
/******************************************************************/

extern  String MonetEntity_get_attribute (entity, id)
     MonetEntity entity;
     MonetAttribute id;
{
  if ( ! entity) {
    return NULL;
  }
  CHECK_ID(id, NULL);
  Monet_type_being_accessed = id->type;
  return ( (String) id->type->access
	  (entity, id->number, attribute_get, attribute_ascii, True, NULL));
}

/******************************************************************/
/* if value is default then returns it, not null */
/******************************************************************/

extern  String MonetEntity_get_attribute_full (entity, id)
     MonetEntity entity;
     MonetAttribute id;
{
  CHECK_ID(id, NULL);
  Monet_type_being_accessed = id->type;
  return ( (String) id->type->access
	  (entity, id->number, attribute_get, attribute_ascii, False, NULL));
}

/******************************************************************/
/*  Gets an attributes value but with no inheritance  */
/*  Converts to string if not already string */
/*  Returns null if value is default  */
/******************************************************************/

extern  String MonetEntity_get_shallow_attribute (entity, id)
     MonetEntity entity;
     MonetAttribute id;
{
  CHECK_ID(id, NULL);
  Monet_type_being_accessed = id->type;
  return ( (String) id->type->access
	  (entity, id->number,
	   attribute_get_shallow, attribute_ascii, True, NULL));
}

/******************************************************************/
/*  Encode atrtribute in compact format  */
/******************************************************************/

static String Attribute2CompactAscii (f, code, value)
     char * f;
     char * value;
     unsigned code;
{
  (void) sprintf (f, "%lu= ", code);
  f += MonetStrlen (f);
  f = MonetStringCopyToEnd (f, value);
  *f = MonitorSepCode; ++f;
  *f = '\0';
  return f;
}

/******************************************************************/
/*  Encode atrtribute in ascii format  */
/******************************************************************/

static String Attribute2Ascii (f, name, value)
     char * f;
     char * value, * name;
{
  String c;
  f = MonetStringCopyToEnd (f, name);
  f = MonetStringCopyToEnd (f, "= ");
  for (c=value; *c!=NULL; ++c) {
    *f = *c; ++f;
    if (*c == CONTINUATION_CHAR) {
      *f = '\n'; ++f;
      *f = '\t'; ++f;
    }
  }
  *f = '\n'; ++f;
  *f = '\0';
  return f;
}

/******************************************************************/
/*  Encode status as readable ascii */
/******************************************************************/

extern String	MonetEntity2Report (entity,
			       header,
			       all,
			       compact,
			       null_if_empty)
     String header;
     MonetBool all, compact;
     MonetEntity entity;
     MonetBool null_if_empty;
{
  static char buffer [8192] = { 0 }; /* !!! */
  MonetAttribute id;
  String f, place;

  if (entity == NULL) {
    MonetSysError ("entity2report: entity is NULL");
    return NULL;
  }
  f = buffer;
  if (header == NULL) {
    f = MonetStringCopyToEnd (f, MonetEntity_title (entity));
    f = MonetStringCopyToEnd (f, "\n");
  } else if (*header) {
    f = MonetStringCopyToEnd (f, header);
    f = MonetStringCopyToEnd (f, "\n");
  }
  place = f;
  FOR_SET_ATTRIBUTES(id)
    if ((id->type->special == False) &&
	(all || MonetMask_ISSET(id -> number, &MASK(entity)))) {
      String value;
      if (value = MonetEntity_get_shallow_attribute (entity, id))
	if (compact)
	  f = Attribute2CompactAscii (f, id->number, value);
	else
	  f = Attribute2Ascii (f, ANAME(id), value);
    }
  ENDFOR;
  if (place == f && null_if_empty)
    return NULL;
  *f = '\0';
  return buffer;
}

/******************************************************************/
/*  Set the change trigger routine  */
/******************************************************************/

extern void MonetEntity_set_change_trigger (changed)
     MonetStatusTrigger changed;
{
  ChangeTrigger = changed;
}

/******************************************************************/
/*  Initialise and finish */
/******************************************************************/

extern void MonetType_init ()
{
  MonetAttribute id;
  Table = MonetStringTable_create (500);  /*!!! Arbitrary value but no danger*/
  ChangeTrigger = NULL;
  FOR_ATTRIBUTES(id)
    id->set = False;
    id->number = c + 1;
  ENDFOR;
  TypeList = NULL;
}

extern void MonetType_finish ()
{
  while (TypeList)
    MonetType_free (TypeList);
  /*  !!! Attributes are not freed  */
  ChangeTrigger = NULL;
  MonetStringTable_free (Table);
}

/******************************************************************/
/******************************************************************/
/* $Header: attribute.c,v 1.3 91/10/01 16:07:06 a38 Exp $ */
/******************************************************************/
/******************************************************************/

