/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "General.h"
#include <ctype.h>
#include <sys/param.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <errno.h>

static char * get_prefix (file)
     char ** file;
{
  char * prefix;
  if (file == NULL || *file == NULL || **file == NULL) {
    prefix = NULL;
  } else if (**file == '/' || **file == '.') {
    prefix = "";
  } else if (**file == '~') {
    if (*(*file+1) == NULL) {
      prefix = Monet_home_dir (NULSTR);
      *file = NULL;
    } else if (*(*file+1) == '\/') {
      prefix = Monet_home_dir (NULSTR);
      *file += 2;
    } else {
      prefix = NULL;
    }
  } else {
    prefix = Monet_dir (NULSTR);
  }
  return prefix;
}

/*****************************************************************************/

/*****************************************************************************/

extern void MonetPrintText (indent, width, text)
     char * indent, * text;
     int width;
{
  char * cp;
  int len, indent_len, out, add;
  MonetBool begin = True;		/* At Beginning of sentence */

  if (text == NULL || *text == NULL)
    return;

  indent_len = MonetStrlen (indent);

  if (width <= indent_len) {
    indent_len = 0;
  }

  for (cp = text ; ; cp += add) {
    int space;
    if (out = indent_len) {
      MonetFwrite (indent, indent_len);
    }
    /* -- Get rid of leading whitespace  */
    space = -1;
    for (;*cp == ' ' || *cp == '\t'; ++cp);
    /* -- Find length of line until next newline or line greater than width */
    add = 1;
    for (len = 0; cp[len] != '\n'; ++len, ++out) {
      if (cp[len] == '\0') {
	MonetFwrite (cp, len);
	if (len > 0 && cp[len-1] != '.') {
	  MonetFwrite (".\n", 2);
	} else {
	  MonetFwrite ("\n", 1);
	}
	return;
      }
      if (len > 0 && (cp[len] == '\t' || cp[len] == ' ')) {
	if (isgraph (cp[len-1])) {
	  space = len;
	}
      }
      if (out >= width) {
	if (space > 0) {
	  len = space;
	}
	add = 0;
	break;
      }
    }
    add += len ;
    if (begin && islower (*cp)) {
      char up = toupper (*cp);
      MonetFwrite (&up, 1);
      MonetFwrite (cp+1, len);
      begin = False;
    } else {
      MonetFwrite (cp, len);
    }
 
    MonetFwrite ("\n", 1);
  }
}

/*****************************************************************************/
/*  Make a filename, mallocs just enough space for the name  */
/*****************************************************************************/

extern char * MonetMakePathname (path, name, extension)
     char * path, * name, *extension;
{
  char * filename = NULL;
  char * prefix;
  int len;

  if (filename) {
    if (path == filename || name == filename) {
      MonetSysError ("MakePathname: Passing malloced filename [%s]", filename);
    }
  }
  if (path && *name != '/' && *name != '~') {
    prefix = get_prefix (&path);
  } else {
    path = NULL;
    prefix = get_prefix (&name);
  }
  if (! prefix) {
    return NULL;
  }

/*  if (! path) {
    path = "";
  }
*/
  if (! name) {
    name = "";
  }

  if (! extension) {
    extension = "";
  }

  if (filename) {
    MonetFree (filename);
  }
  len = MonetStrlen (prefix) + MonetStrlen (path) +
    MonetStrlen (name) + MonetStrlen (extension) + 3;
  if (filename = MonetMalloc (len)) {
    char * cp = filename;
    if (prefix && *prefix) {
      (void) sprintf (cp, "%s/", prefix);
      cp += strlen (cp);
    }
    if (path && *path) {
      (void) sprintf (cp, "%s/", path);
      cp += strlen (cp);
    }
    if (name && *name) {
      (void) sprintf (cp, "%s%s", name, extension);
    }
    return filename;
  }
  MonetFree (filename);
  return NULL;
}

/*****************************************************************************/
/*  Make a filename, check to see if it begins with ~  */
/*****************************************************************************/

extern char * MonetFilename (name)
     char * name;
{
  return MonetMakePathname (NULSTR, name, NULSTR);
}

/*****************************************************************************/
/*  Finds a file using wildcards in a given directory  */
/*****************************************************************************/

extern String Monet_findfile (dir, pattern)
     String dir, pattern;
{
  DIR * dirp;
  struct direct *dp;
  static char result [MAXPATHLEN+1];

  dir = MonetFilename (dir);
  dirp = opendir(dir);
  if (dirp) {
    unsigned matched = 0;
    *result = '\0';
    for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
      if (MonetMatchWildcard (dp->d_name, pattern)) {
	(void) strcpy (result, dp->d_name);
	++ matched;
      }
    closedir(dirp);
    if (! pattern)
      pattern = "*";
    if (matched > 1) {
      MonetPrintfError ("Ambiguous filename [%s] %u matches",
			pattern, matched);
    } else if (matched == 0) {
      MonetPrintfError ("Could not find file matching [%s] in directory [%s]",
			pattern, dir);
    } else {
      return result;
    }
  } else {
    MonetPrintfError ("Could not access directory [%s]", dir);
  }
  return NULL;
}

extern MonetBool Monet_list_dir (dir, pattern)
     String dir;
     String pattern;
{
  DIR * dirp;
  struct direct *dp;

  dir = MonetFilename (dir);
  dirp = opendir(dir);
  if (dirp) {
    unsigned matched = 0;
    MonetPrintfnl ("Listing directory [%s]", dir);
    for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
      if (pattern) {
	if (pattern[0] != '.' && dp->d_name[0] == '.')
	  continue;
	if (! MonetMatchWildcard (dp->d_name, pattern))
	  continue;
      } else if (dp->d_name[0] == '.') {
	continue;
      }
      MonetPrintfnl ("  %s", dp->d_name);
      ++ matched;
    }
    closedir(dirp);
    if (matched == 0)
      MonetPrintfError ("Could not find files matching [%s] in directory [%s]",
		  pattern, dir);
    else
      MonetPrintfnl ("Listed %u files matching [%s] in directory [%s]",
		      matched, pattern, dir);
  } else {
    MonetPrintfError ("Could not access directory [%s]", dir);
    return False;
  }
  return True;
}

/*****************************************************************************/
/*  Files  */
/*****************************************************************************/

extern FILE * MonetFopen (name, mode)
     char * name;
     char * mode;
{
  if ( ! name) {
    return NULL;
  } else if (*name == '/' || *name == '~' || *name == '.') {
    return  fopen (name, mode);
  } else {
    return  fopen (name, mode);
  }
}

extern MonetBool MonetFileExists (name)
     char * name;
{
  struct stat stats;

  if ((name =  MonetFilename (name)) && (stat (name, &stats) == 0)) {
    return True;
  }
  return False;
}

extern FILE * MonetFopenFile (name, mode)
     char * name;
     char * mode;
{
  struct stat stats;
  FILE * f;

  if (!(name = MonetFilename (name)))
    return NULL;

  if (stat (name, &stats) != 0) {
    String message;
    if (errno == ENOENT && (EQS(mode, "a") || EQS(mode, "w"))) {
      if (f = fopen (name, mode)) {
	return f;
      } else {
	MonetPrintfError ("Sorry, can't open file [%s]", name);
      }
      return f;
    }
    if (message = MonetSystemErrorMessage ())
      MonetPrintfError ("%s [%s]", message, name);
    else
      MonetPrintfError ("Error using stat, errno=%d  [%s]", errno, name);
    return NULL;
  }
  switch (stats.st_mode & S_IFMT) {
  case S_IFDIR:
    MonetPrintfError ("[%s] is a directory", name);
    break;
  case S_IFSOCK:
    MonetPrintfError ("[%s] is a socket", name);
    break;
  case S_IFCHR:
  case S_IFBLK:
    MonetPrintfError ("[%s] is a special file", name);
    break;
  case S_IFREG:
    if (f = fopen (name, mode)) {
      return f;
    } else {
      MonetPrintfError ("Sorry, can't open file [%s]", name);
    }
    break;
  default:
    MonetPrintfError ("[%s] not a regular file, mode[%o]",
		name, stats.st_mode);
    break;
  }
  return NULL;
}

/*****************************************************************************/
/*  Open a file with an optional name extension  */
/*****************************************************************************/

extern FILE * MonetFopenFileWithExtension (name, extension, warning, mode)
     char * name, *extension, *warning;
     char * mode;
{
  FILE * f;

  if (extension) {
    char extended_name [2048];
    (void) sprintf (extended_name, "%s%s", name, extension);
    if (MonetFileExists (extended_name)) {
      f = MonetFopenFile (extended_name, mode);
      return f;
    }
  }
  if (f = MonetFopenFile (name, mode)) {
  } else if (warning && *warning) {
    MonetPrintfError ("%s [%s] with mode [%s]", warning, name, mode);
  }
  return f;
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/general/RCS/general.c,v 1.2 1991/07/10 12:50:54 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
