/***************************************************************************/
/***************************************************************************/
/*  $Header$  */
/***************************************************************************/
/*  Provides an asyncrous buffered channel between two processes.          */
/*  An interface to sockets, could be improved to say the least    !!!     */
/*  Uses Non blocking sockets.                                             */
/*  Sends are not done straight away but are buffered and delivered from   */
/*  the main loop in io_server.c,  The buffering is done using the Fifo    */
/*  routines which work but are !!!INEFFICIENT!!! especially when we have  */
/*  the nice writev routine.  */
/***************************************************************************/

#include "General.h"
#include "StrFifo.h"
#include "Channel.h"
#include "NiceSocket.h"
#include <errno.h>
#include <sys/ioctl.h>
#include <monet/msh/Signal.h>
#include <sys/ioctl.h>		/* For FIONBIO  */
#include <sys/param.h>  /*  For MAXHOSTNAMELEN  */

 /*
      if (channel->interactive) {
	switch (*r) {
	case 025:
	  r = *text;
	  *r = '\0';
	  break;
	case 027:
	  -- r;
	  putchar ('\08');
	  break;
	case 06:
	  -- r;
	  putchar ('\08');
	  break;
	case '\02':
	  -- r;
	  putchar ('\08');
	  break;
	default:
	  *r = ' ';
	  break;
	  }
      } else
 */
/***************************************************************************/
/*  Private definitions  */
/*  Exports Server and Channel, these are used externally as identifiers to */
/*  connections  */
/***************************************************************************/

#define END_OF_RECORD  NULL
#define BUFFERLENGTH   (2<<13)

static Channel ChannelList = 0;

struct monet_channel {
  caddr_t data;  /* Data for this connection */
  caddr_t subclass_data;
  MonetStrFifo send_queue;
  int fid;
  ChannelHandler handler;
  ChannelDisconnect disconnect;
  char buffer [BUFFERLENGTH];
  char * pbuffer;
  char previous;
  MonetBool interactive, opened;
  Channel next;
};

typedef struct monet_server * Server;
struct monet_server {
  int fd;
  unsigned port;
  caddr_t data;
  ChannelAcceptHandler handler;
  ChannelAcceptDisconnect disconnect;
  Server next;
};
static Server ServerList;

/**************************************************************************
  *************************************************************************/

static void Server_free (server)
     Server server;
{
  MONETDEBUG(2, ( "Close server listening on port %d", server->port));
  LIST_DEL (Server, server, ServerList, next);
  MonetInputFd_free (server->fd);
  if (server->disconnect)
    server->disconnect (server->fd, server->data);
  MonetFree (server);
}

/***************************************************************************/
/*  Handles a new connection MonetRequest from a client  */
/***************************************************************************/

static void HandleAccept (fd, data)
     int fd;
     caddr_t data;
{
  struct sockaddr_in addr;
  int addrlen, n = 1;
  Server server = (Server) data;

  addrlen = sizeof (addr);
  fd  = accept (fd, (struct sockaddr *) &addr, &addrlen);
    
  if (fd < 0) {
    MonetSysError ("Could not accept, errno %d [%s]",
		   errno, MonetSystemErrorMessage ());
  } else {
    char address [MAXHOSTNAMELEN + 16];

    (void) sprintf (address, "%s+%u",
		    inet_ntoa (addr.sin_addr),
		    ntohs (addr.sin_port));

    /* Non blocking */
    ioctl(fd, FIONBIO, &n);

    server->handler (fd, address, server->data);
  }
}

/***************************************************************************/
/*  Used to start a new channel, supply address of server and callback      */
/*  routines for new messages and disconnection.                           */
/***************************************************************************/

extern int Channel_create_server (port, accept_handler,
				  disconnect_handler, data)
     unsigned port;
     ChannelAcceptHandler accept_handler;
     ChannelAcceptDisconnect disconnect_handler;
     caddr_t data;
{
  Server server;
  int fd;

  if (accept_handler == NULL) {
    MonetSysError ("Channel_create_server: accept_handler is null");
    return -1;
  }
  fd = serversock (port);
  if (fd < 0) {
    MonetPrintfWarning ("Can't listen on port [%d]", port);
  } else if (server = MonetNew (struct monet_server)) {
    server->port = port;
    server -> handler = accept_handler;
    server->disconnect = disconnect_handler;
    server->fd = fd;
    server->data = data;
    LIST_ADD (server, ServerList, next);
    MonetPrintfInform ("Listening on port [%d]", port);
    MonetInputFd_create (fd, HandleAccept, (MonetFdHandler) NULL,
			 (caddr_t) server);
  }
  return fd;
}

extern void Channel_free_server (port)
     unsigned port;
{
  Server server;
  FORALL(server, ServerList, next) {
    if (server->port == port) {
      Server_free (server);
      return;
    }
  }
  MonetSysError ("Not listening on port [%d]", port);
}

/**************************************************************************
  Get the next character for processing
  *************************************************************************/

static unsigned  ReadBufferPosition, ReadBufferLen;
static char  ReadBuffer  [BUFFERLENGTH];

static int NextChar (fid, r, finish)
     int fid;
     char * r;
     MonetBool *finish;
{
  int cc;

  if (ReadBufferPosition == ReadBufferLen) {
    cc = read (fid, ReadBuffer, sizeof(ReadBuffer));
    ReadBufferPosition = 0;
    if (cc < 0) {
      if (errno == EWOULDBLOCK) {
	*finish = False;
      } else if (errno == ECONNRESET) {
	MONETDEBUG(20, ( "recv: channel reset by peer"));
	*finish = True;
      } else if (errno == ENOTCONN) {
	MonetSysError ("Socket/fd is not connected");
	*finish = True;
      } else {
	MonetSysError ("Bad recv: fid [%d] errno [%d][%s]",
		       fid, errno, MonetSystemErrorMessage ());
	*finish = True;
      }
      return False;
    } else if (cc == 0) {
      MONETDEBUG(20, ( "recv: cc == 0, channel seems to have disconnected"));
      *finish = True;
      return False;
    }
    
    MONETDEBUG(9, ( "Read from fid [%d] cc=%d [%.*s]",
		fid, cc, cc, ReadBuffer));

    *finish = False;
    ReadBufferLen = cc;
  }

  *r = ReadBuffer [ReadBufferPosition];
  ++ReadBufferPosition;

  return True;
}

/**************************************************************************
  This is called whenever new data arrives from the server.  Reads in as
  much data as it can, catches some errors and cleanly disconnects.

  If the connection has been shutdown the routine works as normal but
  doesn't call the handler - ie it reads it but then discards it.


  Read one \n or NULL terminated line from an fd and catch errors     
  If it returns 'finish==True' an error occured or disconnected 
  !!! Should check len 
  The routine will return on finding an end of record marker or when the  
  socket would block (if is a non-blocking socket).  In the latter case   
  the user must provide buffering.  
  !!! This is a mess now, clean it up 

  **************************************************************************/

static void ReadHandler (fid, dummy)
	int fid;
	caddr_t dummy;
{
  int len;
  Channel channel;
  MonetBool finish;
  char * r, **text;
  MonetBool max = False;

  if (fid < 0) {
    MonetSysError ("fid [%d] is invalid", fid);
    return;
  }

  FORALL(channel, ChannelList, next)
    if (channel->fid == fid)
      break;
  if (channel == NULL) {
    MonetSysError ("ReadHandler: Couldn't identify channel for fid %d", fid);
    return;
  }
  MONETDEBUG(20, ( "New Messages on connection %d", fid));

  ReadBufferPosition = 0;
  ReadBufferLen = 0;

  len = sizeof(channel->buffer) - (channel->pbuffer - channel->buffer);

  text = &(channel->pbuffer);

  /* -- Read in data and process it all  */

  finish = False;
  for (r = (*text);; r++) {

    if (r-(*text) >= len) {
      if (! max) {
	max = True;
	MonetPrintfError ("Max message length %d exceeded", len);
      }
      r --;  /* Hmmm !!! */
    }

    if (! NextChar (fid, r, &finish)) {
      if (! finish) {
	*r = '\0';  /* -- So it can be printed  */
	MONETDEBUG(20, ( "Socket/fd %d would block, buffer [%s]", fid, *text));
	*text = r;
      }
      break;
    }

    if (channel->previous == '\\' && *r == '\n') {
      channel->previous = NULL;
      if (channel->interactive) {
	MonetPrintf ("%s? ", Monet_program_name ());
	fflush (stdout);
      }
      /*  Continuation line  */
      --r;
      *r = ' ';
    } else if ((*r == '\0' || *r == '\n')) {
      finish = False;
      *r = NULL;   /* Null terminate the string */

      channel->previous = NULL;
      channel->pbuffer = channel->buffer;
      len = sizeof(channel->buffer);
      text = &(channel->pbuffer);
      r = *text;
      -- r;
      if (channel->handler && channel->opened)
	if ((channel->handler (channel, channel->buffer)) == False) {
	  finish = True;
	  break;
	}
    } else if (iscntrl (*r)) {
      /*  Non printing characters are converted into whitespace */
      /*  This is primarily to deal with the problem of \r when */
      /*  using telnet.  */
      --r;
    } else if (!isprint (*r)) {
      *r = ' ';
    } else {
      /*  *r is a printable character */
    }
    channel->previous = *r;
  }

  MONETDEBUG(20, ( "End of messages on connection %d", fid));

  /* -- If the connection is closed for some routine then finish off
     -- by calling the disconnect routine and sending out what is left
     -- to write. */

  if (finish) {
    if (channel -> disconnect) {
      channel->disconnect (channel);
      Channel_shutdown (channel);
    } else {
      MonetSysError ("No disconnect handler given");
      MonetInputFd_free (channel->fid);
    }
  }
}

/**************************************************************************
  This is called when a file descriptor is ready to send
  It reads in buffered messages, one at a time and tries to send them
  any unsent messages remain in the buffer for later.  Any non-sent or
  partially sent messages are returned to the 'top' of the buffer for
  retransmission.

  If something goes really wrong or the other end disconnects then
  this the channel disconnect routine is called.
  **************************************************************************/


#ifndef MONET_SIG_IGN_WORKS
MONET_SIGNAL_ROUTINE(sig_do_nothing)
     /*  -- Do nothing  */
MONET_END_SIGNAL_ROUTINE
#endif


static void write_stacked (channel)
     Channel channel;
{
  char * message;
  unsigned len;
  MONET_PFV sig;
  MonetBool out = False, disconnected = False;
  int fid;
  
  fid = channel->fid;

#ifdef MONET_SIG_IGN_WORKS
  sig = signal (SIGPIPE, SIG_IGN);
#else
  sig = signal (SIGPIPE, sig_do_nothing);
#endif

  while (message = MonetStrFifo_pop (channel->send_queue, &len)) {
    int sent;
     
    if (len == 0) {
      MonetSysError ("Message length is zero but message is not null");
      continue;
    }
    sent = write (channel->fid, message, len);
    if (sent == -1) {
      MONETDEBUG(2, ( "Channel_send [%d], sent == -1, [errno = %d: %s]",
		  channel->fid, errno,
		  MonetSystemErrorMessage ()));
      switch (errno) {
      case ERANGE:
	MonetSysError ("Channel WriteHandler, ERANGE exceeded, sending [%s]%d",
		       message, len);
      case EPIPE:
      case EFAULT:
      case EBADF:
      case EINVAL:
      case ENOTSOCK:
      case EMSGSIZE:  /*  This could easily be handled  */
      default:
	disconnected = True;
	out = True;
	break;
      case EINTR:
      case ENOBUFS:
      case EWOULDBLOCK:
      case EAGAIN:		/* This is a SYSV thing, I just put it in
				   for the sake of doing so. */
	/* -- Just temporary inability to continue  */
	(void) MonetStrFifo_push_on_top (channel->send_queue,
					 message, len);
	out = True;
	break;
      }
    } else if (sent == len) {
      /* -- Okay fine worked perfectly */
      MONETDEBUG(9, ( "Sent full message [%s]", message));
    } else if (sent < len) {
      /*  -- Could only send part of message, so store for retransmission */
      (void) MonetStrFifo_push_on_top (channel->send_queue,
				       message+sent, len - sent);
      MONETDEBUG(9, ( "Sent partial message [%.*s]%d message=[%s] missed %d",
		  sent, message, sent, message, len));
      out = True;
    } else {
      MonetSysError ("Sent absurb value [%d] must be +ve and <= [%d]",
		     sent, len);
    }
    if (out)
      break;
  }

  (void) signal (SIGPIPE, sig);

  if (disconnected) {
    /*  Get rid of all outstanding stuff  */
    while (MonetStrFifo_pop (channel->send_queue, &len))
      ;
    if (channel -> opened) {
      if (channel -> disconnect) {
	channel->disconnect (channel);
      } else {
	MonetSysError ("No disconnect handler given");
	MonetInputFd_free (channel->fid);
      }
    }
    Channel_free (channel);
  } else if ( ! channel -> opened) {
    /*  -- The channel is marked as closed already, so
	-- check to see if there is anything left to write, if not free it.
	*/
    Channel_shutdown (channel);
  } else {
    /*  Everything outstanding has been written so we do not want this
	routine called anymore (until something else is written anyway).
	So alter the existing InputFd so that it does not select
	for write and so this routine will not be called any more.
	*/
    MonetInputFd_create (fid,
			 (MonetFdHandler) ReadHandler,
			 (MonetFdHandler) NULL,
			 (caddr_t) NULL);
  }
}

static void WriteHandler (fid, dummy)
  int fid;
  caddr_t dummy;
{
  Channel channel;

  FORALL(channel, ChannelList, next)
    if (channel->fid == fid)
      break;
  if (channel == NULL) {
    MonetSysError ("WriteHandler: Couldn't identify channel for fid %d", fid);
    return;
  }
  MONETDEBUG(2, ( "Channel ready for writing %d", fid));

  write_stacked (channel);
}

/***************************************************************************/
/*  Create a new channel  */
/***************************************************************************/

extern Channel Channel_create (fid, interactive,
			       result_handler, disconnect, data)
     int fid;
     MonetBool interactive;
     ChannelHandler result_handler;
     ChannelDisconnect disconnect;
     caddr_t data;
{
  Channel channel;

  channel = MonetNew (struct monet_channel);
  if (channel == NULL) {
    MonetSysError ("Could not allocate space for channel");
    return NULL;
  }
  channel->subclass_data = NULL;
  channel->next = ChannelList;
  ChannelList = channel;
  channel -> send_queue = MonetStrFifo_create ();
  channel -> fid = fid;
  channel -> previous = '\0';
  channel -> interactive = interactive;
  channel -> handler = result_handler;
  channel -> disconnect = disconnect;
  channel -> pbuffer = channel->buffer;
  channel -> data = data;
  channel -> opened = True;
  MonetInputFd_create (fid,
		       (MonetFdHandler) ReadHandler,
		       (MonetFdHandler) NULL,
		       (caddr_t) NULL);
  return channel;
}

/***************************************************************************/
/*  Used to start a new channel, supply address of server and callback      */
/*  routines for new messages and disconnection.                           */
/***************************************************************************/

extern Channel Channel_create_client (server_port, server_address,
				      interactive,
				      result_handler, disconnect, data)
     unsigned server_port;
     MonetBool interactive;
     String server_address;
     ChannelHandler result_handler;
     ChannelDisconnect disconnect;
     caddr_t data;
{
  Channel channel;
  int n = 1, fid;



  fid = clientsock (server_address, server_port);
  if (fid == -9999) {
    MonetPrintfError ("Address of pinger [%s] is bad", server_address);
    return NULL_CHANNEL;
  } else if (fid < 0) {
    MonetPrintfInform ("Cannot connect to host [%s] port %u %s",
		       server_address, server_port, MonetPrettyTimeStamp ());
    return NULL_CHANNEL;
  }	
  /* Non blocking */
  ioctl(fid, FIONBIO, &n);
  MONETDEBUG(20, ( "Connected to server [%s] port %d",
		  server_address, server_port));

  channel = Channel_create (fid, interactive,
			    result_handler, disconnect, data);

  /* Send an initial message to get things going, Hack !!! */

  return channel;
}

/***************************************************************************/
/*  Gets rid of a channels datastructure  */
/***************************************************************************/

extern void Channel_free (channel)
     Channel channel;
{
  if (channel) {
    channel -> opened = False;
    MONETDEBUG(2, ( "Freeing channel [%d]", channel->fid));
    LIST_DEL (Channel, channel, ChannelList, next);
    MonetStrFifo_free (channel->send_queue);
    MonetInputFd_free (channel->fid);
    MonetFree (channel);
  }
}

/***************************************************************************/
/*  Send a message from the channel to the server */
/*  Do not send null messages, this avoids problems with strlen too */
/*  If it can't all be sent straight away then queue it for handling */
/*  later on.  */
/***************************************************************************/

extern MonetBool Channel_send (channel, message)
     Channel channel;
     char * message;
{
  int len;
  if (channel == NULL_CHANNEL)
    return False;

    
  len = MonetStrlen(message);
  if (len == 0)
    return True;

  if (MonetStrFifo_push (channel->send_queue, message, len)) {
    /*  This should not create a new InputFd but should change the
	existing one to select for ready for write and call the write
	handler when ready.
	*/

    MonetInputFd_create (channel->fid,
			 (MonetFdHandler) ReadHandler,
			 (MonetFdHandler) WriteHandler,
			 (caddr_t) NULL);

    /*  -- Call 'write' immediately, this is not necessary because
	-- it will be sent later anyway.
	-- But best to try and send right away for good response
	-- and to avoid excessive buffering of data.
	*/
    write_stacked (channel);

  }
  return True;
}

extern MonetBool Channel_end_of_record (channel)
     Channel channel;
{
  return Channel_send (channel, "\n");
}

/***************************************************************************/
/*  Marks channel for closing, will free it if nothing to be sent */
/***************************************************************************/

extern void Channel_shutdown (channel)
     Channel channel;
{
  if (channel) {
    unsigned len;
    char * data;
    if (data = MonetStrFifo_top (channel->send_queue, &len)) {
      /*  -- Do not listen anymore  */
      MONETDEBUG(1, ( "shutdown channel %d, still to write [%s]",
		  channel->fid, data));
      MonetInputFd_create (channel->fid,
			   (MonetFdHandler) 0,
			   (MonetFdHandler) WriteHandler,
			   (caddr_t) NULL);
      channel -> opened = False;
    } else {
      Channel_free (channel);
    }
  }
}

extern MonetBool Channel_send_record (channel, message)
     Channel channel;
     char * message;
{
  if (Channel_send (channel, message) && Channel_send (channel, "\n")) {
    Channel_flush (channel);
    return True;
  }
  return False;
}


extern void Channel_flush (channel)
     Channel channel;
{
  /* 	flush (channel->fid); */
}

/***************************************************************************/
/*  Get fid of channel  */
/***************************************************************************/

extern int Channel2fid (channel)
	Channel channel;
{
	return channel->fid;
}

extern Channel fid2Channel (fid)
	int fid;
{
	Channel c;

	FORALL(c, ChannelList, next)
		if (c->fid == fid)
			return c;

	MonetSysError ("No channel with fid %d", fid);
	return NULL;
}

extern caddr_t Channel_data (channel)
	Channel channel;
{
	return channel->data;
}

extern void Channel_help ()
{
  Channel channel;
  Server server;

  FORALL(channel, ChannelList, next) {
    unsigned len;
    char * top;

    MonetPrintf("Channel %d  ", channel->fid);
    if (! channel->opened) {
      MonetPrintf("Shutdown");
    }
    MonetPrintfnl ("");
    MonetPrintfnl("  Buffer [%s]", channel->buffer);
    if (top = MonetStrFifo_top (channel->send_queue, &len))
      MonetPrintfnl("  Write [%s]", top);
    else
      MonetPrintfnl("  Nothing to Write");
  }
  FORALL(server, ServerList, next) {
    MonetPrintfnl("Server listening on port, fd %d  ",
		  server->port, server->fd);
  }
}

/**************************************************************************
  Writes buffered
  **************************************************************************/

extern unsigned Channel_write_buffered (channel)
     Channel channel;
{
  return MonetStrFifo_size (channel->send_queue);
}

/***************************************************************************/
/*  Int and finish  */
/***************************************************************************/

extern void Channel_init ()
{
  ServerList = NULL;
}

extern void Channel_finish ()
{
  while (ServerList) {
    Server_free (ServerList);
  }
}

/***************************************************************************/
/***************************************************************************/
/*  $Header$  */
/***************************************************************************/
/***************************************************************************/


