/*********************************************************************/
/**********************************************************************/
/*  $Header$  */
/**********************************************************************/
/*  A set of routines for presenting user messages, requires a        */
/*  routine 'write_stderr'.  Could be cleaned up a bit                */
/**********************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include <monet/msh/Context.h>

static MonetStderrProc write_stderr = NULL;
static MonetConfirmProc my_confirm_box;
static MonetStdoutProc write_stdout = NULL;

int yylineno;

/**********************************************************************/
/*  Basic write interface  */
/**********************************************************************/

extern void MonetFwrite (buf, len)
     char * buf;
     int len;
{
  FILE * out;
  if (out = MonetContext_out ()) {
    (void) fwrite (buf, 1, len, out);
  } else {
    write_stdout (buf, len);
  }
}

extern void MonetErrFwrite (buf, len)
     char * buf;
     int len;
{
  write_stderr (buf, len);
}

/*****************************************************************************/
/*  Standard print routine  */
/*****************************************************************************/

/*VARARGS1*/
static void stdio_write_stderr (message, len)
     char * message;
     int len;
{
  fwrite (message, 1, len, stderr);
  fflush (stderr);
}

/*************************************************************************/
/*  Write to the report  */
/*************************************************************************/

/*VARARGS1*/
static void stdio_MonetPrintf (message, len)
     char * message;
     int len;
{
  fwrite (message, 1, len, stdout);
  fflush (stdout);
}

/*************************************************************************/
/*  Set writing routines  */
/*************************************************************************/

extern MonetStderrProc MonetStderrProc_set (writer)
     MonetStderrProc writer;
{
  MonetStderrProc previous = write_stderr;
  write_stderr = writer;
  return previous;
}

extern void Monet_confirm (routine, message)
     ConfirmRoutine routine;
     char * message;
{
  my_confirm_box (routine, message);
}

extern MonetConfirmProc set_confirm_box_routine (confirmer)
     MonetConfirmProc confirmer;
{
  MonetConfirmProc previous = my_confirm_box;
  my_confirm_box = confirmer;
  return previous;
}

extern MonetStdoutProc MonetStdoutProc_set (reporter)
     MonetStdoutProc reporter;
{
  MonetStdoutProc previous = write_stdout;
  write_stdout = reporter;
  return previous;
}

/*************************************************************************/
/*  report routines */
/*************************************************************************/

static void stdio_confirm_box (proc, message)
     ConfirmRoutine proc;
     char * message;
{
  MonetPrintfInform ("%s [yn]", message);
  if (getchar() == 'y')
    proc ();
}

/****************************************************************/
/*  Init and finish  */
/****************************************************************/

/*
static void handler (fd, data)
     int fd;
     char * data;
{
  MonetPrintf ("%s", 
}
*/
extern void MonetPrintf_init ()
{
  write_stderr = stdio_write_stderr;
  write_stdout = stdio_MonetPrintf;
  my_confirm_box  = stdio_confirm_box;
/*
  out = fileno (stdout);
  pipe;
  MonetInputFd_create (fd, fid_read, handler, NULL)
*/
}

extern void MonetPrintf_finish ()
{
}

/*****************************************************************/
/*****************************************************************/
/*  $Header$  */
/*****************************************************************/
/*****************************************************************/


