/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/FileServer/RCS/server.c,v 1.1 1991/05/06 08:17:59 a38 Exp a38 $  */
/**************************************************************************
  This sits and waits with select for new network events.  It is entirely
  syncronous, using no timing interupts. 
  Currently it multiplexes new connections, but I would like to extend it
  to listen for all the pings and requests etc. 
  **************************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Channel.h>
#include <monet/msh/Script.h>
#include <monet/msh/Event.h>
#include "monetP.h"
#include <errno.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <monet/msh/Signal.h>
#include <monet/msh/System.h>  /* For  MonetSystemHandleChildren()  */

static MonetInputFdCreator fd_create;
static MonetInputFdFreer   fd_free;
static long	Select_delay;

extern void MonetInputFd_create (fd, read_handler, write_handler, data)
     int fd;
     MonetFdHandler read_handler, write_handler;
     caddr_t data;
{
  fd_create (fd, read_handler, write_handler, data);
}

extern void MonetInputFd_free (fd)
     int fd;
{
  fd_free (fd);
}


/***************************************************************************/
/*  Private definitions  */
/***************************************************************************/

typedef struct input_fd * InputFd;
struct input_fd {
	int fd;
	MonetFdHandler read_handler, write_handler;
	caddr_t data;
	InputFd next;
};

static InputFd InputFdList = NULL;

static fd_set Readbits, Writebits;
static int Maxfd = -1;

#define SETMAXFD(fd) if(fd>Maxfd) Maxfd = fd;

static InputFd find_fd (fd)
     int fd;
{
  InputFd i;
  FORALL(i, InputFdList, next)
    if (i->fd == fd)
      return i;
  return NULL;
}

/***************************************************************************/
/*  Handle list of inputs waiting for select  */
/***************************************************************************/

static void  my_create_input_fd (fd, read_handler, write_handler, data)
     int fd;
     MonetFdHandler read_handler;
     MonetFdHandler write_handler;
     caddr_t data;
{
  InputFd input_fd;

  if (input_fd = find_fd (fd)) {
    /*  Already exists */
    if (input_fd->read_handler)
      FD_CLR(fd, &Readbits);
    if (input_fd->write_handler)
      FD_CLR(fd, &Writebits);
    MONETDEBUG(9, ( "Changing input fd [%d], read [%d] write [%d]",
		read_handler ? 1 : 0, write_handler ? 1 : 0, fd));
  } else {
    input_fd = MonetNew (struct input_fd);
    input_fd -> fd = fd;
    SETMAXFD(fd);
    LIST_ADD (input_fd, InputFdList, next);
    MONETDEBUG(9, ( "Created new input fd [%d], read [%d] write [%d]",
		read_handler ? 1 : 0, write_handler ? 1 : 0, fd));
  }

  input_fd -> data = data;
  input_fd -> read_handler  = read_handler;
  input_fd -> write_handler = write_handler;

  if (fd < 0) {
    /* bad or psuedo fid */
  } else if (fd >= getdtablesize ()) {
    MonetSysError ("fd greater= than table size [%u]", fd, getdtablesize ());
  } else {
    if (read_handler) {
      FD_SET(fd, &Readbits);
    }
    if (write_handler) {
      FD_SET(fd, &Writebits);
    }
  }
}

static void my_free_input_fd (fd)
     int fd;
{
  InputFd old;

  MONETDEBUG(14, ( "Freeing input fd [%d]", fd));
  old = find_fd (fd);
  if (old == NULL) {
    MonetSysError ("my_free_input_fd: input fd not found %d", fd);
    return;
  }
  if (fd < 0) {
    /* bad or psuedo fid */
  } else if (fd >= getdtablesize ()) {
    MonetSysError ("fd greater= than table size [%u]", fd, getdtablesize ());
  } else {
    if (old->read_handler)
      FD_CLR(fd, &Readbits);
    if (old->write_handler)
      FD_CLR(fd, &Writebits);
  }

  LIST_DEL (InputFd, old, InputFdList, next);
  close (fd);
  MonetFree (old);
}


/***************************************************************************/
/*  Wait for Messages from the client, send requests and return answers.   */
/*  The select will continue until the timer is completed event if not     */
/*  necessary, should replace the Select_delay with an indefinite   */
/*  wait.  */
/***************************************************************************/

MONET_SIGNAL_ROUTINE(sig_quit)

  MonetPrintfInform ("Caught SIGINT, once again to exit!");

MONET_END_SIGNAL_ROUTINE

extern void MonetSelectForever (before, after)
     BeforeSelectHandler * before;
     AfterSelectHandler  * after;
{
  long period = Select_delay;

  for (;;) {
    fd_set readbits, writebits, other;
    struct timeval timer;
    long start;
    int count, numbits;
    InputFd i;
    MONET_PFV previous_quit;

    if (period < 0) {
      MonetPrintfError ("io.delay is negative, reset to 30");
      period = Select_delay = 30;
    }
    bcopy ((char *)(&Readbits),  (char *)(&readbits),  sizeof(Readbits));
    bcopy ((char *)(&Writebits), (char *)(&writebits), sizeof(Writebits));
    FD_ZERO(&other);

    numbits = getdtablesize();
    timerclear(&timer);
    timer.tv_sec = period;
    if (before) {
      before (&numbits, &readbits, &writebits, &other, &timer);
      /*  Modifies the timer if an event is pending in less time than the
	  value of timer.tv_sec, it is debatable whether or not this
	  should be placed here at such a low level.  */
      (void) MonetEvent_next (&timer);
    }
    start = time (0);
    period = timer.tv_sec;

    /*  -- Ignore the first interupt only  */
    previous_quit = signal (SIGINT, sig_quit);
    do {			/* until we have pending connections */
      if ((long) timer.tv_sec < 0) {
	MonetSysError ("Timer less than zero, %d", timer.tv_sec);
      } else {
	MONETDEBUG(15, ( "Starting select for %d seconds", timer.tv_sec));
      }
      count = select(numbits, &readbits, &writebits, &other, &timer);
      MONETDEBUG(15, ( "Select done, %d fd's ready", count));

      (void) signal (SIGINT, previous_quit);
      /* -- Handle children that have died  */
      (void) MonetSystemHandleChildren ();
    } while (count < 0 && errno == EINTR);

    if (after) {
      after (count, &readbits, &writebits, &other);
      MONETDEBUG(14, ( "Done after select procedure"));
    }
    if (count < 0) {
      MonetPError ("select");
    } else if (count == 0) {
      /* Timer ran out */
    } else if (count > 0){
      for (i = InputFdList; i != NULL;) {
	/*  A connection may be removed so we need  */
	/*  to remember where the 'next one is.     */
	InputFd next;
	next = i->next;
	MONETDEBUG(14, ( " (Processing fd %d", i->fd));
	if (i->fd < 0) {
	  /*  Special connections, ie non-socket  */
	} else {
	  /*  -- We check to see if the fd is set AND to see if it is set
	      -- in the whole list, the reason for doing the second test is
	      -- that the 'read_handler' or 'write_handler' may close the
	      -- connections, in which case we the descriptor is no longer
	      -- valid and no longer needs servicing.
	      */
	  if (FD_ISSET (i->fd, &readbits) && FD_ISSET (i->fd, &Readbits)) {
	    MONETDEBUG(14, ( " Call read handler fd %d", i->fd));
	    i->read_handler (i->fd, i->data);
	  }
	  if (FD_ISSET (i->fd, &writebits) && FD_ISSET (i->fd, &Writebits)) {
	    MONETDEBUG(14, ( " Call write handler fd %d", i->fd));
	    i->write_handler (i->fd, i->data);
	  }
	}
	MONETDEBUG(14, ( " )Processed fd %d", i->fd));
	i = next;
      }
    }
    MONETDEBUG(14, ( "Done processing of fids"));
    period -= time (NULL) - start ;
    if (period <= 0)
      period = Select_delay;
  }
  /*NOTREACHED*/
}

/***************************************************************************/
/*  Set create and free routines */
/***************************************************************************/

extern void Monet_set_input_fd_routines (create, freer)
     MonetInputFdCreator create;
     MonetInputFdFreer freer;
{
  fd_create = create;
  fd_free = freer;
}

extern void Monet_reset_input_fd_routines ()
{
  fd_create = my_create_input_fd;
  fd_free = my_free_input_fd;
}

/***************************************************************************/
/*  Initialises a server, supply port number and callbacks for new         */
/*  connections, disconnections and messages,  returns an identifier       */
/***************************************************************************/

extern MonetBool monet_start_server ()
{
  (void) MonetMember_module_class ("io");
  MonetMember_create_write_long ( "delay", &Select_delay,
      "Period(secs) between sending out network queries");
  Select_delay = 30;
  Monet_reset_input_fd_routines ();
  FD_ZERO(&Readbits);
  FD_ZERO(&Writebits);
  return True;
}

/***************************************************************************/
/*  Closes a server */
/***************************************************************************/

extern void monet_close_server ()
{
}

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/server.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/


