/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/FileServer/RCS/server.c,v 1.1 1991/05/06 08:17:59 a38 Exp a38 $  */
/***************************************************************************/
/*  Basically this is my version of 'system(3)'  */
/***************************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Channel.h>
#include <monet/msh/Script.h>
#include <monet/msh/System.h>
#include "monetP.h"
#include <errno.h>
#include <monet/msh/Signal.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/syslog.h>

static MonetBool ChildDied, ChildSig;
static int DeadChildren, LiveChildren;

/*****************************************************************************
  Information about children
  ****************************************************************************/

typedef struct monet_child * Child;
struct monet_child {
  MonetBool async;
  int pid;			/* Process Id, -1 if invalid */
  Boolean alive;		/* Set false in sig_child */
  int output, errors;		/* Sockets/Pipes for comms */
  int exit;			/* Exit status of process */
  Boolean stopped;
  char * after;			/* To be performed after command has finished*/
  Child next;
};
static Child ChildList;
static int MaxChildren = 0;

/*****************************************************************************
  Print Information about children
  ****************************************************************************/

extern void MonetSystem_help ()
{
  Child child;
  MonetPrintfnl ("---- Children ----");
  MonetPrintfnl ("Total ever %u, alive now %u",
		 LiveChildren, LiveChildren - DeadChildren);
  MonetPrintfnl("Total children %u", MaxChildren);

  FORALL(child, ChildList, next) {
    MonetPrintfnl ("Child, pid [%d], out [%d], exit [%d], alive [%d]",
		   child->pid, child->output, child->exit,
		   child->alive);
  }
}

/*****************************************************************************/
/*  Get child  */
/*****************************************************************************/

static Child Child_create ()
{
  Child child;

  if (child = MonetNew (struct monet_child)) {
    LIST_ADD (child, ChildList, next);
    ++ MaxChildren;
  }
  return child;
}

/*****************************************************************************/
/*  Free child  */
/*****************************************************************************/

static void ChildFree (child)
     Child child;
{
  LIST_DEL (Child, child, ChildList, next);
  -- MaxChildren;
  MonetFree (child);
}

/*****************************************************************************/
/*  Sigchild  */
/*****************************************************************************/

MONET_SIGNAL_ROUTINE(sig_child)

  ChildSig = True;

MONET_END_SIGNAL_ROUTINE


static void check_dead_children ()
{
  for (;;) {
    int pid;
    int statusp;
    Child child;

#if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
    pid = wait3 ((union wait*) &statusp,
		 WNOHANG | WUNTRACED, (struct rusage *) 0);
#else
    pid = wait3 (&statusp, WNOHANG | WUNTRACED, (struct rusage *) 0);
#endif

    if (pid <= 0)
      break;

    FORALL (child, ChildList, next) {
      if (pid == child->pid) {
	if (WIFSTOPPED(statusp)) {
	  /* -- child process has stopped, not terminated
	     -- what should be done here?
	   -- maybe just kill it and so disallow stopped processes?
	   */
	  child->stopped = True;
	  break;
	}
	if (WIFSIGNALED(statusp)) {
	  /* -- Died due to a signal  */
	  child->exit  = 1;  /*WTERMSIG(statusp);*/
	} else if (WIFEXITED(statusp)) {
	  /* -- Process exited */
	  child->exit  = WEXITSTATUS(statusp);
	} else {
	  /* -- Very strange, how did it die, did it die?
	     -- should I just kill and return here to make sure?
	     */
	  child->exit  = 1;
	}
	child->alive = False;
	++ DeadChildren;
	ChildDied = 1;
	break;
      }
    } /*  for i */
  } /* for (;;) */
}

/*****************************************************************************/
/*  Check to see if a child died and if so clean up  */
/*****************************************************************************/

extern MonetBool MonetSystemHandleChildren ()
{
  Child child;

  MONETDEBUG(15, ( "Children: live %d, dead %d", LiveChildren, DeadChildren));

  if (ChildDied) {
    /* -- a child died */

    MONETDEBUG(5, ( "Child died"));

    FORALL(child, ChildList, next) {
      if (child->alive) {
	if (child->output >= 0)
	  close (child->output);
	if (child->errors >= 0)
	  close (child->errors);

	syslog(LOG_INFO, "Child [Pid:%d] exit:%d",
	       child->pid, child->exit);

	MONETDEBUG(5, ( "  Dead Child, pid [%d], out/err [%d/%d], exit [%d]",
		    child->pid,
		    child->output,
		    child->errors,
		    child->exit));
	/*  Execute the after command  */
	if (child->after) {
	  MonetEntity_do_action (NULENT, child->after, 0);
	  MonetFree (child->after);
	}
	ChildFree (child);
      }
    }
    ChildDied = 0;
    return True;
  }
  return False;
}

/*****************************************************************************/
/*  Fork off a child  */
/*****************************************************************************/

static Child fork_child (argv, input, after)
     int input;
     char * after;
     char **argv;
{
  int pid;
  int output_pipe [2], errors_pipe [2];
  Child child;

  child = Child_create ();
  if (!child)
    return NULL;

  /* -- set up pipes for stdout and stderr  */
  if (pipe (output_pipe) != 0) {
    MonetPError ("output pipe");
    ChildFree (child);
    return NULL;
  }
  if (pipe (errors_pipe) != 0) {
    close (output_pipe [0]);
    close (output_pipe [1]);
    MonetPError ("error pipe");
    ChildFree (child);
    return NULL;
  }

  /* -- Fork off child  */
  pid = fork ();
  if (pid < 0) {
    close (output_pipe [0]);
    close (output_pipe [1]);
    close (errors_pipe [0]);
    close (errors_pipe [1]);
    ChildFree (child);
    MonetPError ("fork");
    return NULL;
  } else if (pid) {
    MONETDEBUG(5, ( "Parent: just started child, pid = %d", pid));
    /*  Parent  */
    ++ LiveChildren;
    child->pid = pid;
    child->output = output_pipe [0];
    child->errors = errors_pipe [0];
    child->after  = MonetNewString (after);
    child->alive  = True;
    child->stopped = False;
    close (output_pipe [1]);
    close (errors_pipe [1]);
  } else {
    /*  Child  */
    extern char **environ;
    if (input < 0) {
      close (0);
      (void) open ("/dev/null", O_RDONLY , 0); /*  */
    } else if (input > 0) {
      close (0);
      dup (input);
      close (input);
    }
    close (1);
    dup (output_pipe [1]);
    close (output_pipe [0]);
    close (output_pipe [1]);
    close (2);
    dup (errors_pipe [1]);
    close (errors_pipe [0]);
    close (errors_pipe [1]);
    (void) execve(argv[0], argv, environ);
    exit (127);
    /* NOTREACHED */
  }
  return child;
}

/*****************************************************************************/
/*  Initialise children  */
/*****************************************************************************/

extern void child_init ()
{
  /*  -- Just a record of number of spawned children  */
  DeadChildren = 0;
  LiveChildren = 0;

  /* -- Set by signal routine to indicate dead children  */
  ChildDied = 0;

  /*  -- Ignore dying children, none of the masters business once they
      -- have spawned off.  */

  (void) signal (SIGCHLD, sig_child);
}

/**************************************************************************
  Read and Write handlers for asyncronous shell command
  ***************************************************************************/

static void StderrHandler (fid, dummy)
  int fid;
  caddr_t dummy;
{
  Child child = (Child) dummy;
  char buffer [2048]; /* !!! */
  int cc;

  cc = read (fid, buffer, sizeof (buffer) - 1);
  if (cc <= 0) {
    if (cc < 0) {
      MonetPError ("read stderr");
    }
    close (child->errors);
    child -> errors = -1;
    if (child->async) {
      MonetInputFd_free (fid);
    }
  } else {
    MonetErrFwrite (buffer, cc);
  }
}

static void StdoutHandler (fid, dummy)
  int fid;
  caddr_t dummy;
{
  Child child = (Child ) dummy;
  char buffer [2048]; /* !!! */
  int cc;

  cc = read (fid, buffer, sizeof (buffer) - 1);
  if (cc <= 0) {
    if (cc < 0) {
      MonetPError ("read output");
    }
    close (child->output);
    child->output = -1;
    if (child->async) {
      MonetInputFd_free (fid);
    }
  } else {
    MonetFwrite (buffer, cc);
  }
}

static void StdinHandler (fid, dummy)
  int fid;
  caddr_t dummy;
{

}

/**************************************************************************
  asyncronous shell command
  **************************************************************************/

extern int MonetAsyncSystem (after, args)
     char * after, ** args;
{
  Child child;

  if (child = fork_child (args, -1, after)) {
    child -> async = True;
    MonetInputFd_create (child->output, StdoutHandler, NULL, (caddr_t) child);
    MonetInputFd_create (child->errors, StderrHandler, NULL, (caddr_t) child);
    return True;
  }
  return False;
}

/**************************************************************************
  syncronous sh
  **************************************************************************/

extern int MonetSystem (args, input, debug_command)
     char ** args, * debug_command;
     int input;
{
  Child child;
  MONET_PFV prev_sig_child;

  if (child = fork_child (args, input, NULSTR)) {
  } else {
    return 127;
  }

  child -> async = False;

  ChildSig = False;

  prev_sig_child = signal (SIGCHLD, sig_child);
  if (prev_sig_child != sig_child) {
    MonetSysError ("Sig Child procedure has changed");
  }

  /*  -- Loop until child dies  */
  for (;;) {
    int numbits, count;
    struct timeval timer;
    fd_set readbits, other, writebits;

    timerclear(&timer);
    timer.tv_sec = 30;

    FD_ZERO(&readbits);
    FD_ZERO(&writebits);
    FD_ZERO(&other);

/*    if (child->input >= 0)
      FD_SET(child->output, &writebits);
    if (input >= 0)
      FD_SET(input, &readbits);
      */
    if (child->output >= 0)
      FD_SET(child->output, &readbits);
    if (child->errors >= 0)
      FD_SET(child->errors, &readbits);
    if (child->output > child->errors)
      numbits = child->output+1;
    else
      numbits = child->errors+1;

    if (ChildList == NULL) {
      MonetSysError ("Child list is empty");
      break;
    }
    count = select (numbits, &readbits, &writebits, &other, &timer);
    if (count < 0) {
      if (errno == EINTR) {
	/* -- Interupt, probably child died, so ignore */
      } else {
	MonetPError ("%s", debug_command);
      }
    } else if (count == 0) {
      /* Timer ran out */
    } else {
      if (child->output >= 0 && FD_ISSET(child->output, &readbits)) {
	StdoutHandler (child->output, (caddr_t) child);
      }
      if (child->errors >= 0 && FD_ISSET(child->errors, &readbits)) {
	StderrHandler (child->errors, (caddr_t) child);
      }
    }
    /*  A hack because i'm having trouble getting a signal for some reason  */
    check_dead_children ();
    if (ChildSig) {
      MONETDEBUG(1, ( "MonetSystem: Got child signal [%s] pid %d",
		  debug_command, child->pid));
    }
    if (child->stopped) {
      MonetPrintfWarning ("Child Process [PID:%u] has stopped", child->pid);
      child->stopped = False;
    }
    if (! child->alive) {
      break;
    }
    if (MonetException_code () != MONETSH_OKAY) {
      /* -- Kill the child  */
      if (kill (child->pid, SIGKILL) != 0) {
	/*  The kill failed so print a warning and exit  */
	child->exit = 1;
	MonetPError ("Kill Child pid %d [%s]", child->pid, debug_command);
	break;
      }
    }
  }
  if (child->output >= 0) {
    close (child->output);
  }
  if (child->errors >= 0) {
    close (child->errors);
  }
  if (child->exit == 127) {
    MonetPrintfError ("Could not execute shell for [%s]", debug_command);
  }
  ChildFree (child);
  (void) signal (SIGCHLD, prev_sig_child);
  return child->exit;
}

/***************************************************************************/
/*  Init and finish  */
/***************************************************************************/

extern void monet_sh_init ()
{
  ChildDied = False;
  ChildList = NULL;
 (void) signal (SIGCHLD, sig_child);
}

/***************************************************************************/
/*  Closes a server */
/***************************************************************************/

extern void monet_sh_finish ()
{
}

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/Monetd/RCS/server.c,v 1.2 1991/03/22 16:09:27 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/
