/***********************************************************************/
/***********************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/general/RCS/config.c,v 1.3 1991/09/06 10:31:36 a38 Exp a38 $  */
/***********************************************************************/
/* $Log: config.c,v $
 * Revision 1.3  1991/09/06  10:31:36  a38
 * Configuration files changed, these are now monetsh programs.
 * */
/***********************************************************************/

#include "General.h"
#include "scriptP.h"
#include <sys/ioctl.h>

/***********************************************************************/
/*  Configuration files:                                               */
/*  Lowest level system file, ~/.<config>, and those on command line   */
/*  Can configure from the command line too                            */
/***********************************************************************/

static MonetBool  Config_command_line_shell;
static String	Config_startup_command;
static String   Config_file, OnceCommand;

/***********************************************************************/
/*  Configuation options structure  */
/***********************************************************************/

typedef struct monet_config * MonetConfig;
struct monet_config {
  String name, function, default_args, help;
  MonetConfig next;
};
static MonetConfig ConfigList = NULL;

/***********************************************************************/
/*  Create and find configuration options  */
/***********************************************************************/

extern void MonetConfig_create (name, function, default_args, help)
  String name, function, default_args, help;
{
  MonetConfig config;
  config = MonetNew (struct monet_config);
  config -> name = MonetNewString (name);
  config -> function = MonetNewString (function);
  config -> default_args = MonetNewString (default_args);
  config -> help = MonetNewString (help);
  LIST_ADD (config, ConfigList, next);
}

extern MonetConfig MonetConfig_find (name)
     String name;
{
  MonetConfig config;
  FORALL (config, ConfigList, next)
    if (EQS(name, config->name))
      return config;
  return NULL;
}

/***********************************************************************/
/*  Display configuration options                                      */
/***********************************************************************/

MONETCALLBACK(do_help_options)
     MonetConfig config;

     MonetPrintf ("[ ");
     FORALL (config, ConfigList, next) {
       MonetPrintf (" %s", config->name);
     }
     MonetPrintf ("]");
MONETEND

MONETCALLBACK(do_help_config)
     MonetConfig config;
     if (argc > 1) {
       if (EQS (argv[1], "full")) {
	 FORALL (config, ConfigList, next) {
	   if (config->default_args) {
	     MonetPrintfnl ("%8s [%8s %11s] - %s", config->name,
			    config->function,
			    config->default_args,
			    config->help);
	   } else {
	     MonetPrintfnl ("%8s [%20s] - %s", config->name,
			    config->function,
			    config->help);
	   }
	 }
       } else if (EQS (argv[1], "manual")) {
	 FORALL (config, ConfigList, next) {
	   MonetPrintfnl (".TP 8\n.B %12s \\-\n%s",
			  config->name, config->help);
	 }
       } else {
	 char * name = argv[1];
	 if (*name == '-')
	   ++name;
	 FORALL (config, ConfigList, next) {
	   if (EQS (name, config->name + 1)) {
	     MonetPrintfnl ("%8s - %s", config->name, config->help);
	   }
	 }
       }
     } else {
       FORALL (config, ConfigList, next) {
	 MonetPrintfnl ("%8s - %s", config->name, config->help);
       }
     }
MONETEND

/***********************************************************************/
/*  Print options in nroff format for man page                         */
/***********************************************************************/

MONETCALLBACK(do_manpage_syntax)
     MonetConfig config;
     FORALL (config, ConfigList, next) {
       MonetPrintfnl (".IP \\fB\\%s\\fP", config->name);
       MonetPrintfnl ("%s", config->help);
     }
MONETEND

MONETCALLBACK(do_manpage_options)
     MonetConfig config;
     MonetPrintfnl (".B %s", Monet_program_name ());
     FORALL (config, ConfigList, next) {
       MonetPrintf (" [\\%s]", config->name);
     }
     MonetPrintfnl ("");
MONETEND

/***********************************************************************/
/*  Handle command line options                                        */
/***********************************************************************/

extern void Monet_command_line_options (argc, argv)
	int argc;
	char **argv;
{
  unsigned i;

  for (i = 1; i < argc; ) {
    MonetConfig config;
    char buffer [1024];
    String f = buffer;
    *f = NULL;
    if (!argv[i] || !*argv[i]) {
      /*  Ignore null or empty arguments  */
    } else if (EQS(argv[i], "-c")) {
      /* Execute rest of arguments as a command and exit.  */
      OnceCommand = MonetNewString (MonetJoinStrings
				    (argc-i-1, argv+i+1, " "));
      break;
    } else if (config = MonetConfig_find (argv[i])) {
      MonetBool found = False;
      f = MonetStringCopyToEnd (f, config->function);
      for (++i; argv[i]; ++i) {
	if (argv[i][0] == '-') {
	  break;
	}
	found = True;
	*f = ' '; ++f;
	f = MonetString2PrintString (f, argv[i], sizeof (buffer));
      }
      if (found == False) {
	*f = ' '; ++f;
	f = MonetStringCopyToEnd (f, config->default_args);
      }
      *f = NULL;
      (void) MonetshExecute (buffer, NULL, False);
    } else {
      MonetPrintfError ("No such option [%s]", argv[i]);
      exit (1);
    }
  }
}

/***********************************************************************/
/*  Load config files and interpret command line args  */
/***********************************************************************/

static FILE * source = NULL;

extern void Monet_user_config (argc, argv)
     int argc;
     char **argv;
{
  char * file;
  source = stdin;

  /* -- Do options once in case config file is mentioned  */
  Monet_command_line_options (argc, argv);

  /* -- system config file */
  file = MonetMakePathname (Monet_dir (NULL), Config_file, NULSTR);
  (void) Script_source_file (file, ".msh",
			     "Sorry, can't open configuration file");
  /* -- Do options second time, since they have priority  */
  Monet_command_line_options (argc, argv);

  /* -- Initial start up command  */
  MonetshExecute (Config_startup_command, NULSTRSTR, False);

  /* -- Just execute the command specified on the command line and
     -- then quit.  */
  if (OnceCommand) {
    MonetshExecute (OnceCommand, NULSTRSTR, False);
    Monet_quit (0);
    /* NOTREACHED  */
  }
  if (Config_command_line_shell) {
    int n = 1, fd;
    source = stdin;
    printf ("%s> ", Monet_program_name ());
    fflush (stdout);
    fd = fileno (source);
    /*  Read asyncronous commands from 'fd and execute them */
    ioctl(fd, FIONBIO, &n);
    MonetshExecute_fd_async (fd, NULSTRSTR, Config_command_line_shell);
  } else {
    MONETDEBUG(1,
	("Can only use %s interactively just now, 'config command_line_shell'",
	Monet_program_name ()));
  }
}

/***********************************************************************/
/*  Initialise everything, read in config files  */
/***********************************************************************/

extern void MonetConfig_init (file, argc, argv)
     char * file;
     int argc;
     char **argv;
{
  OnceCommand = NULSTR;
  Config_command_line_shell = True;
  Config_file = MonetNewString (file);

  (void) MonetMember_module_class ("config");

  MonetConfig_create ("-c", "",
		      "",
		      "Execute rest of command line as command and exit");

  MonetMember_create ( "help",		do_help_config,
		      "?name",
		      "Lists configuration parameters, if name given \
just lists the one paramenter, can also be 'full'");
  MonetMember_create ( "options",	do_help_options,
		      "?name",
		      "Lists command line options in a brief format");
  MonetMember_create ( "man-options",		do_manpage_options,
		      "",
		      "Lists options for use in OPTIONS section of man page");
  MonetMember_create ( "man-syntax",		do_manpage_syntax,
		      "",
		      "Lists options for use in SYNTAX section of man page");
  MonetMember_create_read ( "startup-file", &Config_file,
			   "Name of the startup configuration file");
  MonetMember_create_write_boolean ( "command-line-shell",
				    &Config_command_line_shell,
			      "Allow interactive input from the command line");
  MonetMember_create_write ( "startup-command", &Config_startup_command,
			    "A monet script that will be executed at startup");

  /***********************************************************************/
  /*  Command line options  */
  /***********************************************************************/

  MonetConfig_create ( "-help", "config->help ; quit", NULSTR,
		      "lists command line options and exits");
  MonetConfig_create ( "-options", "Config->options ; quit", NULSTR,
		      "brief list of command line options");
  MonetConfig_create ("-exec",  "config->startup-command", NULSTR,
		      "A command to execute on start up");
  MonetConfig_create ("-shell", "config->command-line-shell",
		      "$true", "Prompt for input from the command line");
}

/***********************************************************************/
/*  Finish  */
/***********************************************************************/

extern void MonetConfig_finish ()
{
 if (source && source != stdin)
    fclose (source);
  while (ConfigList) {
    MonetFree (ConfigList->name);
    MonetFree (ConfigList->function);
    MonetFree (ConfigList->default_args);
    MonetFree (ConfigList->help);
    ConfigList = ConfigList->next;
  }
}

/***********************************************************************/
/***********************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/general/RCS/config.c,v 1.3 1991/09/06 10:31:36 a38 Exp a38 $  */
/***********************************************************************/
/***********************************************************************/
