/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "scriptP.h"
#include <monet/msh/Channel.h>
#include <monet/msh/Context.h>
#include <monet/msh/System.h>
#include <monet/msh/Signal.h>

struct monet_context {
  MonetStr filename;
  int line_number;
  MonetMember executing;
  MonetBool interactive;
  unsigned stack_depth;
  String * args;
  MonetTree head, tail;
  int depth;
  long started;
  MonetEntityStack local, use;
  Channel channel;
  char * insert_data;
  char * trace;
  FILE * in, * out, * err;
};

static MonetContext Current = NULL;
static MonetStr  UnknownStr;
static MonetEntityStack Local, Use;
static int LocalDepth;
static MonetEntity Global, LocalClass;
static char * Line = NULL;

/***************************************************************************
  FILE I/O
  ****************************************************************************/

extern FILE * MonetContext_in ()
{
  if (Current->in)
    return Current -> in;
  return stdin;
}

extern FILE * MonetContext_set_in (file)
     FILE * file;
{
  FILE * prev;
  prev = Current->in;
  Current->in = file;
  return prev;
}

extern FILE * MonetContext_set_out (file)
     FILE * file;
{
  FILE * prev;
  prev = Current->out;
  Current->out = file;
  return prev;
}

extern FILE * MonetContext_err ()
{
  if (Current->err)
    return Current -> err;
  return NULL;
}

extern FILE * MonetContext_out ()
{
  if (Current->out)
    return Current -> out;
  return NULL;
}

/*****************************************************************************/
/*  A message showing currently executing statement  */
/*****************************************************************************/

extern unsigned MonetContext_message (buffer, message)
     char * message, * buffer;
{
  unsigned len;
  char * space, * place;

  place = buffer;

  if (message) {
    (void) sprintf (place, "[%s", message);
    space = " ";
    len = strlen (place);
    place = buffer + len;
  } else {
    (void) sprintf (place, "[");
    space = "";
    len = 1;
    ++place;
  }

  if (Current) {
    if (Current->executing) {
      (void) sprintf (place, "%sin %s",
		      space,
		      MonetMember2DeclaredString (Current->executing));
      space = ": ";
      len += strlen (place);
      place = buffer + len;
    }
    if (Current->filename) {
      (void) sprintf (place, "%s%s, line %lu",
		      space,
		      MonetStr2String (Current->filename),
		      Current->line_number);
      len += strlen (place);
      place = buffer + len;
    }
  }
  (void) sprintf (place, "]");
  ++ len;
  return len;
}

/*****************************************************************************/
/*  Execute within a context  */
/*****************************************************************************/

MONET_SIGNAL_ROUTINE(sig_quit)

  MonetException_raise (MONETSH_SIGNAL);
  MonetPrintfInform ("Caught SIGINT");

MONET_END_SIGNAL_ROUTINE

static char * MonetContext_execute (context, entity)
     MonetContext context;
     MonetEntity entity;
{
  char * result;
  MONET_PFV previous_quit;

  MonetException_raise (MONETSH_OKAY);
  previous_quit = signal (SIGINT, sig_quit);

  context->started = time(0);

  result = MonetTree_execute
    (entity, context->head, context->args);

  (void) signal (SIGINT, previous_quit);
  if (MonetException_code () == MONETSH_SIGNAL) {
    MonetPrintfWarning ("Execution Broken");
  }
  if (context->interactive && result && *result) {
    MonetPrintfnl ("[%s]", result);
  }
  return result;
}

/*****************************************************************************/
/*  Basic routine for executing string commands  */
/*****************************************************************************/

extern String MonetshEntityExecute (entity, cp, args, interactive)
     MonetEntity entity;
     MonetBool interactive;
     String cp, * args;
{
  String result = NULL;
  /* --- If in interactive mode add the command to the history list */
  if (cp && (! interactive || (cp = MonetHistory_add (cp)))) {
    MonetTree exec, tail = NULL;
    int depth = 0;
    MonetContext previous;
    char * previous_line = Line;

    previous = Current;
    Current = MonetContext_create (interactive, args, True);
    Line = cp;
    MONETDEBUG(20, ( "Begin Execute [%s]", cp));
    depth = 0;
    exec = String2MonetTree (cp, &tail, &depth, True);
    MonetTreeFixTail (exec, tail);
    if (depth != 0) {
      MonetParseError ("Unbalanced parenthesis [%s]", cp);
    } else if (tail && tail->type != lex_end) {
      MonetParseError ("Command not complete [%s] [%s]", cp, LNAME(tail));
    } else if (exec = MonetTree_parse (exec)) {
      Current -> head = exec;
      Current -> args = args;
      Current -> interactive = interactive;
      result = MonetContext_execute (Current, entity);
      MONETDEBUG(20, ( "End Execute [%s] result [%s]", cp, result));
    } else {
      MONETDEBUG(20, ( "Fail Parse Execute or empty [%s]", cp));
    }
    Current -> head = NULL;
    Current -> tail = NULL;
    MonetTree_free (exec);
    MonetContext_free (Current);
    Line = previous_line;
    Current = previous;
  }
  return result;
}

extern String MonetshExecute (cp, args, interactive)
     String cp, * args;
     MonetBool interactive;
{
  return MonetshEntityExecute ( NULENT, cp, args, interactive);
}

/*****************************************************************************/
/*  Same as above but automatically defines a variable  */
/*  The variable is removed before returning.  */
/*****************************************************************************/

extern String MonetshExecute_defined (command, args, names,
				      values, interactive)
     String command, *args;
     String *names;
     String *values;
     MonetBool interactive;
{
  String result;
  unsigned argc;

  MonetMember_deeper ();
  if (names)
    for (argc = 0; names[argc] ; ++argc)
      MonetMember_create_local (names[argc], values[argc]);
  result = MonetshExecute (command, args, interactive);
  MonetMember_shallower ();
  return result;
}

/*****************************************************************************/
/*  Execute successive input from stdin/file etc, contextronously  */
/*****************************************************************************/

static MonetBool process_command (context, command)
     MonetContext context;
     char * command;
{
  MonetContext previous;
  int depth;
  char modechar = '>';

  if (context == NULL) {
    MonetSysError ("NULL context");
    return False; /* Close connection */
  }
  previous = Current;
  Current  = context;

  Line = command;
  depth = context->depth;
  if (context->interactive)
    Line = command = MonetHistory_add (command);

  if (context->head) {
    (void) String2MonetTree (command, &(context->tail), &depth, True);
  } else {
    context->tail = NULL;
    context->head =
      String2MonetTree (command, &(context->tail), &depth, True);
  }
  if (Monet_debug () == 6) {
    MONETDEBUG(6, ( "Depth of parenthesis [%d]", depth));
    MonetTree_dump (context->head, 0);
  }
  if (depth < 0) {
    MonetParseError ("Too many )'s");
    MonetTree_free (context->head);
    context->head = NULL;
    context->tail = NULL;
    depth = 0;
  } else if (context->head == NULL) {
  } else if (context->tail->type != lex_end) {
    modechar = '?';
  } else if (context->head = MonetTree_parse (context->head)) {

    (void) MonetContext_execute (context, NULENT);
    MonetTree_free (context->head);
    context->head = NULL;
    context->tail = NULL;
    depth = 0;
  }
  if (context->interactive) {
    MonetException_report ();
    MonetPrintf ("%s%c ", Monet_program_name (), modechar);
  }
  context->depth = depth;
  if (context->interactive)
    fflush (stdout);  /* !!! Questionable */
  Line = NULL;
  if (context->insert_data) {
    char *prev_data = context->insert_data;
    context->insert_data = NULL;
    process_command (context, prev_data);
    MonetFree (prev_data);
  }
  Current = previous;
  return True;
}

/****************************************************************
  Called everytime new data arrives on the channel
  ***************************************************************/

static MonetBool script_callback (channel, command)
     Channel channel;
     char * command;
{
  MonetContext context;

  context = (MonetContext) Channel_data (channel);

  return process_command (context, command);
}

/****************************************************************/
/*  Called when channel disconnects  */
/****************************************************************/

static void disconnect (chan)
     Channel chan;
{
  MonetContext context = (MonetContext) Channel_data (chan);

  process_command (context, NULSTR);
  MONETDEBUG(20, ( "End of script channel"));
  MonetTree_free (context->head);
  context->head = NULL;
  MonetContext_free (context);
  if (context->interactive && ! MonetIsDaemon ()) {
    /*  This should really be configurable !!! */
    Monet_quit (0);
  }
}

extern void MonetContext_eval (string)
     char * string;
{
  MonetContext context;
  if (context = MonetContext_current ()) {
    context->insert_data =
      MonetReallocAppendString (context->insert_data, string);
  }
}

/****************************************************************/
/*  */
/****************************************************************/

extern void MonetshExecute_fd_async (fd, args, interactive)
     int fd;
     char **args;
     MonetBool interactive;
{
  MonetContext context;

  context = MonetContext_create (interactive, args, False);
  context->channel = Channel_create (fd, interactive, script_callback,
				     disconnect, (caddr_t) context);
}

/***************************************************************************/
/*  Source file  */
/***************************************************************************/

extern MonetBool Script_source_file (in_filename, extension, warning)
     String in_filename, warning, extension;
{
  FILE * f;
  char buf [2048], * filename;
  MonetBool result = False;

  /*  This is a sort of hack, the function MonetMakePathname
      malloces space for its result.  This function is called from
      MonetFopenFileWithExtension.  in_filename might also be the result
      of MonetMakePathname, hence possible confusion.
      */
  in_filename = MonetNewString (in_filename);
  filename = in_filename;

  if (f = MonetFopenFileWithExtension (filename, extension, warning, "r")) {
    MonetContext previous;
    String cp = buf;
    MonetStr previous_file, filestr;
    unsigned space;
    int old_line_number;

    previous = Current;
    Current  = MonetContext_create (False, NULL, True);

    filestr = MonetStr_refer (filename);
    previous_file = MonetContext_filename (filestr);
    old_line_number = MonetContext_line_number (0);

    if (previous_file) {
      MONETDEBUG(10, ( "Sourcing file [%s], previous [%s]",
		  filename, MonetStr2String (previous_file)));
    } else {
      MONETDEBUG(10, ( "Sourcing file [%s]", filename));
    }

    space =  sizeof (buf) - 1;
    while (fgets (cp, space, f) != NULSTR)  {
      unsigned len = MonetStrlen (cp);

      if (MonetException_code ()) {
	break;
      }
      ++Current->line_number;
      if (len >= 2 && cp[len-1] == '\n' && cp[len-2] == '\\') {
	cp[len-2] = '\0';
	cp += len-2;
	space -= len-2;
	if (space <= 0) {
	  MonetPrintfError ("Line longer than buffer size [%d]", sizeof (buf));
	  process_command (Current, buf);
	  space =  sizeof (buf) - 1;
	  cp = buf;
	}
      } else {
	if (len >= 1 && cp[len-1] == '\n')
	  cp[len-1] = NULL;
	process_command (Current, buf);
	space =  sizeof (buf) - 1;
	cp = buf;
      }
    }
    if (cp > buf) {
      process_command (Current, buf);
    }
    process_command (Current, NULSTR);
    fclose (f);


    result = True;

    MonetStr_derefer (filestr);

    MonetContext_free (Current);
    Current = previous;
    (void) MonetContext_filename (previous_file);
    (void) MonetContext_line_number (old_line_number);
  }
  MonetFree (in_filename);
  return result;
}

/***************************************************************************/
/*  There is currently no current context, needs fixed  */
/***************************************************************************/

extern MonetContext MonetContext_current ()
{
  return Current;
}

/***************************************************************************/
/*  When was the command started
    does not make use of the context information yet  */
/***************************************************************************/

extern long MonetContext_get_started (context)
     MonetContext context;
{
  return context->started;
}

/****************************************************************************
  Trace
  ***************************************************************************/

extern void MonetContext_set_trace (pattern)
     char * pattern;
{
  MonetFree (Current->trace);
  Current->trace = MonetNewString (pattern);
}

extern char * MonetContext_get_trace ()
{
  return Current->trace;
}

extern MonetBool MonetContext_trace (id)
     MonetStr id;
{
  if (Current->trace) {
    return MonetMatchWildcard(MonetStr2String (id), Current->trace);
  }
  return False;
}

/*****************************************************************************/
/*  Get name of file being read in and current line number  */
/*****************************************************************************/

extern void MonetContext_filename_line (file, line)
     MonetStr * file;
     int *line;
{
  if (*file) {
    *file = Current->filename;
  }
  if (*line) {
    *line = Current->line_number;
  }
}

extern MonetStr MonetContext_filename (name)
     MonetStr name;
{
  MonetStr previous;

  previous = Current->filename;
  Current->filename = name;

  return previous;
}

extern MonetMember MonetContext_executing_member (member)
     MonetMember member;
{
  MonetMember previous_member;

  previous_member = Current->executing;
  Current->executing = member;

  return previous_member;
}

extern int MonetContext_line_number (number)
     int number;
{
  int previous;

  previous = Current->line_number;
  Current->line_number = number;

  return previous;
}

/*****************************************************************************/
/*  Create and destory contexts  */
/*****************************************************************************/

extern MonetContext MonetContext_create (interactive, args, inherit)
     MonetBool interactive, inherit;
     char ** args;
{
  MonetContext context, prev = Current;

  context = MonetNew (struct monet_context);
  context->executing = NULL;
  context->filename = NULL;
  context->line_number = 0;
  context->interactive = interactive;
  context->started = 0;
  context->depth = 0;
  context->stack_depth = 0;
  context->args = (String *) MonetNewString ((String) args);
  context->head = NULL;
  context->tail = NULL;
  context->channel = NULL;
  context->started = time (0);
  context->local = Local;
  context->use = Use;
  context->insert_data = NULL;
  if (Current && inherit) {
    context->in  = Current->in;
    context->err = Current->err;
    context->out = Current->out;
    context->trace = MonetNewString (Current->trace);
  } else {
    context->in  = NULL;
    context->err = NULL;
    context->out = NULL;
    context->trace = NULSTR;
  }
  Current = context;
  MonetMember_deeper ();
  Current = prev;
  return context;
}

extern void MonetContext_free (context)
  MonetContext context;
{
  MonetContext prev = Current;

  Current = context;
  MonetMember_shallower ();
  if (context -> channel) {
    Channel_shutdown (context->channel);
  }
/*  MonetEntityStack_free (context->local);
  MonetEntityStack_free (context->use);
*/
  MonetFree (context->trace);
  MonetFree (context->args);
  MonetFree (context);

  if (context == prev) {
    Current = NULL;
  } else {
    Current = prev;
  }
}

/***************************************************************************/
/*  Nesting  */
/***************************************************************************/

extern MonetEntity MonetLocalStackTop ()
{
  return MonetEntityStack_top (Current->local);
}

extern MonetEntityStack MonetLocalStack ()
{
  return Current->local;
}

extern MonetEntityStack MonetUseStack ()
{
  return Current->use;
}

extern MonetEntity MonetGlobal ()
{
  return Global;
}

extern void MonetMember_deeper ()
{
  MonetEntity entity;
  char buffer [16];

/*  (void) sprintf (buffer, "local%d", Current->stack_depth);*/
  (void) sprintf (buffer, "l%d", LocalDepth);
  if (entity = MonetEntity_create_instance (LocalClass, buffer)) {
    MonetEntityStack_push (Current->local, entity);
    Current->stack_depth ++;
    ++ LocalDepth;
  } else {
    MonetSysError ("Entity not created [%s]", buffer);
  }
  MONETDEBUG(11, ( "Deeper depth [%d]", Current->stack_depth));
}

extern void MonetMember_shallower ()
{
  MonetEntity local;
  MONETDEBUG(11, ( "Shallow depth [%d]", Current->stack_depth));
  if (local = MonetEntityStack_pop (Current->local)) {
    Current->stack_depth--;
    -- LocalDepth;
    MonetEntity_free (local);
  }
}

/***************************************************************************
  -- Search list of known commands given the entity          
  -- This is an intensely inefficient routine.               

  Records last looked up member,  the cache is cleared every time a      
  member is created or destroyed, or when a new entity is created.       
  This is a trivial optimisation aimed at repeated instructions and is   
  probably dangerous.

**************************************************************************/

static MonetMember find (entity, id, which)
     MonetEntity entity, * which;
     MonetStr id;
{
  MonetMember member;
  MonetEntity e, topuse, top;
  static MonetMember PrevMember = NULL;
  static MonetEntity PrevEntity, PrevWhich;
  static long Change = 0;
  long change;

  if (! Current) {
    MonetSysError ("MemberFind, Current is null");
    return NULL;
  }

  change = MonetMembers_changed ();
  if (change == Change && PrevMember) {
    if (entity == PrevEntity && id == MonetMember2Str (PrevMember)) {
      MONETDEBUG(20, ( "Got Cache Entity[%s] Id[%s] Which[%s]",
		  entity ? MonetEntity_title (entity) : "",
		  MonetStr2String (id),
		  MonetEntity_title (PrevWhich)));
      *which = PrevWhich;
      return PrevMember;
    }
  } else {
    PrevMember = False;
    Change = change;
  }

  for (e = MonetEntityStack_top (Current->local); e;
       e = MonetEntityStack_next (Current->local))
    if (member = MonetMember_lookup (e, id)) {
      *which = e;
      PrevWhich = *which;
      PrevMember = member;
      PrevEntity = entity;
      return member;
    }

  /*  Try entity, check everything including top class */
  if (member = MonetEntity_inherit (entity, id, NULL)) {
    *which = entity;
    PrevWhich = *which;
    PrevMember = member;
    PrevEntity = entity;
    return member;
  }

  top = MonetEntityTopClass ();
  topuse = MonetEntityStack_top (Current->use);
  for (e = topuse; e; e = MonetEntityStack_next (Current->use))
    if (member = MonetEntity_inherit (e, id, top)) {
      *which = e;
      PrevWhich = *which;
      PrevMember = member;
      PrevEntity = entity;
      return member;
    }

  /*  If no entity then the top class was not checked, so try it
      now.  If anything found set 'which' to the top of the use
      stack.  If no use stack set it to 'top'.  */
  if (!entity && (member = MonetMember_lookup (top, id))) {
    if (topuse) {
      *which = topuse;
    } else {
      *which = top;
    }
    PrevWhich = *which;
    PrevMember = member;
    PrevEntity = entity;
    return member;
  }
    PrevMember = NULL;
    Change = 0;

    return NULL;
}

extern MonetMember MonetEntity_find_member (entity, id, which)
  MonetEntity entity, * which;
  MonetStr id;
{
  MonetMember member;
  MonetEntity dummy;

  if (which == NULL) {
    which = &dummy;
  }
  if (member = find (entity, id, which)) {
    return member;
  }
  return find (entity, UnknownStr, which);
}

/***************************************************************************/
/*  -- Search list of known commands  */
/***************************************************************************/

extern MonetMember MonetEntity_qualified_member (entity, id)
  MonetEntity entity;
  MonetStr id;
{
  MonetMember member;

  if (member = MonetEntity_inherit (entity, id, NULL))
    return member;
  return NULL;
}

/***************************************************************************/
/*  -- Search list of known commands  */
/***************************************************************************/

extern MonetMember MonetMember_find (name)
  String name;
{
  MonetStr id;
  if (id = String2MonetStr (name)) {
    return MonetEntity_find_member (NULL, id, NULL);
  }
  return NULL;
}

/*****************************************************************************/
/*  Use and unuse  */
/*****************************************************************************/

extern void  MonetContext_use (entity)
     MonetEntity entity;
{
  MonetEntityStack_push (Current->use, entity);
  /* Always put the global entity on top  */
  MonetEntityStack_push (Current->use, Global);
}

extern void  MonetContext_unuse (entity)
     MonetEntity entity;
{
  (void) MonetEntityStack_remove (Current->use, entity);
}

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void	monetsh_init_context ()
{
  Current = NULL;
  LocalDepth = 0;

  LocalClass = MonetEntity_create_class (MonetEntityTopClass(), "local");

  Local = MonetEntityStack_create ();
  Use = MonetEntityStack_create ();
  Current = MonetContext_create (True, (char **) NULL, False);

  Global = MonetMember_module_class ("global");

  MonetContext_use (Global);
  MonetMember_deeper ();
  UnknownStr = MonetStr_refer ("unknown");
  monetsh_init_src ();
}

extern void	monetsh_finish_context ()
{
  monetsh_init_src ();
  MonetStr_derefer (UnknownStr);
  MonetMember_shallower ();
  MonetContext_free (Current);
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

