/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "scriptP.h"

/***************************************************************************/
/* History */
/***************************************************************************/

#define MAX_HISTORY 32
static int HistorySize, HistoryTop;
static char * HistoryList [MAX_HISTORY];
static MonetBool HistoryOn;

static char * hget (n)
     int n;
{
  if (n >= MAX_HISTORY || n < 0) {
    MonetSysError ("history_get: n not it range n=%d", n);
    return NULSTR;
  }
  return HistoryList[n];
}

static void hset (n, value)
     int n;
     char * value;
{
  if (n >= MAX_HISTORY || n < 0) {
    MonetSysError ("history_get: n not it range n=%d", n);
  } else {
    HistoryList[n] = value;
  }
}

static void History_free ()
{
  int i;
  for (i = 0; i < MAX_HISTORY; ++i) {
    if (hget(i)) {
      MonetFree (hget(i));
      hset(i, NULSTR);
    }
  }
  HistorySize = 0;
  HistoryTop  = 0;
}

static void History_list (num)
     int num;
{
  int i, p;
  MonetPrintfnl ("[%u] entries in the history list", HistorySize);
  if (HistorySize < num)
    num = HistorySize;
  for (i=num; i > 0; --i) {
    p = HistoryTop - i;
    MonetPrintfnl ("%u \t%s", p+1, hget(p % MAX_HISTORY));
  }
}

/*
static char * History_top ()
{
  if (HistorySize > 0)
    return hget((HistoryTop -1) % MAX_HISTORY);
  return NULL;
}
*/

static char * History_get (num)
     int num;
{
  int number;
  if (num < 0) {
    number = - num;
  } else {
    number = HistoryTop - num + 1;
  }
  if (number > HistorySize) {
    MonetPrintfError ("No such event [!%d] on the history list", num);
    return NULL;
  }
  number = (HistoryTop - number) % MAX_HISTORY;
  return hget(number);
}

extern char * MonetHistory_add (cp)
     char * cp;
{
  char * command = NULL;
  int top;
  char * p, * ins = NULL;
  int i, size = 0, len;
  MonetBool in_s = False, in_d = False;

  if (HistoryOn == False || MonetStrlen (cp) == 0)
    return cp;

  for (p = cp, i = 0; *p; ++p, ++i) {
    int n;
    switch (*p) {
    case '\\':
      if (i >= size) {
	size += (size) / 2 + 2;
	command = MonetRealloc (command, size);
      }
      command[i] = *p;
      ++i;
      ++p;
      break;
    case '"':
      in_d = ! in_d;
      break;
    case '\'':
      in_s = ! in_s;
      break;
    case '!':
      if (in_d || in_s)
	break;
      ++p;
      if (*p == '!') {
	n = -1;
      } else {
	if (*p == '-' && isdigit (*(p+1))) {
	  ++p;
	  n = - atoi (p);
	} else if (isdigit (*p)) {
	  n = atoi (p);
	} else {
	  --p;
	  break;
	}
	for (; isdigit (*p); ++p);
	--p;
      }
      if (ins = History_get (n)) {
	len = MonetStrlen (ins);
	if (i + len >= size) {
	  size += len + 1;
	  command = MonetRealloc (command, size);
	}
	(void) bcopy (ins, command+i, len);
	i += len - 1;
      }
      continue;
    default:
      break;
    }
    if (i >= size) {
      size += (size) / 2 + 1;
      command = MonetRealloc (command, size);
    }
    command[i] = *p;
  }
  if (command == NULL)
    return NULL;
  command[i] = NULL;

  /* -- Put on history list */
  top = HistoryTop % MAX_HISTORY;
  if (HistorySize < MAX_HISTORY) {
    ++HistorySize;
  } else {
    MonetFree (hget(top));
  }
  hset(top, command);
  ++ HistoryTop;
  if (ins) {
    /*  If somethign inserted then echo the result, this assumes
	we are working interactively  */
    MonetPrintfnl ("%s", command);
  }
  return hget(top);
}

MONETCALLBACK(do_history)
     int n = MonetString2int (argv[1]);
     if (n > 0) {
       History_list (n);
     } else {
       History_list (HistorySize);
     }
MONETEND

MONETCALLBACK(do_reset)
     History_free ();
MONETEND

MONETCALLBACK(do_on)
     HistoryOn = True;
MONETEND

MONETCALLBACK(do_off)
     HistoryOn = False;
MONETEND

extern int History_check (name)
     String name;
{
  if (HistorySize != 0) {
    printf ("History size %d top %d [%s]\n", HistorySize, HistoryTop, name);
    return 1;
}
  return 0;
}

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_history ()
{
  int i;

  HistoryTop = 0;
  HistorySize = 0;
  HistoryOn = True;

  for (i = 0; i < MAX_HISTORY; ++i)
    hset(i, NULSTR);

  (void) MonetMember_module_class ("history");
  MonetMember_create ( "history", 	do_history,
		      "?n",
		      "print history list");
  MonetMember_create ( "reset", 	do_reset,
		      "",
		      "Resets the history list");
  MonetMember_create ( "on", 	do_on,
		      "",
		      "Switches the history list on");
  MonetMember_create ( "off", 	do_off,
		      "",
		      "Switches the history list off");
}

extern void monetsh_finish_history ()
{
  History_free ();
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/


