/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*  Lexical stuff for monet script. Could maybe better use lex               */
/*  Two levels, one level is more relaxed, second is quite tight and is      */
/*  used within composite statements (...).  This means that there is more   */
/*  leniency taken with commands issued on the command line that those       */
/*  written as scripts.  */
/*  MUST NOT ALTER THE SOURCE STRING */
/*  !!! Does not check if string is over run, defined as 8192, which is long */
/*  !!! enough for a lot of situations but not all !!! */
/*****************************************************************************/

#include "scriptP.h"

static char ShortGetAttribute;

static MonetBool LooseLex;
#define MAXEXPAND 32
#define IGNORESPACE(c) (SPACEORTAB(c))

#define LEX_BUF_SIZE 8192
static char DataBuf [LEX_BUF_SIZE];

#ifdef MonetDebug
static int Malloced [LEX_NUM] = { 0 };

#define MALLOCED_INIT { int i; for (i = 0; i < LEX_NUM; ++i) Malloced[i] = 0; }
#define ADD(op) {Malloced[(int) op] ++; };
#define SUB(op) {Malloced[(int) op] --; };
#endif

/*
      if (*cp == '!') {
	*lex = lex_call_function;
	++cp;
      } else if (*cp == '-') {
	++cp;
	*lex = lex_call_function;
	for (++cp; isdigit(*cp); ++cp);
      } else if (isdigit(*cp)) {
	*lex = lex_call_function;
	do {
	  ++cp;
	} while (isdigit(*cp));
      } else {
	
*/
/*****************************************************************************/
/*  Some help information  */
/*****************************************************************************/

static int LexFreed = 0,  LexMalloced = 0;
extern void MonetTree_help ()
{
  MonetPrintfnl ("---- Tree ----");
  MonetPrintfnl ("sizeof (struct monet_tree): %d", sizeof(struct monet_tree));
  MonetPrintfnl ("Malloced                  : %d", LexMalloced);
  MonetPrintfnl ("Freed                     : %d", LexFreed);
  MonetPrintfnl ("Total space               : %d", LexMalloced - LexFreed);
}

/*****************************************************************************/
/*  Lexical Scanner  */
/*  returns the next item  */
/*****************************************************************************/

static String start_lex (source, buffer, lex)
     MonetLex *lex;
     String buffer;
     String *source;
{
  String result;
  String cp;

  if ( ! source || *source == NULL) {
    *lex = lex_end;
    return NULL;
  }
  cp = *source;

  /*  -- Ignore whitespace  */
  
  while (IGNORESPACE(*cp))
    ++ cp;

  /*  Ignore comments */
  while (*cp == SCRIPT_COMMENT_CHAR)
    for (; *cp && *cp != '\n'; ++cp);

  result = cp;
  switch (*cp) {
  case '\0':
    *lex = lex_end;
    break;
  case '\"':
    result = PrintString2MonetString (&cp, buffer, True);
    if (*cp == '\0') {
      MonetLexError ("End of command found where [\"] expected");
    } else {
      ++cp;
    }
    *lex = lex_string;
    break;
  case '\'':
    result = PrintString2MonetString (&cp, buffer, False);
    if (*cp == '\0') {
      MonetLexError ("End of command found where ['] expected");
    } else {
      ++cp;
    }
    *lex = lex_string;
    break;
  case '\\':
    {
      ++cp;
      buffer[0] = MonetPrintChar2char (*cp);
      buffer[1] = '\0';
      result = buffer;
      *lex = lex_string;
      ++cp;
    }
    break;
  case '{':
    ++cp;
    *lex = lex_braces;
    break;
  case '}':
    ++cp;
    *lex = lex_end_braces;
    break;
  case '$':
    ++cp;
    *lex = lex_var;
    break;
  case SCRIPT_OPEN_BRACKET:
    ++cp;
    *lex = lex_parenthesis;
    break;
  case SCRIPT_CLOSE_BRACKET:
    ++cp;
    *lex = lex_end_parenthesis;
    break;
  case '[':
    ++cp;
    *lex = lex_open_array;
    break;
  case ']':
    ++cp;
    *lex = lex_end_array;
    break;
  case '>':
    ++cp;
    if (*cp == '=') {
      *lex = lex_greater_less;
      ++cp;
    } else if (*cp == '>') {
      *lex = lex_pipe_append_out;
      ++cp;
    } else
      *lex = lex_pipe_out;
    break;
  case '<':
    ++cp;
    if (*cp == '=') {
      *lex = lex_greater_less;
      ++cp;
    } else if (*cp == '<') {
      *lex = lex_pipe_append_in;
      ++cp;
    } else {
      *lex = lex_pipe_in;
    }
    break;
  case '^':
    *lex = lex_bitwise_xor;
    ++cp;
    break;
  case '&':
    ++cp;
    if (*cp == '&') {
      *lex = lex_and;
      ++cp;
      break;
    } else {
      *lex = lex_bitwise_and;
    }
    break;
  case '!':
  case '=':
    result = cp;
    ++cp;
    if (*cp == '=' || *cp == '~') {
      *lex = lex_eq_ne;
      ++cp;
    } else if (*cp == '>') {
      *lex = lex_entity_name;
      ++cp;
    } else if (*result == '!') {
      *lex = lex_not;
    } else {
      *lex = lex_assign;
    }
    break;
  case '\n':
  case ';':
    /*  -- ignore blanks and command serarators */
    while (*cp == ' ' || *cp == '\t' || *cp == '\n' || *cp == ';')
      cp++;
    (void) strcpy (buffer, ";");
    result = buffer;
    *lex = lex_sep;
    break;
  case '|':
    ++cp;
    if (*cp == '|') {
      *lex = lex_or;
      ++cp;
    } else
      *lex = lex_pipe;
    break;
  case '-':
    if (*(cp+1) == '>') {
      cp += 2;
      *lex = lex_get_attribute;
    } else {
      *lex = lex_null;
    }
    break;
  case '?':
    if (MonetNextOnLine (cp+1)) {
      *lex = lex_null;
    } else {
      *lex = lex_question;
      ++cp;
    }
    break;
  case '.':
    ++cp;
    if (*cp == '.' && *(cp + 1) == '.') {
      cp += 2;
      *lex = lex_call_function;
    } else if (*cp == '.' || '.' != ShortGetAttribute) {
      *lex = lex_null;
      --cp;
    } else {
      *lex = lex_dot;
    }
    break;
  default:
    if (*cp == ShortGetAttribute) {
      *lex = lex_dot;
    } else if (isdigit (*cp)) {
      do {
	++ cp;
      } while (isdigit (*cp));
      if (*cp == '.') {
	do {
	  ++ cp;
	} while (isdigit (*cp));
      }
      *lex = lex_number;
    } else {
      *lex = lex_null;
    }
    break;
  }
  *source = cp;
  return result;
}

static String next_symbol (source, lex, buffer)
     MonetLex *lex;
     String *source, buffer;
{
  String cp, result;

#define VALID "\"'&|;)}><"

  buffer[0] = NULL;
  result = start_lex (source, buffer, lex);
  cp = *source;
  if (! cp)
    return cp;
  if (*lex == lex_null) {
    result = cp;
    if (LooseLex) {
      if (*cp == '-' && ! isalnum (*(cp+1))) {
	++cp;
      } else {
	while (*cp && isgraph (*cp) &&
	       (! MonetCharInString (*cp, VALID)) &&
	       *cp != ShortGetAttribute) {
	  if (*cp == '-' && ! isalnum (*(cp+1))) {
	    break;
	  }
	  ++cp;
	}
      }
      *lex = lex_call_function;
    } else {
      result = cp;
      switch (*cp) {
      case '~':
	++cp;
	*lex = lex_bitwise_not;
	break;
      case '+':
	++cp;
	if (*(cp+1) == '+') {
	  ++ cp;
	  *lex = lex_inc_dec;
	} else {
	  *lex = lex_add;
	}
	break;
      case '-':
	++cp;
	if (*cp == '-') {
	  *lex = lex_inc_dec;
	  ++cp;
	} else if (*cp == '>') {
	  *lex = lex_get_attribute;
	  ++cp;
	} else {
	  *lex = lex_sub;
	}
	break;
      case '/':
      case '%':
      case '*':
	++cp;
	*lex = lex_mul_div;
	break;
      default:
	if (isdigit (*cp)) {
	  do {
	    ++ cp;
	  } while (isdigit (*cp));
	  *lex = lex_number;
	} else if (isalnum (*cp) || MonetCharInString (*cp, "?_")) {
	  do {
	    ++ cp;
	    if (*cp == '-' && ! isalnum (*(cp+1))) {
	      break;
	    }
	  } while (*cp && isgraph(*cp) &&
		   (!MonetCharInString (*cp, VALID)) &&
		   *cp != ShortGetAttribute);
	  *lex = lex_call_function;
	} else {
	  MonetLexError ("Unexpected symbol [%s]", cp);
	  ++cp;
	  *lex = lex_sep;
	}
	break;
      }
    }
  }
  if (result != buffer) {
    int size;
    size = cp - result;
    if (size >= LEX_BUF_SIZE) {
      size = LEX_BUF_SIZE - 1;
      MonetSysError ("Lex value bigger than buffer");
    }
    bcopy (result, buffer, size);
    buffer[size] = '\0';
  }
  while (*cp == ' ' || *cp == '\t')
    ++cp;
  *source = cp;
  return buffer;
}

/*****************************************************************************/
/*  next symbol  */
/*****************************************************************************/

extern void     MonetTree_change_type (tree, lex)
     MonetTree tree;
     MonetLex lex;
{
#ifdef MonetDebug
  SUB(tree->type);
#endif
  tree->type = lex;
#ifdef MonetDebug
  ADD(tree->type);
#endif
}

extern void MonetTree_change_name (tree, name)
     MonetTree tree;
     char * name;
{
  MonetStr_derefer (tree->id);
  tree->id = MonetStr_refer (name);
}

extern MonetTree MonetTree_create (lex, value)
     String value;
     MonetLex lex;
{
  MonetTree tree;

  if (tree = MonetNew (struct monet_tree)) {
    tree->argc=0;
    tree->next = NULL;
    tree->arg = NULL;
    tree->id = MonetStr_refer (value);
    tree->type  = lex;
    ++ LexMalloced;
#ifdef MonetDebug
    tree->debris = False;
    ADD(lex);
#endif
  } else {
    MonetSysError ("Could not allocate lex [%s]", value);
  }
  return tree;
}

extern void MonetTree_free (tree)
     MonetTree tree;
{
  if (tree) {
    if (tree->arg) {
      switch (tree->type) {
      case lex_end:
      case lex_end_braces:
      case lex_end_parenthesis:
	MonetSysError ("MonetTree_free: lex op[%d] has args, memory leak !!!",
		   tree->type);
      default:
	break;
      }
      MonetTree_free (tree->arg);
    }
    MonetTree_free (tree->next);
    MonetStr_derefer (tree->id);
    tree->id = NULL;
    tree->arg = NULL;
    tree->next = NULL;
    tree->type = lex_null;
    MonetFree (tree);
    ++ LexFreed;
#ifdef MonetDebug
    SUB(tree->type);
#endif
  }
}

static MonetTree MonetTree_lex (source)
     String *source;
{
  String value, buffer;
  MonetTree tree;
  MonetLex lex;

  buffer = DataBuf;
  value = next_symbol (source, &lex, buffer);
  if (lex == lex_end)
    return NULL;
  tree = MonetTree_create (lex, value);

  return tree;
}

/*****************************************************************************/

/*****************************************************************************/

#define ONECHR(c) {*buf = (c); ++buf;}
#define SPACE ONECHR(' ')

static String tree2string (tree, buf, len)
     MonetTree tree;
     String buf;
     unsigned len;
{
  if (tree) {
    char * name;
    if (LEX_BINOP (tree->type)) {
      switch (tree->type) {
      case lex_pipe:
      case lex_pipe_in:
      case lex_pipe_out:
      case lex_pipe_append_out:
      case lex_pipe_append_in:
	if (isalpha (LNAME(tree)[0])) {
	  name = (LNAME(tree)) + 4;
	} else {
	  MonetSysError ("Pipe op name not changed [%s]", LNAME(tree));
	  name = LNAME(tree);
	}
	break;
      default:
	name = LNAME(tree);
	break;
      }
      buf = tree2string (tree->arg, buf, len);
      buf = MonetStringCopyToEnd (buf, name);
      SPACE;
      if (tree->arg->next) {
	buf = tree2string (tree->arg->next, buf, len);
      } else if (tree->type != lex_sep) {
	MonetSysError ("tree2string: Bin op with only one arg [%s]", LNAME(tree));
      }
    } else {
      MonetTree arg;
      switch (tree->type) {
      case lex_parenthesis:
	ONECHR('(');
	SPACE;
	buf = tree2string (tree->arg, buf, len);
	ONECHR(')');
	break;
      case lex_braces:
	ONECHR('{');
	SPACE;
	buf = tree2string (tree->arg, buf, len);
	ONECHR('}');
	break;
      case lex_string:
	buf = MonetString2PrintString (buf, LNAME(tree), (int) len);
	SPACE;
	buf = tree2string (tree->arg, buf, len);
	break;
      case lex_not:
      case lex_call_function:
	buf = MonetStringCopyToEnd (buf, LNAME(tree));
	SPACE;
	FORALL(arg, tree->arg, next)
	  buf = tree2string (arg, buf, len);
	break;
      default:
	buf = MonetStringCopyToEnd (buf, LNAME(tree));
	SPACE;
	buf = tree2string (tree->arg, buf, len);
	break;
      }
    }
    SPACE;
  }
  *buf = NULL;
  return buf;
}

extern String MonetTree2String (tree)
     MonetTree tree;
{

  DataBuf[0] = NULL;
  if (tree)
    tree2string (tree, DataBuf, LEX_BUF_SIZE);
  return DataBuf;
}

/*****************************************************************************/
/*  Print tree */
/*****************************************************************************/

extern void MonetTree_dump (tree, depth)
     MonetTree tree;
     unsigned depth;
{
  unsigned i;
  if (tree) {
    for (i=0; i<=depth; ++i)
      printf (" ");
    MonetPrintf ("( [%s] lex [%d]", LNAME(tree), tree->type);
    if (tree->type == lex_call_function) {
      if (tree->arg) {
	MonetPrintf (",  args[%d]", tree->arg->argc);
      }
    }
    MonetPrintfnl ("");
    MonetTree_dump (tree->arg, depth+2);
    for (i=0; i<=depth; ++i)
      MonetPrintf (" ");
    MonetPrintf (")\n");
    MonetTree_dump (tree->next, depth);
  }
}

/*****************************************************************************/

/*****************************************************************************/

extern MonetTree String2MonetTree (cp, previous, depth, loose)
     String cp;
     MonetTree *previous;
     int *depth;
     MonetBool loose;
{
  MonetTree head, tree;

  if (*depth == 0 && loose) {
    LooseLex = True;
  } else {
    LooseLex = False;
  }
  if (cp == NULL || (previous == NULL && *cp == NULL)) {
    head = MonetTree_create (lex_end, "");
    if (previous) {
      if (*previous) {
	(*previous)->next = head;
      }
      *previous = head;
    }
    return head;
  }
  head = tree = MonetTree_lex (&cp);
  if (previous && *previous) {
    (*previous)->next = head;
  }
  if ( ! head)
    return NULL;

  while (1) {
    switch (tree->type) {
    case lex_parenthesis:
    case lex_braces:
      ++ (*depth);
      LooseLex = False;
      break;
    case lex_end_parenthesis:
    case lex_end_braces:
      -- (*depth);
      if (*depth == 0 && loose) {
	LooseLex = True;
      } else {
	LooseLex = False;
      }
    case lex_end:
      break;
    default:
      break;
    }
    tree->next = MonetTree_lex (&cp);
    if (tree->next == NULL) {
      if (LEX_OPEN(tree->type) || LEX_BINOP(tree->type) || (*depth != 0)) {
      } else {
	tree->next = MonetTree_create (lex_end, "");
	tree = tree->next;
	tree->next = NULL;
      }
      break;
    } else if (tree->next->type == lex_sep && tree->type == lex_sep) {
      /* Compress multiple seperators into one */
      MonetTree_free (tree->next);
      tree->next = NULL;
    } else {
      tree = tree->next;
    }
  }

  if (previous) {
    *previous = tree;
  }
  return head;
}


extern void MonetTreeFixTail (tree, tail)
     MonetTree tree, tail;
{
  if (tail && tree != tail) {
    if (tail->type == lex_sep && tail->next == NULL) {
      MonetTree_change_type (tail, lex_end);
      tail->next = NULL;
    }
  }
}

extern char MonetLex_set_short_get (c)
     char c;
{
  char p;
  p = ShortGetAttribute;
  ShortGetAttribute = c;
  return p;
}

extern void monet_lex_init ()
{
  ShortGetAttribute = '\0';
}


extern void monet_lex_finish ()
{

}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
