/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/* This entity provides support for a simple (extensible) script programming */
/* language.  It provides basic functionality, loops, conditionals, vars,    */
/* pipes, members and a set of standard functions.  Other entitys can extend  */
/* on the set of standard functions using the C interface.                   */
/* The main bias is towards short scripts for text processing, especially    */
/* processing the output for normal operating system programs, so that use   */
/* sh, awk etc can be avoided when they aren't really necessary and impose   */
/* an unwanted processor overhead.                                           */
/* Input can come from a file, stdin or from a string.                       */
/* !!! This stuff is very similar to the code for config and they could be   */
/* !!! combined.                                                             */
/* This entity provides support for members/functions and help                */
/*****************************************************************************/

#include "scriptP.h"
#include <monet/msh/Context.h>
#include <monet/msh/Entity.h>
#include <monet/msh/Event.h>

static long MallocedArgs = 0, FreedArgs = 0,
  CreatedMembers = 0, FreedMembers = 0;
static MonetEntity ModuleClass;

/***************************************************************************/

typedef enum {
  member_attribute,
  member_var,
  member_read,
  member_write,
  member_write_long,
  member_write_boolean,
  member_environment,
  member_callback,
  member_function
  }  MemberType;
#define MEMBER_DEFAULT member_var;
#define MEMBER_FUNC(type) (((int) (type)) >= (int) member_environment)

struct monet_member {
  int linenumber;
  MonetStr id, filename;
  MonetEntity entity;
  MonetEvent event;
  MemberType type;
  unsigned min_arg, max_arg;
  String * argv;
  MonetBool unlimited_args;
  ScriptEnvironmentCallback environment_callback;
  ScriptCallback callback;
  MonetValue value;
  String syntax;
  String help, malloced_help;
  MonetMember next;
};


#define MNAME(member) (MonetStr2String ((member)->id))


static MonetEntity CreateClass;


#define MAXSCRIPTARG 1023

static MonetBool PreviousResult = False;

/***************************************************************************
  Records whether or not the members have been changed
  ie  member is created or destroyed, or when a new entity is created.
  This can be used for maintaining EXACTLY ONE cache.
  *************************************************************************/

static long MembersChanged = 0;

static void members_changed ()
{
  ++MembersChanged;
}

extern long MonetMembers_changed ()
{
  return MembersChanged;
}

/***************************************************************************
  Free all of an entities members, this should be called every time an
  entity is freed.
***************************************************************************/

extern void MonetEntity_free_members (entity)
     MonetEntity entity;
{
  members_changed ();
  while (entity->members) {
    MonetMember_free (entity->members);
  }
}

/***************************************************************************/
/*  Given a list of members and a name find the member  */
/***************************************************************************/

static MonetMember lookup_member (name, memberlist)
     char * name;
     MonetMember memberlist;
{
  MonetStr id;
  MonetMember member;

  if (memberlist && (id = String2MonetStr (name)))
    FORALL(member, memberlist, next)
      if (id == member->id)
	return member;
  return NULL;
}

extern MonetMember MonetMember_lookup (entity, id)
     MonetStr id;
     MonetEntity entity;
{
  MonetMember member;

  FORALL(member, entity->members, next) {
    if (id == member->id) {
      return member;
    }
  }
  return NULL;
}

/***************************************************************************/
/*  Print a member as string  */
/***************************************************************************/

static String args2string (f, member)
     MonetMember member;
     String f;
{
  if (member->syntax && *member->syntax) {
    f  = MonetStringCopyToEnd (f, "  ");
    f  = MonetStringCopyToEnd (f, member->syntax);
  } else if (member->argv) {
    unsigned c;
    for (c = 0; c < member->min_arg; ++c) {
      *f = ' ';
      ++f;
      f  = MonetStringCopyToEnd (f, member->argv[c]);
    }
    for (; c < member->max_arg; ++c) {
      f  = MonetStringCopyToEnd (f, "  ?");
      f  = MonetStringCopyToEnd (f, member->argv[c]);
    }
    if (member->unlimited_args) {
      f  = MonetStringCopyToEnd (f, " ...");
    }
  } else {
    f  =  MonetStringCopyToEnd (f, " ...");
  }
  return f;
}

extern MonetStr MonetMember2Str (member)
     MonetMember member;
{
  return member->id;
}

extern String MonetMember2DeclaredString (member)
     MonetMember member;
{
  static char buffer [1024]; /* !!! */
  if (member->filename) {
    (void) sprintf (buffer, "%s->%s [%s, line %s]",
		    MonetEntity_title (member->entity),
		    MNAME(member),
		    MonetStr2String (member->filename),
		    member->linenumber);
  } else {
    (void) sprintf (buffer, "%s->%s",
		    MonetEntity_title (member->entity),
		    MNAME(member));
  }
  return buffer;
}

extern String MonetMember2String (member)
     MonetMember member;
{
  static char buffer [1024]; /* !!! */
  (void) sprintf (buffer, "%s -> \"%s\"",
		  MonetEntity_title (member->entity),
		  MNAME(member));
  return buffer;
}

static String MonetMember2Syntax (member)
     MonetMember member;
{
  char * f, *start;
  start = MonetMember2String (member);
  f = start + strlen (start);
  (void) args2string (f, member);
  return start;
}

static String member2string (member, def)
     MonetMember member;
     MonetBool def;
{
  static char buffer [1024]; /* !!! */
  char * f;
  f = buffer;
  if (MEMBER_FUNC(member->type)) {
    f = MonetStringCopyToEnd (f, "function ");
    f = MonetString2PrintString (f, MNAME(member), sizeof buffer);
    f = args2string (f, member);
    f = MonetStringCopyToEnd (f, " {");
    if (def && member->type == member_function) {
      f = MonetStringCopyToEnd (f, member->value.pointer);
    }
    (void) MonetStringCopyToEnd (f, "};");
  } else if (member->type == member_attribute) {
    (void) sprintf (f, "attribute %s",
		    MonetAttribute2FullName
		    ((MonetAttribute)member->value.pointer));
    f += MonetStrlen (f);
/*    f = MonetString2PrintString (f, MNAME(member), sizeof buffer); */
    (void) MonetStringCopyToEnd (f, ";");
  } else {
    String value;
    value = MonetMember_execute (member->entity,
				 member,
				 NULL,
				 member->id, NULTREE);
    f = MonetStringCopyToEnd (f, "variable ");
    f = MonetString2PrintString (f, MNAME(member), sizeof buffer);
    if (value) {
      f = MonetStringCopyToEnd (f, "  ");
      f = MonetString2PrintString (f, value, 1024);
    }
    (void) MonetStringCopyToEnd (f, ";");
  }
  return buffer;
}

extern String MonetMember_declaration  (member)
     MonetMember member;
{
  return member2string (member, False);
}

extern String MonetMember_definition  (member)
     MonetMember member;
{
  return member2string (member, True);
}

extern String MonetMember_documentation  (member)
     MonetMember member;
{
  if (member->malloced_help) {
    return member->malloced_help;
  } else {
    return member->help;
  }
}


/**********************************************************/
/*  Set entity for next set of members  */
/**********************************************************/

extern void MonetMember_set_entity (entity)
     MonetEntity entity;
{
  CreateClass = entity;
}

/**********************************************************/
/*  Free all the events belonging to an entity  */
/**********************************************************/

extern void MonetEvent_free_entity (entity)
     MonetEntity entity;
{
  MonetMember member;
  FORALL(member, entity->members, next)
    while (member->event)
      MonetEvent_free (member->event);
}

/***************************************************************************/
/*  String is false  */
/***************************************************************************/

extern MonetBool Script_true (v)
     String v;
{
  return (v != NULL && (*v != '0' || MonetString2long (v) != 0));
}

/***************************************************************************/
/*  Evaluate arguments  */
/***************************************************************************/

static unsigned execute_args (entity, name, arglist, arg)
     MonetEntity entity;
     String *arg, name;
     MonetTree arglist;
{
  unsigned argc;

  arg[0] = MonetNewString (name);
  ++ MallocedArgs;
  for (argc=1 ; arglist ; ++argc, arglist = arglist->next) {
    String result;
    if (argc >= MAXSCRIPTARG-1) {
      MonetParseError ("Too many arguments, maximum is %d", MAXSCRIPTARG);
      break;
    }
    if (arglist->type == lex_braces) {
      result = MonetTree2String (arglist);
    } else {
      result = MonetTree_execute (entity, arglist, NULL);
    }
    ++ MallocedArgs;
    arg[argc] = MonetNewString (result);
  }
  arg [argc] = '\0';
  return argc;
}

static void free_execute_args (result, arg, argc)
        unsigned argc;
        String * arg, result;
{
  static String Result = NULL;
  while (argc) {
    -- argc;
    if (arg[argc] == result) {
      if (Result) {
	++ FreedArgs;
	MonetFree (Result);
      }
      Result = arg[argc];
    } else {
      ++ FreedArgs;
      MonetFree (arg[argc]);
    }
  }
}

/***************************************************************************/
/*  Execute a member function/variable/attribute
    The arguments are executed under the context 'arg_entity' while the
    member itself is executed with 'entity'  */
/***************************************************************************/

extern String MonetMember_execute (entity, member, arg_entity, id, arg)
     MonetMember member;
     MonetEntity entity, arg_entity;
     MonetStr id;
     MonetTree arg;
{
  int argc;
  char * name, * result = NULL;

  if (id) {
    name = MonetStr2String (id);
  } else {
    name = MNAME(member);
  }

  if (entity == NULL) {
    entity = member->entity;
  }
  if (arg) {
    argc= arg->argc;
  } else {
    argc = 0;
  }

  if (MonetContext_trace (member->id)) {
    MonetPrintfWarning ("(Trace calling [%s->%s]: %d arguments",
		   MonetEntity_title (entity),
		   MonetMember2DeclaredString (member), argc);
  }

  if (member->unlimited_args == False && member->max_arg < argc ) {
    if ( member->max_arg == 0) {
      MonetParseError("%s given but none required\n\  [%s]",
		      MonetPrettyCount (argc, "Argument"),
		      MonetMember2Syntax (member));
    } else {
      MonetParseError("%s given but at most %d required\n\  [%s]",
		      MonetPrettyCount (argc, "Argument"),
		      member->max_arg,
		      MonetMember2Syntax (member));
    }
  } else if (member->min_arg > argc) {
    if (member->unlimited_args)
      MonetParseError ("At least %s expected, %d given\n  [%s]",
		       MonetPrettyCount (member->min_arg, "argument"),
		       argc,
		       MonetMember2Syntax (member));
    else if (member->min_arg == member->max_arg)
      MonetParseError ("%s expected, %d given [%s]",
		       MonetPrettyCount (member->min_arg, "Argument"),
		       argc,
		       MonetMember2Syntax (member));
    else
      MonetParseError ("From %d to %s expected, %d given\n  [%s]",
		       member->min_arg,
		       MonetPrettyCount (member->max_arg, "argument"),
		       argc,
		       MonetMember2Syntax (member));
  } else {
    switch (member->type) {
    case member_read:
      result =  (String) *(member->value.string_pointer);
      break;
    case member_write_boolean:
      if (argc > 0) {
	if (MonetTree_execute (arg_entity, arg, (char **) 0)) {
	  *(member->value.boolean_pointer) = True;
	} else {
	  *(member->value.boolean_pointer) = False;
	}
      }
      if (*(member->value.boolean_pointer)) {
	result = "true";
      } else {
	result = NULL;
      }
      MONETDEBUG(2, ( "Write MonetBool [%s] argc %d, result [%s]",
		  name, 1, result))
      break;
    case member_write_long:
      if (argc > 0) {
	*(member->value.long_pointer) =
	  MonetString2long (MonetTree_execute (arg_entity, arg, (char **) 0));
      }
      result = Monetlong2String (*(member->value.long_pointer));
      break;
    case member_write:
      if (argc != 0) {
	if (member->value.pointer) {
	  MonetFree (*(member->value.string_pointer));
	}
	*(member->value.string_pointer) =
	  MonetNewString (MonetTree_execute (arg_entity, arg, (char **) 0));
      }
      result =  *(member->value.string_pointer);
      break;
    case member_environment:
      result = member->environment_callback (entity, name, arg_entity,
					     argc, arg);
      break;
    case member_attribute:
      result = MonetEntity_get_attribute
	(entity, (MonetAttribute) member->value.pointer);
      break;
    case member_var:
      result = member->value.pointer;
      break;
    case member_callback:
    case member_function:
    default:
      {
	String args[MAXSCRIPTARG+1];
	MonetMember prev_member;

	prev_member = MonetContext_executing_member (member);

	(void) execute_args (arg_entity, name, arg, args);

	if (member->callback) {
	  result = member->callback (argc+1, args, entity);
	} else if (member->value.pointer) {
	  int i, min;
	  min = MONETMIN (argc, member->max_arg);

	  MonetMember_deeper ();
	  for (i = 0; i < min; ++i) {
	    MonetMember_create_local (member->argv[i], args[i+1]);
	  }
	  for (; i < member->max_arg; ++i) {
	    MonetMember_create_local (member->argv[i], (char *) 0);
	  }
	  if (member->unlimited_args) {
	    char * value;
	    if (argc > member->max_arg) {
	      value = MonetJoinStrings((int) (argc - member->max_arg),
				       args + member->max_arg +1, " ");
	    } else {
	      value = "";
	    }
	    MonetMember_create_local ("...", value);
	  }
	  MonetMember_create_local ("0", name);
	  result = MonetshEntityExecute (entity,
					 member->value.pointer,
					 args, False);
	  MonetMember_shallower ();
	  if (MonetException_code () == MONETSH_RETURN) {
	    MonetException_raise (MONETSH_OKAY);
	  }
	}
	free_execute_args (result, args, argc+1);

	(void) MonetContext_executing_member (prev_member);
      }
    }
  }
  if (MonetContext_trace (member->id)) {
    MonetPrintfWarning (")trace called  [%s->%s]: result [%s]",
			MonetEntity_title (entity),
			MonetMember2DeclaredString (member), result);
  }
  if (result) {
    PreviousResult = True;
  } else {
    PreviousResult = False;
  }
  return result;
}

/***************************************************************************/
/* Execute a statement or a function  */
/***************************************************************************/

extern String MonetTree_execute_qualified (entity, qualifier, exec, args)
     MonetEntity entity, qualifier;
     MonetTree exec;
     String * args;
{
  String result;

  if (MonetException_code ())
    return NULL;
  if (exec == NULL) {
    return NULL;
  }

  switch (exec->type) {
  case lex_braces:
    if (exec->arg)
      result = MonetTree_execute (qualifier ? qualifier :entity,
				  exec->arg, (char **) 0);
    else
      result = "";
    break;
  default:
    {
      MonetMember member;

      if (qualifier)
	member = MonetEntity_qualified_member (qualifier, exec->id);
      else
	member = MonetEntity_find_member (entity, exec->id, &qualifier);
      if (member) {
	MONETDEBUG(20, ( "Execute Function [%s]", LNAME(exec)));
	result = MonetMember_execute (qualifier, member,
				      entity, exec->id, exec->arg);
      } else {
	if (qualifier) {
	  MonetParseError ("Don't recognise command [%s -> %s]",
			   MonetEntity_title (qualifier), LNAME(exec));
	} else {
	  MonetParseError ("Don't recognise command [%s]", LNAME(exec));
	}
	result = NULL;
      }
    }
    break;
  case lex_number:
  case lex_string:
    MONETDEBUG(20, ( "Execute String [%s]", LNAME(exec)));
    result = LNAME(exec);
    break;
  case lex_sep:
    if (exec->arg) {
      result = MonetTree_execute (entity, exec->arg, (char **) 0);
      if (exec->arg->next && ! MonetException_code ()) {
	result = MonetTree_execute (entity, exec->arg->next, (char **) 0);
      }
    }
    break;
  case lex_null:
  case lex_end:
    MonetSysError ("execute tree: Illegal node found [%s][%d]",
	       LNAME(exec), exec->type);
    result = "";
    break;
  }
  MONETDEBUG(13, ( "[%s] [%s]", LNAME(exec), result));
  return result;
}

extern String MonetTree_execute (entity,  exec, args)
     MonetEntity entity;
     MonetTree exec;
     String * args;
{
  return MonetTree_execute_qualified (entity, NULL, exec, args);
}

/***************************************************************************/
/*  Managing members  */
/***************************************************************************/

/***************************************************************************/
/* True if the previous statement/function succeeded */
/***************************************************************************/

extern MonetBool MonetPreviousResult ()
{
  return PreviousResult;
}

/***************************************************************/
/*  member to entity  */
/***************************************************************/

extern MonetEntity MonetMember_entity (member)
     MonetMember member;
{
  return member->entity;
}

/***************************************************************/
/*  member to event  */
/***************************************************************/

extern MonetEvent MonetMember_event (member)
     MonetMember member;
{
  return member->event;
}

/***************************************************************************/
/*  assign  */
/***************************************************************************/

extern void MonetMember_assign (entity, member, value)
     MonetEntity entity;
     MonetMember member;
     String value;
{
  switch (member->type) {
  case member_var:
    if (member->value.pointer) {
      MonetFree (member->value.pointer);
    }
    member->value.pointer = (caddr_t) MonetNewString (value);
    break;
  case member_attribute:
    MonetEntity_set_attribute
      (entity, (MonetAttribute) member->value.pointer, value);
    break;
  default:
    MonetParseError ("Can only assign to variables, not [%s]", MNAME(member));
    break;
  }
}

/***************************************************************************/
/*  free */
/***************************************************************************/

static void basic_free (member)
     MonetMember member;
{
  /*  Clear the cache when anything changes  */
  members_changed ();

  if (member->filename) {
    MonetStr_derefer (member->filename);
    member->filename = NULL;
  }
  while (member->event) {
    MonetEvent_free (member->event);
  }
  if (member->argv) {
    while (member->max_arg > 0) {
      member->max_arg --;
      MonetFree (member->argv[member->max_arg]);
    }
    MonetFree (member->argv);
  }
  switch (member->type) {
  case member_environment:
  case member_callback:
  case member_write_long:
  case member_write_boolean:
  case member_read:
  case member_attribute:
    break;
  case member_write:
  case member_function:
  case member_var:
  default:
    MonetFree (member->value.pointer);
    break;
  }
  MonetFree (member->malloced_help);
  ++ FreedMembers;
}

extern void MonetMember_free (member)
     MonetMember member;
{
  basic_free (member);
  LIST_DEL (MonetMember, member, member->entity->members, next);
  MonetStr_derefer ( member->id);
  MonetFree (member);
}

/***************************************************************************/
/*  Creating members  */
/***************************************************************************/

static MonetMember member_create (entity, name, type, help)
     MonetEntity entity;
     String name, help;
     MemberType type;
{
  MonetMember member;

  if (! name || !*name) {
    MonetSysError ("Null name for member");
    return NULL;
  }
  if (entity == NULL) {
    entity = MonetGlobal ();
  }

  /* Clear the cache, just in case */
  members_changed ();

  if (member = lookup_member (name, entity->members)) {
    basic_free (member);
  } else {
    member = MonetNew (struct monet_member);
    LIST_ADD (member, entity->members, next);
    member->id = MonetStr_refer (name);
  }
  member -> entity = entity;
  member->value.pointer = NULL;
  member -> event = NULL;
  member -> type = type;
  member -> callback = NULL;
  member -> syntax = NULL;
  member -> help = help;
  member -> malloced_help = NULSTR;
  member->argv = NULL;
  member->min_arg = 0;
  member->max_arg = 0;
  member->environment_callback = NULL;
  member->unlimited_args = False;
  member->linenumber = 0;
  member->filename = NULL;

  ++ CreatedMembers;
  return member;
}

/***************************************************************************/

/***************************************************************************/

static void member_args (member, argc, argv)
     MonetMember member;
     String * argv;
     int argc;
{
  if (argc > 0) {
    unsigned i;
    member->max_arg = 0;
    member->min_arg = 0;
    member->unlimited_args = False;
    member->argv = (char **) MonetMalloc (sizeof (char *) * argc);
    for (i = 0; i < argc; ++i) {
      if (EQS(argv[i], "...")) {
	member->unlimited_args = True;
      } else if (argv[i] == NULL || *argv[i] == NULL) {
	MonetPrintfError ("Argument of [%s] is empty", MNAME(member));
      } else {
	char * name;
	if ('?' != *argv[i]) {
	  ++(member->min_arg);
	  name = argv[i];
	} else {
	  name = argv[i] + 1;
	}
	++(member->max_arg);
	member->argv[i] = MonetNewString (name);
      }
    }
  }
}

/***************************************************************************/

/***************************************************************************/

static void member_syntax (member, syntax)
     MonetMember member;
     String syntax;
{
  if (syntax) {
    String s;
    for (s=syntax; *s; ) {
      if (isalpha (*s)) {
	++(member->min_arg);
	++(member->max_arg);
	do {
	  ++s;
	} while (isgraph (*s));
      }
      switch (*s) {
      case '\0':
	break;
      case '?':
	++(member->max_arg);
	do {
	  ++s;
	} while (isgraph (*s));
	break;
      case '.':
	if (*(s+1) == '.' && *(s+2) == '.') {
	  s += 3;
	  member->unlimited_args = True;
	  break;
	}
      default:
	++s;
	break;
      }
    }
  }
  member -> syntax = MonetNewString (syntax);
}

/***************************************************************************/

/***************************************************************************/

extern MonetMember MonetMember_create (name, callback, syntax, help)
     String name, help, syntax;
     ScriptCallback callback;
{
  MonetMember member;

  if (member = member_create (CreateClass, name, member_callback,help)) {
    member_syntax (member, syntax);
    member -> callback = callback;
  }
  return member;
}

extern MonetMember MonetMember_create_read (name, address, help)
     String name, help, *address;
{
  MonetMember member;

  if (address == NULL) {
    MonetSysError ("MonetMember_create_read: NULL address [%s]", name);
    return NULL;
  }
  if (member = member_create (CreateClass, name, member_read,
			    help)) {
    member->value.string_pointer = address;
  }
  return member;
}

extern MonetMember MonetMember_create_write (name, address, help)
     String name, help, *address;
{
  MonetMember member;

  if (address == NULL) {
    MonetSysError ("MonetMember_create_write: NULL address [%s]", name);
    return NULL;
  }
  if (member = member_create (CreateClass, name, member_write,
			    help)) {
    member_syntax (member, "?new_value");
    member->value.string_pointer = address;
  }
  return member;
}

extern MonetMember MonetMember_create_write_long (name, address, help)
     String name, help;
     long *address;
{
  MonetMember member;

  if (address == NULL) {
    MonetSysError ("MonetMember_create_write_long: NULL address [%s]", name);
    return NULL;
  }
  if (member = member_create (CreateClass, name, member_write_long,
			    help)) {
    member_syntax (member, "?int_value");
    member->value.long_pointer = address;
  }
  return member;
}

extern MonetMember MonetMember_create_write_boolean (name, address,
					    help)
     String name, help;
     MonetBool *address;
{
  MonetMember member;

  if (address == NULL) {
    MonetSysError ("MonetMember_create_write_boolean: NULL address [%s]", name);
    return NULL;
  }
  if (member = member_create (CreateClass, name, member_write_boolean, help)) {
    member_syntax (member, "?boolean_value");
    member->value.boolean_pointer = address;
  }
  return member;
}

extern MonetMember MonetMember_create_local (name, value)
     String name, value;
{
  MonetMember member;
  if (member = member_create (MonetLocalStackTop (), name, member_var, ""))
      member -> value.pointer = MonetNewString (value);
  return member;
}

extern MonetMember MonetMember_create_environment (name, callback,syntax, help)
     ScriptEnvironmentCallback callback;
     String name, syntax, help;
{
  MonetMember member;

  if (member = member_create (CreateClass, name, member_environment, help)) {
    member_syntax (member, syntax);
    member->environment_callback = callback;
  }
  return member;
}

/***************************************************************************/
/*  Set up attributes  */
/***************************************************************************/

MONETCALLBACK(do_attribute)
     String s_type = argv[2];
     String s_id = argv[1];
     String name = argv[3];
     unsigned num;
     MonetAttribute id;
     MonetType type;
     MonetMember member;

     if (! isdigit (*s_id) || (num = atoi (s_id)) == 0) {
       MonetPrintfError ("Attribute id must be an integer > than 1, not [%s]",
		   s_id);
     } else if (!(type = String2MonetType (s_type))) {
       MonetPrintfError ("No such attribute type [%s]", s_type);
     } else if (id = MonetAttribute_create (name, num, type, "")) {
       if (member = member_create (entity, name, member_attribute, "")) {
	 member->value.pointer = (caddr_t) id;
       }
       RETURNSC;
     } else {
       MonetPrintfError ("Could not define attribute [%d:%s]", id, name);
     }
     result = NULL;
MONETEND

/***************************************************************************/
/*  Help  */
/***************************************************************************/

MONETCALLBACK(do_help)
     MonetPrintfnl ("Malloced arguments = %ld, freed = %ld, diff = %ld",
		    MallocedArgs, FreedArgs, MallocedArgs - FreedArgs);
     MonetPrintfnl ("Malloced members = %ld, freed = %ld, diff = %ld",
		    CreatedMembers, FreedMembers,
		    CreatedMembers - FreedMembers);
MONETEND

/***************************************************************************/
/*  User defined members  */
/***************************************************************************/

MONETCALLBACK(do_define)
     String name, value;
     MonetMember member;

     name = argv[1];
     value = argv[argc-1];
     MONETDEBUG(80, ( "define [%s] as [%s]", name, value));

     if (member = member_create (entity, name, member_function, "")) {
       member_args (member, argc -3, argv+2);
       member->value.pointer = (caddr_t) MonetNewString (value);
       MonetContext_filename_line (&member->filename, &member->linenumber);
       MonetStr_refer (MonetStr2String(member->filename));
     }
MONETEND

MONETCALLBACK(do_document)
     MonetEntity which;
     MonetStr id;
     MonetMember member;
     if ((id = String2MonetStr (argv[1])) && (member =
	 MonetEntity_find_member (entity, id, &which))) {
       char * help;
       help = MonetJoinStrings (argc-2, argv+2, " ");
       MonetFree (member->malloced_help);
       member->malloced_help = MonetNewString (help);
     } else {
       MonetPrintfError ("No such member [%s]", argv[1]);
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_var)
     String name, value;
     MonetMember member;

     name = argv[1];
     value = MonetJoinStrings (argc-2, argv+2, NULL);
     if (member = member_create (entity, name, member_var, "")) {
       member -> value.pointer = MonetNewString (value);
     }
MONETEND

/***************************************************************************
  Private routine for inheriting.
  If inheriting a variable then copies the variable from the class definition.
  ***************************************************************************/

extern MonetMember MonetEntity_inherit (entity, id, top)
     MonetEntity entity, top;
     MonetStr id;
{
  MonetMember member;
  MonetEntity e;

  if (entity) {
    if (member = MonetMember_lookup (entity, id)) {
      return member;
    }
    for (e = MonetEntity_classOf (entity);
       e && e != top; e = MonetEntity_classOf(e))
      if (member = MonetMember_lookup (e, id)) {
	if (member->type == member_var) {
	  MonetMember new_member;
	  if (new_member = member_create(entity,MNAME(member),member_var,"")) {
	    new_member -> value.pointer =
	      MonetNewString (member->value.pointer);
	  }
	  return new_member;
	}
	return member;
      }
  }
  return NULL;
}

/***************************************************************************/
/*  Declare local variable  */
/***************************************************************************/

MONETCALLBACK(do_local_var)
     while (argc > 1) {
       -- argc;
       (void) MonetMember_create_local (argv[argc], "");
     }
MONETEND

/***************************************************************************/
/*  Members are grouped into layers  */
/***************************************************************************/

extern MonetEntity MonetMember_module_class (name)
     String name;
{
  MonetEntity entity;

  if (EQS(name, "Top")) {
    CreateClass = MonetEntityTopClass();
  } else if (entity = String2MonetEntity_subclass (ModuleClass, name)) {
    CreateClass = entity;
  } else if (entity = MonetEntity_create_class (ModuleClass, name)) {
    MonetLibrary_register (entity);
    CreateClass = entity;
  } else {
    MonetSysError ("Entity not created [%s]", name);
  }
  return CreateClass;
}

/***************************************************************************/
/* Help messages */
/***************************************************************************/

static void print_help (member)
     MonetMember member;
{
  MonetPrintfnl (" \t%s", member2string (member, True));
  if (member->malloced_help) {
    MonetPrintText (" \t   # ", 70, member->malloced_help);
  } else {
    MonetPrintText (" \t   # ", 70, member->help);
  }
}

extern int MonetEntity_help_members (entity, class_pattern, pattern,
				     header, trailer)
     MonetEntity entity;
     String pattern, class_pattern, header, trailer;
{
  MonetMember member;
  unsigned found = 0;
  char * title;

  title = MonetEntity_title (entity);
  if (MonetMatchWildcard (title, class_pattern)) {
    MonetBool head = True;
    FORALL(member, entity->members, next) {
      if (MonetMatchWildcard (MNAME(member), pattern)) {
	if (head) {
	  MonetPrintfnl ("%s", header);
	  head = False;
	}
	++ found;
	print_help (member);
      }
    }
    if (!header) {
      MonetPrintfnl ("%s", trailer);
    }
  }
  return found;
}

extern int MonetEntity_list_members (entity, class_pattern)
     MonetEntity entity;
     String class_pattern;
{
  MonetMember member;
  unsigned found = 0;
  char * title;

  title = MonetEntity_title (entity);
  if (MonetMatchWildcard (title, class_pattern)) {
    MonetBool header = True;
    int len = 0;
    FORALL(member, entity->members, next) {
      char * name = MNAME(member);
      len += MonetStrlen (name) + 2;
      if (header) {
	MonetPrintfnl ("\n%s", title);
	header = False;
      } else if (len >= 60) {
	MonetPrintfnl ("");
	len = 0;
      }
      MonetPrintf ("  %s", name);
      ++ found;
    }
    if (! header) {
      MonetPrintfnl ("");
    }
  }
  return found;
}

/***************************************************************************/

/***************************************************************************/

extern MonetMember MonetMember_next (member)
     MonetMember member;
{
  if (member)
    return member->next;
  return NULL;
}

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_member ()
{
  MonetEntity top = MonetEntityTopClass ();

  members_changed ();

  MonetMember_set_entity (top);
  MonetLibrary_register (top);

  ModuleClass = MonetEntity_create_class (top, "module");
  MonetLibrary_register (ModuleClass);

  MonetMember_create ( "document",		do_document,
		      "name ?info ...",
		      "Documents a member, you must insert new lines");
  MonetMember_create ( "function",		do_define,
		      "name ?arguments ... value",
		      "Define a function");
  MonetMember_create ( "variable",		do_var,
		      "name ?value ... ",
	      "Define a Non Local variable member of an entity. \
The value arguments will be concatinated.");
  MonetMember_create ( "attribute",		do_attribute,
		      "#num type name",
		      "Defines an attribute, types are counter,sting,enum. \
These behave like variables but can be distributed");
  MonetMember_create ("var",		do_local_var,
		      "name ...",
		      "Creates a local variable.  This will disappear at the \
end of the current block.");
  (void) MonetMember_module_class ("memory");
  MonetMember_create ("members",		do_help,
		      "",
		      "A bit of internal info on the functions and vars");
}

extern void monetsh_finish_member ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
