/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/****************************************************************************
  A set of routines which use variable argument lists.
  These are all included here for the sake of convenience and portability.
  ****************************************************************************/

#include "General.h"
#include "Script.h"
#include "Context.h"  /* for MonetContext_message (buffer, message); */
#include <syslog.h>

/*********************************************************************
  Some definitions
  *********************************************************************/

/*  Three choices are given,
    ANSI-C/C++     - will use stdlib.h and assume you have vprintf/vsprintf
    MonetVprintf   - if defined will use varargs
    fmt,a,b,c,d,e, - if the others aren't available
    */

#if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)

#include <stdarg.h>

#define VARPROC(name) \
   extern void name  (char *fmt,...) { va_list args; va_start(args,fmt); {
#define FPRINT(fid) vfprintf(fid, fmt,args);
#define SPRINT(buf) vsprintf(buf, fmt,args);
#define ENDPROC } va_end(args); }

#elif MONET_VPRINTF

#include <varargs.h>

#define VARPROC(name) \
   extern void name  (va_alist) va_dcl { va_list args; char *fmt; \
   va_start(args); fmt = va_arg(args, char *); {
#define FPRINT(fid)  vfprintf(fid,fmt,args);
#define SPRINT(buf)  vsprintf(buf,fmt,args);
#define ENDPROC } va_end(args); }

#else

#define ARGLIST  fmt,a,b,c,d,e,f,g,h,i
#define VARPROC(name) \
   extern void name (ARGLIST) char * fmt; {
#define FPRINT(fid) (void) fprintf (fid, ARGLIST);
#define SPRINT(buf) (void) sprintf (buf, ARGLIST);
#define ENDPROC }
#undef ARGLIST

#endif

/**********************************************************************/
/*  Globals  */
/**********************************************************************/

#define SIZEMESSAGEBUFFER  2048   /* !!! */

static MonetBool SysLogMessages = False;

/*****************************************************************************/
/*  Syslog stuff, here for convenience sake  */
/*****************************************************************************/

extern void MonetSyslog (enable)
     MonetBool enable;
{
  if (SysLogMessages != enable) {
    if (SysLogMessages) {
      /* All messages will be logged to local0 and will send to
	 the console if syslog doesn't work.
	 */
      syslog(LOG_INFO, "End %s", Monet_program_name ());
      closelog ();
    }
    SysLogMessages = enable;
    if (SysLogMessages) {
      openlog (Monet_program_name (),
	       LOG_PID | LOG_CONS | LOG_NOWAIT, LOG_LOCAL0);
      syslog(LOG_INFO, "Start %s", Monet_program_name ());
    }
  }    
}

/*********************************************************************
  Messages that may also be logged
  *********************************************************************/

#define  LEADER(message) \
  { unsigned len; \
    char buffer[SIZEMESSAGEBUFFER]; \
  len = MonetContext_message (buffer, message); \
  buffer [len] = ' '; \
  buffer [len+1] = '\0'; \
  SPRINT(buffer + len + 1); \
  MonetErrFwrite (buffer, strlen (buffer)); \
  MonetErrFwrite ("\n", 1); \
  if (SysLogMessages) syslog (LOG_INFO, "%s", buffer); \
  }


VARPROC(MonetParseError)

  LEADER ( "Parse Error" );
ENDPROC

VARPROC(MonetParseWarning)

  LEADER ( "Parse Warning" );
ENDPROC

VARPROC(MonetLexError)

  LEADER ( "Lex Error" );
ENDPROC

VARPROC(MonetPrintfWarning)

  LEADER ( "Warning" );
ENDPROC

VARPROC(MonetPrintfError)

     LEADER ( "Error" );
ENDPROC

VARPROC(MonetPrintfInform)

     char buffer[SIZEMESSAGEBUFFER];
     SPRINT(buffer);
     MonetErrFwrite (buffer, strlen (buffer));
     MonetErrFwrite ("\n", 1);
     if (SysLogMessages) syslog (LOG_INFO, "%s", buffer);
ENDPROC

VARPROC(MonetPError)

  char buf [1024]; /* !!! */
  (void) sprintf (buf, "errno:%d %s", errno, MonetSystemErrorMessage ());
  LEADER (buf);
ENDPROC


/*************************************************************************/
/*  Script failed with error  */
/*************************************************************************/

VARPROC(MonetScriptFail)

  char buf[SIZEMESSAGEBUFFER];
  SPRINT(buf);
  MonetException_raise_message (MONETSH_ERROR, buf);
ENDPROC

/*********************************************************************
  real system (monet) errors and warnings
  *********************************************************************/

VARPROC(MonetPrintfStderr)

  FPRINT(stderr);
ENDPROC

VARPROC(MonetSysError)

  MonetPrintfStderr ("[Internal Error] ");
  FPRINT(stderr);
  MonetPrintfStderr ("\n");
  fflush (stderr);
ENDPROC

VARPROC(MonetSysWarn)

  MonetPrintfStderr ("[System Warning in %s] ", Monet_program_name ());
  FPRINT(stderr);
  MonetPrintfStderr ("\n");
  fflush (stderr);
ENDPROC

/******************************************************************
  printf routines
  *****************************************************************/


VARPROC(MonetPrintf)
     char buffer[SIZEMESSAGEBUFFER];
     if (*fmt && fmt) {
       SPRINT(buffer);
       if (*buffer) {
	 MonetFwrite (buffer, strlen (buffer));
       }
     }
ENDPROC

VARPROC(MonetPrintfnl)
     char buffer[SIZEMESSAGEBUFFER];
     if (*fmt && fmt) {
       SPRINT(buffer);
       if (*buffer) {
	 MonetFwrite (buffer, strlen (buffer));
       }
     }
     MonetFwrite ("\n", 1);
ENDPROC

/******************************************************************
  stderr routines
 *****************************************************************/

VARPROC(MonetErrPrintf)
     char buffer[SIZEMESSAGEBUFFER];
     if (fmt && *fmt) {
       SPRINT(buffer);
       if (*buffer) {
	 MonetErrFwrite (buffer, strlen (buffer));
       }
     }
ENDPROC

VARPROC(MonetErrPrintfnl)
     char buffer[SIZEMESSAGEBUFFER];
     if (fmt && *fmt) {
       SPRINT(buffer);
       if (*buffer) {
	 MonetErrFwrite (buffer, strlen (buffer));
       }
     }
     MonetErrFwrite ("\n", 1);
ENDPROC

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

