/***********************************************************/
/***********************************************************/
/*  $Header$  */
/***********************************************************/
/*  This is taken from cmusnmp/mib.c  */
/*  It provides more flexible printing routines  */
/***********************************************************/

/***********************************************************
	Copyright 1988, 1989 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/time.h>

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include "snmpP.h"

extern struct tree * Mib;

/***********************************************************/
/**** jbarr ****/
/***********************************************************/

static char StrBuf [2048];

#define F_TYPE 1
#define F_VALUE 2
#define F_ALL 3


/**********************************************************************/

/**********************************************************************/

struct tree *get_symbol();

static oid RFC1066_MIB[] = { 1, 3, 6, 1, 2, 1 };
static unsigned char RFC1066_MIB_text[] = ".iso.org.dod.internet.mgmt.mib";
struct tree *Mib;

static struct tree *
find_rfc1066_mib(root)
    struct tree *root;
{
    oid *op = RFC1066_MIB;
    struct tree *tp;
    int len;

    for(len = sizeof(RFC1066_MIB)/sizeof(oid); len; len--, op++){
	for(tp = root; tp; tp = tp->next_peer){
	    if (tp->subid == *op){
		root = tp->child_list;
		break;
	    }
	}
	if (tp == NULL)
	    return NULL;
    }
    return root;
}

static int
lc_cmp(s1, s2)
    char *s1, *s2;
{
    char c1, c2;

    while(*s1 && *s2){
	if (isupper(*s1))
	    c1 = tolower(*s1);
	else
	    c1 = *s1;
	if (isupper(*s2))
	    c2 = tolower(*s2);
	else
	    c2 = *s2;
	if (c1 != c2)
	    return ((c1 - c2) > 0 ? 1 : -1);
	s1++;
	s2++;
    }

    if (*s1)
	return -1;
    if (*s2)
	return 1;
    return 0;
}


static parse_subtree(subtree, input, output, out_len)
    struct tree *subtree;
    char *input;
    oid	*output;
    int	*out_len;   /* number of subid's */
{
    char buf[128], *to = buf;
    int subid = 0;
    struct tree *tp;

    /*
     * No empty strings.  Can happen if there is a trailing '.' or two '.'s
     * in a row, i.e. "..".
     */
    if ((*input == '\0') ||
	(*input == '.'))
	return (0);

    if (isdigit(*input)) {
	/*
	 * Read the number, then try to find it in the subtree.
	 */
	while (isdigit(*input)) {
	    subid *= 10;
	    subid += *input++ - '0';
	}
	for (tp = subtree; tp; tp = tp->next_peer) {
	    if (tp->subid == subid)
		goto found;
	}
	tp = NULL;
    }
    else {
	/*
	 * Read the name into a buffer.
	 */
	while ((*input != '\0') &&
	       (*input != '.')) {
	    *to++ = *input++;
	}
	*to = '\0';

	/*
	 * Find the name in the subtree;
	 */
	for (tp = subtree; tp; tp = tp->next_peer) {
	    if (lc_cmp(tp->label, buf) == 0) {
		subid = tp->subid;
		goto found;
	    }
	}

	/*
	 * If we didn't find the entry, punt...
	 */
	if (tp == NULL) {
	    MonetPrintfError( "sub-identifier not found: [%s]", buf);
	    return (0);
	}
    }

found:
    if(subid > MAX_SUBID){
	MonetPrintfError( "sub-identifier too large: %s", buf);
	return (0);
    }

    if ((*out_len)-- <= 0){
	MonetPrintfError( "object identifier too long");
	return (0);
    }

    *output++ = subid;

    if (*input != '.')
	return (1);

/* jbarr */
	/*
	 * If we didn't find the entry, punt...
	 * But since it's a number we can do something with it.
	 */
    if (tp == NULL) {
      int n = 0;
      /* MonetPrintfError("numeric sub-identifier not found: %d", subid);*/
      while (*input == '.') {
	subid = 0;
	++input;
	if (!isdigit (*input)) {
	  MonetPrintfError ("A digit was expected not [%s]", input);
	  return 0;
	}
	do {
	  subid *= 10;
	  subid += *input - '0';
	  input++;
	} while (isdigit(*input));
	if(subid > MAX_SUBID) {
	  MonetPrintfError( "sub-identifier too large: %s", buf);
	  return (0);
	}
	if (n >= *out_len){
	  MonetPrintfError( "object identifier too long");
	  return (0);
	}
	++n;
	*output = subid;
	++output;
      }
      if (*input) {
	MonetPrintfError("sub-identifier not found: %d", subid);
	return (0);
      }
      *out_len = n + 1;
      return (*out_len);
    }
/* jbarr */

    if ((*out_len =
	 parse_subtree(tp->child_list, ++input, output, out_len)) == 0)
	return (0);
    return (++*out_len);
}

extern int read_objid(input, output, out_len)
    char *input;
    oid *output;
    int	*out_len;   /* number of subid's in "output" */
{
    struct tree *root = Mib;
    oid *op = output;
    int i;

    if (*input == '.')
	input++;
    else {
	root = find_rfc1066_mib(root);
	for (i = 0; i < sizeof (RFC1066_MIB)/sizeof(oid); i++) {
	    if ((*out_len)-- > 0)
		*output++ = RFC1066_MIB[i];
	    else {
		MonetPrintfError( "object identifier too long");
		return (0);
	    }
	}
    }

    if (root == NULL){
	MonetPrintfError( "Mib not initialized.  Exiting.\n");
	exit(1);
    }
    if ((*out_len =
	 parse_subtree(root, input, output, out_len)) == 0)
	return (0);
    *out_len += output - op;

    return (1);
}

/***********************************************************/

/***********************************************************/

static void sprint_by_type
  PR((char *buf,
      struct variable_list *var,
      int form,
      struct enum_list	    *enums));

/***********************************************************/
/*  sprint various object types  */
/***********************************************************/

static void
sprint_octet_string(buf, var, form, enums)
     char *buf;
     struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    int hex, x;
    u_char *cp;

    if (var->type != ASN_OCTET_STR){
	sprintf(buf, "Wrong Type (should be OCTET STRING): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    hex = 0;
    for(cp = var->val.string, x = 0; x < var->val_len; x++, cp++){
	if (!(isprint(*cp) || isspace(*cp)))
	    hex = 1;
    }
    if (var->val_len <= 4)
	hex = 1;    /* not likely to be ascii */
    if (hex){
      if (form & F_TYPE)
	buf = MonetStringCopyToEnd (buf, "OCTET STRING-   (hex):\t");
      if (form & F_VALUE)
	MonetString2HexString(buf, (char *) var->val.string, var->val_len);
    } else {
      if (form & F_TYPE)
	buf = MonetStringCopyToEnd (buf, "OCTET STRING- (ascii):\t");
      if (form & F_VALUE)
	MonetString2PrintString(buf, (char *) var->val.string, 1024); /* !!! */
    }
}

static void sprint_opaque(buf, var, form, enums)
     char *buf;
     struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{

    if (var->type != OPAQUE){
	sprintf(buf, "Wrong Type (should be Opaque): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd(buf, "OPAQUE -   (hex):\t");
    if (form & F_VALUE)
      MonetString2HexString(buf, (char *) var->val.string, var->val_len);
}

static void
sprint_object_identifier(buf, var, form, enums)
     char *buf;
     struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != ASN_OBJECT_ID){
	sprintf(buf, "Wrong Type (should be OBJECT IDENTIFIER): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd(buf, "OBJECT IDENTIFIER:\t");
    if (form & F_VALUE)
      sprint_objid(buf, (oid *)(var->val.objid), var->val_len / sizeof(oid));
}

static void
sprint_timeticks(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != TIMETICKS){
	sprintf(buf, "Wrong Type (should be Timeticks): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
      if (form & F_TYPE)
	sprintf(buf, "Timeticks: (%d) %s", *(var->val.integer),
		Monet_uptime_string(*(var->val.integer)));
      if (form & F_VALUE)
	sprintf(buf, "%d", *(var->val.integer));

}

static void
sprint_integer(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
    struct enum_list	    *enums;
     int form;
{
    char    *enum_string = NULL;

    if (var->type != ASN_INTEGER){
	sprintf(buf, "Wrong Type (should be INTEGER): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    for (; enums; enums = enums->next)
	if (enums->value == *var->val.integer){
	    enum_string = enums->label;
	    break;
	}
    if (form & F_TYPE) {
      if (enum_string == NULL)
	sprintf(buf, "INTEGER: %d", *var->val.integer);
      else
	sprintf(buf, "INTEGER: %s(%d)", enum_string, *var->val.integer);
    }
    if (form & F_VALUE)
	sprintf(buf, "%d", *var->val.integer);
}

static void
sprint_gauge(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != GAUGE){
	sprintf(buf, "Wrong Type (should be Gauge): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd(buf, "Gauge:");
    if (form & F_VALUE)
      sprintf(buf, "%lu", *var->val.integer);
}

static void
sprint_counter(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != COUNTER){
	sprintf(buf, "Wrong Type (should be Counter): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd(buf, "Counter:");
    if (form & F_VALUE)
      sprintf(buf, "%lu", *var->val.integer);
}

static void
sprint_networkaddress(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    int x, len;
    u_char *cp;

    if (form & F_TYPE)
      buf = MonetStringCopyToEnd(buf, "Network Address:\t");
    if (form & F_VALUE) {
      cp = var->val.string;    
      len = var->val_len;
      for(x = 0; x < len; x++) {
	sprintf(buf, "%02X", *cp++);
	buf += MonetStrlen(buf);
	if (x < (len - 1))
	  *buf++ = ':';
      }
    }
}

static void
sprint_ipaddress(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    u_char *ip;

    if (var->type != IPADDRESS){
	sprintf(buf, "Wrong Type (should be Ipaddress): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    ip = var->val.string;
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd (buf, "IpAddress:");
    if (form & F_VALUE)
      sprintf(buf, "%d.%d.%d.%d",ip[0], ip[1], ip[2], ip[3]);
}
/*
static void
sprint_unsigned_short(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != ASN_INTEGER){
	sprintf(buf, "Wrong Type (should be INTEGER): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    if (form & F_TYPE)
      buf = MonetStringCopyToEnd (buf,"INTEGER (0..65535):");
    if (form & F_VALUE)
      sprintf(buf,  "%lu", *var->val.integer);
}
*/
static void
sprint_null(buf, var, form, enums)
     char *buf;
     struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    if (var->type != ASN_NULL){
	sprintf(buf, "Wrong Type (should be NULL): ");
	buf += MonetStrlen(buf);
	sprint_by_type(buf, var, form, (struct enum_list *)NULL);
	return;
    }
    sprintf(buf, "NULL");
}

static void
sprint_badtype(buf, var, form, enums)
     char *buf;
     struct variable_list *var;
     int form;
     struct enum_list	    *enums;
{
    sprintf(buf, "Variable has bad type");
}

static void
sprint_by_type(buf, var, form, enums)
    char *buf;
    struct variable_list *var;
    struct enum_list	    *enums;
     int form;
{
    switch (var->type){
	case ASN_INTEGER:
	    sprint_integer(buf, var, form, enums);
	    break;
	case ASN_OCTET_STR:
	    sprint_octet_string(buf, var, form, enums);
	    break;
	case OPAQUE:
	    sprint_opaque(buf, var, form, enums);
	    break;
	case ASN_OBJECT_ID:
	    sprint_object_identifier(buf, var, form, enums);
	    break;
	case TIMETICKS:
	    sprint_timeticks(buf, var, form, enums);
	    break;
	case GAUGE:
	    sprint_gauge(buf, var, form, enums);
	    break;
	case COUNTER:
	    sprint_counter(buf, var, form, enums);
	    break;
	case IPADDRESS:
	    sprint_ipaddress(buf, var, form, enums);
	    break;
	case ASN_NULL:
	    sprint_null(buf, var, form, enums);
	    break;
	default:
	    sprint_badtype(buf, var, form, enums);
	    break;
    }
}


static char *  type2string (type)
     u_char type;
{
  char * result;
  switch (type){
  case ASN_INTEGER:
    result = "Integer";
    break;
  case ASN_OCTET_STR:
    result = "Octet String";
    break;
  case OPAQUE:
    result = "Opaque";
    break;
  case ASN_OBJECT_ID:
    result = "Oid";
    break;
  case TIMETICKS:
    result = "Timeticks";
    break;
  case GAUGE:
    result = "Gauge";
    break;
  case COUNTER:
    result = "Counter";
    break;
  case IPADDRESS:
    result = "IpAddress";
    break;
  case ASN_NULL:
    result = "NULL";
    break;
  default:
    result = "Unknown";
    break;
  }
  return result;
}


/**********************************************************************/
/*  Print subtree */
/**********************************************************************/

static void sprint_subtree (buf, subtree, variable, form)
     struct tree    *subtree;
     String buf;
     struct variable_list *variable;
     int form;
{
  if (subtree->printer)
    (*subtree->printer)(buf, variable, form, subtree->enums);
  else {
    sprint_by_type(buf, variable, form, subtree->enums);
  }
}

/**********************************************************************/

/**********************************************************************/

static void
set_functions(subtree)
    struct tree *subtree;
{
    for(; subtree; subtree = subtree->next_peer){
	switch(subtree->type){
	    case TYPE_OBJID:
		subtree->printer = sprint_object_identifier;
		break;
	    case TYPE_OCTETSTR:
		subtree->printer = sprint_octet_string;
		break;
	    case TYPE_INTEGER:
		subtree->printer = sprint_integer;
		break;
	    case TYPE_NETADDR:
		subtree->printer = sprint_networkaddress;
		break;
	    case TYPE_IPADDR:
		subtree->printer = sprint_ipaddress;
		break;
	    case TYPE_COUNTER:
		subtree->printer = sprint_counter;
		break;
	    case TYPE_GAUGE:
		subtree->printer = sprint_gauge;
		break;
	    case TYPE_TIMETICKS:
		subtree->printer = sprint_timeticks;
		break;
	    case TYPE_OPAQUE:
		subtree->printer = sprint_opaque;
		break;
	    case TYPE_NULL:
		subtree->printer = sprint_null;
		break;
	    case TYPE_OTHER:
	    default:
		subtree->printer = sprint_badtype;
		break;
	}
	set_functions(subtree->child_list);
    }
}

static struct tree * find_oid_mib (root, op, len)
     struct tree *root;
     oid *op;
     int len;
{
  struct tree *tp = NULL;

  for(; len; len--, op++) {
    for(tp = root; tp; tp = tp->next_peer){
      if (tp->subid == *op){
	root = tp->child_list;
	break;
      }
    }
    if (tp == NULL)
      return NULL;
  }
  return tp;
}

static struct tree * find_mib_tree (root, name)
     char * name;
     struct tree *root;
{
  int len;
  oid objid[MAX_NAME_LEN];

  len = MAX_NAME_LEN;
  if (!read_objid(name, objid, &len)){
    MonetPrintfError ("Invalid object identifier: %s", name);
    return NULL;
  }
  return find_oid_mib (root, objid, len);
}

/*
static struct tree * find_oid_parent (root, op, len)
     struct tree *root;
     oid * op;
     int len;
{
  if (len == 0)
    return NULL;
  return find_oid_mib (root, op, len-1);
}
*/

static struct tree * find_mib_parent (root, name)
     char * name;
     struct tree *root;
{
  int len;
  oid objid[MAX_NAME_LEN];

  len = MAX_NAME_LEN;
  if (!read_objid(name, objid, &len)){
    MonetPrintfError ("Invalid object identifier: %s", name);
    return NULL;
  }
  if (len == 0)
    return NULL;
  return find_oid_mib (root, objid, len-1);
}

/**********************************************************************/
/*  sprint variables, external routines */
/**********************************************************************/

static void print_subtree(subtree, name_prefix)
     struct tree *subtree;
     char * name_prefix;
{
  if (subtree) {
    struct tree *tp;
    char * message, * type;
    message = (char *) MonetMalloc(MonetStrlen(name_prefix)+
			       MonetStrlen(subtree->label)+3);
    if (message) {
      (void) sprintf (message, "%s.%s", name_prefix, subtree->label);
      type = type2string (subtree -> type);
      if (type) {
	MonetPrintf("%s\t%s\n", message, type);
      } else {
	MonetPrintf("%s\n", message);
      }
      for(tp = subtree->child_list; tp; tp = tp->next_peer) {
	if (MonetException_code ())
	  break;
	print_subtree(tp, message);
      }
      MonetFree (message);
    }
  }
}

MONETCALLBACK (do_snmp_dump_mib)
     extern struct tree * Mib;
     struct tree * mib;
     char * start;
     if (argc == 1) {
       mib = Mib;
       start = "";
     } else if (mib = find_mib_tree (Mib, argv[1])) {
       start = argv[1];
     } else {
       RETURNSC;
     }
     print_subtree(mib, start);
MONETEND

MONETCALLBACK (do_snmp_dump_rfc1066)
     extern struct tree * Mib;
     struct tree * root;
     root = find_rfc1066_mib(Mib);
     print_subtree(root, "");
MONETEND

static char * parent_name (string)
     char * string;
{
  oid objid[MAX_NAME_LEN];
  int objid_length;

  /*     SNMPOKAY; */

  objid_length = MAX_NAME_LEN;
  if (!read_objid(string, objid, &objid_length)) {
    MonetPrintfError ("Invalid object identifier: %s", string);
  } else if (objid_length > 0) {
    (void) sprint_objid(StrBuf, objid, objid_length - 1);
    return StrBuf;
  }
  return NULL;
}

MONETCALLBACK (do_parent)
     result = parent_name (argv[1]);
MONETEND

MONETCALLBACK (do_previous_sibling)
     extern struct tree * Mib;
     struct tree * tree;
     char * name;

     name = (find_mib_tree (Mib, argv[1]))->label;

     if (tree = find_mib_parent (Mib, argv[1])) {
       char * parent = parent_name (argv[1]);
       struct tree *tp, *previous = NULL;
       for(tp = tree->child_list; tp; tp = tp->next_peer) {
	 if (EQS(tp->label, name)) {
	   if (previous) {
	     (void) sprintf (StrBuf, "%s.%s", parent, previous->label);
	     result = StrBuf;
	     RETURNSC;
	   }
	 }
	 previous = tp;
       }
       if (previous) {
	 (void) sprintf (StrBuf, "%s.%s", parent, previous->label);
	 result = StrBuf;
	 RETURNSC;
       }
     }
     result = NULL;
MONETEND

MONETCALLBACK (do_next_sibling)
     extern struct tree * Mib;
     struct tree * tree;
     char * name;

     name = (find_mib_tree (Mib, argv[1]))->label;

     if (tree = find_mib_parent (Mib, argv[1])) {
       char * parent = parent_name (argv[1]);
       struct tree *tp, *first;
       first = tree->child_list;
       for(tp = first; tp; tp = tp->next_peer) {
	 if (EQS(tp->label, name)) {
	   if (tp->next_peer) {
	     (void) sprintf (StrBuf, "%s.%s", parent, tp->next_peer->label);
	     result = StrBuf;
	     RETURNSC;
	   }
	 }
       }
       if (first) {
	 (void) sprintf (StrBuf, "%s.%s", parent, first->label);
	 result = StrBuf;
	 RETURNSC;
       }
     }
     result = NULL;
MONETEND

/*
MONETCALLBACK (do_next)
     extern struct tree * Mib;
     struct tree * tree;

     if (tree = find_mib_tree (Mib, argv[1])) {
       if (tree->child_list) {
	 result = tree->child_list->label;
       }
     } else {

     }
MONETEND
*/

MONETCALLBACK (do_child)
     extern struct tree * Mib;
     struct tree * tree;

     result = NULL;

     if (tree = find_mib_tree (Mib, argv[1])) {
       if (tree->child_list) {
	 (void) sprintf (StrBuf, "%s.%s", argv[1], tree->child_list->label);
	 result = StrBuf;
       }
     }
MONETEND

/*
MONETCALLBACK (do_complete)

MONETEND
*/

MONETCALLBACK (do_rfc)
     result = (char *) RFC1066_MIB_text;
MONETEND

/***************************************************************************/
/*  Oid in numeric format  */
/***************************************************************************/

MONETCALLBACK(do_oid_numeric)
     MonetPrintfWarning ("not implemented yet");
     result = NULL;
MONETEND

MONETCALLBACK(do_oid_alpha)
     oid objid[MAX_NAME_LEN];
     int objid_length;
     String string = argv[1];

/*     SNMPOKAY; */

     objid_length = MAX_NAME_LEN;
     if (!read_objid(string, objid, &objid_length)){
       MonetPrintfError ("Invalid object identifier: %s", string);
       result = NULL;
     } else {
       (void) sprint_objid(StrBuf, objid, objid_length);
       result =  StrBuf;
     }
MONETEND

extern MonetBool MonetMib_init (file)
     String file;
{
  extern struct tree * Mib;

  (void) MonetMember_module_class ("oid");
  MonetMember_create
    ("alpha",  do_oid_alpha,
     "oid",
     "Returns the given Object Identifier in named format. \
Uses the Snmp mib for reference");
  MonetMember_create
    ("numeric",  do_oid_numeric,
     "oid",
     "Returns the given Object Identifier in numeric format. \
Uses the Snmp mib for reference");
  MonetMember_create
    ("dump",  do_snmp_dump_mib,
     "?oid",
     "Dumps the known oid/mib tree, below a certain level");
  MonetMember_create
    ("dump-rfc1066",  do_snmp_dump_rfc1066,
     "",
     "Returns the top of the rfc1066 asn.1 mib tree, \
     .iso.org.dod.internet.mgmt.mib");
  MonetMember_create
    ("next", do_next_sibling,
     "oid",
     "Returns the next oid on the same level");
  MonetMember_create
    ("previous",  do_previous_sibling,
     "oid",
     "Returns the previous oid on the same level");
  MonetMember_create
    ("rfc1066",  do_rfc,
     "",
     "Returns the rfc1066 mib");
/*  MonetMember_create
    ("next",  do_next,
     "oid",
     "Returns the next oid");
*/
  MonetMember_create
    ("child",  do_child,
     "oid",
     "Returns the child of the oid");
  MonetMember_create
    ("parent",  do_parent,
     "oid",
     "Returns the parent of the oid");
/*  MonetMember_create
    ("complete",  do_child,
     "oid",
     "Returns a complete 'oid' given a prefix");
*/

  if (! file) {
    file = "mib.txt";
  }
  file = (char *) MonetFilename (file);
  Mib = read_mib(file);
  if (!Mib) {
    MonetPrintfError ("Couldn't open SNMP MIB file [%s]", file);
    return False;
  }
  set_functions(Mib);

  return True;
}

extern String MonetOid_sprint_short_objid(objid, objidlen)
    oid	    *objid;
    int	    objidlen;	/* number of subidentifiers */
{
  static char    tempbuf[1024];
  char *cp;
  struct tree    *subtree = Mib;

  tempbuf[0] = '.';	/* this is a fully qualified name */
  tempbuf[1] = '\0';
  subtree = get_symbol(objid, objidlen, subtree, tempbuf + 1);
  if ((MonetStrlen(tempbuf) >= MonetStrlen((char *)RFC1066_MIB_text)) &&
      !bcmp(tempbuf, (char *)RFC1066_MIB_text,
	    MonetStrlen((char *)RFC1066_MIB_text))){
    cp = tempbuf + sizeof(RFC1066_MIB_text);
  } else {
    cp = tempbuf;
  }
  return cp;
}

extern String MonetOid_sprint_variable(objid, objidlen, variable)
    oid     *objid;
    int	    objidlen;
    struct  variable_list *variable;
{
    char  *result, *cp;
    struct tree    *subtree = Mib;
    cp = result = MonetOid_sprint_short_objid(objid, objidlen);
    cp += MonetStrlen (cp);
    sprint_subtree (cp, subtree, variable, F_ALL);
    return result;
}

extern String MonetOid_sprint_value(objid, objidlen, variable)
    oid     *objid;
    int	    objidlen;
    struct  variable_list *variable;
{
    static char    buf[512];
    struct tree    *subtree = Mib;

    subtree = get_symbol(objid, objidlen, subtree, buf);
    *buf = '\0';
    sprint_subtree (buf, subtree, variable, F_VALUE);
    return buf;
}

extern String MonetOid_sprint_type(objid, objidlen, variable)
    oid     *objid;
    int	    objidlen;
    struct  variable_list *variable;
{
    static char    buf[512];
    struct tree    *subtree = Mib;

    subtree = get_symbol(objid, objidlen, subtree, buf);
    *buf = '\0';
    sprint_subtree (buf, subtree, variable, F_TYPE);
    return buf;
}


/***********************************************************/
/*  Obsolete unused stuff */
/***********************************************************/

#ifdef UNUSEDjjj

print_value(objid, objidlen, variable)
    oid     *objid;
    int	    objidlen;
    struct  variable_list *variable;
{
    char    tempbuf[512];
    struct tree    *subtree = Mib;

    subtree = get_symbol(objid, objidlen, subtree, tempbuf);
    sprint_subtree (tempbuf, subtree, variable, form);
    printf("%s", tempbuf);
}

struct tree *
get_symbol(objid, objidlen, subtree, buf)
    oid	    *objid;
    int	    objidlen;
    struct tree    *subtree;
    char    *buf;
{
    struct tree    *return_tree = NULL;

    for(; subtree; subtree = subtree->next_peer){
	if (*objid == subtree->subid){
	    strcpy(buf, subtree->label);
	    goto found;
	}
    }

    /* subtree not found */
    while(objidlen--){	/* output rest of name, uninterpreted */
	sprintf(buf, "%d.", *objid++);
	while(*buf)
	    buf++;
    }
    *(buf - 1) = '\0'; /* remove trailing dot */
    return NULL;

found:
    if (objidlen > 1){
	while(*buf)
	    buf++;
	*buf++ = '.';
	*buf = '\0';
	return_tree = get_symbol(objid + 1, objidlen - 1, subtree->child_list, buf);
    } 
    if (return_tree != NULL)
	return return_tree;
    else
	return subtree;
}


print_objid(objid, objidlen)
    oid	    *objid;
    int	    objidlen;	/* number of subidentifiers */
{
    char    buf[256];
    struct tree    *subtree = Mib;

    *buf = '.';	/* this is a fully qualified name */
    get_symbol(objid, objidlen, subtree, buf + 1);
    printf("%s\n", buf);
        
}

sprint_objid(buf, objid, objidlen)
    char *buf;
    oid	    *objid;
    int	    objidlen;	/* number of subidentifiers */
{
    struct tree    *subtree = Mib;

    *buf = '.';	/* this is a fully qualified name */
    get_symbol(objid, objidlen, subtree, buf + 1);
}

print_variable(objid, objidlen, variable)
    oid     *objid;
    int	    objidlen;
    struct  variable_list *variable;
{
    char    buf[512], *cp;
    struct tree    *subtree = Mib;

    *buf = '.';	/* this is a fully qualified name */
    subtree = get_symbol(objid, objidlen, subtree, buf + 1);
    cp = buf;
    if ((MonetStrlen(buf) >= MonetStrlen((char *)RFC1066_MIB_text)) && !bcmp(buf, (char *)RFC1066_MIB_text,
	MonetStrlen((char *)RFC1066_MIB_text))){
	    cp += sizeof(RFC1066_MIB_text);
    }
    printf("Name: %s\n", cp);
    *buf = '\0';
    sprint_subtree (buf, subtree, variable, form);
    printf ("%s", buf);
}

int
read_rawobjid(input, output, out_len)
    char *input;
    oid *output;
    int	*out_len;
{
    char    buf[12], *cp;
    oid	    *op = output;
    int	    subid;

    while(*input != '\0'){
	if (!isdigit(*input))
	    break;
	cp = buf;
	while(isdigit(*input))
	    *cp++ = *input++;
	*cp = '\0';
	subid = atoi(buf);
	if(subid > MAX_SUBID){
	    fprintf(stderr, "sub-identifier too large: %s\n", buf);
	    return 0;
	}
	if((*out_len)-- <= 0){
	    fprintf(stderr, "object identifier too long\n");
	    return 0;
	}
	*op++ = subid;
	if(*input++ != '.')
	    break;
    }
    *out_len = op - output;
    if (*out_len == 0)
	return 0;
    return 1;
}


#ifdef testing
int snmp_dump_packet = 0;

main(argc, argv)
     int argc;
     char *argv[];
{
    oid objid[64];
    int objidlen = sizeof (objid);
    int count;
    struct variable variable;

    init_mib(&Mib);
    if (argc < 2)
	print_subtree(Mib, 0);
    variable.type = ASN_INTEGER;
    variable.val.integer = 3;
    variable.val_len = 4;
    for (argc--; argc; argc--, argv++) {
	objidlen = sizeof (objid);
	printf("read_objid(%s) = %d\n",
	       argv[1], read_objid(argv[1], objid, &objidlen));
	for(count = 0; count < objidlen; count++)
	    printf("%d.", objid[count]);
	printf("\n");
	print_variable(objid, objidlen, &variable);
    }
}

#endif testing

int read_objid(input, output, out_len)
    char *input;
    oid *output;
    int	*out_len;   /* number of subid's in "output" */
{
    struct tree *root = Mib;
    oid *op = output;
    int i;

    if (*input == '.')
	input++;
    else {
	root = find_rfc1066_mib(root);
	for (i = 0; i < sizeof (RFC1066_MIB)/sizeof(oid); i++) {
	    if ((*out_len)-- > 0)
		*output++ = RFC1066_MIB[i];
	    else {
		fprintf(stderr, "object identifier too long\n");
		return (0);
	    }
	}
    }

    if (root == NULL){
	fprintf(stderr, "Mib not initialized.  Exiting.\n");
	exit(1);
    }
    if ((*out_len =
	 parse_subtree(root, input, output, out_len)) == 0)
	return (0);
    *out_len += output - op;

    return (1);
}


static parse_subtree(subtree, input, output, out_len)
    struct tree *subtree;
    char *input;
    oid	*output;
    int	*out_len;   /* number of subid's */
{
    char buf[128], *to = buf;
    int subid = 0;
    struct tree *tp;

    /*
     * No empty strings.  Can happen if there is a trailing '.' or two '.'s
     * in a row, i.e. "..".
     */
    if ((*input == '\0') ||
	(*input == '.'))
	return (0);

    if (isdigit(*input)) {
	/*
	 * Read the number, then try to find it in the subtree.
	 */
	while (isdigit(*input)) {
	    subid *= 10;
	    subid += *input++ - '0';
	}
	for (tp = subtree; tp; tp = tp->next_peer) {
	    if (tp->subid == subid)
		goto found;
	}
	tp = NULL;
    }
    else {
	/*
	 * Read the name into a buffer.
	 */
	while ((*input != '\0') &&
	       (*input != '.')) {
	    *to++ = *input++;
	}
	*to = '\0';

	/*
	 * Find the name in the subtree;
	 */
	for (tp = subtree; tp; tp = tp->next_peer) {
	    if (MonetLowerCaseCompare(tp->label, buf) == 0) {
		subid = tp->subid;
		goto found;
	    }
	}

	/*
	 * If we didn't find the entry, punt...
	 */
	if (tp == NULL) {
	    fprintf(stderr, "sub-identifier not found: %s\n", buf);
	    return (0);
	}
    }

found:
    if(subid > MAX_SUBID){
	fprintf(stderr, "sub-identifier too large: %s\n", buf);
	return (0);
    }

    if ((*out_len)-- <= 0){
	fprintf(stderr, "object identifier too long\n");
	return (0);
    }
    *output++ = subid;

    if (*input != '.')
	return (1);
    if ((*out_len =
	 parse_subtree(tp->child_list, ++input, output, out_len)) == 0)
	return (0);
    return (++*out_len);
}

#endif

/***********************************************************/
/***********************************************************/
/*  $Header$  */
/***********************************************************/
/***********************************************************/
