/* ===========================================================================

	x25_getbill - io.c	Marten Terpstra, NIKHEF-H

	Here the (host dependent) input/output routines
	reside. The first two functions put() and get() are
	pretty straight forward and don't have to be changed,
	but the routines openx25() and getcause() may have to
	be altered or completely rewritten (see README).

============================================================================ */	

#include <stdio.h>

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>

#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/ioctl.h>
#include <netdb.h>

#include <sundev/syncstat.h>
#include <netx25/x25_pk.h>
#include <netx25/x25_ctl.h>
#include <netx25/x25_ioctl.h>

#define	CON_ID		0x01

/* CONN_DB 	r_address = {0, 0, 4, { CON_ID, 0, 0, 0}};*/
CONN_DB 	r_address ;
FACILITY_DB 	facils;
X25_CAUSE_DIAG 	diag;

/* ====================================================================	*/
/* 	char2bcd - convert ascii numer to bcd.  */
/* ====================================================================	*/

static u_char char2bcd (s, d)
register char   *s;
register u_char *d;
{
    register int    c,
		    i;

    for (i = 0; *s; i++) {
	if ((c = *s++) >= 'a' && c <= 'f')
	    c -= 'a' + 0x0a;
	else
	    if (c >= 'A' && c <= 'F')
		c -= 'A' + 0x0a;
	    else
		if (c >= '0' && c <= '9')
		    c -= '0';
		else
		    c = 0;

	if (i & 1)
	    *d++ |= c & 0xf;
	else
	    *d = (c & 0xf) << 4;
    }

    return i;
}

/* ====================================================================	*/
/* 	openx25() - open a connection to address PSE_DTELOG		*/
/* ====================================================================	*/

static openx25(nua, user_data)
char *nua, *user_data;
{
	extern          		errno;
	register int    		sd;

	int error;

	r_address.hostlen = char2bcd( nua, r_address.host);
	r_address.datalen = 4;
	r_address.data[0] = 0x01;

        if ((sd = socket (AF_X25, SOCK_STREAM, 0)) < 0) {
		MonetPrintfError ("openx25: socket");
		return(-1);
	} 

	facils.recvpktsize = 128;
	facils.sendpktsize = 128;
	facils.recvwndsize = 2;
	facils.sendwndsize = 2;
	if ( ioctl(sd, X25_WR_FACILITY, &facils) < 0 ) {
                MonetPrintfError ("openx25: facilities");
                return(-1);
	}
        if (connect (sd, &r_address, sizeof (CONN_DB)) < 0) {
	  if (Monet_debug () > 10) {
	    getcause (sd, errno, nua); 
	  }
	  return(-2);
	}

	return (sd);
}


/* ====================================================================	*/
/*	closex25 - close socket and set status-variable                 */
/* ====================================================================	*/
static closex25(fd)
int	fd;
{
  getcause(x25_connection, errno, dte);
  close(fd);
  restart_x25 = TRUE;
}


/* ==================================================================== */
/*      getcause(fd, error) - Diagnose error on openening a connection  */
/* ==================================================================== */
static getcause (sd, error, nua)
int             sd;
int             error;
char		*nua;
{
        int             cause_code, diag_code, len;
	char            buf[80];


	if ( ioctl(sd, X25_RD_CAUSE_DIAG, &diag) < 0 ) {
		MonetPrintfError ("openx25: connect cause read failed");
	} else if (( diag.flags & ( (u_char)1 << RECV_DIAG)) == 0 
		|| diag.datalen < 1) {
		MonetPrintfError ("openx25: call failed : no cause given");
	} else {
	  cause_code = diag.data[0];
	  if ( diag.datalen == 1 ) 
		diag_code = 0;
	  else
		diag_code = diag.data[1];

	  if (( diag.flags & ( (u_char)1 << DIAG_TYPE)) == 0 ) {
		sprintf(buf, "%s %s %s %x diag %x\n",
		"openx25: connect ", nua, ": reset cause",
		diag.data[0], diag.data[1]);
		MonetPrintfError(buf);
	  } else {
		sprintf(buf, "%s %s %s %x diag %x\n",
		"openx25: connect ", nua, ": clear cause",
		diag.data[0], diag.data[1]);
		MonetPrintfError(buf);
	  }
	}
		

}

static int input_pending(fd)
int fd;
{   /* check to see if input is pending on fd */
        fd_set readfds;
	long   time_now;
        struct timeval tid;

        FD_ZERO( &readfds);
        FD_SET( fd, &readfds);
        tid.tv_sec=0;
        tid.tv_usec=0;
	return(select( getdtablesize(), &readfds, 0, 0, &tid) != 0);

}


static int except_pending(fd)
int fd;
{  
/* check to see if our X.25-connection is up */
        fd_set exceptfds;
        struct timeval tid;

        FD_ZERO( &exceptfds);
        FD_SET( fd, &exceptfds);
        tid.tv_sec=0;
        tid.tv_usec=0;
	return( select( getdtablesize(), 0, 0, &exceptfds, &tid) != 0);
}


#define OPEN_CLOSE_X25
