/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/********************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include "x25P.h"
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>


/* ====================================================================	*/
/*	defs.h - x25elogd						*/
/*									*/
/*	(c) Marten Terpstra, NIKHEF-H					*/
/*	Mar 1990							*/
/* ====================================================================	*/

#include <stdio.h>

#define	MAXLINKS	20
#define	POLL_INTERVAL	20
#define	MAX_LINE	133

#define	TRUE		1
#define	FALSE		0

#define	NOT_OCCURRED		50
#define	DONT_PRINT		51
#define	OCCURRED_BUT_PRINT	52

#define	UP_SIGNAL	100
#define	DOWN_SIGNAL	101

extern int  display_state;
#define	CONTINUE        0
#define	CLOSE	        1
#define	TERMINATE	2
#define	RESTART         3


#define TIMEDIFF	30

#define POLL_INITIAL	60
#define LOG_INTERVAL    3600

#define NAME_REC	1
#define HEAD_REC	2
#define DETAIL_REC	3
#define END_REC		4

#define HEADER "LINK   STATE CALLS CRC  RETR   REJ  FRAME-I  FRAME-O   SET UP FR/SEC-I FR/SEC-O"
#define FOOTER "p - poll switch; r - read config; q - quit display;   t - terminate elogd"
#define CONNECTION "Connection to switch down";

/* Here are the global variables.				*/
/* I know I shouldn't have them, but they make life so much	*/
/* easier ...							*/

typedef struct linkdef {
	char	name[5];
	int	down_at;
	int	warning;
	int	notified;
	char    status[MAX_LINE];
} LINKDEF;

typedef struct server_desc {
	struct server_desc	*suc;
	struct server_desc	*pred;
	int     socket;
} ;

typedef struct server_rec {
	int	type;
	int	size;
	char	data[MAX_LINE];
};

static char	*X25_get();
static		X25_put();
static int      input_pending();
static int      except_pending();
static          getcause();

/*
static		fclose();
FILE		*fopen(); 
static char	*malloc();
char    *sprintf();
*/

#ifdef sklsdlksdkl
char		errorfile[80],programfile[80],panicfile[80];
char		pollfile[80];
char		billfile[80];
char		configfile[80];


static struct linkdef  x25_link[];


int      x25_connection,  /* descriptor */
  restart_x25,       /* TRUE - when X.25 conncetion is broken */
  mydisplay_open,    /* TRUE  */
  restart_display,   /* TRUE = restart display when ready */
  just_monitoring,   /* TRUE = just poll and display */
  poll_interval;     /* seconds between polling */
static int      nr_of_links;   
static long     next_logtime,
                next_midnight,
                log_interval; /* seconds between logg links to file */
static char	*switch_name;
char	dte[20];
static char	last_error[MAX_LINE];
static char     display_name[80];
#endif

/***************************************************************************/
/* System specific stuff */
/***************************************************************************/

#ifdef apollo
#include "apollo-x25.c"
#elif SUN_X25
#include "sun-x25.c"
#elif GOULD_NP1
#include "gould-x25.c"
#elif UCB_X25
#include "ucb-x25.c"
#else

MONETCALLBACK(do_x25_ping_once)
     MonetPrintfError ("X25 not compiled in");
MONETEND

#endif

#ifdef OPEN_CLOSE_X25

/* ====================================================================	*/
/* 	OCCUR() - Check for occurence of pat in str			*/
/* ====================================================================	*/

static int occurs(str, pat)
char *str, *pat;
{
	register char *point = str;

	while (point=index(point, *pat))
	{
		if (strncmp(point, pat, strlen(pat)) == 0)
			return(1);
		point++;
	}
	return(0);
}

/* ====================================================================	*/
/*	put(fd, msg) - write msg to file descriptor fd			*/
/* ====================================================================	*/

static int X25_put(fd, msg)
int	fd;
char	*msg;
{
	return write(fd, msg, strlen(msg));
}

/* ====================================================================	*/
/*	char *get(fd) - get string from file descriptor fd.		*/
/*			if "NONE" occurs in string, return (char *)0	*/
/* ====================================================================	*/

static char *X25_get(fd)
int	fd;
{
	int		n;
	static char	buf[512];

	if ((n=recv(fd, buf, 512,0)) <= 0 ) {
	  closex25(fd); 
	  return ((char *) 0);
	}

	buf[n] = '\0';

	if (occurs(buf, "NONE")) return((char *) 0);

	return(&buf[0]);
}

/********************************************************************/
/*  Probe for seel boxes  */
/********************************************************************/

MONETCALLBACK(do_telepac_probe)
     /*     int check_links(fd,nr_of_links,link) */
     int	fd;
     int	nr_of_links;
     LINKDEF	x25_link[MAX_LINE];

     char		*msg,*command,buf[20];
     int		time_now,i;

     command = buf;

     for (i=1;i<=nr_of_links;i++) {

       /* Construct command for TelePac switch to poll status of a link	*/

       command = sprintf(command,"\020poll %s\r",x25_link[i].name);
	
       X25_put(fd,command);
       msg = X25_get(fd);
 
       if (msg && MonetIsStringInString(msg,"DATA")) {

	 /* If status is DATA check if a notification has gone out. If not   */
	 /* everything is OK, if it has, sent out a UP_SIGNAL.		     */
			
	 x25_link[i].down_at = 0;
	 
	 if (x25_link[i].notified == TRUE) {
	   /*	   notify(x25_link[i].name,UP_SIGNAL,x25_link[i].warning); */
	   x25_link[i].notified = FALSE;
	 }
       } else {
    
	 /* If the status != DATA check if a notification has gone out. If not	*/
	 /* check if it's time to do so, else wait.				*/

	 if (x25_link[i].notified == FALSE) {

	   time_now = time((long *) 0);
		
	   if (x25_link[i].down_at == 0) 
	     x25_link[i].down_at = time_now;

	   if (time_now - x25_link[i].down_at >= x25_link[i].warning) {
	     x25_link[i].notified = TRUE;
	     /*	     notify(x25_link[i].name,DOWN_SIGNAL,x25_link[i].warning); */
	   }
	 }
       }
     }
}

/********************************************************************/
/*  Generic pinger  */
/********************************************************************/

MONETCALLBACK(do_x25_ping_once)
     String host = argv[1];
     int    		fd;
     MonetLookupX25 lookup;

     if ((lookup = MonetHost2X25 (host)) == NULL) {
       result = "invalid";
     } else {
       fd = openx25 (lookup->address, lookup->userdata);
       if (fd == -1) {
	 result = "unknown";
       } else if (fd == -2) {
	 result = "unreachable";
       } else if (fd >= 0) {
	 result = "reachable";
	 closex25 (fd);
       } else {
	 result = "unknown";
       }
     }
MONETEND

#endif /* OPEN_CLOSE_X25 */

/********************************************************************/
/*  Init and finish  */
/********************************************************************/

extern void MonetProbe_x25_init ()
{
  MonetX25Lookup_init ();

#ifdef OPEN_CLOSE_X25
  MonetMember_create ("probe-telepac", do_telepac_probe,
     "host",
     "Probes a seel/telepac",
     "");
#endif

  MonetMember_create ("ping-once", do_x25_ping_once,
     "host",
     "Succeeds if host is reachable.  Take this with a pinch of salt.");
  MonetX25Lookup_periodic_check_file ();
}

extern void MonetProbe_x25_finish ()
{
  MonetX25Lookup_finish ();
}

/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/********************************************************************/


