/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/********************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include <monet/msh/Channel.h>

#include <sys/errno.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>

/********************************************************************/
/*  Write  */
/********************************************************************/

MONETCALLBACK(do_write)
     int fd;
     char * message;
     Channel channel;

     fd = MonetString2int (argv[1]);
     message = MonetJoinStrings (argc-2, argv+2, " ");

     if (channel = fid2Channel (fd)) {
       Channel_send (channel, message);
     } else {
       MonetPrintfError ("No channel connected to fd [%d]", fd);
     }
MONETEND

MONETCALLBACK(do_writeln)
     int fd;
     char * message;
     Channel channel;

     fd = MonetString2int (argv[1]);
     message = MonetJoinStrings (argc-2, argv+2, " ");

     if (channel = fid2Channel (fd)) {
       Channel_send_record (channel, message);
     } else {
       MonetPrintfError ("No channel connected to fd [%d]", fd);
     }
MONETEND

/**************************************************************************
  Message come in
  **************************************************************************/

static MonetBool input (channel, text)
     Channel channel;
     char * text;
{
  char * command  = (char *) Channel_data (channel);
  char * names[5], * values[5];

  names[0] = "fd-channel";
  values[0] = Monetint2String (Channel2fid (channel));
  names[1] = "accept-channel";
  values[1] = NULSTR;
  names[2] = "close-channel";
  values[2] = NULSTR;
  names[3] = "data-channel";
  values[3] = text;
  names[4] = NULSTR;
  values[4] = NULSTR;

  /* -- If the command succeeds return true, else return false in which case
     -- the channel closes itself.  */
  if (MonetshExecute_defined (command, NULL, names, values, False)) {
    MONETDEBUG(2, ( "input: not closed, still to read"));
    return True;
  }
  MONETDEBUG(2, ( "input: closing connection, returned false"));
  return False;
}

/**************************************************************************
  Channel disconnected
  **************************************************************************/

static void serv_disconnect (channel)
     Channel channel;
{
  char * command  = (char *) Channel_data (channel);
  char * names[4], * values[4];

  names[0] = "fd-channel";
  values[0] = Monetint2String (Channel2fid (channel));
  names[1] = "accept-channel";
  values[1] = NULSTR;
  names[2] = "close-channel";
  values[2] = "";
  names[3] = NULSTR;
  values[3] = NULSTR;

  (void) MonetshExecute_defined (command, NULL, names, values, False);

}

/***************************************************************************/
/*  Handles a new connection MonetRequest from a client  */
/***************************************************************************/

static void HandleAccept (fid, address, data)
     int fid;
     char * address;
     caddr_t data;
{
  char * names[4], * values[4];
  Channel channel;
  char * command = (caddr_t) data;

  /* -- Create a channel  */
  channel = Channel_create (fid, False, input,
			    serv_disconnect, (caddr_t) command);

  /* -- Call 'command' indicating that this is a new connection  */

  names[0] = "fd-channel";
  values[0] = Monetint2String (fid);
  names[1] = "accept-channel";
  values[1] = address;
  names[2] = NULSTR;
  values[2] = NULSTR;

  if (MonetshExecute_defined (command, NULL, names, values, False)) {
    /* -- Okay channel accepted  */
  } else {
    /* -- Reject connection  */
    Channel_shutdown (channel);
  }
}

static void HandleDisconnect (fid, data)
     int fid;
     caddr_t data;
{
  MonetFree ((char *) data);
}

/********************************************************************/
/*  Create server side */
/********************************************************************/

MONETCALLBACK(do_server)
     int port;

     port = MonetString2int (argv[1]);
     if (port <= 0) {
       MonetPrintfError ("Port number is must be greater than 0 [%d]", port);
       result = NULSTR;
     } else {
       int fid;

       fid = Channel_create_server(port,
				   HandleAccept,
				   HandleDisconnect,
				   MonetNewString (argv[2]));
       if (fid >= 0)
	 result = Monetint2String (fid);
       else
	 result = NULSTR;
     }
MONETEND

/***************************************************************************/
/*  Called when channel disconnects  */
/***************************************************************************/

static void disconnect (channel)
     Channel channel;
{
  char * command  = (char *) Channel_data (channel);
  serv_disconnect (channel);
  MonetFree (command);
}

/********************************************************************/
/*  Create client side */
/********************************************************************/

MONETCALLBACK(do_client)
     int port;
     Channel channel;

     if (! argv[1]) {
       MonetPrintfError ("Address is null");
       result = NULSTR;
     } else if ((port = MonetString2int (argv[2])) <= 0) {
       MonetPrintfError ("Port number is must be greater than 0 [%d]", port);
     } else if (channel = Channel_create_client
		(port, argv [1],
		 False,
		 input,
		 disconnect,
		 (caddr_t) MonetNewString (argv[3]))) {
       result = Monetint2String (Channel2fid (channel));
     } else {
       MonetPrintfError ("Could not connect channel [%s+%d]", argv[1], port);
       result = NULSTR;
     }
MONETEND

/********************************************************************/
/*  Shutdown  */
/********************************************************************/

MONETCALLBACK(do_shutdown)
     int fd;
     Channel channel;

     fd = MonetString2int (argv[1]);

     if (channel = fid2Channel (fd)) {
       Channel_shutdown (channel);
     } else {
       MonetPrintfError ("No channel connected to fd [%d]", fd);
     }
MONETEND

/********************************************************************/
/*  Help  */
/********************************************************************/

MONETCALLBACK(do_help)
     Channel_help ();
MONETEND

/********************************************************************/
/*  Init and finish  */
/********************************************************************/

extern void monetsh_init_channel ()
{
  (void) MonetMember_module_class ("channel");

  MonetMember_create ( "server",		do_server,
		      "port command",
		      "Creates a socket listening on a \
on 'port', when a line of data is read in 'command' is executed with the \
variable 'data-channel' set to the data.  If a new connection the variable \
'accept-channel' will be set to address of other side, when closed the \
variable 'close-channel' will be set true.  'fd-channel' will be set to the \
descriptor for the channels socket.  \
If the command fails the channel will be closed.");

  MonetMember_create ( "client",		do_client,
		      "address port command",
		      "Creates a channel connected to 'address' listening \
on 'port', when a line of data is read in 'command' is executed with the \
variable 'data-channel' set to the data.  If a new connection the variable \
'accept-channel' will be set to address of other side, when closed the \
variable 'close-channel' will be set true.  'fd-channel' will be set to the \
descriptor for the channels socket.  \
If the command fails the channel will be closed.");

  MonetMember_create ( "write",		do_write,
		      "fd message ...",
		      "Writes a message to channel connected to \
descriptor 'fd', this is done asyncronously but in order, so messages \
are sent later but the command does not block.  The arguments are \
concatinated but divided by spaces before being sent.");

  MonetMember_create ( "writeln",		do_writeln,
		      "fd message ...",
		      "Same as 'write' but adds a newline at the end, \
newline is the end of record mark so use this when communicating with a \
remote server channel.");

  MonetMember_create ( "shutdown",		do_shutdown,
		      "fd",
		      "Closes down channel connected to descriptor 'fd' \
but only after clearing its write buffer.  You shouldn't need to use \
this with channels created by 'server' and rarely with 'client' channels \
since the 'command' registered with these channels can close the channel \
simply by returning $false.");

  MonetMember_create ( "help",		do_help,
		      "",
		      "List all the channels and there status");

}

extern void monetsh_finish_channel ()
{
}

/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/********************************************************************/


