
/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*  Defines some basic functions for monet script  */
/*****************************************************************************/

#include "stdP.h"
#include <monet/msh/Context.h>

/***************************************************************************/
/*  Statement variable  */
/***************************************************************************/

SCRIPTENVIRONMENT(do_get_var)
     String name;
     MonetStr id;
     MonetMember member;
     MonetEntity qualifier;

     name = MonetTree_execute (arg_entity, arg, NULL);

     if (!(id = String2MonetStr (name))) {
       MonetParseError ("No variables with name [%s]", name);
     } else if (MonetStrlen (name) == 0) {
       MonetParseError ("Empty variable name [%s]", name);
     } else if (member = MonetEntity_find_member
		(arg_entity, id, &qualifier)) {
       result = MonetMember_execute
	 (qualifier, member, arg_entity, id, arg->next);
     } else {
       MonetParseError ("Don't recognise variable [%s]", name);
       result = NULL;
     }
MONETEND

/***************************************************************************/
/*  Statement sequences  */
/***************************************************************************/

SCRIPTENVIRONMENT(do_braces)
    if (arg) {
      MonetMember_deeper ();
      result = MonetTree_execute (arg_entity, arg, NULL);
      MonetMember_shallower ();
    }
MONETEND

SCRIPTENVIRONMENT(do_parenthesis)
     if (arg) {
       result = MonetTree_execute (arg_entity, arg, NULL);
     }
MONETEND

/***************************************************************************/
/*  Loops  */
/***************************************************************************/

MONETCALLBACK(do_break)
     MonetException_raise (MONETSH_BREAK);
MONETEND

MONETCALLBACK(do_continue)
     MonetException_raise (MONETSH_CONTINUE);
MONETEND

MONETCALLBACK(do_return)
     result = MonetJoinStrings (argc-1, argv+1, NULL);
     MonetException_raise (MONETSH_RETURN);
MONETEND

MONETCALLBACK(do_exit)
     MonetException_raise (MONETSH_EXIT);
MONETEND

MONETCALLBACK(do_halt)
     result = MonetJoinStrings (argc-1, argv+1, NULL);
     MonetException_raise (MONETSH_HALT);
MONETEND

MONETCALLBACK(do_raise)
     int num;
     num = MonetString2int (argv[1]);
     if (num >= MONETSH_MIN_EXCEPTION && num < MONETSH_MAX_EXCEPTION) {
       MonetException_register (num, argv[2]);
       MonetException_raise_message (num,
				     MonetJoinStrings (argc-3, argv+3, NULL));
     } else {
       MonetPrintfError ("raise: must be between %d and %d",
			 MONETSH_MIN_EXCEPTION, MONETSH_MAX_EXCEPTION);
     }
MONETEND

SCRIPTENVIRONMENT(do_catch)
     char * pattern;
     pattern = MonetTree_execute (arg_entity, arg, NULL);
     (void) MonetTree_execute (arg_entity, arg, NULL);
     if (MonetException_code () == MONETSH_OKAY) {
       result = NULL;
     } else {
       static char buf[1024];
       (void) strcpy (result = buf, MonetException_name ());
       if (! MonetMatchWildcard (pattern, MonetException_name ())) {
	 MonetException_raise (MONETSH_OKAY);
       }
     }
MONETEND

SCRIPTENVIRONMENT(do_while)
     MonetMember_deeper ();
     while (Script_true (MonetTree_execute (arg_entity, arg, NULL))) {
       (void) MonetTree_execute (arg_entity, arg->next, NULL);
       switch (MonetException_code ()) {
       case MONETSH_OKAY:
	 break;
       case MONETSH_CONTINUE:
	 MonetException_raise (MONETSH_OKAY);
	 break;
       case MONETSH_BREAK:
	 MonetException_raise (MONETSH_OKAY);
       default:
	 goto out;
       }
     }
   out:
       MonetMember_shallower ();
MONETEND

SCRIPTENVIRONMENT(do_until)
     MonetMember_deeper ();
     while (Script_false (MonetTree_execute (arg_entity, arg, NULL))) {
       (void) MonetTree_execute (arg_entity, arg->next, NULL);
       switch (MonetException_code ()) {
       case MONETSH_OKAY:
	 break;
       case MONETSH_CONTINUE:
	 MonetException_raise (MONETSH_OKAY);
	 break;
       case MONETSH_BREAK:
	 MonetException_raise (MONETSH_OKAY);
       default:
	 goto out;
       }
     }
   out:
     MonetMember_shallower ();
MONETEND

MONETCALLBACK(do_foreach)
     if (argv[1] && *argv[1]) {
       int i;
       for (i=2; i < argc - 1; ++i) {
	 MonetMember_deeper ();
	 MonetMember_create_local (argv[1], argv[i]);
	 result = MonetshEntityExecute (entity, argv[argc-1], NULL, False);
	 MonetMember_shallower ();
	 switch (MonetException_code ()) {
	 case MONETSH_OKAY:
	   break;
	 case MONETSH_CONTINUE:
	   MonetException_raise (MONETSH_OKAY);
	   break;
	 case MONETSH_BREAK:
	   MonetException_raise (MONETSH_OKAY);
	 default:
	   RETURNSC;
	 }
       }
     }
MONETEND

/***************************************************************************/
/*  if/else  */
/***************************************************************************/

static MonetBool IfResult, IfSet;

SCRIPTENVIRONMENT(do_if)
     MonetBool succeeded;
     MonetMember_deeper ();
     if (succeeded =
	 Script_true (MonetTree_execute (arg_entity, arg, NULL))) {
       result = MonetTree_execute (arg_entity, arg->next, NULL);
     } else {
       result = NULL;
     }
     IfSet = True;
     IfResult = succeeded;
     MonetMember_shallower ();
MONETEND

SCRIPTENVIRONMENT(do_else_if)
     if (! IfSet) {
       MonetPrintfError ("If statement must come before [elseif]");
     }
     MonetMember_deeper ();
     if (IfResult == False) {
       MonetBool succeeded;
       if (succeeded =
	   Script_true (MonetTree_execute (arg_entity, arg, NULL))) {
	 result = MonetTree_execute (arg_entity, arg->next, NULL);
       }
       IfResult = succeeded;
     } else {
       result = NULL;
     }
     IfSet = True;
     MonetMember_shallower ();
MONETEND

SCRIPTENVIRONMENT(do_else)
     if (! IfSet) {
       MonetPrintfError ("If statement must come before [else]");
     }
     if (arg && arg->type == lex_string && arg->id == String2MonetStr ("if")) {
       MonetPrintfWarning ("Should use 'elseif' not 'else if'");
     }
     MonetMember_deeper ();
     if (IfResult == False) {
       result = MonetTree_execute (arg_entity, arg, NULL);
     } else {
       result = NULL;
     }
     MonetMember_shallower ();
     IfSet = False;
MONETEND

MONETCALLBACK(do_quit)
      Monet_quit (0);
MONETEND

/***************************************************************************/
/*  Source form file  */
/***************************************************************************/

MONETCALLBACK(do_source)
     if (Script_source_file (argv[1], ".msh", "Cannot source file")) {
       if (MonetException_code () == MONETSH_EXIT) {
	 MonetException_raise (MONETSH_OKAY);
       }
     } else {
       result = NULL;
     }
MONETEND

/***************************************************************************/
/*  Execute argument as a script command  */
/***************************************************************************/

MONETCALLBACK(do_eval)
     char * command;
     command = MonetJoinStrings (argc-1, argv+1, " ");
     result = MonetshEntityExecute (entity, command, (char **) NULL, False);
MONETEND

MONETCALLBACK(do_context_eval)
     (void) MonetContext_eval (argv[1]);
MONETEND

/***************************************************************/
/*  Add and remove from the use stack  */
/***************************************************************/

MONETCALLBACK(do_use)
     int i;
     MonetEntity e;
     for (i = 1; i < argc; ++i) {
       if (e = String2MonetEntity (argv[i])) {
	 MonetContext_use (e);
       } else {
	 MonetPrintfError ("No such entity [%s]", argv[i]);
       }
     }
MONETEND

MONETCALLBACK(do_unuse)
     int i;
     MonetEntity e;
     for (i = 1; i < argc; ++i) {
       if (e = String2MonetEntity (argv[i])) {
	 MonetContext_unuse (e);
       } else {
	 MonetPrintfError ("No such entity [%s]", argv[i]);
       }
     }
MONETEND

/***************************************************************************/
/*  Check subclass  */
/***************************************************************************/

MONETCALLBACK (do_is_subclass)
     MonetEntity e;

     if (e = String2MonetEntity (argv[1])) {
       MonetEntity superclass;
       if (! MonetEntity_isClass (e)) {
	 e = MonetEntity_classOf (e);
       }
       if (superclass = String2MonetEntity (argv[2])) {
	 if (! MonetEntity_isClass (superclass)) {
	   superclass = MonetEntity_classOf (superclass);
	 }
	 if (MonetEntity_subclass (superclass, e)) {
	   RETURNSC;
	 }
       } else {
	 MonetPrintfError ("[%s] does not exist", argv[2]);
       }
     } else {
       MonetPrintfError ("[%s] does not exist", argv[1]);
     }
     result = NULL;
MONETEND

/***************************************************************************/
/* Tracing */
/***************************************************************************/

MONETCALLBACK(do_trace)
     if (argc > 1) {
       MonetContext_set_trace (argv[1]);
     }
     result = MonetContext_get_trace ();
MONETEND

/*****************************************************************************/
/*  True and false  */
/*****************************************************************************/

MONETCALLBACK(do_false)
     result = NULL;
MONETEND

MONETCALLBACK(do_true)
     result = "";
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_control ()
{
  (void) MonetMember_module_class ("control");

  MonetMember_create ("trace",		do_trace,
		      "?pattern",
		      "Traces execution of any functions matching 'pattern'. \
If no argument given switches tracing off");
  MonetMember_create ( "continue", 	do_continue,
		      "",
	    "Proceed from begining of loop, raises the 'continue' exception");
  MonetMember_create ( "break", 	do_break,
		      "",
	"Break execution of current loop, raises the 'break' exception");
  MonetMember_create ( "exit", 	do_exit,
		      "",
	"Closes the current input stream, exits 'source', \
raises the 'exit' exception");
  MonetMember_create ( "halt", 	do_halt,
		      "?reason ...",
		      "Break program execution, raises the 'halt' exception");
  MonetMember_create ( "return", 	do_return,
		      "?value",
		      "Returns a value from currently executing procedure \
		      raises the 'return' exception");
  MonetMember_create ( "raise",	 do_raise,
		      "num name ?message ...",
		      "Raises an exception, an error usually");
  MonetMember_create_environment ( "while",	do_while,
				  "condition ?sequence" ,
				  "Repeat a sequence until condition fails");
  MonetMember_create_environment
    ("catch",	do_catch,
     "pattern sequence",
     "Catches any exceptions matching pattern, \
     returns the exception name, or false if okay"
     );
  MonetMember_create_environment ( "until",	do_until,
				  "condition ?sequence" ,
				 "Repeat a sequence until condition succeeds");
  MonetMember_create ( "foreach",	do_foreach,
		      "var  ..." ,
	"Repeat for every item in list, 'var' is set to the item name");
  MonetMember_create_environment ( "if",	do_if,
				  "condition statement",
				  "");
  MonetMember_create_environment ( "elseif",	do_else_if,
				  "condition statement",
				  "");
  MonetMember_create_environment ( "else",	do_else,
				  "statement",
				  "");
  MonetMember_create ("use",		do_use,
		      "entity ...",
		      "Put entities on the use stack");
  MonetMember_create ("unuse",		do_unuse,
		      "entity ...",
		      "Remove entities from the use stack");
  MonetMember_create ( "subclass",  do_is_subclass,
		      "entity class",
		      "Returns true if <entity> is a subclass of <class>, \
		      if either is an instance then will use their class");
  MonetMember_create ( "quit",		do_quit,
		      "", "quit");
  MonetMember_create ( "source",		do_source,
		      "filename",
		      "commands from a file");
  MonetMember_create ( "eval",		do_eval,
		      "string ..." ,
		      "Execute its arguments as a script command");
  MonetMember_create ( "context-eval",		do_context_eval,
		      "string" ,
		      "Execute its argument as script command. \
Inserts command in history and alters local vars");

  MonetMember_create ( "false",		do_false,
		      "",
		      "returns false");
  MonetMember_create ( "true",		do_true,
		      "",
		      "returns true");

  MonetMember_module_class ("Top");
  MonetMember_create_environment ( "(", do_parenthesis,
				  "?sequence",
				  "Execute sequence");
  MonetMember_create_environment ( "{", do_braces,
				  "?sequence",
				  "Execute sequence");
  MonetMember_create_environment ( "$", do_get_var,
				  "string",
				  "Treats string as an identifier");

}

extern void monetsh_finish_control ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
