/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************
  This module implements procedures for providing interactive help and
  automatic documentation.
  ****************************************************************************/

#include "stdP.h"
#include <monet/msh/StrFifo.h>
#include <monet/msh/Context.h>
#include <monet/msh/Entity.h>

/***************************************************************************/
/*  Print table of classes  */
/***************************************************************************/

static void list_class(entity_class, indent, tab)
     MonetEntity          entity_class;
     int indent;
     char * tab;
{
  if (entity_class) {
    MonetEntity c;
    int i;
    for (i = indent; i > 0; -- i) {
      MonetPrintf ("%s", tab);
    }
    MonetPrintfnl ("%s", MonetEntity_title (entity_class));
    FOR_ENTITYLIST(c, MonetEntity_subclasses(entity_class)) {
      list_class(c, indent+1, tab);
    } END_FORLIST;
  }
}

MONETCALLBACK(do_class_tree)
     MonetEntity entity_class;
     char * tab;
     if (entity_class = String2MonetEntity (argv[1])) {
       if (!MonetEntity_isClass (entity_class)) {
	 entity_class = MonetEntity_classOf (entity_class);
       }
     } else {
       entity_class = MonetEntityTopClass ();
     }
     if (argc == 3) {
       tab = argv[2];
     } else {
       tab = "     ";
     }
     list_class (entity_class, 0, tab);
MONETEND

/***************************************************************/
/*  List declared names  */
/***************************************************************/

static int MonetContext_help (class_pattern, pattern)
     String pattern, class_pattern;
{
  MonetEntity entity;
  unsigned found = 0;
  MonetEntityStack use, local;
  char * title;

  local = MonetLocalStack ();
  use   = MonetUseStack   ();
  MonetPrintfnl ("-- Local Stack");
  for (entity = MonetEntityStack_top (local); entity;
       entity = MonetEntityStack_next (local)) {
    title = MonetEntity_title (entity);
    found += MonetEntity_help_members (entity, class_pattern, pattern,
				       title, NULL);
  }
  MonetPrintfnl ("-- Use Stack");
  for (entity = MonetEntityStack_top (use); entity;
       entity = MonetEntityStack_next (use)) {
    title = MonetEntity_title (entity);
    found += MonetEntity_help_members (entity, class_pattern, pattern,
				       title, NULL);
  }
  if (found == 0 && (entity = String2MonetEntity (class_pattern))) {
    title = MonetEntity_title (entity);
    found = MonetEntity_help_members (entity, "*", pattern, title, NULL);
  }
  return found;
}

/***************************************************************/
/*  Help on a specific entity  */
/***************************************************************/

MONETCALLBACK(do_stack)
     int found;

     char * class_pattern, * pattern;
     if (argc == 3 && argv[1] && argv [2]) {
       class_pattern = argv[1];
       pattern = argv [2];
     } else if (argv [1]) {
       class_pattern = "*";
       pattern = argv [1];
     } else {
       class_pattern = "*";
       pattern = "*";
     }
     found = MonetContext_help (class_pattern, pattern);
     if (found == 0) {
       MonetPrintfInform ("Could not find anything matching [%s -> %s]",
			  class_pattern, pattern);
     } else {
       MonetPrintfInform ("Matched %lu commands", found);
     }
MONETEND

MONETCALLBACK(do_brief)
     MonetEntity e;
     MonetEntityStack use;
     char * class_pattern;

     if (argc == 2 && argv[1]) {
       class_pattern = argv[1];
     } else {
       class_pattern = "*";
     }

     use   = MonetUseStack   ();

     MonetPrintfnl ("Brief list of commands");
     for (e = MonetEntityStack_top (use); e;
	  e = MonetEntityStack_next (use)) {
       (void) MonetEntity_list_members (e, class_pattern);
     }
MONETEND

MONETCALLBACK(do_long)
     MonetEntity e;
     MonetEntityStack use;
     char * class_pattern;

     if (argc == 2 && argv[1]) {
       class_pattern = argv[1];
     } else {
       class_pattern = "*";
     }

     use   = MonetUseStack   ();

     for (e = MonetEntityStack_top (use); e;
	  e = MonetEntityStack_next (use)) {
       char  header [1024];
       (void) sprintf (header, "class %s %s {", MonetEntity_title (e),
		       MonetEntity_title (MonetEntity_classOf (e)));
       (void) MonetEntity_help_members (e, class_pattern, "*", header, "}");
     }
MONETEND

MONETCALLBACK(do_list)
     MonetMember member;
     char * attributes;
     entity = String2MonetEntity (argv[1]);
     if (entity == NULL) {
       MonetPrintfError ("No such entity [%s]", argv[1]);
       RETURNSC;
     }
     if (MonetEntity_isClass (entity)) {
       MonetPrintf ("class ");
     } else {
       MonetPrintf ("entity ");
     }
     MonetPrintfnl ("%s %s {",
		    MonetEntity_title (entity),
		    MonetEntity_title (MonetEntity_classOf (entity)));
     if (attributes = MonetEntity2Report (entity, "", True, False, True)) {
       attributes = MonetString_indent (attributes, ' ', 6);
       MonetPrintfnl ("\tset");
       MonetPrintfnl ("%s", attributes);
     }
     for (member = entity->members; member ; member = MonetMember_next (member)) {
       MonetPrintfnl ("\t %s", MonetMember_definition (member));
     }
     MonetPrintfnl ("}");
MONETEND

MONETCALLBACK(do_class_members)
     MonetMember member;
     entity = String2MonetEntity (argv[1]);
     if (entity == NULL) {
       MonetPrintfError ("No such entity [%s]", argv[1]);
       RETURNSC;
     }
     for (member = entity->members; member ;
	  member = MonetMember_next (member)) {

       MonetPrintfnl (MonetStr2String (MonetMember2Str (member)));
     }
MONETEND

MONETCALLBACK(do_ancestors)
     MonetMember member;
     MonetEntity ent;

     if (ent = String2MonetEntity (argv[1])) {
       MonetEntity e, top;
       top = MonetEntityTopClass ();

       for (e = MonetEntity_classOf(ent);
	    e;
	    e = MonetEntity_classOf(e)) {
	 MonetPrintf (" %s", TITLE(e));
       }
     } else {
       MonetPrintfError ("No such entity [%s]", argv[1]);
     }
MONETEND

/*****************************************************************************/
/*  List defined types and attributes  */
/*****************************************************************************/

MONETCALLBACK(do_attributes)
     MonetAttribute_report ();
MONETEND

MONETCALLBACK(do_types)
     MonetType_report ();
MONETEND

MONETCALLBACK(do_ordset)
     MonetOrdSet_help ();
MONETEND

MONETCALLBACK(do_tree)
     MonetTree_help ();
MONETEND

MONETCALLBACK(do_system)
     MonetSystem_help ();
MONETEND

MONETCALLBACK(do_string)
     MonetStr_help ();
MONETEND

MONETCALLBACK(do_global_table)
     MonetStr_dump ();
MONETEND

MONETCALLBACK(do_entity)
     MonetEntity_dump ();
MONETEND

MONETCALLBACK(do_fifo)
     MonetStrFifo_help ();
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_help ()
{
  (void) MonetMember_module_class ("help");
  MonetMember_create ("class-tree",  do_class_tree,
		      "?root ?tab-string", 
		      "Prints class heirarchy, default root is Top, \
each level of the tree is indented with <tab-string>.");
  MonetMember_create ("stack",  do_stack,
		      "?class ?command",
		      "General information on entities and functions");
  MonetMember_create ("brief",  do_brief,
		      "?class",
		      "General information on commands on use stack");
  MonetMember_create ("long",  do_long,
		      "?class",
		      "General information on commands on use stack");
  MonetMember_create ("list",  do_list,
		      "entity",
		      "List definitions of everything defined");
  MonetMember_create ("class-members",  do_class_members,
		      "entity",
		      "List names of all class members of entity");
  MonetMember_create ( "attributes",		do_attributes,
		      "",
		      "list defined attributes");
  MonetMember_create ( "types",		do_types,
		      "",
		      "list defined types");
  MonetMember_create ( "ancestors",		do_ancestors,
		      "entity",
		      "prints all the ancestor classes of an entity");
  (void) MonetMember_module_class ("memory");
  MonetMember_create ("system",		do_system,
		      "",
		      "Internal information about shell and child processes");
  MonetMember_create ("tree",		do_tree,
		      "",
		      "Internal information about parse trees");
  MonetMember_create ("ordset",		do_ordset,
		      "",
		      "Internal information about sets");
  MonetMember_create ("string",		do_string,
		      "",
		      "Internal information about string hash tables");
  MonetMember_create ("table",		do_global_table,
		      "",
		      "Internal information about global string table");
  MonetMember_create ("entity",		do_entity,
		      "",
		      "Internal information about entities");
  MonetMember_create ("fifo",		do_fifo,
		      "",
		      "Internal information about string fifo's");

  /* -- These will be defined in the standard module help.msh  */
  MonetConfig_create ( "-manual", "help->manual ; quit", NULSTR,
		      "produces an nroff formatted manual page and exits");
  MonetConfig_create ( "-reference", "help->reference ; quit", NULSTR,
		     "produces an latex formatted reference manual and exits");
}

extern void monetsh_finish_help ()
{
  MonetMember_shallower ();
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
