/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/*  Internet/Ip address conversions and lookups  */
/********************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include <monet/probe/Probe.h>

#include <sys/errno.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>

static char *GetHostBuf = NULL;

/***************************************************************************/
/*  Host name to address stuff  */
/***************************************************************************/

MONETCALLBACK(do_get_domain_name)
     char * name, * fullname;
     fullname = name = Monet_host_name ();
     while (*name) {
       if (*name == '.') {
	 result = name+1;
	 RETURNSC;
       }
       ++name;
     }
     MonetSysError ("Your local host seems not to have a domain name",
		    fullname);
     result = NULSTR;
MONETEND

/********************************************************************/

/********************************************************************/

MONETCALLBACK(do_get_host_name)
     struct hostent  *host;
     String title = argv[1];

     result = NULL;
     if (! title || ! *title) {
       RETURNSC;
     }
     if (isdigit (*title)) {
       unsigned long num;
       num = inet_addr (title);
       host = gethostbyaddr ((char *) &num, sizeof (unsigned long), AF_INET);
     } else {
       host = gethostbyname (title);
     }
     if (host == NULL) {
       if (h_errno == HOST_NOT_FOUND) {
	 /* MonetScriptFail ("Host not found: [%s]", title); */
	 RETURNSC;
       }
       herror ("");
       MonetScriptFail ("Error");
       RETURNSC;
     } else {
       result = host->h_name;
     }
     if (result) {
       MonetStringToLowerCase (result);
     }
MONETEND

MONETCALLBACK(do_get_host)
     struct hostent  *host;
     String title = argv[1];

     result = NULL;
     if (! title || ! *title) {
       RETURNSC;
     }
     if (isdigit (*title)) {
       unsigned long num;
       num = inet_addr (title);
       host = gethostbyaddr ((char *) &num,
				   sizeof (unsigned long), AF_INET);
     } else {
       host = gethostbyname (title);
     }
     if (host == NULL) {
       if (h_errno == HOST_NOT_FOUND) {
	 MonetScriptFail ("Host not found: [%s]", title);
	 RETURNSC;
       }
       herror ("");
       RETURNSC;
     } else {
       char ** cp;
       String f;

       if (GetHostBuf == NULL) {
	 /*!!!  Should use an appropriate space */
	 GetHostBuf = MonetMalloc (2048);
       }
       f = GetHostBuf;
       f = MonetStringCopyToEnd (f, host->h_name);
       if (host->h_addrtype != AF_INET || host->h_length != 4) {
	 MonetScriptFail ("Bad Address type [%d] length [%d]\n",
			  host -> h_addrtype, host-> h_length);
	 MonetFree (GetHostBuf);
	 GetHostBuf = NULL;
	 RETURNSC;
       }
       for (cp = host->h_addr_list; *cp; ++cp) {
	 *f = '\n';
	 ++f;
	 f = MonetStringCopyToEnd (f, inet_ntoa(*(struct in_addr *)*cp));
       }
       *f = NULL; ++f;
       result = GetHostBuf;
       MonetStringToLowerCase (result);
     }
MONETEND


static struct netent * get_net (title)
     char * title;
{
  struct netent  *net;

  if (title && *title) {
    if (isdigit (*title)) {
      unsigned long num;
      char name [17], *np = name, *c;
      for (c = title; *c && np - name < sizeof (name) -1; ++c) {
	if (isdigit (*c)) {
	  *np = *c;
	  ++np;
	} else if (*c == '.') {
	  do {
	    ++c;
	  } while (*c == '0');
	  if (isdigit(*c)) {
	    *np = '.';
	    ++np;
	    *np = *c;
	    ++np;
	  } else {
	    break;
	  }
	} else {
	  break;
	}
      }
      *np = NULL;
      num = inet_network (name);
      net = getnetbyaddr (num, AF_INET);
    } else {
      net = getnetbyname (title);
    }
    if (net) {
      return net;
    }
  }
  return NULL;
}

MONETCALLBACK(do_get_network_name)
     struct netent  *net;

     if (net = get_net (argv[1])) {
       result = net->n_name;
       MonetStringToLowerCase (result);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_get_network_address)
     struct netent  *net;

     if (net = get_net (argv[1])) {
       result = inet_ntoa(inet_makeaddr(net->n_net, 0));
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_in_addr_arpa)
     if (argv[1]) {
       int a[4], n;
       static char buf [48];
       char * cp;
       if (n = sscanf (argv[1], "%u.%u.%u.%u", &a[0], &a[1], &a[2], &a[3])) {
	 for (cp = buf, --n; n>=0; --n) {
	   if (a[n] > 0) {
	     (void) sprintf (cp, "%u.", a[n]);
	   cp += strlen (cp);
	   }
	 }
	 (void) strcpy (cp, "in-addr.arpa");
	 result = buf;
       } else {
	 result = NULL;
       }
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_reverse_address)
     result = NULL;

     if (argv[1]) {
       int a[4], n;

       if (n = sscanf (argv[1], "%u.%u.%u.%u", &a[0], &a[1], &a[2], &a[3])) {
	 static char buf [48];
	 char * cp;
	 for (cp = buf, --n; n>0; --n) {
	   (void) sprintf (cp, "%u.", a[n]);
	   cp += strlen (cp);
	 }
	 (void) sprintf (cp, "%u", a[0]);
	 result = buf;
       }
     }
MONETEND

/********************************************************************/

/********************************************************************/

MONETCALLBACK(do_netof)
     struct in_addr in;
     if (argv[1]) {
       in.s_addr = inet_addr(argv[1]);
       result = inet_ntoa(in);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_long_netof)
     struct in_addr in;
     if (argv[1]) {
       in.s_addr = inet_addr(argv[1]);
       in = inet_makeaddr(inet_netof(in), 0);
       result = inet_ntoa(in);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_lnaof)
     struct in_addr in;
     if (argv[1]) {
       in.s_addr = inet_addr(argv[1]);
       result = Monetint2String (inet_lnaof(in));
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_dotted_lnaof)
     struct in_addr in;
     if (argv[1]) {
       long  a;
       static char buf [32];
       in.s_addr = inet_addr(argv[1]);
       a = htons (inet_lnaof(in));
       if (a < 256) {
	 (void) sprintf (buf, "%u", a & 255);
       } else if (a < 256 * 256) {
	 (void) sprintf (buf, "%u.%u", (a & 0xff00) >> 8, a & 0xff);
       } else {
	 (void) sprintf (buf, "%u.%u.%u",
			 (a & 0xff0000) >> 16, (a & 0xff00) >> 8, a & 0xff);
       }
       result = buf;
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_dotted_netof)
     struct in_addr in;
     if (argv[1]) {
       long  a;
       static char buf [32];
       in.s_addr = inet_addr(argv[1]);
       a = htons (inet_netof(in));
       if (a < 256) {
	 (void) sprintf (buf, "%u", a & 255);
       } else if (a < 256 * 256) {
	 (void) sprintf (buf, "%u.%u", (a & 0xff00) >> 8, a & 0xff);
       } else {
	 (void) sprintf (buf, "%u.%u.%u",
			 (a & 0xff0000) >> 16, (a & 0xff00) >> 8, a & 0xff);
       }
       result = buf;
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_makeaddr)
     struct in_addr in;
     in = inet_makeaddr(MonetString2int (argv[1]), MonetString2int (argv[2]));
     result = inet_ntoa(in);
MONETEND

/********************************************************************/
/*  Init and finish  */
/********************************************************************/

extern void monetsh_init_inet ()
{
  GetHostBuf = NULL;
  MonetMember_module_class ("inet");
  MonetMember_create ("get-domain",  do_get_domain_name,
		      "",
		      "Returns the domain name for the local host");
  MonetMember_create ("get-host",  do_get_host,
		      "hostname|address",
   "Returns information from unix 'gethostbyname' or 'gethostbyaddr'");
  MonetMember_create ("get-in-addr-arpa",  do_in_addr_arpa,
		      "address",
		      "Given an ip address gives the reverse mapping.");
  MonetMember_create ("reverse-address",  do_reverse_address,
		      "address",
		      "Returns address a.b.c.d in form d.c.b.a");
  MonetMember_create ("get-network-name",  do_get_network_name,
		      "address",
		      "Returns information from unix 'getnetbyaddr' \
		      given the address returns the name.");
  MonetMember_create ("get-network-address",  do_get_network_address,
		      "name",
		      "Returns information from unix 'getnetbyname' \
given the name, returns the address");
  MonetMember_create ("get-host-name",  do_get_host_name,
		      "hostname|address",
		      "Returns official hostname using unix 'gethostbyname' \
or 'gethostbyaddr'");
  MonetMember_create ("netof", do_netof,
		      "address",
		      "Returns the IP network part of 'address'. \
See unix inet_netof(3N)");
  MonetMember_create ("long-netof", do_long_netof,
		      "address",
		      "Returns the IP network address in as a dotted quad.\
See unix inet_netof(3N)");
  MonetMember_create ("lnaof", do_lnaof,
		      "address",
     "Returns the IP local part of 'address'. See unix inet_lnaof(3N)");
  MonetMember_create ("dotted-lnaof", do_dotted_lnaof,
		      "address",
		      "Returns IP local part of 'address' in dotted format. \
See unix inet_lnaof(3N)");
  MonetMember_create ("dotted-netof", do_dotted_netof,
		      "address",
		      "Returns IP network part of 'address' in dotted format. \
See unix inet_netof(3N)");
  MonetMember_create ("makeaddr", do_makeaddr,
		      "net local",
		      "Returns an Ip Address. See unix inet_makeaddr(3N)");
}

extern void monetsh_finish_inet ()
{
  MonetFree (GetHostBuf);
  GetHostBuf = NULL;
}

/********************************************************************/
/********************************************************************/
/* $Header$ */
/********************************************************************/
/********************************************************************/


