/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/

#include "stdP.h"
#include <monet/msh/Channel.h>
#include <monet/msh/Context.h>
#include <monet/msh/System.h>

static String Script_redirect_in PR((MonetEntity entity,
				     MonetTree left, String filename));
static String Script_redirect_out PR((MonetEntity entity,
				      MonetTree left, String filename,
				      String mode));

static MonetBool DaemonMode = False;

/***************************************************************************/
/*  Become a daemon  */
/***************************************************************************/

MONETCALLBACK(do_daemon)
     if (argc == 2) {
       MonetBecomeDaemon (True);
     } else {
       MonetBecomeDaemon (False);
     }
MONETEND

/***************************************************************************/
/*  Read record from input  */
/***************************************************************************/

MONETCALLBACK(do_get_line)
     static char buffer [2048]; /* !!! */
     if (result = fgets (buffer, sizeof(buffer), MonetContext_in ())) {
       int len;
       if (len = strlen (buffer))
	 if (buffer[len-1] == '\n') {
	   buffer[len -1] = '\0';
	 }
     }
MONETEND

MONETCALLBACK(do_fgets)
     static char buffer [2048]; /* !!! */
     result = fgets (buffer, sizeof(buffer), MonetContext_in ());
MONETEND

#ifdef dsdsd
/***************************************************************************/
/*  Open and close a file  */
/***************************************************************************/

MONETCALLBACK(do_fopen)
     FILE * f;
     if (argv[1] && argv[2]) {
       if (f = fopen (argv[1] && argv[2])) {
	 result = Monetlong2String (fileno(f));
	 RETURNSC;
       }
     }
     result = NULL;
MONETEND
/***************************************************************************/
/*  Open and close a file  */
/***************************************************************************/

MONETCALLBACK(do_fclose)
/*     FILE * f; */
MONETEND
#endif

/*****************************************************************************/
/* Execute a shell command using my implementation */
/*****************************************************************************/

MONETCALLBACK(do_monet_sh)
     char * arg [4];

     arg[0] = "/bin/sh";
     arg[1] = "-c";
     arg[2] = MonetJoinStrings (argc-1, argv+1, " ");
     arg[3] = NULL;

     if (!arg[2]) {
       MonetPrintfError ("[%s]: null command", argv[0]);
       result = "127";
     } else {
       static char buf [16];
       int exit;

       exit = MonetSystem (arg, -1, arg[2]);
       (void) sprintf (buf, "%d", exit);
       result = buf;
     }
MONETEND

/*****************************************************************************/
/* Execute a shell command using my implementation */
/*****************************************************************************/

MONETCALLBACK(do_forkexec)
     if (!argv[0]) {
       MonetPrintfError ("[%s]: null command", argv[0]);
       result = "127";
     } else {
       static char buf [16];
       int exit, in_number;
       FILE * in;

       if (in = MonetContext_in ()) {
	 in_number = fileno (in);
       } else {
	 in_number = 0;
       }

       exit = MonetSystem (argv+1, in_number, argv[1]);
       (void) sprintf (buf, "%d", exit);
       result = buf;
     }
MONETEND

/*****************************************************************************/
/* Execute a shell command using my implementation */
/*****************************************************************************/

MONETCALLBACK(do_monet_nul_in_sh)
     char * arg [4];

     arg[0] = "/bin/sh";
     arg[1] = "-c";
     arg[2] = MonetJoinStrings (argc-1, argv+1, " ");
     arg[3] = NULL;

     if (!arg[2]) {
       MonetPrintfError ("MonetAsyncSystem: null command");
       result = "127";
     } else {
       static char buf [16];
       int exit;

       exit = MonetSystem (arg, -1, arg[2]);
       (void) sprintf (buf, "%d", exit);
       result = buf;
     }
MONETEND

/*****************************************************************************/
/* Execute a shell command in the background  */
/*****************************************************************************/

MONETCALLBACK(do_background_sh)
     char * arg [4];

     arg[0] = "/bin/sh";
     arg[1] = "-c";
     arg[2] = MonetJoinStrings (argc-1, argv+1, " ");
     arg[3] = NULL;

     if (!arg[2]) {
       MonetPrintfError ("[%s]: null command", argv[0]);
       result = "127";
     } else {
       if (MonetAsyncSystem ((char *) argv+1, arg)) {
       } else {
	 result = NULSTR;
       }
     }
MONETEND

/***************************************************************************/
/*  Redirect output to a file, can write on top of old one or append to it */
/***************************************************************************/

static String Script_redirect_out (entity, left,filename, mode)
     MonetTree left;
     String filename;
     String mode;
     MonetEntity entity;
{
  String value;
  FILE * f;

  if (! filename || *filename == NULL) {
    MonetScriptFail ("File name is null");
    value = NULL;
  } else if (EQS(filename, "/dev/stdout")) {
    value = MonetTree_execute (entity, left, NULL);
/*  } else if (EQS(filename, "/dev/stderr")) {
    FILE * saved;
    MONETDEBUG(10, ( "Redirect to file <stderr>, mode [%s]", filename, mode));
    saved = MonetContext_set_out_stderr ();
    value = MonetTree_execute (entity, left, NULL);
    fclose (f);
    (void) MonetContext_set_out (saved);
*/
  } else if (f = MonetFopenFileWithExtension
	     (filename, NULL, "Cannot open", mode)) {
    FILE * saved;
    MONETDEBUG(10, ( "Redirect to file [%s], mode [%s]", filename, mode));
    saved = MonetContext_set_out (f);
    value = MonetTree_execute (entity, left, NULL);
    fclose (f);
    (void) MonetContext_set_out (saved);
  } else {
    MonetScriptFail ("Could not open file");
    value = NULL;
  }
  return value;
}

/***************************************************************************/
/*  Redirect input from a file */
/***************************************************************************/

static String Script_redirect_in (entity, left,filename)
     MonetTree left;
     MonetEntity entity;
     String filename;
{
  String value;
  FILE * file, * previous;

  if (! filename || *filename == NULL) {
    MonetPrintfError ("File name is null");
    value = NULL;
  } else if (file = MonetFopenFileWithExtension (filename, NULL,
						   "Cannot open", "r")) {
    previous = MonetContext_set_in (file);
    MONETDEBUG(10, ("Redirect from file [%s]", filename));
    value = MonetTree_execute (entity, left, NULL);
    fclose (file);
    (void) MonetContext_set_in (previous);
  } else {
    MonetPrintfError ("Could not open file [%s]", filename);
    value = NULL;
  }
  return value;
}

/***************************************************************************/
/*  pipe and redirect */
/***************************************************************************/

SCRIPTENVIRONMENT(do_pipe)
     char file [32];
     (void) sprintf (file, "/tmp/monetshpipe_XXXXXX");
     (void) mktemp (file);
     result = Script_redirect_out (arg_entity, arg, file, "w");
     result = Script_redirect_in (arg_entity, arg->next, file);
     unlink (file);
MONETEND

SCRIPTENVIRONMENT(do_pipe_in)
    result =
       Script_redirect_in (arg_entity, arg,
			   MonetTree_execute (arg_entity,
					      arg->next, (char **)0));
MONETEND

SCRIPTENVIRONMENT(do_pipe_append_out)
    result =
       Script_redirect_out (arg_entity, arg,
			    MonetTree_execute (arg_entity,
					       arg->next, (char **)0),
			    "a");
MONETEND

SCRIPTENVIRONMENT(do_pipe_out)
     result =Script_redirect_out
       (entity, arg, MonetTree_execute (arg_entity,arg->next, (char **)0),
	"w");
MONETEND

/*****************************************************************************/
/*  Write out  */
/*****************************************************************************/

MONETCALLBACK(do_echo)
     int i;
     for (i = 1; i < argc; ++i) {
       unsigned len;
       if (len = MonetStrlen (argv[i]))
	 MonetFwrite (argv[i], len);
     }
     MonetFwrite ("\n", 1);
MONETEND

MONETCALLBACK(do_put)
     int i;
     for (i = 1; i < argc; ++i) {
       unsigned len;
       if (len = MonetStrlen (argv[i]))
	 MonetFwrite (argv[i], len);
     }
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void Script_io_init ()
{
  (void) MonetMember_module_class ("io");

  MonetMember_create ( "echo",		do_echo,
		      "?text ...",
		     "echo to output, with spaces between args and a newline");
  MonetMember_create ( "put",		do_put,
		      "?text ...",
		      "echo to output, no spaces between args, no newline");
  MonetMember_create ( "forkexec",		do_forkexec,
		      "command ?args ...",
      "Forks and execs 'command' directly without calling /bin/sh");
  MonetMember_create ( "sh",		do_monet_sh,
		      "command ?args ...",
"Unix (sh)ell command, if returns 127 then can't execute.  See sh-nul-in");
  MonetMember_create ( "sh-nul-in",		do_monet_nul_in_sh,
		      "command ?args ...",
"Unix (sh)ell command, if returns 127 then can't execute. \
Rredirects input to /dev/null, similar to 'rsh(1)");
  MonetMember_create ( "background-sh",		do_background_sh,
		      "after command ?args ...",
		      "Unix (sh)ell command in the background. \
Executes 'after' when the process is finished");
  MonetMember_create ( "get-line", 	do_get_line,
		      "",
		      "read one line of input with no trailing newline");
  MonetMember_create ( "fgets", 	do_fgets,
		      "",
		     "read one line of input, may be terminated by a newline");
  MonetMember_create_environment ( "pipe|", do_pipe,
				  "left right",
				  "pipe out");
  MonetMember_create_environment ( "pipe>", do_pipe_out,
				  "source file",
				  "Redirect to file");
  MonetMember_create_environment ( "pipe>>", do_pipe_append_out,
				  "source file",
				  "Redirect to file, appending");
  MonetMember_create_environment ( "pipe<", do_pipe_in,
				  "destination file",
				  "Redirect from file");
  MonetMember_create ( "become-daemon",		do_daemon,
		      "?no-fork",
"Fork off close descriptors and become a daemon. Be careful with this one. \
if an argument is given then 'fork' is not called.");

  MonetMember_create_write_boolean ( "daemon-mode",
				    &DaemonMode,
				    "Sets daemon mode");
  MonetConfig_create ("-daemon", "daemon-mode", NULL,
		      "Set daemon mode for this program, careful");

}

extern void Script_io_finish ()
{

}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/


