/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*  Defines interface for posix  */
/*****************************************************************************/

#include "stdP.h"

#include <monet/msh/Channel.h>

#include <sys/param.h>  /* For MAXPATHLEN */
#ifdef apollo
#include <sys/types.h>
#else
#include <utime.h>/* For struct utimebuf */
#endif

#include <sys/stat.h>  /*  For umask mkstemp  */
#include <sys/wait.h>		/* For wait */

/*****************************************************************************/
/*  util  */
/*****************************************************************************/

static char * getpath (arg)
     char * arg;
{
  if (arg)
    return arg;
  return "";
}

/*****************************************************************************/
/*  Definitions  */
/*****************************************************************************/

MONETCALLBACK(do_chdir)
     char * dir;
     dir = getpath (argv[1]);

     if (chdir (dir) != 0) {
       MonetPError ("chdir [%s]", dir);
       result = NULL;
     }
MONETEND


MONETCALLBACK(do_chmod)
     char * dir;
     int mode;

     dir = getpath (argv[1]);
     mode = MonetString2int (argv[2]);
     if (chmod (dir, mode) != 0) {
       MonetPError ("chmod [%s %s]", dir, argv[2]);
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_fork)
     result = Monetint2String (fork ());
MONETEND


MONETCALLBACK(do_getcwd)
     char buf [MAXPATHLEN];
     result = getcwd (buf, sizeof (buf));
     if ( ! result) {
       MonetPError ("getcwd");
     }
MONETEND


MONETCALLBACK(do_getpid)
     result = Monetint2String (getpid ());
MONETEND


MONETCALLBACK(do_getppid)
     result = Monetint2String (getppid ());
MONETEND


MONETCALLBACK(do_kill)
     int pid, number;
     pid = MonetString2int (argv[1]);
     number = MonetString2int (argv[2]);
     result = Monetint2String (kill (pid, number));
MONETEND

MONETCALLBACK(do_mkdir)
     char * dir;
     int mode;

     dir = getpath (argv[1]);
     if (argc == 2) {
       mode = MonetString2int (argv[2]);
     } else {
       mode = umask (umask (0));
     }
     if (mkdir (dir, mode) != 0) {
       MonetPError ("mkdir [%s]", dir);
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_readlink)
     int cc;
     static char buf [MAXPATHLEN];
     char * dir;

     dir = getpath (argv[1]);
     cc = readlink (dir, buf, sizeof (buf));
     if (cc < 0) {
       result = NULSTR;
     } else {
       result = buf;
     }
MONETEND

MONETCALLBACK(do_rmdir)
     char * dir;
     dir = getpath (argv[1]);

     if (rmdir (dir) != 0) {
       MonetPError ("rmdir [%s]", dir);
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_rename)
     char * path1, * path2;
     path1 = getpath (argv[1]);
     path2 = getpath (argv[2]);
     result = Monetint2String (rename (path1, path2));
MONETEND

MONETCALLBACK(do_link)
     char * path1, * path2;
     path1 = getpath (argv[1]);
     path2 = getpath (argv[2]);
     result = Monetint2String (link (path1, path2));
MONETEND

MONETCALLBACK(do_symlink)
     char * path1, * path2;
     path1 = getpath (argv[1]);
     path2 = getpath (argv[2]);
     result = Monetint2String (symlink (path1, path2));
MONETEND

MONETCALLBACK(do_unlink)
     char * path;
     path = getpath (argv[1]);
     result = Monetint2String (unlink (path));
MONETEND


MONETCALLBACK(do_system)
     if (argv[1]) {
       system (argv[1]);
     }
MONETEND


MONETCALLBACK(do_umask)
     int mask;
     mask = MonetString2int (argv[1]);
     result = Monetint2String (umask (mask));
MONETEND

MONETCALLBACK(do_wait)
     result =  Monetint2String (wait (NULL));
MONETEND

/*  -- Sleep  */

MONETCALLBACK(do_sleep)
     int seconds;
     seconds = MonetString2int (argv[1]);
     result = Monetint2String (sleep (seconds));
MONETEND

#ifdef apollo
MONETCALLBACK(do_utime)
     char * dir;

     dir = getpath (argv[1]);
     if (argc > 2) {
       time_t times[2];
       times[0]  = MonetString2int (argv[2]);
       times[1] = MonetString2int (argv[3]);
       if (utime (dir, &times) != 0) {
	 MonetPError ("utime [%s %s %s]", dir, argv[2], argv[3]);
	 result = NULL;
       }
     } else {
       if (utime (dir, NULL) != 0) {
	 MonetPError ("utime [%s]", dir);
	 result = NULL;
       }
     }
MONETEND
#else
MONETCALLBACK(do_utime)
     char * dir;

     dir = getpath (argv[1]);
     if (argc > 2) {
       struct utimbuf times;
       times.actime  = MonetString2int (argv[2]);
       times.modtime = MonetString2int (argv[3]);
       if (utime (dir, &times) != 0) {
	 MonetPError ("utime [%s %s %s]", dir, argv[2], argv[3]);
	 result = NULL;
       }
     } else {
       if (utime (dir, NULL) != 0) {
	 MonetPError ("utime [%s]", dir);
	 result = NULL;
       }
     }
MONETEND
#endif

/*****************************************************************************/
/*  Return effective and real user id's  */
/*****************************************************************************/

MONETCALLBACK(do_getuid)
     result = unsigned2string ((unsigned) getuid ());
MONETEND

MONETCALLBACK(do_geteuid)
     result = unsigned2string ((unsigned) geteuid ());
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_posix ()
{
  (void) MonetMember_module_class ("posix");

  MonetMember_create ("chdir", do_chdir,
		      "path",
"Change the current working directory to <path>.");

  MonetMember_create ("chmod", do_chmod,
		      "path, mode",
"Change the mode of <path> to the numeric <mode>.");

  MonetMember_create ("fork", do_fork,
		      "",
"Fork a child process.  Return 0 in the child, the child's process id \
in the parent.");


  MonetMember_create ("getcwd", do_getcwd,
		      "",
"Return a string representing the current working directory.");

  MonetMember_create ("getpid", do_getpid,
		      "",
"Return the current process id.");


  MonetMember_create ("getppid", do_getppid,
		      "",
"Return the parent's process id.");


  MonetMember_create ("sleep", do_sleep,
		      "seconds",
"Suspend processing for <seconds>");


  MonetMember_create ("kill", do_kill,
		      "pid, sig",
"Kill the process <pid> with signal <sig>.");


  MonetMember_create ("link", do_link,
		      "src, dst",
"Create a hard link pointing to <src> named <dst>.");


  MonetMember_create ("mkdir", do_mkdir,
		      "path ?mode",
"Create a directory named <path> with numeric mode <mode>. \
If no mode is given uses the current value of <umask>");

  MonetMember_create ("readlink", do_readlink,
		      "path",
"Return a string representing the path to which the symbolic link \
points.");

  MonetMember_create ("rename", do_rename,
		      "src, dst",
"Rename the file or directory <src> to <dst>.");

  MonetMember_create ("rmdir", do_rmdir,
		      "path",
"Remove the directory <path>.");

  MonetMember_create ("symlink", do_symlink,
		      "src, dst",
"Create a symbolic link pointing to <src> named <dst>.");

  MonetMember_create ("system", do_system,
		      "command",
"Execute the command (a string) in a subshell.  This is implemented by \
calling the Standard C function <system()>, and has the same \
limitations.  It is far better to use the 'io->sh' command.");

  MonetMember_create ("umask", do_umask,
		      "mask",
"Set the current numeric umask and returns the previous umask.");

  MonetMember_create ("unlink", do_unlink,
		      "path",
"Unlink <path>.");


  MonetMember_create ("utime", do_utime,
		      "path ?atime ?mtime",
"Set the access and modified time of the file to the given values.");


  MonetMember_create ("wait", do_wait,
		      "",
"Wait for completion of a child process and returns its process id.");

  MonetMember_create ( "geteuid",		do_geteuid,
		      "",
		      "Returns effective uid of user. \
See the relevant unix manual page");
  MonetMember_create ( "getuid",		do_getuid,
		      "",
		      "Returns real uid of user. \
See the relevant unix manual page");
}

extern void monetsh_finish_posix ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
