/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*  Defines some basic functions for monet script  */
/*****************************************************************************/

#include "stdP.h"


/***************************************************************************/
/*  Return errno  */
/***************************************************************************/

MONETCALLBACK(do_errno)
     if (argc == 1) {
       result = Monetint2String (errno);
     } else {
       result = MonetSystemErrorMessage ();
     }
MONETEND

/***************************************************************************/
/*  Date/Time  */
/***************************************************************************/

MONETCALLBACK(do_date)
     if (EQS(argv[1], "gmt")) {
       result = MonetTimeStamp ();
     } else if (EQS(argv[1], "long") || argv[1] == NULL || *argv[1] == NULL) {
       result = MonetPrettyTimeStamp ();
     } else if (EQS(argv[1], "since70")) {
       result = Monetlong2String ((long) time (NULL));
     } else {
       MonetScriptFail ("date: no such option [%s]", argv[1]);
     }
MONETEND

/*****************************************************************************/
/*  Return effective and real user id's  */
/*****************************************************************************/

MONETCALLBACK(do_getuid)
     result = unsigned2string ((unsigned) getuid ());
MONETEND

MONETCALLBACK(do_geteuid)
     result = unsigned2string ((unsigned) geteuid ());
MONETEND

/***************************************************************************/
/*  Change and directory  */
/***************************************************************************/

MONETCALLBACK(do_chdir)
     char * dir;
     if (argc == 2 && argv[1]) {
       dir = argv[1];
     } else {
       dir = "";
     }
     if (chdir (dir) != 0) {
       MonetPrintfError ("%s [%s]", MonetSystemErrorMessage ());
       result = NULL;
     }
MONETEND

/***************************************************************************/
/*  Get and set environment variables  */
/***************************************************************************/

MONETCALLBACK(do_getenv)
     if (argv[1]) {
       result = getenv (argv[1]);
     } else {
       result = NULL;
     }
MONETEND

MONETCALLBACK(do_putenv)
     char * name =argv[1], * value = argv[2];
#define MAXENV 48
     static int num = 0;
     static char * namelist [MAXENV];
     int i, len;
     extern char **environ;

     if (argc == 1) {
       char ** p;
       for (p = environ; p && *p; ++p) {
	 MonetPrintfnl ("%s", *p);
       }
       RETURNSC;
     }

     if (name == NULL) {
       result = NULL;
       RETURNSC;
     }

     /*  Need to be careful here because we need to allocate the
	 space used by the environment variables.  To stop a
	 potential memory leak, the variables are allocated and
	 kept in an array.  If an attempt to put a new value is
	 made the list is searched to see if it is in the allocated
	 list, if so it is removed.
	 */
     len = MonetStrlen (name);
     for (i = 0; i < num; ++i) {
       if (strncpy (namelist [i], name, len) == 0 &&
	   name[len] == '=') {
	 MonetFree (namelist [i]);
	 if (num > 1) {
	   num--;
	   namelist [i] = namelist [num];
	   break;
	 }
       }
     }
     if (num == MAXENV) {
       MonetPrintfError ("Sorry, can only allocate %d environment vars",
			 MAXENV);
       RETURNSC;
     }
     namelist [num] = MonetMalloc (len + 2 + MonetStrlen (value));
     (void) sprintf (namelist [num], "%s=%s", name, value);
     if (putenv (namelist [num]) != 0) {
       MonetPrintfError ("Could not malloc space for environment list");
       result = NULL;
     }
     ++num;
MONETEND

/*****************************************************************************/
/*  Init finish  */
/*****************************************************************************/

extern void monetsh_init_unix ()
{
  (void) MonetMember_module_class ("unix");


  MonetMember_create ( "errno",		do_errno,
		      "?bool",
		      "Returns value of errno(3), if an argument is given\
it returns the corresponding system message");
  MonetMember_create ( "date",		do_date,
		      "?gmt|long|since70",
		      "Returns formatted date, default is long \
since70 is the #seconds since 1970");
  MonetMember_create ( "geteuid",		do_geteuid,
		      "",
		      "Returns effective uid of user. \
See the relevant unix manual page");
  MonetMember_create ( "getuid",		do_getuid,
		      "",
		      "Returns real uid of user. \
See the relevant unix manual page");
  MonetMember_create ( "getenv",		do_getenv,
		      "name",
		      "Gets an environment variable. \
See unix command 'getenv(3)'");
  MonetMember_create ( "setenv",		do_putenv,
		      "?name ?value",
		      "Sets an environment variable. \
See unix command 'putenv(3)'");

  MonetMember_create ( "chdir",		do_chdir,
		      "?filename",
		      "Calls 'chdir(2)', changes current working directory");

}

extern void monetsh_finish_unix ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header: //gaia/group.local/computer/monet/Xmonet/RCS/main.c,v 1.1 1991/03/26 12:00:14 a38 Exp a38 $ */
/*****************************************************************************/
/*****************************************************************************/
