
/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*  Useful Callback and Actions for use with X11                             */
/*****************************************************************************/

#include <monet/msh/General.h>
#include <monet/x/Widget.h>

/****************************************************************************/
/*  Take the name of a widget, find it, if it is a dialog or a text widget  */
/*  return its value, NULL is empty or error  */
/****************************************************************************/

extern String XmonetWidget2value (widget, w_name)
     Widget widget;
     String w_name;
{
  Widget text_widget;
  String name;
  char  clean_name[1024]; /* !!! */

  if (w_name == NULL || *w_name == NULL) {
    MonetPrintfError ("text_widget name is NULL or empty");
    return NULL;
  }
  if (! widget)
    widget = XmonetTopWidget;

  WcCleanName (w_name, clean_name);
  text_widget = WcFullNameToWidget (widget, clean_name);
  if ( ! text_widget) {
    MonetPrintfError ("No such text_widget [%s]", w_name);
    return NULL;
  }
  if (XtIsSubclass (text_widget, dialogWidgetClass)) {
    name = XawDialogGetValueString (text_widget);
  } else if (XtIsSubclass (text_widget, labelWidgetClass)) {
    Arg args[1];
    XtSetArg(args[0], XtNlabel, &name);
    XtGetValues(text_widget, args, ONE);
  } else if (XtIsSubclass (text_widget, smeBSBObjectClass)) {
    Arg args[1];
    XtSetArg(args[0], XtNlabel, &name);
    XtGetValues(text_widget, args, ONE);
  } else if (XtIsSubclass (text_widget, asciiTextWidgetClass)) {
    Arg args[1];
    XtSetArg(args[0], XtNstring, &name);
    XtGetValues(text_widget, args, ONE);
  } else {
    MonetPrintfError ("Widget [%s] [%s] not dialog,label or asciitext widget",
		      clean_name, XtName (text_widget));
    return NULL;
  }
  return name;
}

/****************************************************************************/
/*  Take the name of a widget, find it, if it is a dialog or a text widget  */
/*  set and return its value, NULL is empty or error  */
/****************************************************************************/

extern String TextWidget_set_value (widget, w_name, name)
     Widget widget;
     String w_name, name;
{
  Widget text_widget;
  char  clean_name[1024]; /* !!! */
  Arg args[1];

  if (w_name == NULL) {
    MonetPrintfError ("text_widget name is NULL");
    return NULL;
  }
  if (! widget)
    widget = XmonetTopWidget;

  WcCleanName (w_name, clean_name);
  text_widget = WcFullNameToWidget (widget, clean_name);
  if ( ! text_widget) {
    MonetPrintfError ("No such widget [%s]", w_name);
    return NULL;
  }
  if (XtIsSubclass (text_widget, dialogWidgetClass)) {
    XtSetArg(args[0], XtNvalue, name);
  } else if (XtIsSubclass (text_widget, labelWidgetClass)) {
    XtSetArg(args[0], XtNlabel, name);
  } else if (XtIsSubclass (text_widget, smeBSBObjectClass)) {
    XtSetArg(args[0], XtNlabel, name);
  } else if (XtIsSubclass (text_widget, asciiTextWidgetClass)) {
    XtSetArg(args[0], XtNstring, name);
  } else {
    MonetPrintfError ("Widget [%s] [%s] not dialog, label or asciitext widget",
		      clean_name, XtName (text_widget));
    return NULL;
  }
  if (XtIsRealized(text_widget))
    XtSetValues(text_widget, args, ONE);
  return name;
}

/*****************************************************************************/
/*  Takes the name of a widget a returns a transient shell widget by that    */
/*  name if such a widget exists.                                            */
/*****************************************************************************/

extern Widget XmonetWidget2Popup (widget, w_name)
     Widget widget;
     String w_name;
{
  Widget popup;
  char clean_name[1024]; /* !!! */
  
  (void) WcCleanName (w_name, clean_name);
  popup = WcFullNameToWidget (widget, clean_name);
  if ( ! popup) {
    MonetPrintfError ("No such popup widget [%s]", w_name);
    return NULL;
  }
  if ( ! (XtIsSubclass (popup, shellWidgetClass))) {
    MonetPrintfError ("Widget is not a shell widget [%s]", clean_name);
    return NULL;
  }
  return popup;
}
/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
