
/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include <monet/x/Widget.h>

/*****************************************************************************/
/*  Confirm Box  */
/*****************************************************************************/

static char * ConfirmMessage;
static ConfirmRoutine ConfirmProc;
static Widget  ConfirmLabelWidget;
static Widget ConfirmBox;

static void xio_confirm_box (proc, message)
     ConfirmRoutine proc;
     char * message;
{
  if (!ConfirmBox) {
    fprintf(stderr, "Null confirm_box widget\n");
    ConfirmBox = WcFullNameToWidget (XmonetTopWidget, "*confirm");
    if (ConfirmBox) {
      fprintf(stderr, "Found confirm_box widget\n");
    }
  }
  if (ConfirmBox) {
    Arg args [1];
    if (XtIsSubclass (ConfirmBox, shellWidgetClass)) {
      ShellWidget shell;

      shell = (ShellWidget) ConfirmBox;

      if (ConfirmMessage) {
	MonetFree (ConfirmMessage);
      }
      ConfirmMessage = MonetNewString (message);
      ConfirmProc = proc;
    
      if (ConfirmLabelWidget) {
	XtSetArg(args[0], XtNlabel, ConfirmMessage);
	XtSetValues(ConfirmLabelWidget, args, ONE);
      }
      XMonetPopupCentral (XmonetTopWidget, ConfirmBox, XtGrabExclusive);

      XFlush(XmonetDisplay);

      while (shell -> shell.popped_up) {
	XEvent event;
	XtAppNextEvent(XmonetAppCon, &event);
	XtDispatchEvent(&event);
      }
    } else {
      MonetPrintfError ("Confirm box is not a shell widget");
    }
  } else {
      MonetPrintfError ("No confirm box");
  }
} 

MONETCALLBACK(do_confirm)
     if (ConfirmProc) {
       ConfirmProc ();
     } else {
       MonetSysError ("do_confirm: no confirm action supplied");
     }
MONETEND

MONETCALLBACK(do_register)
     String w_name = argv[1], w_message = argv[2];
     Widget popup, message;

     if ( ! (popup = XmonetWidget2Popup (XmonetCurrentWidget, w_name))) {
       MonetPrintfError ("No such popup widget [%s]", w_name);
       RETURNSC;
     }
     if ( ! (message = WcFullNameToWidget (XmonetCurrentWidget, w_message))) {
       MonetPrintfError ("No such widget [%s]", w_name);
       RETURNSC;
     }
     (void) set_confirm_box_routine (xio_confirm_box);
     ConfirmBox = popup;
     ConfirmLabelWidget = message;
MONETEND

/*****************************************************************************/
/*  Init  */
/*****************************************************************************/

extern void xmonet_confirm_init ()
{
  ConfirmMessage     = NULSTR;
  ConfirmProc        = NULL;
  ConfirmLabelWidget = (Widget) 0;
  ConfirmBox         = (Widget) 0;

  (void) MonetMember_module_class ("xwidget");
  MonetMember_create ("confirm",  do_confirm,
		      "",
		      "Do confirm action");
  MonetMember_create ("register-confirm",  do_register,
		      "popup-widget message-widget",
		      "Register popup and message widgets for confirmation. \
		      Confirm boxes are activated internally.");
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
