/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

#include "xmonetP.h"
#include <monet/msh/Script.h>
#include <X11/ObjectP.h>

static Cursor    SizingCursor, WaitCursor, ExecCursor;


/*****************************************************************************/
/* This function is called by  MonetException_code  */
/* The idea is to provide an interupt mechanism for the X windows interface. */
/* With a command line shell a simple ^C causes an interupt and the program  */
/* stops.  This isn't easy with a windows interface and besides we still     */
/* want processing of X events while a job is going on.  */
/*****************************************************************************/

static void confirm ()
{
  MonetException_raise_message
    (MONETSH_SIGNAL, "Interuptted by a key or button");
}

static MonetBool button_pending ()
{
  XtInputMask mask;

  XFlush(XmonetDisplay);
  if (mask = XtAppPending(XmonetAppCon)) {
    XEvent          event;
    if (mask & XtIMAlternateInput) {
      XtProcessEvent((unsigned) XtIMAlternateInput);
    } else if (mask & XtIMTimer) {
      XtProcessEvent((unsigned) XtIMTimer);
    } else /* mask & XtIMXEvent */ {
      XtAppPeekEvent(XmonetAppCon, &event);
      switch(event.type) {
      case ButtonPress:
      case ButtonRelease:
	return True;
      case KeyRelease:
	return True;
      case LeaveNotify:
      case EnterNotify:
      case MotionNotify:
      default:
	XtAppNextEvent(XmonetAppCon, &event);
	break;
      }
    }
  }
  return False;
}

static void check_fail ()
{
  XEvent event;

  if (button_pending ()) {
    XtAppNextEvent(XmonetAppCon, &event);
    Monet_confirm (confirm, "Do you really want to break execution");
  }
}

/******************************************************************/
/*  Cursor stuff                                                  */
/******************************************************************/

static void map_cursor (widget, cursor)
     Cursor cursor;
     Widget widget;
{
  Window window, bounded;

  if (widget) {
    if (! XtIsRealized (widget)) {
      XtRealizeWidget (widget);
    }
    bounded = XtWindow(widget);
  } else {
    widget = XmonetTopWidget;
    bounded = None;
  }
  if (! (window = XtWindow(widget))) {
    MonetPrintfError ("No window");
  } else if (cursor) {
    XGrabPointer (XtDisplay (widget), window, True,
		  (unsigned long) PointerMotionMask |
		  ButtonPressMask | ButtonReleaseMask,
		  GrabModeAsync, GrabModeAsync,
		  bounded, cursor, CurrentTime);
  } else {
    MonetSysError ("Cursor is null");
  }
  MonetException_register_fail (check_fail);
}

extern void Xmonet_sizing_cursor (widget)
     Widget widget;
{ map_cursor (widget, SizingCursor); }

extern void Xmonet_normal_cursor (widget)
     Widget widget;
{
  if (! widget) {
    widget = XmonetTopWidget;
  }
  XUngrabPointer (XtDisplay (widget), CurrentTime);
  MonetException_register_fail (NULL);
}

extern void Xmonet_wait_cursor (widget)
     Widget widget;
{ map_cursor (widget, WaitCursor); }

extern void Xmonet_exec_cursor (widget)
     Widget widget;
{ map_cursor (widget, ExecCursor); }


/*****************************************************************************/
/*  Init and finish off  */
/*****************************************************************************/

extern void xmonet_init_cursors ()
{
  /*  Set up cursors  */
/*  NormCursor   = XCreateFontCursor(XmonetDisplay, XC_crosshair); */
  SizingCursor = XCreateFontCursor(XmonetDisplay, XC_sizing);
  WaitCursor   = XCreateFontCursor(XmonetDisplay, XC_watch);
  ExecCursor   = XCreateFontCursor(XmonetDisplay, XC_leftbutton);
}

extern void xmonet_finish_cursors ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

