/******************************************************************/
/******************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/xio/draw.c,v 1.1 1991/06/27 18:09:52 a38 Exp a38 $ */
/*****************************************************************
  Draw library for X windows, extension to basic draw library   
  includes limited scaling of drawings.  Would like a proper    
  Zoom feature, but alas 
  Notice that FillRectangle needs 1 added to width and height   
  to draw a width by height rectangle.                          

  Each Font has a set of GC's, one for each 'shade'.

  In addition there is a base set of GC's for drawing no-text
  graphics, one for thick lines and one for thin.
  *****************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/Script.h>
#include <monet/x/Widget.h>
#include <X11/Xmu/Drawing.h>

/******************************************************************/
/*  Definitions  */
/******************************************************************/

typedef struct scaled_font * ScaledFont;
DrawLibrary DrawX11Lib = (DrawLibrary) 0;

typedef struct segment_request {
  GC gc;
  Cardinal cursor;
  XSegment * data;
} LinesRequest;

/*  This needs some fixing  */
static struct {
	Window          window;
	Region          region;
	LinesRequest	lines_gc[NUM_SHADE], lines_widegc[NUM_SHADE];
	} Def;

/**** GC's   [ font | wide/thin lines | shade ] ****/
/****          4    * 2               * NUM_SHADE  = 40 ****/
/**** Comprimise for now, just one font ****/

static float
  PreviousXScale, PreviousYScale,
  ThisXScale, ThisYScale;

#define MAXSCALENAMELEN 80
struct scaled_font {
	char name[MAXSCALENAMELEN];
	XFontStruct * font;
	float x_scale, y_scale;
	Dimension width, height;
	GC text_gc[NUM_SHADE];
	ScaledFont next;
};

static GC GcNorm[NUM_SHADE], GcWide[NUM_SHADE];

#define Shade2TextGC(scale, shade) (scale->text_gc[(int)shade])
#define Shade2NormGC(shade)     GcNorm[(int)shade]
#define Shade2wideGC(shade) GcWide[(int)shade]

static ScaledFont ScaledFontList = NULL;

static void LinesRequest_send (r)
     LinesRequest * r;
{
  XDrawSegments (
		 XmonetDisplay,
		 Def.window,
		 r->gc,
		 r->data,
		 r->cursor);
  if (r->cursor > 0) {
    MONETDEBUG(10, ( "Flushed %d lines", (int) r->cursor));
    r->cursor = 0;
  }
}

static void FLUSH_REQUESTS()
{
  int shade;
  MONETDEBUG(5, ( "Flushing lines"));
  for (shade = 0; shade<NUM_SHADE; shade++) {
    LinesRequest_send (&Def.lines_gc[(int)shade]);
    LinesRequest_send (&Def.lines_widegc[(int)shade]);
  }
  XFlush (XmonetDisplay);
}

static void FINISH_REQUESTS()
{
  int shade;
  for (shade = 0; shade<NUM_SHADE; shade++) {
    XtFree ((char *) Def.lines_widegc[(int)shade].data);
    XtFree ((char *) Def.lines_gc[(int)shade].data);
  }
}

static void LinesRequest_create (r, gc, size)
     LinesRequest * r;
     GC gc;
     unsigned size;
{
  r-> gc = gc;
  r->data = (XSegment *) XtMalloc (size);
  r->cursor = 0;
}

static unsigned MaxLines;


static void DRAW_LINE(request, xl, yl, xr, yr)
     LinesRequest * request;
     Position        xl, yl, xr, yr;
{
  XSegment * segment;

  if (request->cursor >= MaxLines) {
    LinesRequest_send (request);
    }
  segment = & (request -> data [request->cursor]);
  segment->x1 = xl;
  segment->x2 = xr;
  segment->y1 = yl;
  segment->y2 = yr;
  request -> cursor ++;
}

/*****************************************************************************/
/*  Check region  */
/*****************************************************************************/

static MonetBool in_region (x,y,w,h)
     Position x,y;
     Dimension w,h;
{
  if (Def.region && XRectInRegion (Def.region, x,y,w,h) == RectangleOut) {
    return False;
  }
  return True;
}

/*****************************************************************************/
/*  Set up the GC's for all maps, won't work for multiple display etc  */
/*  !!! Should not rely on 6x10 font existing, doesn't always work  */
/*****************************************************************************/

static void GetTextGCs(scale)
     ScaledFont scale;
{
  Widget widget = XmonetTopWidget;
  GC graygc, normgc, revgc, heavydotgc;
  int             screen;
  XGCValues       values;
  unsigned long    mask = GCForeground | GCBackground | GCLineStyle |
    GCLineWidth | GCFillStyle | GCJoinStyle | GCCapStyle | GCFunction,
    fontMask;
  Pixel           background, foreground;
  XFontStruct * font;



  /**** Set up back and foreground pixels ****/
  /* !!! Not guaranteed to work on colour or greyscale displays */
  screen = DefaultScreen(XmonetDisplay);
  /*	if (Config.reverse_video) {
	foreground = WhitePixel(XmonetDisplay, screen);
	background = BlackPixel(XmonetDisplay, screen);
	*/
  background = BlackPixel(XmonetDisplay, screen);
  foreground = WhitePixel(XmonetDisplay, screen);
  /* !!! Hack */
  foreground = BlackPixel(XmonetDisplay, screen) ^ WhitePixel(XmonetDisplay, screen);
  background = 0;

  /**** Set up the GC's ****/
  values.function   = GXxor;
  values.foreground = foreground;
  values.background = background;
  if (font = scale->font) {
    values.font = font->fid;
    fontMask = GCFont;
  } else {
    values.font = NULL;
    fontMask = 0;
  }
  values.line_style = LineSolid;
  values.line_width = 1;
  values.cap_style  = CapRound;
  values.fill_style = FillSolid;
  values.join_style = JoinRound;

  normgc = XtGetGC(widget, mask | fontMask , &values);
  values.line_width = 3;
  values.cap_style  = CapNotLast;

  values.line_width = 1;
  
  values.background = foreground;
  values.foreground = background;
  values.cap_style  = CapRound;
  revgc = XtGetGC(widget, mask, &values);
  values.tile = XmuCreateStippledPixmap(XtScreen(widget),background,foreground,
					widget->core.depth);
  values.fill_style = FillTiled;
  values.function   = GXxor;
  mask = GCTile | GCFillStyle | GCFunction | GCLineWidth |
    GCCapStyle | fontMask;
  graygc     = XtGetGC(widget, mask, &values);
  values.line_width = 3;
  values.cap_style  = CapNotLast;

  /********************************  Heavy dotted *************************/
  
  values.line_width = 3;
  values.tile = XmuCreateStippledPixmap(XtScreen(widget),background,foreground,
					widget->core.depth);
  values.fill_style = FillTiled;
  values.function   = GXxor;
  values.dashes = 4;
  mask = GCTile | GCFillStyle | GCFunction |
    GCLineWidth | GCCapStyle | GCDashList | fontMask;
  heavydotgc = XtGetGC(widget, mask, &values);
  

  /**** Set up the 'shade' to GC translation table ****/
  scale->text_gc[(int)shade_background]	= revgc;
  scale->text_gc[(int)shade_1]			= revgc;   /* Not used */
  scale->text_gc[(int)shade_grey]		= graygc;
  scale->text_gc[(int)shade_3]			= graygc;  /* Not used */
  scale->text_gc[(int)shade_dark]		= normgc;
  scale->text_gc[(int)shade_heavy]		= heavydotgc;
}

/*****************************************************************************/
/*  Convert between ScaledFont and string  */
/*****************************************************************************/

static ScaledFont	ScaledFont_create	(name)
     String name;
{
  Dimension width, height;
  ScaledFont scale;
  XFontStruct * font;

  if (name == NULL) {
    MonetPrintfError ("Font name is NULL");
    return NULL;
  }

  width = 0;

  /*  Check to see if ScaledFont already exists  */
  FORALL(scale, ScaledFontList, next)
    if (EQS (name, scale->name))
      return scale;

  /*  ScaledFont does not already exist, so try to find a font for it  */
  if (font = XLoadQueryFont(XmonetDisplay, name)) {
    width  = XTextWidth(font, "ABCDEFGHIJKLMNOPQRSTUVWXYZw_", 28) / 28 + 1;
    height = font->ascent + font->descent;
  } else {
    MonetPrintfError("Cannot open [%s] font", name);
    return NULL;
  }

  /*  Create a new ScaledFont with the font  */
  scale = MonetNew (struct scaled_font);
  if ( ! scale) {
    MonetSysError ("Could not allocate space for ScaledFont");
    return NULL;
  }
  scale-> height = height;
  scale-> width  = width;
  scale-> font = font;
  scale-> x_scale = (float) width   / (float) (DrawCharWidth+1);
  scale-> y_scale = (float) height  / (float) DrawCharHeight;
  (void) strncpy (scale->name, name, MAXSCALENAMELEN - 1);
  scale->name[MAXSCALENAMELEN-1] = NULL;
  LIST_ADD (scale, ScaledFontList, next);
  
  GetTextGCs (scale);
  
  return scale;
}

/**************************************************************
  Set region for all defined GC's
  ****************************************************************/

static void set_gc_regions (region)
     Region region;
{
  unsigned shade;

  for (shade = 0; shade < NUM_SHADE; ++ shade) {
    ScaledFont scale;
    XSetRegion (XmonetDisplay, GcNorm[shade], region);
    XSetRegion (XmonetDisplay, GcWide[shade], region);
    FORALL(scale, ScaledFontList, next) {
      XSetRegion (XmonetDisplay, scale->text_gc[shade], region);
    }
  }
}

static void clear_gc_regions ()
{
  unsigned shade;

  for (shade = 0; shade < NUM_SHADE; ++ shade) {
    ScaledFont scale;
    XSetClipMask (XmonetDisplay, GcNorm[shade], None);
    XSetClipMask (XmonetDisplay, GcWide[shade], None);
    FORALL(scale, ScaledFontList, next) {
      XSetClipMask (XmonetDisplay, scale->text_gc[shade], None);
    }
  }
}

/*****************************************************************/
/*  Call this before a batch of routines.  */
/*****************************************************************/

extern MonetBool start_draw_X11 (net, x, y, x_scale, y_scale, region)
     Widget          net;
     Position x,y;
     float x_scale, y_scale;
     Region region;
{
  if (XtIsRealized (net) == False)
    return False;

  if (XmonetDisplay != XtDisplay(net)) {
    /* -- this won't happen but better check anyway  */
    MonetSysError ("start_draw_X11: window not on default xmonet display");
    return False;
  }

  if (ScaledFontList == NULL) {
    MonetPrintfWarning ("No fonts loaded");
  }

  PreviousXScale = ThisXScale;
  PreviousYScale = ThisYScale;
  ThisXScale = x_scale;
  ThisYScale = y_scale;

  Def.window  = XtWindow(net);

  Draw_start_draw (DrawX11Lib, x,y, x_scale, y_scale);

  if (region) {
    XGCValues       values;
    unsigned long    mask = GCClipXOrigin | GCClipYOrigin;

    XGetGCValues (XmonetDisplay, GcNorm[0], mask, &values);
    XOffsetRegion (region, values.clip_x_origin, values.clip_y_origin);
    Def.region = region;

    set_gc_regions (region);
  } else {
    Def.region = (Region) 0;
    /*  Clear clipping  */
    clear_gc_regions ();
  }

  return True;
}

static void end_draw ()
{
  ThisXScale = PreviousXScale;
  ThisYScale = PreviousYScale;

  Def.region = (Region) 0;
  FLUSH_REQUESTS();
  XFlush (XmonetDisplay);
}

/******************************************************************/
/*  Drawing routines  */
/******************************************************************/

static void draw_line(shade, xl, yl, xr, yr)
     MonetShade		shade;
     Position        xl, yl, xr, yr;
{
  DRAW_LINE (&(Def.lines_gc[(int)shade]), xl, yl, xr, yr);
}

static void draw_percentage_line(shade, x,y, xl, yl, xr, yr)
     MonetShade		shade;
     Position        xl, yl, xr, yr, x, y;
{
  DRAW_LINE(&(Def.lines_gc[(int)shade]), xl, yl, x, y);
  DRAW_LINE(&(Def.lines_widegc[(int)shade]), x, y, xr, yr);
}


static void draw_elipse(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  XDrawArc(XmonetDisplay, Def.window, Shade2NormGC(shade), x, y, width, height,
	   0, 360 * 64);
}

static void fill_elipse(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  XFillArc(XmonetDisplay, Def.window, Shade2NormGC(shade),
	   x, y, width, height, 0, 360 * 64);
}

static void draw_rectangle(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  XDrawRectangle(XmonetDisplay, Def.window, Shade2NormGC(shade),
		 x, y, width, height);
}

static void draw_roundedRectangle(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  GC gc =  Shade2NormGC(shade);

  XDrawArc(XmonetDisplay, Def.window, gc, x, y, 5, height, 90 * 64, 180 * 64);
  XDrawArc(XmonetDisplay, Def.window, gc,
	   x+width-5, y, 5, height, 270 * 64, 180 * 64);
  XDrawLine(XmonetDisplay, Def.window, gc, x+5, y, x+width-5, y);
  XDrawLine(XmonetDisplay, Def.window, gc, x+5, y+height, x+width-5, y+height);
  /* !!! This routine is not part of the standard,
     !!! don't know how to work it either */
  /*XmuDrawRoundedRectangle(XmonetDisplay, Def.window, gc,
    x, y, width, height,width,height); */
}

static void fill_roundedRectangle(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  GC gc =  Shade2NormGC(shade);

  XDrawArc(XmonetDisplay, Def.window, gc, x, y, 5, height, 90 * 64, 180 * 64);
  XDrawArc(XmonetDisplay, Def.window, gc, x+width-5, y, 5, height,
	   270 * 64, 180 * 64);
  XFillRectangle(XmonetDisplay, Def.window, gc, x+5, y, width-5+1, height+1);
  /* !!! This routine is not part of the standard,
     !!! don't know how to work it either */
  /*XmuDrawRoundedRectangle(XmonetDisplay, Def.window, gc,
    x, y, width, height,width,height); */
}


static void fill_rectangle(shade, x, y, width, height)
     MonetShade		shade;
     Position        x, y;
     Dimension       width, height;
{
  XFillRectangle(XmonetDisplay, Def.window,  Shade2NormGC(shade), x, y,
		 width+1, height+1);
}

static void do_draw_string(shade, x, y, w,h, string, len, boxed)
     MonetShade		shade;
     Position        x, y;
     Dimension       w, h;
     String          string;
     unsigned len;
     MonetBool boxed;
{
  ScaledFont s, scale;
  Dimension s_width;

  if (w == 0 || h == 0 || len == 0) {
    return;
  }
  /*
    MONETDEBUG(13, ( "String [%s]:[%d] w,h [%d,%d]", string, len, w, h));
    */
  scale = NULL;
  FORALL (s, ScaledFontList, next) {
    float x_scale, y_scale;
    Dimension width;
    width =  XTextWidth (s->font, string, len);
    if (width == 0)
      continue;
    x_scale = (float) w / (float) width;
    y_scale = (float) h / (float) s -> height;
    /*
    MONETDEBUG(13, ( "Try Scale [%s] w,h [%d,%d] xs ys [%f,%f] sxs sys [%f,%f]",
		s->name, s->width, s->height,
		x_scale, y_scale,
		s->x_scale, s->y_scale));
		*/
    if (s->x_scale <= x_scale && s->y_scale <= y_scale)
      if ( (!scale) || (
			s->x_scale >= scale->x_scale &&
			s->y_scale >= scale->y_scale)) {
	if (width <= w) {
	  scale = s;
	  s_width = width;
	  /*
	    MONETDEBUG(13, ( "Scaled [%s] width, height [%d,%d]",
	    s->name, s->width, s->height));
	    */
	}
      }
  }
  if (scale) {
    GC gc;
    y += (scale->height + h) / 2;
    x += (w - s_width) / 2;
    if (shade == shade_grey) {
      XFillRectangle(XmonetDisplay, Def.window,   Shade2NormGC(shade),
		     x, y - scale->height, s_width +1,
		     scale->height + 1);
      boxed = ! boxed;
    }
    gc =  Shade2TextGC(scale, shade_dark);
    if (boxed) {
      XFillRectangle(XmonetDisplay, Def.window, gc,
		     x, y - scale->height, s_width +1,
		     scale->height + 1);
    }
    XDrawString(XmonetDisplay, Def.window, gc, x, y, string, len);
  }
}

static void draw_string(shade, x, y, w,h, string, len)
     MonetShade		shade;
     Position        x, y;
     Dimension       w, h;
     String          string;
     unsigned len;
{
  do_draw_string(shade, x, y, w,h, string, len, False);
}

static void draw_boxed_string(shade, x, y, w,h, string, len)
     MonetShade		shade;
     Position        x, y;
     Dimension       w, h;
     String          string;
     unsigned len;
{
  do_draw_string(shade, x, y, w,h, string, len, True);
}

static void set_reverse ()
{
	/*  does nothing, because XOR is used  */
}

static void set_scale (x,y)
     float x,y;
{
}

/*****************************************************************/
/*  For giving the names of new scalable fonts.  */
/*****************************************************************/

MONETCALLBACK(do_add_fonts)
     while (argc > 1 ) {
       --argc;
       (void) ScaledFont_create (argv[argc]);
     }
MONETEND

MONETCALLBACK(do_help)
  ScaledFont s;

  FORALL (s, ScaledFontList, next) {
    MonetPrintfnl ("[%s] scale [%f %f] width %d, height %d",
		   s->name, s->x_scale, s->y_scale, s->width, s->height);
  }
MONETEND

/*****************************************************************************/
/*  Set up the GC's for drawing non-text graphics  */
/*****************************************************************************/

static void GetLineGCs()
{
  Widget widget = XmonetTopWidget;
  GC graygc,  normgc, widegraygc, widenormgc, revgc,
      heavydotgc, wideheavydotgc;
  int             screen;
  XGCValues       values;
  unsigned long    mask = GCForeground | GCBackground | GCLineStyle |
    GCLineWidth | GCFillStyle | GCJoinStyle | GCCapStyle | GCFunction;
  Pixel           background, foreground;

  /**** Set up back and foreground pixels ****/
  /* !!! Not guaranteed to work on colour or greyscale displays */

  screen = DefaultScreen(XmonetDisplay);
  background = BlackPixel(XmonetDisplay, screen);
  foreground = WhitePixel(XmonetDisplay, screen);
  /* !!! Hack */
  foreground = BlackPixel(XmonetDisplay, screen) ^
    WhitePixel(XmonetDisplay, screen);

  background = 0;

  /**** Set up the GC's ****/
  values.function   = GXxor;
  values.foreground = foreground;
  values.background = background;
  values.line_style = LineSolid;
  values.line_width = 1;
  values.cap_style  = CapRound;
  values.fill_style = FillSolid;
  values.join_style = JoinRound;

  normgc = XtGetGC(widget, mask, &values);
  values.line_width = 3;
  values.cap_style  = CapNotLast;
  widenormgc = XtGetGC(widget, mask, &values);
  values.line_width = 1;
  
  values.background = foreground;
  values.foreground = background;
  values.cap_style  = CapRound;
  revgc = XtGetGC(widget, mask, &values);
  values.tile = XmuCreateStippledPixmap(XtScreen(widget),background,foreground,
					widget->core.depth);
  values.fill_style = FillTiled;
  values.function   = GXxor;
  mask = GCTile | GCFillStyle | GCFunction | GCLineWidth |
    GCCapStyle;
  graygc     = XtGetGC(widget, mask, &values);
  values.line_width = 3;
  values.cap_style  = CapNotLast;
  widegraygc = XtGetGC(widget, mask, &values);

  /********************************  Heavy dotted *************************/
  
  values.line_width = 3;
  values.tile = XmuCreateStippledPixmap(XtScreen(widget),background,foreground,
					widget->core.depth);
  values.fill_style = FillTiled;
  values.function   = GXxor;
  values.dashes = 4;
  mask = GCTile | GCFillStyle | GCFunction |
    GCLineWidth | GCCapStyle | GCDashList;
  heavydotgc = XtGetGC(widget, mask, &values);
  
  values.line_width = 4;
  wideheavydotgc = XtGetGC(widget, mask, &values);


  /**** Set up the 'shade' to GC translation table ****/
  GcNorm[(int)shade_0]	= revgc;
  GcNorm[(int)shade_1]	= revgc;   /* Not used */
  GcNorm[(int)shade_2]	= graygc;
  GcNorm[(int)shade_3]	= graygc;  /* Not used */
  GcNorm[(int)shade_4]	= normgc;
  GcNorm[(int)shade_5]	= heavydotgc;
  GcWide[(int)shade_0]	= revgc;
  GcWide[(int)shade_1]	= revgc;   /* Not used */
  GcWide[(int)shade_2]	= widegraygc;
  GcWide[(int)shade_3]	= widegraygc;  /* Not used */
  GcWide[(int)shade_4]	= widenormgc;
  GcWide[(int)shade_5]	= wideheavydotgc;
}

/*****************************************************************/
/* Init and finish drawing of entitys */
/*****************************************************************/

extern void xmonet_init_draw_X11 ()
{
  unsigned requestsize, size;
  int shade;

  NEWDRAWLIB(DrawX11Lib, True);

  (void) MonetMember_module_class ("xwidget");
  MonetMember_create
    ("scaled-fonts", do_add_fonts,
     "fontname ...",
     "Give a list of fonts to be used for scaling");
  MonetMember_create
    ("help-fonts", do_help,
     "",
     "Lists details about scaled fonts");
  PreviousXScale = 0;
  PreviousYScale = 0;
  ThisXScale     = 0;
  ThisYScale     = 0;

  GetLineGCs ();

  ScaledFontList     = NULL;
  (void) ScaledFont_create ("5x8");


  /*  -- Initialise the stuff for batching 'draw line' requests  */

  requestsize = XMaxRequestSize (XmonetDisplay);
  size = (requestsize - 3) / 2;

  MaxLines = size / sizeof (XSegment);

  for (shade = 0; shade<NUM_SHADE; shade++) {
    LinesRequest_create (& Def.lines_gc[(int)shade],
			 Shade2NormGC(shade), size);
    LinesRequest_create (& Def.lines_widegc[(int)shade],
			 Shade2wideGC(shade), size);
  }
}

extern void xmonet_finish_draw_X11()
{
  ScaledFont scale;

  FINISH_REQUESTS();
  MonetFree (DrawX11Lib);
  DrawX11Lib = NULL;
  FORALL (scale, ScaledFontList, next)
    MonetFree (scale);
  ScaledFontList = NULL;
}


/*****************************************************************/
/*****************************************************************/
/* $Header: //gaia/group.local/computer/monet/lib/xio/draw.c,v 1.1 1991/06/27 18:09:52 a38 Exp a38 $ */
/*****************************************************************/
/*****************************************************************/

