/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

#include "xmonetP.h"
#include <monet/msh/Channel.h>
#include <monet/msh/Script.h>

/*****************************************************************************/
/*  Handling listening to fds  */
/*****************************************************************************/

typedef struct input_fd * InputFd;
struct input_fd {
	int fd;
	XtInputId xtid_r, xtid_w;
	InputFd next;
	MonetFdHandler read_handler, write_handler;
	caddr_t data;
};

static InputFd InputFdList = NULL;

static InputFd find_fd (fd)
     int fd;
{
  InputFd i;
  FORALL(i, InputFdList, next)
    if (i->fd == fd)
      return i;
  return NULL;
}

/***************************************************************************/
/*  Handle list of inputs waiting for select  */
/***************************************************************************/

static void ReadHandler (closure, source, id)
    XtPointer closure;
    int*      source;
    XtInputId* id;
{
  InputFd call = (InputFd) closure;
  call->read_handler (call->fd,call->data);
}

static void WriteHandler (closure, source, id)
    XtPointer closure;
    int*      source;
    XtInputId* id;
{
  InputFd call = (InputFd) closure;
  call->write_handler (call->fd,call->data);
}

static void x_create_input_fd (fd, read_handler, write_handler, data)
     int fd;
     MonetFdHandler read_handler, write_handler;
     caddr_t data;
{
  InputFd input_fd;

  if (input_fd = find_fd (fd)) {
    if (input_fd->xtid_r) {
      XtRemoveInput (input_fd->xtid_r);
    }
    if (input_fd->xtid_w) {
      XtRemoveInput (input_fd->xtid_w);
    }
  } else {
    input_fd = MonetNew (struct input_fd);
    input_fd -> fd = fd;
    LIST_ADD (input_fd, InputFdList, next);
  }

  input_fd -> data = data;
  input_fd -> read_handler  = read_handler;
  input_fd -> write_handler = write_handler;
  input_fd -> xtid_r = NULL;
  input_fd -> xtid_w = NULL;

  if (fd < 0) {
    /* bad or psuedo fid */
  } else if (fd >= getdtablesize ()) {
    MonetSysError ("fd greater= than table size [%u]", fd, getdtablesize ());
  } else {
    if (read_handler) {
      input_fd -> xtid_r = XtAppAddInput (XmonetAppCon, fd,
					  (XtPointer) XtInputReadMask,
					  ReadHandler, input_fd);
    }
    if (write_handler) {
      input_fd -> xtid_w = XtAppAddInput (XmonetAppCon, fd,
					  (XtPointer) XtInputWriteMask,
					  WriteHandler, input_fd);
    }
  }
}

static void x_free_input_fd (fd)
     int fd;
{
  InputFd old;

  old = find_fd (fd);

  if (old == NULL) {
    MonetSysError ("x_free_input_fd: input fd not found %d", fd);
    return;
  }
  if (old->xtid_r != NULL)
    XtRemoveInput (old->xtid_r);
  if (old->xtid_w != NULL)
    XtRemoveInput (old->xtid_w);
  
  LIST_DEL (InputFd, old, InputFdList, next);
  close (fd);
  MonetFree (old);
}

/*****************************************************************************/
/* Start X client                                                            */
/* !!! Should probably load fonts and such like at this stage                */
/*****************************************************************************/

extern void xmonet_io_init ()
{
  Monet_set_input_fd_routines (x_create_input_fd, x_free_input_fd);
}

extern void xmonet_io_finish ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

