/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*  Rubber band routines  */
/*  This is a first attempt to generalise a set of routines that were        */
/*  originally embedded in the code for 'xmonet'. Hence their rather strange */
/*  structure.                                                               */
/*****************************************************************************/

#include <monet/msh/General.h>
#include <monet/x/Widget.h>

static GC XMonetRubberGC;

#define DRAW(x,y,width,height, elipse) \
  if (elipse) \
    XDrawArc(XmonetDisplay, XtWindow(widget), \
		 XMonetRubberGC, x, y, width, height, 0, 360 * 64); \
  else \
    XDrawRectangle(XmonetDisplay, XtWindow(widget), \
		 XMonetRubberGC, x, y, width, height);

/*****************************************************************************/
/*  Set up GC for rubber lines  */
/*  Done everytime in case the colours change  */
/*****************************************************************************/

static void setup_gc ()
{
  int             screen;
  XGCValues       values;
  unsigned long    mask;
  Pixel           background, foreground;

  XFlush(XmonetDisplay);

  /**** Set up back and foreground pixels ****/
  /* !!! Not guaranteed to work on colour or greyscale displays */
  screen = DefaultScreen(XmonetDisplay);
  background = BlackPixel(XmonetDisplay, screen);
  foreground = WhitePixel(XmonetDisplay, screen);
  /* !!! Hack */
  if (foreground == 0) {
    foreground = background;
    background = 0;
  }
  /**** Set up the GC's ****/
  mask  = GCForeground | GCBackground | GCLineStyle |
    GCLineWidth | GCFillStyle | GCJoinStyle | GCCapStyle | GCFunction;
  values.function   = GXxor;
  values.foreground = foreground;
  values.background = background;
  values.line_style = LineSolid;
  values.line_width = 1;
  values.cap_style  = CapRound;
  values.fill_style = FillSolid;
  values.join_style = JoinRound;
  XMonetRubberGC = XtGetGC(XmonetTopWidget, mask,  &values);
}

/*****************************************************************************/

/*****************************************************************************/

static void adjust_forGrid (grid_x, grid_y, x,y, newx,newy)
     unsigned grid_x, grid_y;
     Position x,y, *newx, *newy;
{
  *newx = (x >> grid_x) << grid_x;
  *newy = (y >> grid_y) << grid_y;
}

/*****************************************************************************/

/*****************************************************************************/

extern MonetBool Xio_rubberband_drag(widget, grid_x, grid_y, drag, done, event)
     Widget widget;
     XioDragCallback drag, done;
     unsigned grid_x, grid_y;
     XEvent *event;
{
  Position x, y, px, py;

  /*	SELECTINPUT(window); */

  Xmonet_sizing_cursor (widget);

  setup_gc ();
  px = event->xbutton.x;
  py = event->xbutton.y;
  adjust_forGrid (grid_x, grid_y, px, py, &px, &py);
  x = px;
  y = py;
  drag (px, py, x,y);
  for (;;) {
    if (event->type == ButtonRelease) {
      break;
    } else if (event->type == LeaveNotify) {
      MonetPrintfError ("Sorry, can't drag something out of a window");
      break;
    } else if (event->type == MotionNotify) {
      Position motion_x, motion_y;
      motion_x = event->xbutton.x;
      motion_y = event->xbutton.y;
      while (XPending(event->xmotion.display)) {
	XEvent nextEvent;
	XPeekEvent(event->xmotion.display, &nextEvent);
	if (nextEvent.type == MotionNotify &&
	    event->xmotion.window == nextEvent.xmotion.window &&
	    event->xmotion.subwindow == nextEvent.xmotion.subwindow) {
	  motion_x = event->xbutton.x;
	  motion_y = event->xbutton.y;
	  /* replace the current event with the next one */
	  XNextEvent(event->xmotion.display, event);
	} else {
	  break;
	}
      }
      if (event->type == ButtonRelease) {
	break;
      }
      drag (px, py, x,y);
      x = motion_x;
      y = motion_y;
      adjust_forGrid (grid_x, grid_y, x, y, &x, &y);
      drag (px, py, x,y);
    } else {
      /*  do nothing  */
    }
    XNextEvent (XmonetDisplay, event);
  }

  drag (px, py, x,y);
  Xmonet_normal_cursor (widget);
  if (px==x && py == y)
    return False;
  done (px, py, x,y);
  return True;
}

extern MonetBool Xio_rubberband_drag_rectangle (widget, grid_x, grid_y,
					      xloc, yloc, w, h, elipse, event)
     Widget widget;
     Position *xloc, *yloc;
     unsigned grid_x, grid_y;
     Dimension w, h;
     MonetBool elipse;
     XEvent *event;
{
  Position x, y;

  /*	SELECTINPUT(window) */

  Xmonet_sizing_cursor (widget);

  setup_gc ();
  x = *xloc;
  y = *yloc;

  DRAW(x, y, w, h, elipse);
  do {
    XNextEvent (XmonetDisplay, event);
    /* !!! Does no bound checking or anything */
    if (event->type == MotionNotify) {
      DRAW(x, y, w, h, elipse);
      x = event->xbutton.x;
      y = event->xbutton.y;
      adjust_forGrid (grid_x, grid_y, (Position) x, (Position) y, &x, &y);
      DRAW(x, y, w, h, elipse);
    } else if (event->type == LeaveNotify) {
      MonetPrintfError ("Sorry, can't drag something out of a window");
      break;
    }
  } while (event->type != ButtonRelease);
  DRAW(x, y, w, h, elipse);
  Xmonet_normal_cursor (widget);
  if (*xloc == x && *yloc == y)
    return False;
  *xloc = x;
  *yloc = y;
  return True;
}

static void draw_outlinebox (widget, xl,yl, xr,yr, elipse)
     Widget widget;
     Position xl,yl, xr,yr;
     MonetBool elipse;
{
  Position x,y;
  Dimension w,h;

  if (xl < xr)
    x = xl;
  else
    x = xr;
  if (yl < yr)
    y = yl;
  else
    y = yr;
  w = abs (xr-xl);
  h = abs (yr-yl);

  DRAW(x, y, w, h, elipse);
}

extern MonetBool Xio_rubberband_makebox (widget, grid_x, grid_y, xloc, yloc,
					 width, height, elipse, event)
     Widget widget;
     unsigned grid_x, grid_y;
     Position *xloc, *yloc;
     Dimension *width, *height;
     MonetBool elipse;
     XEvent *event;
{
  /* The bounds of the box */
  Position x, y;
  Dimension w,h;

  /* The cursor Position */
  Position cx, cy;
  Position fixedx, fixedy;

  Xmonet_sizing_cursor (widget);


  setup_gc ();

  x = *xloc;
  y = *yloc;
  w = *width;
  h = *height;
  cx = event->xbutton.x;
  cy = event->xbutton.y;

  if (cx < x + w /2) {
    fixedx = x + w;
  } else {
    fixedx = x;
  }
  if (cy < y + h /2) {
    fixedy = y + h;
  } else {
    fixedy = y;
  }

  draw_outlinebox (widget, cx,cy, fixedx,fixedy, elipse);
  for (;;) {
    /* !!! Does no bound checking or anything */
    if (event->type == ButtonRelease) {
      break;
    } else if (event->type == LeaveNotify) {
      MonetPrintfError ("Sorry, can't resize something out of a window");
      break;
    } else if (event->type == MotionNotify) {
      draw_outlinebox (widget, cx,cy, fixedx,fixedy, elipse);
      cx = event->xbutton.x;
      cy = event->xbutton.y;
      adjust_forGrid (grid_x, grid_y, (Position) cx, (Position) cy, &cx, &cy);
      draw_outlinebox (widget, cx,cy, fixedx,fixedy, elipse);
    }
    XNextEvent (XmonetDisplay, event);
  }
  
  draw_outlinebox (widget, cx,cy, fixedx,fixedy, elipse);  

  Xmonet_normal_cursor (widget);

  if (cx < fixedx) {
    x = cx;
    w = fixedx - cx;
  } else {
    x = fixedx;
    w = cx - fixedx;
  }
  if (cy < fixedy) {
    y = cy;
    h = fixedy - cy;
  } else {
    y = fixedy;
    h = cy - fixedy;
  }
  if (*xloc == x && *yloc == y && *width == w && *height == h)
    return False;
  *xloc = x;
  *yloc = y;
  *width = w;
  *height = h;
  return True;
}

/*****************************************************************************/

/*****************************************************************************/




/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
