/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

#include <monet/msh/General.h>
#include <monet/x/Widget.h>
#include <monet/msh/Script.h>
#include <X11/bitmaps/gray>
#include <sys/dir.h>

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/


/*************************************************************************/
/*  Globals  */
/*************************************************************************/

static Widget Stdout_widget = (Widget) 0;
static Widget Stderr_widget = (Widget) 0;

/*************************************************************************/
/*  Find widget  */
/*************************************************************************/

extern Widget Xmonet_get_widget (name)
     String name;
{
  Widget widget;
  char  clean_name[1024]; /* !!! */

  if (name) {
    WcCleanName (name, clean_name);
    widget = WcFullNameToWidget( XmonetCurrentWidget, name );
    if ( ! widget) {
      MonetPrintfError ("No such widget [%s]", name);
    }
    return widget;
  }
  return NULL;
}

extern Widget Xmonet_sub_widget (name, wclass, class_name)
     String name, class_name;
     WidgetClass wclass;
{
  Widget w;
  if (name) {
    if (w = Xmonet_get_widget (name)) {
      if ( ! XtIsSubclass (w, wclass)) {
       MonetPrintfError ("Widget [%s] is not %s", name, class_name);
       return NULL;
     }
    }
    return w;
  }
  return NULL;
}

/*************************************************************************/
/*  Menupopup  */
/*************************************************************************/

static Widget my_XtFindPopup(widget, name)
    Widget widget;
    String name;
{
    register Cardinal i;
    register XrmQuark q;
    register Widget w;

    q = XrmStringToQuark(name);

    for (w=widget; w != NULL; w=w->core.parent)
        for (i=0; i<w->core.num_popups; i++)
            if (w->core.popup_list[i]->core.xrm_name == q)
                return w->core.popup_list[i];

    return NULL;
}

MONETCALLBACK(do_xt_menu_popup)
     char *params[2];
     Widget widget;
     XEvent *event;
     MonetBool spring_loaded;
     register Widget popup_shell;

     if (! (event = XmonetCurrentEvent)) {
       MonetPrintfError ("No current event, must call from MonetACT");
       RETURNSC;
     }

     if (! (widget = XmonetCurrentWidget)) {
       MonetPrintfError ("No current widget, must call from MonetACT");
       RETURNSC;
     }

     params[0] = argv[1];
     params[1] = NULL;

     if (event->type == ButtonPress) {
       spring_loaded = True;
     } else if (event->type == KeyPress || event->type == EnterNotify) {
       spring_loaded = False;
     } else {
       XtAppWarningMsg(XtWidgetToApplicationContext(widget),
		       "invalidPopup","unsupportedOperation","XtToolkitError",
		       "Pop-up menu creation is only supported on ButtonPress, KeyPress or EnterNotify events.",
		       (String *)NULL, (Cardinal *)NULL);
       spring_loaded = False;
     }
     
     popup_shell = my_XtFindPopup(widget, params[0]);


     if (popup_shell == NULL) {
       Cardinal num;
       num = 1;
       XtAppWarningMsg(XtWidgetToApplicationContext(widget),
		       "invalidPopup","xtMenuPopup","XtToolkitError",
		       "Can't find popup widget \"%s\" in XtMenuPopup",
		       params, &num);
       RETURNSC;
     }
     spring_loaded = True;
     XGrabButton (XtDisplay (popup_shell),
		  Button1,
		  AnyModifier,
		  XtWindow (popup_shell),
		  True,
		  ButtonPressMask | ButtonReleaseMask,
		  GrabModeAsync, GrabModeAsync, None,
		  None);
     if (spring_loaded) {
       XtPopupSpringLoaded (popup_shell);
     } else {
       XtPopup (popup_shell, XtGrabNonexclusive);
     }
MONETEND

/*************************************************************************/
/*  Menu option  */
/*************************************************************************/

MONETCALLBACK(do_xt_menu_option)
     Widget menu;
     if (menu =
	 Xmonet_sub_widget (argv[1],simpleMenuWidgetClass,
			    "a simpleMenu widget")) {
       if (XtCreateManagedWidget(argv[2], smeBSBObjectClass,
				 menu, (Arg *) 0, (Cardinal) 0)) {
	 char buf [2048];
	 (void) sprintf (buf, "MonetACT(%s.%s.callback: %s)",
			 argv[1], argv[2], argv[3]);
	 WcSetValueCB (NULL, buf, NULL);
       } else {
	 MonetPrintfError ("Could not create menu option [%s]", argv[2]);
       }
     }
MONETEND

MONETCALLBACK(do_xt_set_child_value)
     CompositeWidget parent;
     if (!argv[2]) {
       MonetPrintfError ("Resource is null");
     } else if (!argv[3]) {
       MonetPrintfError ("value is null");
     } else if (parent = (CompositeWidget)
		Xmonet_sub_widget (argv[1], compositeWidgetClass,
				   "a composite widget")) {
       int num_children = parent->composite.num_children;
       WidgetList children = parent->composite.children;
       Widget *childP;
       for (childP = children; childP - children < num_children; childP++) {
	 WcSetValueFromString (*childP, argv[2],  argv[3]);
       }
     }
MONETEND

MONETCALLBACK(do_xt_set_value)
     if (argv[1]) {
       WcSetValueCB (XmonetTopWidget, argv[1], NULL);
     } else {
       /*  Ignore  */
     }
MONETEND

MONETCALLBACK(do_xt_menu_line)
     Widget menu;
     if (menu =
	 Xmonet_sub_widget (argv[1],simpleMenuWidgetClass,
			    "a simpleMenu widget")) {
       if (XtCreateManagedWidget("line", smeLineObjectClass,
				 menu, (Arg *) 0, (Cardinal) 0)) {
       } else {
	 MonetPrintfError ("Could not create menu line [%s]", argv[2]);
       }
     }
MONETEND

/*************************************************************************/
/*  Write to the stderr widget  */
/*************************************************************************/

static void xio_write_stderr (form, len)
     char * form;
     int len;
{
  if (form && *form) {
    if (Stderr_widget) {
      Xmonet_put_widget (Stderr_widget, form, len);
    } else {
      (void) fwrite (form, 1, len, stderr);
    }
  }
}

MONETCALLBACK(do_SetStderrWidgetCB)
     String w_name = argv[1];

     Widget text_widget;
     char  clean_name[1024]; /* !!! */

     if (w_name == NULL) {
       MonetPrintfError ("Stderr text widget name is NULL");
       RETURNSC;
     }
     WcCleanName (w_name, clean_name);
     if ( !XmonetTopWidget) {
       MonetSysError ("Toplevel widget is NULL");
       RETURNSC;
     }
     text_widget = WcFullNameToWidget (XmonetTopWidget, clean_name);
     if ( ! text_widget) {
       MonetPrintfError ("No such text widget [%s]", clean_name);
       RETURNSC;
     }
     if ( ! XtIsSubclass (text_widget, textWidgetClass)) {
       MonetPrintfError ("Stderr Widget [%s] is not an text widget",
			 clean_name);
       RETURNSC;
     }
     (void) MonetStderrProc_set (xio_write_stderr);
     Stderr_widget = text_widget;
MONETEND

/*************************************************************************/
/*  Write to the stdout widget */
/*************************************************************************/

static void xio_Write_Stdout (form, len)
     char * form;
     int len;
{
  if (Stdout_widget) {
    Xmonet_put_widget (Stdout_widget, form, len);
  } else {
    (void) fwrite (form, 1, len, stdout);
  }
}

MONETCALLBACK(do_SetStdoutWidget)
     String w_name = (String) argv[1];

     Widget text_widget;
     char  clean_name[1024]; /* !!! */

     if (w_name == NULL) {
       MonetPrintfError ("Stderr text widget name is NULL");
     } else {
       WcCleanName (w_name, clean_name);
       if ( !XmonetTopWidget) {
	 MonetSysError ("Toplevel widget is NULL");
       } else {
	 text_widget = WcFullNameToWidget (XmonetTopWidget, clean_name);
	 if ( ! text_widget) {
	   MonetPrintfError ("No such text widget [%s]", clean_name);
	 } else if ( ! XtIsSubclass (text_widget, textWidgetClass)) {
	   MonetPrintfError ("Stdout Widget [%s] is not an text widget",
			     clean_name);
	 } else {
	   (void) MonetStdoutProc_set (xio_Write_Stdout);
	   Stdout_widget = text_widget;
	 }
       }
     }
MONETEND

/*****************************************************************************/
/*  Get x and y position of cursor when current action started  */
/*****************************************************************************/
/*
MONETCALLBACK(do_get_x)

MONETEND
*/
/*****************************************************************************/
/*  Disable/force layout of a form  */
/*****************************************************************************/

MONETCALLBACK(do_xt_form_do_layout)
     Widget w;

     if (w = Xmonet_get_widget (argv[1])) {
       XawFormDoLayout (w, String2MonetBool (argv[2]));
     }
MONETEND

/*****************************************************************************/
/*  Get Shell Ancestor  */
/*****************************************************************************/

extern ShellWidget XmonetShellAncestor (widget, toplevel)
     Widget widget;
     MonetBool * toplevel;
{
  while (widget) {
    if (XtIsSubclass (widget, shellWidgetClass)) {
      ShellWidget shell = (ShellWidget) widget;
      if (toplevel)
	*toplevel = widget == XmonetTopWidget;
      return shell;
    }
    widget = XtParent (widget);
  }
  return NULL;
}

/*****************************************************************************/
/*  Is widget popped up  */
/*****************************************************************************/

MONETCALLBACK(do_xt_is_popped_up)
     Widget widget;
     if (widget = Xmonet_get_widget (argv[1])) {
       ShellWidget shell = (ShellWidget) widget;
       if (shell = XmonetShellAncestor (widget, (MonetBool *) 0)) {
	 if (! shell -> shell.popped_up) {
	   result = NULL;
	 }
	 RETURNSC;
       }
       MonetPrintfError ("[%s] has no shell ancestor", argv[1]);
     }
     result = NULL;
MONETEND

/*****************************************************************************/
/*  Is widget popped up  */
/*****************************************************************************/

MONETCALLBACK(do_xt_parent)
     Widget widget;
     if (widget = Xmonet_get_widget (argv[1])) {
       if (widget = XtParent (widget)) {
	 result = NULL;
       } else {
	 result = WcWidgetToFullName (widget);
       }
     }
MONETEND

/******************************************************************/
/* directory listing  */
/******************************************************************/

MONETCALLBACK(do_list_current)
     Widget w;
     result = NULL;
     if (w = Xmonet_sub_widget (argv[1],listWidgetClass,"a list widget")) {
       XawListReturnStruct * r;
       if (r = XawListShowCurrent (w)) {
	 result = r->string;
       }
     }
MONETEND

/******************************************************************/
/* directory listing, only alter if changed  */
/******************************************************************/

MONETCALLBACK(do_list_dir)
     Widget w;
     if (w = Xmonet_sub_widget (argv[1],listWidgetClass,"a list widget")) {
       char * dir, * pattern, ** list;
       static char ** InPlaceList = NULL;
       DIR * dirp;
       struct direct *dp;
       static int Matched = 0;
       int matched;
       int  longest = 0;

       matched = 0;
       list = (char **) MonetMalloc (8 * sizeof (char *));
       pattern = argv[3];
       dir = argv[2];
       dir = MonetFilename (dir);
       dirp = opendir(dir);
       if (dirp) {
	 for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	   if (MonetMatchWildcard (dp->d_name, pattern)) {
	     int len = MonetStrlen (dp->d_name);
	     if (len > 0) {
	       list[matched] = MonetNewString (dp->d_name);
	       ++ matched;
	       list = (char **) MonetRealloc
		 (list, matched * sizeof(char *)+1);
	       if (len > longest)
		 longest = len;
	     }
	   }
	 closedir(dirp);
	 if (matched == 0) {
	   MonetPrintfError ("Found no files matching [%s] in directory [%s]",
			     pattern, dir);
	 }
	 list[matched] = 0;
	
	 /*  Checking for equality is probably less expensive than drawing
	     the stuff and besides it gets rid of an irritating flash.  */
	 if (! MonetEqStringOfStrings (InPlaceList, list, Matched, matched)) {
	   /*  Longest has to be in pixels unfortunately  */
	   XawListChange (w, list, matched, 0 /*longest*/, True);
	   MonetFreeStringOfStrings (InPlaceList, Matched);
	   Matched = matched;
	   InPlaceList = list;
	 } else {
	   MonetFreeStringOfStrings (list, matched);
	 }
       } else {
	 MonetPrintfError ("Could not access directory [%s]", dir);
       }
     }
MONETEND

/*****************************************************************************/
/*  Create widgets */
/*****************************************************************************/

MONETCALLBACK(do_xt_create)
     WcCreateChildrenCB ( XmonetCurrentWidget,
			 MonetJoinStrings (argc--, argv+1, " "), NULL);
MONETEND

/*****************************************************************************/
/*    Manage and Unmanage widgets  */
/*****************************************************************************/

MONETCALLBACK(do_xt_ismanaged)
	MonetPrintfWarning ("Not implemented yet");
/*
       Widget w;
       if (w = Xmonet_get_widget ( argv[argc] ))
	 XtManageChild( w );
	else {
		result = NULL;
     }
*/
MONETEND

MONETCALLBACK(do_xt_manage)
     Widget w;
     int i;
     for (i = 1; i < argc; ++i)
       if (w = Xmonet_get_widget ( argv[i] ))
	 XtManageChild( w );
MONETEND

MONETCALLBACK(do_xt_unmanage)
     Widget w;
     int i;
     for (i = 1; i < argc; ++i)
       if (w = Xmonet_get_widget ( argv[i] ))
	 XtUnmanageChild( w );
MONETEND

/*****************************************************************************/
/*    This function destroys the widget tree.  */
/*****************************************************************************/

MONETCALLBACK(do_x_destroy_shell)
     while (argc > 1) {
       Widget condemned;
       --argc;
       if (condemned = Xmonet_sub_widget
	   ( argv[argc], shellWidgetClass, "a shell" ))
	 XtDestroyWidget( condemned );
     }
MONETEND

MONETCALLBACK(do_xt_destroy)
     while (argc > 1) {
       Widget condemned;
       --argc;
       if (condemned = Xmonet_get_widget ( argv[argc] ))
	 XtDestroyWidget( condemned );
     }
MONETEND

/*****************************************************************************/
/*  Realize a widget  */
/*****************************************************************************/

MONETCALLBACK(do_xt_realize)
     while (argc > 1) {
       Widget condemned;
       --argc;
       if (condemned = Xmonet_get_widget ( argv[argc] ))
	 XtRealizeWidget( condemned );
     }
MONETEND

/*****************************************************************************/
/*  Disable/force layout of a form  */
/*****************************************************************************/

MONETCALLBACK(do_xt_install_accelerators)
     String widget_name;
     Widget source, dest;
     char  clean_name[1024]; /* !!! */

     widget_name = argv[1];
     WcCleanName (widget_name, clean_name);
     dest = WcFullNameToWidget (XmonetTopWidget, clean_name);
     if ( ! dest) {
       MonetPrintfError ("No such widget [%s]", widget_name);
     }
     widget_name = argv[2];
     WcCleanName (widget_name, clean_name);
     source = WcFullNameToWidget (XmonetTopWidget, clean_name);
     if ( ! source) {
       MonetPrintfError ("No such widget [%s]", widget_name);
     }
     XtInstallAccelerators (dest, source);
MONETEND

/*****************************************************************************/
/*  Returns the name of a widget  */
/*****************************************************************************/

MONETCALLBACK(do_x_name_widget)
     String widget_name;
     Widget w;
     char  clean_name[1024]; /* !!! */

     widget_name = argv[1];
     WcCleanName (widget_name, clean_name);
     w = WcFullNameToWidget (XmonetCurrentWidget, clean_name);
     if ( ! w) {
       MonetPrintfError ("No such widget [%s]", widget_name);
     } else 
       result = XtName (w);
MONETEND

/*****************************************************************************/
/*  Searchs for a widget  */
/*****************************************************************************/

MONETCALLBACK(do_x_name_find)
     String widget_name;
     Widget w;
     char  clean_name[1024]; /* !!! */

     widget_name = argv[1];
     WcCleanName (widget_name, clean_name);
     w = WcFullNameToWidget (XmonetCurrentWidget, clean_name);
     if ( ! w) {
       result = NULL;
     } else 
       result = XtName (w);
MONETEND

/*****************************************************************************/
/*  Loads file into widget  */
/*****************************************************************************/

MONETCALLBACK(do_x_load_widget)
     String widget_name, filename;
     Widget text, scrolwin;
     FILE *file;
#define STRINGLEN 8192
     int count;
     Arg args[9];
     Dimension width, height;
     XFontStruct * font;
     char  clean_name[1024]; /* !!! */

     widget_name = argv[1];
     filename = argv[2];

     text = Xmonet_get_widget (widget_name);
     if ( ! text) {
       RETURNSC;
     }
     if ( ! XtIsSubclass (text, asciiTextWidgetClass)) {
       MonetPrintfError ("[%s] is not an ascii text widget", widget_name);
       RETURNSC;
     }
     scrolwin = XtParent (text);
     while (scrolwin = XtParent (scrolwin))
       if (XtIsSubclass (scrolwin, shellWidgetClass))
	 break;
     if (! scrolwin) {
       MonetPrintfError ("Shell Parent of text widget does not exist");
       RETURNSC;
     }

     if (!(file = MonetFopen (filename, "r"))) {
       MonetPrintfError ("File [%s] not found", filename);
       RETURNSC;
     }

     if ( ! XtIsRealized (scrolwin)) {
       XtRealizeWidget (scrolwin);
     }

     if (file) {
       static char  help_buf[STRINGLEN];
       String place = help_buf;
       help_buf [1] = '\0';
       height = 0;
       while (fgets(place, STRINGLEN - (place - help_buf) - 1, file)) {
	 height ++;
	 place += MonetStrlen (place);
	 if (place - help_buf >= STRINGLEN) {
	   MonetSysError ("do_x_load_widget: buffer overflow");
	   break;
	 }
       }
       (void) fclose(file);
       *place = '\0';
       count = 0;
       XtSetArg(args[count], XtNwidth, &width); count++;
       XtGetValues(scrolwin, args, count);
       MONETDEBUG(5, ( "do_x_load_widget: width %d", width));
       count = 0;
       XtSetArg(args[count], XtNfont, &font); count++;
       XtGetValues(text, args, count);
       height *= (font->ascent + font->descent);
       MONETDEBUG(5, ( "do_x_load_widget: height %d", (int) height));
       MONETDEBUG(5, ( "do_x_load_widget: font %d %d",
		   (int) font->ascent, (int) font->descent));
       count = 0;
       XtSetArg(args[count], XtNtype, XawAsciiString); count++;
       XtSetArg(args[count], XtRAsciiType, XtEstring); count++;
       XtSetArg(args[count], XtNheight, height); count++;
       XtSetValues(text, args, count);
       count = 0;
       XtSetArg(args[count], XtNstring, ""); count++;
       XtSetValues(text, args, count);

       count = 0;
       XtSetArg(args[count], XtNwidth, width - 18); count++;
       XtSetArg(args[count], XtNheight, height); count++;
       XtSetValues(text, args, count);
       count = 0;
       XtSetArg(args[count], XtNstring, help_buf); count++;
       XtSetValues(text, args, count);
     }
MONETEND

/*****************************************************************************/
/*  Set the value of a text widget  */
/*****************************************************************************/

MONETCALLBACK(do_x_set_widget)
     char *value;
     /*  Fills in a text or dialog widgets value with the  */
     /*  result of a monet script function.  */
     if (argc == 2) {
       value = "";
     } else {
       value = argv [2];
     }
     TextWidget_set_value (XmonetCurrentWidget, argv[1], value);
MONETEND

MONETCALLBACK(do_x_get_widget)
     result = XmonetWidget2value (XmonetCurrentWidget, argv[1]);
MONETEND

/*****************************************************************************/

/*****************************************************************************/
 
static char ** prepare_shell_command (command)
     char * command;
{
  static char *argv[4];

  {
    /*
     * Build a display string using the current screen number, so that
     * X programs which get fired up from a menu come up on the screen
     * that they were invoked from, unless specifically overridden on
     * their command line.
     */
    static char buf[256];
    char *ds = DisplayString (XmonetDisplay);
    char *colon, *dot1;

    colon = rindex (ds, ':');
    if (colon) {			/* if host[:]:dpy */
      (void) strcpy (buf, "DISPLAY=");
      (void) strcat (buf, ds);
      colon = buf + 8 + (colon - ds);	/* use version in buf */
      dot1 = index (colon, '.');	/* first period after colon */
      if (!dot1) dot1 = colon + MonetStrlen (colon);  /* if not there,append */
      (void) sprintf (dot1, ".0");
      putenv (buf);
    }
  }
  argv[0] = "/bin/sh";
  argv[1] = "-c";
  argv[2] = command;
  argv[3] = NULL;
  
  return argv;
}

/*****************************************************************************/
/*  The following callbacks for positioning popups was adapted from the  */
/*  MIT source  */
/*****************************************************************************/
/*
 * Copyright 1989 Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */



/*	Function Name: MoveMenu
 *	Description: Actually moves the menu, may force it to
 *                   to be fully visable if menu_on_screen is TRUE.
 *	Arguments: w - the simple menu widget.
 *                 x, y - the current location of the widget.
 *	Returns: none 
 */

static void
MoveMenu(w, x, y)
Widget w;
Position x, y;
{
	Arg arglist[3];
	Cardinal num_args = 0;
    
	int width = w->core.width + 2 * w->core.border_width;
	int height = w->core.height + 2 * w->core.border_width;
	
	if (x < 0) 
	    x = 0;
	else {
	    int scr_width = WidthOfScreen(XtScreen(w));
	    if (x + width > scr_width)
		x = scr_width - width;
	}
	
	if (y < 0)
	    y = 0;
	else {
	    int scr_height = HeightOfScreen(XtScreen(w));
	    if (y + height > scr_height)
		y = scr_height - height;
	}
    
    XtSetArg(arglist[num_args], XtNx, x); num_args++;
    XtSetArg(arglist[num_args], XtNy, y); num_args++;
    XtSetValues(w, arglist, num_args);
}

extern MonetBool Xmonet_pointer (widget, x, y)
     Widget widget;
     Position *x, *y;
{
  Window junk1, junk2;
  int root_x, root_y;
  unsigned int junkM;

  if (XQueryPointer(XmonetDisplay, XtWindow(widget),
      &junk1, &junk2, &root_x, &root_y, (int *) x, (int *) y, &junkM))
    return True;
  return False;
}

MONETCALLBACK(do_PopupAtCursor)
     String w_name = argv[1];
     Widget popup;
     Position x, y;
     Window junk1, junk2;
     int root_x, root_y, win_x, win_y;
     unsigned int junkM;
     
     if ( ! (popup = XmonetWidget2Popup (XmonetCurrentWidget, w_name))) {
       RETURNSC;
     }
     /* The width will not be correct unless it is realized. */
     if (!XtIsRealized(popup)) {
       XtRealizeWidget(popup);
     }
     if (XQueryPointer(XtDisplay(popup), XtWindow(XmonetTopWidget),
		       &junk1, &junk2, 
		       &root_x, &root_y, &win_x, &win_y, &junkM) == False) {
       MonetPrintfError ("Could not find location of mouse pointer");
       RETURNSC;
     }

     x = XmonetTopWidget->core.x + win_x;
     y = XmonetTopWidget->core.y + win_y;
     /*
       x = (Position) root_x;
       y = (Position) root_y;
       */    
     x -= (Position) popup->core.width/2;

     XtPopdown (popup);
     MoveMenu(popup, x, y);
     XtPopup (popup, XtGrabNone);
     if (XtWindow(popup)) {
       /*  Even if already popped up, put it on top  */
       XRaiseWindow (XmonetDisplay, XtWindow(popup));
     }
MONETEND

MONETCALLBACK(do_PopupCommandPopup)
     String w_name = argv[1];
     Widget popup, widget;
     Arg arglist[3];
     Cardinal num_args;
     int popup_x, popup_y, popup_width, popup_height,
       button_width, button_height;
     Position button_x, button_y;

     widget = XmonetCurrentWidget;
     if ( ! (popup = XmonetWidget2Popup (widget, w_name))) {
       RETURNSC;
     }

     if (!XtIsRealized(popup)) {
       XtRealizeWidget(popup);
     }
  
     popup_width = popup->core.width + 2 * popup->core.border_width;
     button_width = widget->core.width + 2 * widget->core.border_width;
     button_height = widget->core.height + 2 * widget->core.border_width;
     popup_height = popup->core.height + 2 * popup->core.border_width;
     
     XtTranslateCoords(widget, 0, 0, &button_x, &button_y);
     popup_x = button_x;
     popup_y = button_y + button_height;

     if (popup_x < 0) {
       popup_x = 0;
     } else {
       int scr_width = WidthOfScreen(XtScreen(popup));
       if (popup_x + popup_width > scr_width)
	 popup_x = scr_width - popup_width;
     }

     if (popup_y < 0) {
       popup_y = 0;
     } else {
       int scr_height = HeightOfScreen(XtScreen(popup));
       if (popup_y + popup_height > scr_height)
	 popup_y = scr_height - popup_height;
     }
     
     num_args = 0;
     XtSetArg(arglist[num_args], XtNx, popup_x); num_args++;
     XtSetArg(arglist[num_args], XtNy, popup_y); num_args++;
     XtSetValues(popup, arglist, num_args);

     if (argc > 2 && String2MonetBool (argv[2])) {
       XtPopupSpringLoaded(popup);
     } else {
       XtPopup(popup, XtGrabNone);
     }
     if (XtWindow(popup)) { /*  Even if already popped up, put it on top  */
       XRaiseWindow (XmonetDisplay, XtWindow(popup));
     }
MONETEND

MONETCALLBACK(do_PopupCentral)
     Widget popup;

     if ((popup = XmonetWidget2Popup (XmonetCurrentWidget, argv[1]))) {
       XMonetPopupCentral (XmonetTopWidget, popup, XtGrabNonexclusive);
   }
MONETEND


MONETCALLBACK(do_busy)
     if (argv[1]) {
       Xmonet_wait_cursor (NULL);
     } else {
       Xmonet_normal_cursor (NULL);
     }
MONETEND

MONETCALLBACK(do_graypixmap)
     while (argc > 1) {
       --argc;
     (void) XMonetSetWidgetPixmap
       (argv[argc], False, gray_bits, gray_width, gray_height);
     }
MONETEND

MONETCALLBACK(do_colours)
     Widget w;
     if (w = Xmonet_get_widget (argv[1])) {
       WcSetValueFromString (w, "foreground",  argv[2]);
       WcSetValueFromString (w, "background",  argv[3]);
     }
MONETEND

/*****************************************************************************/
/*  Init and finish  */
/*****************************************************************************/

extern void xmonet_init_script ()
{

  (void) MonetMember_module_class ("xwidget");

  MonetMember_create ("popup-under-command",	 do_PopupCommandPopup,
		      "widget-name ?Boolean grab",
		      "Popups up a widget underneath a command widget. \
Default grab is no grab.");
  MonetMember_create ("popup-at-cursor",	 do_PopupAtCursor,
		      "widget-name",
		      "Popups up a widget under the cursor");
  MonetMember_create ("popup-central",	 do_PopupCentral,
		      "widget-name",
		     "Popups a widget in the centre of the application shell");
  MonetMember_create ("gray",	 do_graypixmap,
		      "widget  ...",
		   "Sets the background pixmap of widgets to X11/bitmap/gray");
  MonetMember_create ("colours",	 do_colours,
		      "widget  foreground background",
		     "Sets the foreground and background colours of a widget");
  MonetMember_create ("get",	 do_x_get_widget,
		      "name",
		      "Returns the value of a label, text or dialog widget");
  MonetMember_create ("set",	 do_x_set_widget,
		      "name ?value",
		      "Set the value of a label, text or dialog widget. \
Default value is the empty string");
  MonetMember_create ("load",	 do_x_load_widget,
		      "widget filename",
		      "Loads the contents of a file into a text widget");
  MonetMember_create ("name",	 do_x_name_widget,
		      "widget",
		      "Returns the last part of the name of a widget");
  MonetMember_create ("find",	 do_x_name_find,
		      "widget",
		     "Succeeds if it finds the named widget, false otherwise");
  MonetMember_create ("stdout", 
		      do_SetStdoutWidget,
		      "widget",
		      "Sets the widget used for sending printf");
  MonetMember_create ("stderr", 
		      do_SetStderrWidgetCB,
		      "widget",
		      "Sets the widget used for sending printf");
  MonetMember_create ("is-popped-up",  do_xt_is_popped_up,
		      "widget",
		      "True if widget or its shell ancestor is popped up");
  MonetMember_create ("form-do-layout",  do_xt_form_do_layout,
		      "widget do_layout",
		      "Executes the X toolkit function XawFormDoLayout");
  MonetMember_create ("install-accelerators", 
		      do_xt_install_accelerators,
		      "destination widget source-widget",
		      "Executes the X toolkit function XtInstallAccelerators");
  MonetMember_create ("realize", 
		      do_xt_realize,
		      "widget ...",
		      "Executes the X toolkit function XtRealizeWidget");
  MonetMember_create ("destroy-widget", 
		      do_xt_destroy,
		      "widget ...",
		      "Executes the X toolkit function XtDestroyWidget");
  MonetMember_create ("destroy-shell", 
		      do_x_destroy_shell,
		      "widget ...",
		      "Destroys a shell widget");
  MonetMember_create ("menu-option", 
		      do_xt_menu_option,
		      "widget label command",
     "Adds an option to a simple menu widget. \
The command is a monet shell script");
  MonetMember_create ("menu-popup", 
		      do_xt_menu_popup,
		      "widget",
		      "Popups up a shell widget.  See XtMenuPopup (3Xt)");
  MonetMember_create ("menu-line", 
		      do_xt_menu_line,
		      "widget",
		      "Adds an 'line' to a simple menu widget");
  MonetMember_create ("set-value", 
		      do_xt_set_value,
		      "widget_name.res:value",
		      "Sets value of a widget resource");
  MonetMember_create ("set-child-values",
		      do_xt_set_child_value,
		      "parent resource value",
		      "Sets resource values of all of a widgets children");
  MonetMember_create ("create", 
		      do_xt_create,
		      "parent children ...",
		      "Creates widgets");
  MonetMember_create ("manage", 
		      do_xt_manage,
		      "children ...",
		      "Manage widgets");
  MonetMember_create ("unmanage", 
		      do_xt_unmanage,
		      "children ...",
		      "UnManage widgets");
  MonetMember_create ("ismanaged", 
		      do_xt_ismanaged,
		      "child",
		      "Returns true if the child widget is managed");
  MonetMember_create ("parent", 
		      do_xt_parent,
		      "child",
		      "Returns the parent of the child widget");
  MonetMember_create ("ls", do_list_dir,
		      "listwidget directory pattern",
		      "Fills listwidget with directory contents");
  MonetMember_create ("list-current", do_list_current,
		      "listwidget",
		      "Returns currently highlighted item in list widget. \
See XawListShowCurrent(3Xt)");
  MonetMember_create ("busy", do_busy,
		      "boolean",
		      "Sets/Resets the busy cursor and enables/disables. \
Interupting by the cursor.");

}

extern void xmonet_finish_script ()
{
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/
