/*****************************************************************************/
/*****************************************************************************/
/* $HEADER$ */
/*****************************************************************************/
/*****************************************************************************/

#include "xmonetP.h"
#include <monet/msh/Script.h>
#include <monet/msh/System.h>

#include <X11/Xmu/Error.h>

/*****************************************************************************/
/*  I don't particularly like making these global */
/*****************************************************************************/

Widget          XmonetTopWidget;
XtAppContext    XmonetAppCon;
Display        *XmonetDisplay;
Widget   XmonetCurrentWidget = NULL;
XEvent * XmonetCurrentEvent = NULL;
Position XmonetCurrentX = 0, XmonetCurrentY = 0;

/*****************************************************************************/
/*  Error handler  */
/*****************************************************************************/

static int myhandler (display, myerr)
     Display *display;
     XErrorEvent *myerr;
{
  (void) XmuPrintDefaultErrorMessage (display, myerr, stderr);
  return 0;
}

/*****************************************************************************/
/*  Return position of widget  */
/*****************************************************************************/

extern void Widget_get_dimensions (w, x,y,width,height)
	Widget w;
	Position *x,*y;
	Dimension *width, *height;
{
	/*  If not realised then core values are rubbish  !!! */
	if (w != (Widget) 0) {
		*x = w->core.x;
		*y = w->core.y;
		*width = w->core.width;
		*height = w->core.height;
	} else {
		*x = 0;
		*y = 0;
		*width = 0;
		*height = 0;
	}
}

/*****************************************************************************/
/*  Utility to write messages to end of text widgets */
/*****************************************************************************/

extern void Xmonet_put_widget (w, add_this, length)
     Widget w;
     String add_this;
     int length;
{
  XawTextBlock text;
  /* !!!  This depends on the widget used */
  static XawTextPosition pos = 0;

  if (w && add_this && *add_this && length > 0 && XtIsRealized (w)) {
    text.length = length;
    text.ptr = add_this;
    text.firstPos = 0;
    text.format = FMT8BIT;

    XawTextReplace (w, pos, pos, &text);

    pos += text.length;
    XawTextSetInsertionPoint(w, pos);
  }
}

/*****************************************************************************/
/*  Resize child widget, simplified  */
/*****************************************************************************/

extern XtGeometryResult XmonetMakeGeometryRequest (widget, request, reply)
    Widget         widget;
    XtWidgetGeometry *request, *reply;
{
    XtGeometryResult r;

    r = XtMakeGeometryRequest(widget, request, reply);
    if (r == XtGeometryAlmost) {
      XEvent event;
      /*  Remove any expose events already in the queue, no  */
      /*  need to repeat ourselves.  */
      while (XCheckWindowEvent(XmonetDisplay,
			       XtWindow (widget),Expose, &event));
      /* -- If can't get exact values then make do with those given */
      (void) XtMakeGeometryRequest(widget, reply, NULL);
    } else {
      reply->width = request->width;
      reply->height = request->height;
      reply->x = request->x;
      reply->y = request->y;
    }
    return r;
} /* XmonetMakeResizeRequest */

/*****************************************************************************/
/*  Set pix map value to one widget  */
/*****************************************************************************/

static void FreeWidgetPixmap(w, closure, calldata)
     Widget w;
     XtPointer closure, calldata;
{
  XFreePixmap(XtDisplay (w), (Pixmap) closure);
}


extern MonetBool XMonetSetWidgetPixmap (name, set_sides, bits, width, height)
     String name;
     char *bits;
     Cardinal width, height;
     MonetBool set_sides;
{
  Widget widget;
  Arg args[8];
  int count;
  Pixmap bitmap;
  Screen *screen;

  widget = WcFullNameToWidget( XmonetTopWidget, name );
  if ( widget == NULL ) {
    MonetPrintfError ("No such widget [%s]", name);
    return False;
  }
  if (!XtIsRealized(widget))
    XtRealizeWidget(widget);

  screen = XtScreen (widget);
  count = 0;
  XtSetArg(args[count], XtNlabel, ""); count++;
  if (set_sides) {
    XtSetArg(args[count], XtNheight, height); count++;
    XtSetArg(args[count], XtNwidth, width); count++;
  }
  XtSetValues(widget, args, count);
  bitmap = XCreatePixmapFromBitmapData(XtDisplay(widget),
                                       XtWindow(widget), bits,
                                       width, height,
                                       BlackPixelOfScreen(screen),
                                       WhitePixelOfScreen(screen),
                                       DefaultDepthOfScreen(screen));

  count = 0;
  XtSetArg(args[count], XtNbackgroundPixmap, bitmap); count++;
/*  XtSetArg(args[count], XtNresizable, FALSE); count++; */
  XtSetValues(widget, args, count);

  XtAddCallback(widget, XtNdestroyCallback,
                FreeWidgetPixmap, (XtPointer) bitmap);
  return True;
}

extern void XMonetPopupCentral (parent, popup, grab)
	Widget parent, popup;
	XtGrabKind grab;
{
	Position x,y;
	Dimension width, height;
	Arg             args[10];

	if (!XtIsRealized(popup))
		XtRealizeWidget(popup);

	if (!XtIsRealized(parent))
		XtRealizeWidget(parent);

	width = popup->core.width;
	height = popup->core.height;
	if (width == 0)
	  width = 300;
	if (height == 0)
	  height = 100;

	x = parent->core.x + parent->core.width /2  - width /2;
	y = parent->core.y + parent->core.height /2 - height /2;


	XtSetArg(args[0], XtNx, x);
	XtSetArg(args[1], XtNy, y);
	XtSetArg(args[2], XtNwidth, width);
	XtSetArg(args[3], XtNheight, height);
	XtSetArg(args[4], XtNresizable, False);
	XtSetValues(popup, args, 5);
	XtPopup (popup, grab);
}

/***********************************************************************/
/*  Execute a monet script command  */
/***********************************************************************/

CALLBACK(MonetScriptCommandCB)
     String command = (String) client_data;

     XmonetCurrentEvent = NULL;
     XmonetCurrentWidget = widget;
     (void) MonetshExecute (command, NULL, False);
     XmonetCurrentWidget = NULL;
ENDCB


ACTION(MonetScriptCommandACT, 1)
     char * command;
     command = MonetJoinStrings (*num_params, params, " ");
     XmonetCurrentEvent = event;
     XmonetCurrentX = XmonetCurrentEvent->xbutton.x;
     XmonetCurrentY = XmonetCurrentEvent->xbutton.y;
     XmonetCurrentWidget = widget;
     (void) MonetshExecute (command, NULL, False);
     XmonetCurrentWidget = NULL;
     XmonetCurrentEvent = NULL;
ENDACT

/*****************************************************************************/
/*  Register callbacks and actions  */
/*****************************************************************************/

static void register_actions ()
{
#define NUM_ACTIONS 2

  static XtActionsRec actions[NUM_ACTIONS] = {
    {"MonetACT", MonetScriptCommandACT},
    {"MonetScriptCommandACT", MonetScriptCommandACT},
  };

  (void) XtAppAddActions(XmonetAppCon, (XtActionList) actions,
			 (Cardinal) NUM_ACTIONS);
}

static void register_callbacks ()
{
/*  ONCE_PER_XtAppContext( XmonetAppCon ); */
  /* -- Command interpreters  */
  RCALL("MonetCB",	MonetScriptCommandCB,	NULL);
  /*  !!! Throw this synonym away  replaced by "MonetCB"  */
  RCALL("MonetScriptCommandCB",	MonetScriptCommandCB,	NULL);
}

/*****************************************************************************/
/*  Resource Converters  from Xmu*/
/*****************************************************************************/

static XtConvertArgRec res_screenConvertArg[] = {
    {XtBaseOffset, (caddr_t) XtOffset(Widget, core.screen), sizeof(Screen *)}
};

static void register_converters ()
{
  XtAddConverter( XtRString, XtRBackingStore, XmuCvtStringToBackingStore,
		 NULL, 0 );
  XtAddConverter (XtRString, XtRBitmap, XmuCvtStringToBitmap,
		  res_screenConvertArg, XtNumber(res_screenConvertArg));
  /* -- Converting pixmaps, this is needed for
     -- backgroundPixmap and borderPixmap.
     -- Could use XmuCvtStringToBitmap but this does not work for
     -- monocrome or colour screens.  */
  XtAddConverter (XtRString, XtRPixmap, XmonetCvtStringToPixmap,
		  res_screenConvertArg, XtNumber(res_screenConvertArg));
}

/*****************************************************************************/
/* Start X client                                                            */
/* !!! Should probably load fonts and such like at this stage                */
/*****************************************************************************/

extern MonetBool XMonet_init_x (appClass, argc, argv)
     char*        appClass;
     int             *argc;
     char          **argv;
{
  XSetErrorHandler (myhandler);
  /* initialize first letter to make class, or first two if
   ** first is already capitalized, or don't worry about it.
   */
  if (islower(appClass[0]))
    appClass[0] = toupper(appClass[0]);
  else if (islower(appClass[1]))
    appClass[1] = toupper(appClass[1]);
    
  /* With the Athena widgets, we MUST register the Wc
   ** converters first, so WcCvtStringToWidget is invoked
   ** instead of the far less useful XmuCvtStringToWidget.
   */
  WcAddConverters(NULL);
  register_converters ();

  /*  -- Intialize Toolkit creating the application shell */
  XmonetTopWidget = XtInitialize
    (argv[0], appClass,		/* app name and class */
     NULL, 0, 			/* description of cmd line options */
     argc, argv 
     );
  if (XmonetTopWidget == NULL) {
    MonetPrintfError ("Could not create XmonetTopWidget shell");
    return False;
  }

  XmonetDisplay = XtDisplay (XmonetTopWidget);
  XmonetAppCon = XtWidgetToApplicationContext(XmonetTopWidget);
  if (XmonetAppCon == NULL) {
    MonetPrintfError ("Could not create application context");
    return False;
  }

  return True;
}

extern MonetBool XMonet_init ()
{

  MonetLibrary_begin (monetlib, "xmonet", "X11 Utilities ");

  WcAddConverters(XmonetAppCon);

  register_converters ();
  register_actions ();
  register_callbacks ();

  /*  -- Register all Athena widget classes */
  AriRegisterAthena ( XmonetAppCon );

#ifdef HAS_MOTIF
  /*  -- Register all Motif classes and constructors */
  MriRegisterMotif ( XmonetAppCon );
#endif /* HAS_MOTIF */

  /*  -- Init drawing routines  */
  xmonet_io_init ();
  xmonet_init_draw_X11();
  xmonet_init_script ();
  xmonet_confirm_init ();
  xmonet_init_cursors ();

  MonetLibrary_done ();

  return True;
}

/****************************************************************************
  Handles monet events
  ****************************************************************************/

static XtIntervalId Interval;
static MonetBool IsInterval;

static void work_proc (dummy, interval)
     XtPointer dummy;
     XtIntervalId * interval;
{
  struct timeval timer;
  timerclear (&timer);
  timer.tv_sec = 3600;  /* -- This is an arbitrarily large number
			   -- MonetEvent_next will reset the tv_sec
			   -- with the time until the next event
			   -- provided this time is less than 3600 seconds.
			   */
  if (MonetEvent_next (&timer) == -1) {
    /*  -- No more events to handle so remove this work proc  */
    IsInterval = False;
  } else {
    IsInterval = True;
    Interval = XtAppAddTimeOut (XmonetAppCon,
				(unsigned long) timer.tv_sec,
				work_proc,
				(XtPointer) 0);
  }
}

/*****************************************************************************/
/* Top Level Loop                                                    */
/*****************************************************************************/

extern void XMonet_EventLoop()
{
  XEvent          event;

  XtRealizeWidget(XmonetTopWidget);
  /* -- this ensures that all requests are sent without this line,
     -- linking with cc means that the requests are not all sent */
  XFlush(XtDisplay(XmonetTopWidget));

  IsInterval = False;

  for (;;) {
    /*  -- check for dead children, do this before the event handling because
	-- this may cause some actions itself.  */
    MonetSystemHandleChildren ();

    /*  -- Handle the next monet event if there is one,
	-- if there are any following then set up an alarm that will
	-- be called by XtAppNextEvent  */
    work_proc ((XtPointer) 0, (XtIntervalId *) 0);
  
    /* -- Next queued event */
    XtAppNextEvent(XmonetAppCon, &event);

    /*  -- Remove the monet event timer,
	-- this will be reset next time around the loop  */
    if (IsInterval) {
      XtRemoveTimeOut (Interval);
      IsInterval = False;
    }

    /* -- Actually perform the event  */
    XtDispatchEvent(&event);


  }
  /* XtAppMainLoop(XmonetAppCon); */
  /* NOTREACHED */
}

/*****************************************************************************/
/*****************************************************************************/
/* $Header$ */
/*****************************************************************************/
/*****************************************************************************/

